/*****************************************************************************/

/*
 *      playfile.c -- REALmagic trivial file player example
 *
 *      Copyright (C) 2000 Sigma Designs
 *                   written by Emmanuel Michon
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#define _DEFINE_FMP_TYPES_
#include "fmp.h"

int main(int argc,char **argv)
{
	int fd;
	int streamtype;
	int bytesread=1,rc=FMPE_OK;
	
	if (argc!=3) {
		fprintf(stderr,"Syntax is playfile <streamtype> <filename>\n"
		       "<streamtype=t (transport) d (dvd) s (system)\n\n"
		       "Example: ./playfile s /samplempeg/movie.mpg\n");
		exit(1);
	}

	switch (argv[1][0]) {
	case 't':
		streamtype=FMPF_TRANSPORT;
		break;
	case 'd':
		streamtype=FMPF_DVD;
		break;
	case 's':
		streamtype=FMPF_SYSTEM;
		break;
	default:
		printf("Syntax is playfile <streamtype> <filename>\n<streamtype=t (transport) d (dvd) s (system)\n");
		exit(1);
	}

	MPEGDriverEntry(NO_DRIVE);
//	MPEGDriverEntry(5);
	FMPSet(FMPI_VIDEOOUT,FMPV_VIDEOOUT_PAL);
	FMPOpen(streamtype,32*1024,8,NULL,0);
	FMPPlay();

        fd=open(argv[2],O_RDONLY);
	if (fd==-1) {
		perror("open failed"); 
		exit(1); 
	}

	while (rc==FMPE_OK) {
		while (bytesread) {
			FMP_BUFFER FMPBuf;
			static int count=0;
			
			// get a pointer to a free buffer
			rc = FMPGetBuffer (&FMPBuf, TRUE);
			if (rc != FMPE_OK) {
				printf ("FMPError : %s\n", FMPGetLastError(rc));
				break;
			}	
			
			// signals can cause harmless EINTR
			while (((bytesread=read(fd,FMPBuf.pBuffer,FMPBuf.dwBufferSize))==-1)
			       &&(errno==EINTR));
			if (bytesread==-1) bytesread=0;
			
			FMPBuf.dwDataSize=bytesread;
			
			// the buffer is ready to be queued
			FMPPush (&FMPBuf);
			
			count++;
#define ONEEVERY 20			
			if (count%ONEEVERY==0) printf("PTS %ld / STC %ld\n",
						      FMPGet (FMPI_PTS),
						      FMPGet (FMPI_STC));
		}
		
		lseek (fd,0,SEEK_SET);
		bytesread=1;
	}
	
	// never reached
	close (fd);
	exit(0);
}
