/*****************************************************************************/

/*
 *      callbacks.c -- REALmagic simple windowing application in GTK
 *
 *      Copyright (C) 2000 Sigma Designs
 *                    written by Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *                    modified by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                                Pascal Cannenterre <pascal_cannenterre@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/


#include "define.h"
#include "gtk_callbacks.h"
#include "interface.h"

/*
 * ID for the connected function to the buuton
 * press GTK signal
 * I need to have acces to this gint in order to
 * disconnect the signal handler after the fullscreen mode
 */
gint id_fullscreen_cb_mouse;

// cannot #include <X11/extensions/xf86vmode.h> because BOOL and BYTE conflict.
Bool XF86VidModeGetViewPort(
			    Display *display,
			    int screen,
			    int *x_return,
			    int *y_return);

void loadconfig()
{  
	DWORD FMPreturn;
	
	G_MESSAGE("Ok clicked\nLoad parameters");

	getres(&xresolution,&yresolution,&depth,&freq);
	FMPreturn=LoadParamForThisResolution(xresolution,yresolution,depth,freq);
	
	G_MESSAGE("Load parameters: %ld",FMPreturn);
	
	if (FMPreturn==2) {
		printf("Please, make the autocalibration for this resolution before launching miniplayer\n");
		_exit(0);
	}
}

void fullscreen_on_cb (GtkWidget *widget, gpointer data)
{
  DWORD FMPreturn; 
  gint xof,yof;
 
  if(b_msg_connect_ready)
    {
      while (gdk_pointer_grab(window->window, TRUE, 
			      GDK_BUTTON_PRESS_MASK, 
			      NULL, NULL, 
			      GDK_CURRENT_TIME))
	sleep(1);
      
      id_fullscreen_cb_mouse = 
	gtk_signal_connect (GTK_OBJECT (window), "button-press-event",
			    GTK_SIGNAL_FUNC (fullscreen_off_cb),
			    NULL);
      /* 
	 id_fullscreen_cb_keyb = 
	 gtk_signal_connect (GTK_OBJECT (window), "key-press-event",
	 GTK_SIGNAL_FUNC (fullscreen_off_cb),
	 NULL); 
      */
      
      if (!XF86VidModeGetViewPort(dpy,scr,&xof,&yof)) 
	{ 
	  g_print("Oulllllllllaaaaaaaaaa !");
	  _exit(1);
	}

      if(!b_vga)
	{
	  // Switch output to VGA
	  FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_VGA);
	  b_vga = TRUE;
	}
      FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,1,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
      FMPreturn = FMPSetDestination ((DWORD)(0-xof),(DWORD)(0-yof),(DWORD)xresolution,(DWORD)yresolution);
    }
}

gboolean fullscreen_off_cb (GtkWidget *widget, GdkEvent *event,gpointer data)
{
  DWORD FMPreturn;  

  gtk_signal_disconnect (GTK_OBJECT (window),id_fullscreen_cb_mouse);
  // gtk_signal_disconnect (GTK_OBJECT (window),id_fullscreen_cb_keyb);
  gdk_pointer_ungrab(GDK_CURRENT_TIME);
  
  // Set overlay mode
  FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,2,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
  // Set key color
  FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_VGAKEY,0x004000,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
  // loadconfig();
  window_move_toplevel(window,NULL,NULL);
  
  return TRUE;
}

gboolean
window_move_toplevel                            (GtkWidget       *widget,
					GdkEventConfigure *event,
					gpointer         user_data)
{
  gint x,y,xof,yof,width,height;
  GtkWidget * draw;
  DWORD FMPreturn;
  draw = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (widget), "drawingarea"));
  gdk_window_get_origin(draw->window , &x ,&y);
  gdk_window_get_size(draw->window ,&width,&height);

  G_MESSAGE("New position window x=%d y=%d size w = %d  h = %d\n",x,y,width,height);
  // be careful of viewport.
  if (!XF86VidModeGetViewPort(dpy,scr,&xof,&yof)) _exit(1);
  // change the overlay position and overlay size
  if(b_msg_connect_ready == TRUE)
    {
      FMPreturn = FMPSetDestination ((DWORD)(x-xof),(DWORD)(y-yof),(DWORD)width,(DWORD)height);
    }
  G_MESSAGE("Change the window size or position return: %ld",FMPreturn);
  return FALSE;
}

gboolean
window_move_drawingarea                            (GtkWidget       *widget,
					GdkEventConfigure *event,
					gpointer         user_data)
{
  gint x,y,xof,yof,width,height;
  // GtkWidget *window;
  DWORD FMPreturn;
  // window = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (widget), "drawingarea"));
  gdk_window_get_origin(widget->window , &x ,&y);
  gdk_window_get_size(widget->window ,&width,&height);

  G_MESSAGE("New position window x=%d y=%d size w = %d  h = %d\n",x,y,width,height);
  // be careful of viewport.
  if (!XF86VidModeGetViewPort(dpy,scr,&xof,&yof)) _exit(1);
  // change the overlay position and overlay size
  if(b_msg_connect_ready == TRUE)
    {
      FMPreturn = FMPSetDestination ((DWORD)(x-xof),(DWORD)(y-yof),(DWORD)width,(DWORD)height);
    }
  G_MESSAGE("Change the window size or position return: %ld",FMPreturn);
  return FALSE;
}

/* Get the selected filename and print it to the console */
void file_ok_sel( GtkWidget        *w,
                  GtkFileSelection       *fs)
{
  char * filename;
  DWORD FMPreturn;

  gtk_widget_hide(GTK_WIDGET(fs));
  if(b_playing == TRUE)
    {
      G_MESSAGE("\n\n\n     Appel CB OPEN     OVERLAY\n\n\n");
      switch_overlay_back();
      G_MESSAGE("\n\n\n     Appel CB OPEN     SHUTDOWN\n\n\n");
      FMPCloseFile();
    }
  filename = (char *) (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  
  if(filename == "" || filename == NULL || open(filename,O_RDONLY) == -1)
    {
      // g_print("\nY'a probleme de fichier ...\n");
      gtk_message_box("Error","File error",
		      g_strconcat("The file ",(gchar *) filename," couldn't be opened ! Sorry.",NULL));
    }
  else
    {
      b_playing = TRUE;
      
      FMPreturn = FMPOpen(getStreamType(filename),32*1024,8,NULL,0);
      waitForAnalogOverlayAccess();    
      
      b_msg_connect_ready = TRUE;
      
      PLAYER_mode = "PLAY";
      // Test the VGA cable
      if (!FMPAnalogOverlay(FMP_ANALOG_OVERLAY_GET,FMP_ANALOG_OVERLAY_ATTRIBUTE_OVERLAY_CABLE_PRESENT,
			    UNUSED_ARG,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG)) 
	{
	  fprintf(stderr,"Please attach VGA cable.\n");
	  _exit(1);
	}
      // Switch output to VGA
      FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_VGA);
      b_vga = TRUE;
      // Set overlay mode
      FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,2,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
      // Set key color
      FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_VGAKEY,0x004000,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
      loadconfig();
      
      FMPPlayFile(filename);
      gtk_window_set_title (GTK_WINDOW (window),filename);
      
      /*** RESIZING THE TOP-LEVEL WINDOW ***/
      window_move_toplevel(window,NULL,NULL);
      gtk_widget_destroy(GTK_WIDGET(fs));
      filew = NULL;
    }
}

/* Obligatory basic callback */
void print_hello( GtkWidget *w,
                         gpointer   data )
{
  g_message ("Hello, World!\n");
}


void switch_overlay_back()
{
  DWORD FMPreturn;
  if(b_msg_connect_ready == TRUE)
    {
      // Switch back overlay mode
      FMPreturn = FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,0,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
      // Switch back output to TV
      FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_TV);
      b_vga = FALSE;
    }
}

void close_cb (GtkWidget *widget, gpointer data)
{ 

  DWORD FMPreturn;

  if(b_playing == TRUE) 
    { 
      PLAYER_mode = "STOP";
      switch_overlay_back();
      b_msg_connect_ready = FALSE; 
      b_playing = FALSE; 
      FMPreturn = FMPCloseFile();
      gtk_window_set_title (GTK_WINDOW (window), "RealMagic GTK FilePlayer");
  }
  G_MESSAGE("\nCB CLOSE ...\n");
}

/** Toolbar a partir d'une fenetre et d'un packer existant dans cette fenetre */
void play_cb (GtkWidget *widget, gpointer data)
{
  DWORD FMPreturn;
  
  if(b_playing == FALSE)
    get_file_selector(window);
  else if(PLAYER_mode != "PLAY")
    {
      PLAYER_mode = "PLAY"; 
      FMPreturn = FMPPlay();
    }
}

void stop_cb (GtkWidget *widget, gpointer data)
{
	 DWORD FMPreturn;
	 
	 if(PLAYER_mode != "STOP" && b_playing == TRUE) 
	   { 
	     if (PLAYER_mode == "PAUSE")
	       FMPreturn = FMPPlay();
	     FMPreturn = FMPStopFile();
	     PLAYER_mode = "STOP";
	   }
	 G_MESSAGE("\nCB STOP ...\n");
}

void pause_cb (GtkWidget *widget, gpointer data)
       {
	 DWORD FMPreturn;
	 if(b_playing == TRUE)
	   {
	     if(PLAYER_mode == "PAUSE")
	       {
		 FMPreturn = FMPPlay();
		 PLAYER_mode = "PLAY";
		 G_MESSAGE("\nCB PLAY ...\n");
	       }
	     else if(PLAYER_mode == "PLAY")
	       {
		 FMPreturn = FMPPause ();
		 PLAYER_mode = "PAUSE";
		 G_MESSAGE("\nCB PAUSE ...\n");
	       }
	     else if(PLAYER_mode != "STOP") 
	       {
		 FMPreturn = FMPPause ();
		 PLAYER_mode = "PAUSE";
		 G_MESSAGE("\nCB PAUSE ...\n");
	       }
	   }
       }
 
 void toolbar_cb (GtkWidget *widget, gpointer data)
   {
     g_print("\nOk, dockey !!\n");
   }


void switch_TVout_cb(GtkWidget *widget, gpointer data)
{
  DWORD FMPreturn;
  
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      if( b_vga == TRUE)
	{
	  FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_TV);
	  b_vga = FALSE;
	}
    } 
  else 
    { 
      // If control reaches here, the toggle button is up 
      if(b_vga == FALSE)
	{
	  FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_VGA);
	  b_vga = TRUE;  
	}
    }
}

void set_mute(const BOOL mute)
{
  DWORD FMPreturn;
  FMPreturn = FMPSet(FMPI_MUTE, mute);
}

void updateAudio(int volume, int balance)
{
	DWORD volumeleft;
  DWORD volumeright;
	DWORD percentleft;
  DWORD percentright;

  DWORD FMPreturn;

	volumeleft=FMPGet(FMPI_LEFT_VOLUME);
	volumeright=FMPGet(FMPI_RIGHT_VOLUME);

	G_MESSAGE("Old volume :\nleft : %ld right : %ld",volumeleft,volumeright);
	G_MESSAGE("Balance : %d Volume %d",balance,volume);

	if (balance<=0){
		percentleft=100;
		percentright=100+balance;
	}
	if (balance>0){
		percentright=100;
		percentleft=100-balance;
	}

	volumeleft =(100-volume)*percentleft/100;
	volumeright=(100-volume)*percentright/100;
	
	G_MESSAGE("New volume :\nleft : %ld right : %ld",volumeleft,volumeright);

	FMPreturn=FMPSet(FMPI_LEFT_VOLUME,volumeleft);

#ifdef DEBUG
	G_MESSAGE("Set new volume left return %ld",FMPreturn);
	FMPreturn=FMPGet(FMPI_LEFT_VOLUME);
	G_MESSAGE("Get new volume left return %ld",FMPreturn);
#endif
	FMPreturn=FMPSet(FMPI_RIGHT_VOLUME,volumeright);

#ifdef DEBUG
	G_MESSAGE("Set new volume right return %ld",FMPreturn);
	FMPreturn=FMPGet(FMPI_RIGHT_VOLUME);
	G_MESSAGE("Get new volume right return %ld",FMPreturn);
#endif
}

void set_TVmode(DWORD  newtvmode) 
{
  DWORD tvmode, FMPreturn;
  //    newtvmode = FMPV_VIDEOOUT_NTSC;
  //    newtvmode = FMPV_VIDEOOUT_PAL;
  //    newtvmode = FMPV_VIDEOOUT_PAL60;
  //    newtvmode = FMPV_VIDEOOUT_PAL
  tvmode = FMPGet(FMPI_VIDEOOUT);
  tvmode = tvmode & (~FMPV_VIDEOOUT_STANDARDTV_MASK); // reset
  tvmode = tvmode | newtvmode;                        // set

  FMPreturn = FMPSet(FMPI_VIDEOOUT, tvmode);
}

void volume_changed( GtkAdjustment *volume, GtkAdjustment *balance)
{
	updateAudio((int)volume->value,(int)balance->value);
}


void balance_changed( GtkAdjustment *balance,GtkAdjustment *volume)
{
	updateAudio((int)volume->value,(int)balance->value);
}


void brightness_changed( GtkAdjustment *adj,gpointer   data )
{
	DWORD FMPreturn;

	G_MESSAGE("Brightness changed new brightness %d",1000-(unsigned int)adj->value);
	FMPreturn=FMPSet(FMPI_BRIGHTNESS,1000-(unsigned int)adj->value);
	G_MESSAGE("set new brightness return: %ld",FMPreturn);
}



void contrast_changed( GtkAdjustment *adj,gpointer   data )
{
	DWORD FMPreturn;

	G_MESSAGE("Contrast changed new contrast %d",1000-(unsigned int)adj->value);
	FMPreturn=FMPSet(FMPI_CONTRAST,1000-(unsigned int)adj->value);
	G_MESSAGE("set new contrast return: %ld",FMPreturn);
}



void saturation_changed( GtkAdjustment *adj,gpointer   data )
{
	DWORD FMPreturn;
	
	G_MESSAGE("Saturation changed new saturation %d",1000-(unsigned int)adj->value);
	FMPreturn=FMPSet(FMPI_SATURATION,1000-(unsigned int)adj->value);
	G_MESSAGE("set new saturation return: %ld",FMPreturn);
}

void C_Ok_clicked( GtkWidget *widget, gpointer   data )
{
	GtkWidget *window;
	window = gtk_widget_get_toplevel(widget);
	/* we used to save the settings, but things have changed ! */
	gtk_widget_destroy(window);
	Configuration = NULL;
}
