/*****************************************************************************/

/*
 *      interface.c -- REALmagic simple windowing application in GTK
 *
 *      Copyright (C) 2000 Sigma Designs
 *                    written by Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *                    modified by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                                Pascal Cannenterre <pascal_cannenterre@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include "define.h"

#include "gtk_callbacks.h"
#include "interface.h"

/* include pixmaps */
#include "./pixmaps/play.xpm"
#include "./pixmaps/stop.xpm"
#include "./pixmaps/pause.xpm"
#include "./pixmaps/frewind.xpm"
#include "./pixmaps/fforward.xpm"
#include "./pixmaps/fileopen.xpm"
#include "./pixmaps/next.xpm"
//#include "./pixmaps/prev.xpm"
//#include "./pixmaps/REALmagic-mini.xpm"
#include "./pixmaps/REALmagic.xpm"
#include "./pixmaps/audiol.xpm"
#include "./pixmaps/audior.xpm"
#include "./pixmaps/brightns.xpm"
#include "./pixmaps/contrast.xpm"
#include "./pixmaps/saturatn.xpm"
#include "./pixmaps/logo.xpm"

GtkWidget *filew = NULL;
GtkWidget *Configuration = NULL;

void my_gtk_widget_destroy(GtkWidget ** widget)
{
  gtk_widget_destroy(*widget);
  *widget = NULL;
}

GtkWidget* create_pixmap(GtkWidget *widget, char ** filename)
{
  GtkWidget *pixmap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GdkColormap *colormap;
  
  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask, NULL, filename );
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  
  return pixmap;
}

void set_window_icon(GtkWidget *widget, char ** pixmap_data)
{
	GdkPixmap *gdk_pixmap;
	GdkBitmap *gdk_mask;
      
 	gtk_widget_realize(widget);

	gdk_pixmap = gdk_pixmap_create_from_xpm_d (widget->window, &gdk_mask, NULL,pixmap_data);

	gdk_window_set_icon(widget->window, NULL, gdk_pixmap, gdk_mask);
}

void gtk_message_box(gchar * window_title, gchar * frame_title, gchar * msg)
{
  static GtkWidget *msg_window = NULL;
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *button;

  msg_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  
  gtk_signal_connect_object (GTK_OBJECT (msg_window), "destroy",
			     GTK_SIGNAL_FUNC(my_gtk_widget_destroy),
			     (GtkObject*)&msg_window);
  
  gtk_window_set_title (GTK_WINDOW (msg_window), window_title);
  gtk_window_set_position (GTK_WINDOW (msg_window), GTK_WIN_POS_CENTER);

  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (msg_window), vbox);
  gtk_container_set_border_width (GTK_CONTAINER (msg_window), 5);
  
  frame = gtk_frame_new (frame_title);
  label = gtk_label_new (msg);

  // gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_container_add (GTK_CONTAINER (frame), label);
  gtk_container_set_border_width (GTK_CONTAINER (frame), 5);

  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
  
  button = gtk_button_new_with_label ("Close");
  
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC(my_gtk_widget_destroy),
			     (GtkObject*)&msg_window);
  
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 5);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);
  
  gtk_window_set_modal(GTK_WINDOW (msg_window),TRUE);
  
  gtk_widget_show_all (msg_window);
} 

GtkWidget *get_toolbar(GtkWidget *parent, GtkWidget *vbox_parent)
{
  GtkWidget * frewind_button, *play_button, *stop_button, *pause_button, *fforward_button;
  GtkWidget* handlebox;
  
  /* Ok, we need a toolbar, an icon with a mask (one for all of 
     the buttons) and an icon widget to put this icon in (but 
     we'll create a separate widget for each button) */
  GtkWidget * toolbar;
  GtkStyle * style;
  GdkPixmap * icon_play, *icon_stop, *icon_pause, *icon_fforward, 
    *icon_frewind;
  GdkBitmap * mask_play, *mask_stop, *mask_pause, *mask_fforward,
    *mask_frewind;
  GtkWidget * playw, *stopw, *pausew, *fforwardw, *frewindw;
  
  /* to make it nice we'll put the toolbar into the handle box, 
   * so that it can be detached from the main window */
  handlebox = gtk_handle_box_new ();
  gtk_box_pack_end ( GTK_BOX (vbox_parent),
		       handlebox, FALSE, FALSE,0);
  /* toolbar will be horizontal, with both icons and text, and
   * with 5pxl spaces between items and finally, 
   * we'll also put it into our handlebox */
  toolbar = gtk_toolbar_new ( GTK_ORIENTATION_HORIZONTAL,
			      GTK_TOOLBAR_ICONS);
  gtk_container_set_border_width ( GTK_CONTAINER ( toolbar ) ,0);
  gtk_toolbar_set_space_size ( GTK_TOOLBAR ( toolbar ), 5 );
  gtk_container_add ( GTK_CONTAINER ( handlebox ) , toolbar );
  
  /* now we create icon with mask: we'll reuse it to create
   * icon widgets for toolbar items */
  /* Get the style of the button to get the */
  /* background color. */
  style = gtk_widget_get_style(parent);
  
  /* Now on to the xpm stuff */
  
  icon_frewind = gdk_pixmap_create_from_xpm_d (parent->window, &mask_frewind,
					     &style->bg[GTK_STATE_NORMAL],
					       frewind_xpm);
  frewindw = gtk_pixmap_new (icon_frewind, mask_frewind);
  
  icon_play = gdk_pixmap_create_from_xpm_d (parent->window, &mask_play,
					  &style->bg[GTK_STATE_NORMAL],
					    play_xpm);
  playw = gtk_pixmap_new (icon_play, mask_play);
  
  icon_stop = gdk_pixmap_create_from_xpm_d (parent->window, &mask_stop,
					  &style->bg[GTK_STATE_NORMAL],
					    stop_xpm);
  stopw = gtk_pixmap_new (icon_stop, mask_stop);
  
  icon_pause = gdk_pixmap_create_from_xpm_d (parent->window, &mask_pause,
						 &style->bg[GTK_STATE_NORMAL],
					     pause_xpm);
  pausew = gtk_pixmap_new (icon_pause, mask_pause);
  
  icon_fforward = gdk_pixmap_create_from_xpm_d (parent->window, &mask_fforward,
						  &style->bg[GTK_STATE_NORMAL],
						fforward_xpm);
  fforwardw = gtk_pixmap_new (icon_fforward, mask_fforward);
  
  frewind_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
			      "F Rewind",               /* button label */
			      "F Rewind",   /* this button's tooltip */
			      "F Rewind",     /* tooltip private info */
			      frewindw,                 /* icon widget */
			      GTK_SIGNAL_FUNC (toolbar_cb), /* a signal */
			      NULL );
  
  play_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
			      "Play",               /* button label */
			      "Play file",   /* this button's tooltip */
			      "Play",          /* tooltip private info */
			      playw,                 /* icon widget */
			      GTK_SIGNAL_FUNC (play_cb), /* a signal */
			      NULL );
  stop_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
			      "Stop",               /* button label */
			      "Stop",   /* this button's tooltip */
			      "Stop",          /* tooltip private info */
			      stopw,                 /* icon widget */
			      GTK_SIGNAL_FUNC (stop_cb), /* a signal */
			      NULL );
  
  pause_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
			      "Pause",               /* button label */
			      "Pause",   /* this button's tooltip */
			      "Pause",          /* tooltip private info */
			      pausew,                 /* icon widget */
			      GTK_SIGNAL_FUNC (pause_cb), /* a signal */
			      NULL );
  fforward_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
			      "F Forward",               /* button label */
			      "F Forward",   /* this button's tooltip */
			      "F Forward",          /* tooltip private info */
			      fforwardw,                 /* icon widget */
			      GTK_SIGNAL_FUNC (toolbar_cb), /* a signal */
			      NULL );
  
  /*gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar));*//* space after item */
    
    
    gtk_widget_show (toolbar);
    
    return (handlebox);
}

/* Create a new hbox with an image and a label packed into it */
/* and return the box. */

GtkWidget *xpm_button( GtkWidget *parent,
		       gchar     * xpm_filename)
{
  GtkWidget *button1;
  GtkWidget *pixmapwid;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle *style;
  
  /* Create a new button */
  button1 = gtk_button_new ();
  
  /* Realize the button - to have his style... */
  /* gtk_widget_realize(button1); */
  /* Get the style of the button to get the */
  /* background color. */
  style = gtk_widget_get_style(parent);
  
  /* Now on to the xpm stuff */
  pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
				       &style->bg[GTK_STATE_NORMAL],
				       xpm_filename);
  pixmapwid = gtk_pixmap_new (pixmap, mask);
  
  gtk_container_add (GTK_CONTAINER (button1), pixmapwid);
  
  gtk_widget_show(pixmapwid);
    
    return( button1 );
}


/* The Gtk File Selector */
void get_file_selector( GtkWidget  *window )
{
/* I test if there is already a visible or an iconified window of the file selector */
 if (filew != NULL && filew->window != NULL)
   {
     gdk_window_show(filew->window);
     gdk_window_raise(filew->window);
   }
 else
   {
     
     /* Create a new file selection widget */
     filew = gtk_file_selection_new ("Open a file ...");

     /* Setting the file selector in a modal bahaviour */
     /*gtk_window_set_modal(filew,TRUE);*/

     gtk_signal_connect_object (GTK_OBJECT (filew), "destroy",
				(GtkSignalFunc) my_gtk_widget_destroy , (GtkObject*)&filew);
     /* Connect the ok_button to file_ok_sel function */
     gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_sel, filew );
     
     /* Connect the cancel_button to destroy the widget */
     gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION
					   (filew)->cancel_button),
			       "clicked", (GtkSignalFunc) my_gtk_widget_destroy,
			       (GtkObject*)&filew);
     
     /* Lets set the filename, as if this were a save dialog, and we are giving
     a default filename */
     
     gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), 
				      "sample.mpg");

     /* filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(filew));*/
     set_window_icon(filew,fileopen_xpm);
     gtk_widget_show(filew);
   }
}


/* This is the GtkItemFactoryEntry structure used to generate new menus.
   Item 1: The menu path. The letter after the underscore indicates an
           accelerator key once the menu is open.
   Item 2: The accelerator key for the entry
   Item 3: The callback function.
   Item 4: The callback action.  This changes the parameters with
           which the function is called.  The default is 0.
   Item 5: The item type, used to define what kind of an item it is.
           Here are the possible values:

           NULL               -> "<Item>"
           ""                 -> "<Item>"
           "<Title>"          -> create a title item
           "<Item>"           -> create a simple item
           "<CheckItem>"      -> create a check item
           "<ToggleItem>"     -> create a toggle item
           "<RadioItem>"      -> create a radio item
           <path>             -> path of a radio item to link against
           "<Separator>"      -> create a separator
           "<Branch>"         -> create an item to hold sub items (optional)
           "<LastBranch>"     -> create a right justified branch 
*/

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL, 0, "<Branch>" },
  { "/File/_Open",    "<control>O",get_file_selector, 0, NULL },
  { "/File/_Close",    "<control>C", close_cb, 0, NULL },
  { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/File/Quit",     "<control>Q", gtk_main_quit, 0, NULL },
  { "/_Controls",      NULL,         NULL, 0, "<Branch>" },
  { "/Controls/Play",  "P",     play_cb, 0, NULL },
  { "/Controls/Stop",  "S",     stop_cb, 0, NULL },
  {"/Controls/Pause",  "<alt>P",     pause_cb, 0, NULL },
  { "/_View",      NULL,         NULL, 0, "<Branch>" },
  { "/View/FullScreen","<control>F", fullscreen_on_cb, 0, NULL},
  { "/View/sep2",     NULL,         NULL, 0, "<Separator>" },
  { "/View/Options...","<alt>O", get_options_window, 0, NULL},
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About",   NULL,         NULL, 0, NULL },
};


void get_main_menu( GtkWidget  *window,
                    GtkWidget **menubar )
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */ 
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}


GtkWidget*
create_window(void)
{
  GtkWidget *window;
  GtkWidget *drawingarea;
  GtkWidget *main_vbox;
  GtkWidget *menubar;
  GtkWidget *handlebox;

  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), "RealMagic GTK FilePlayer");
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (window), 640, 480);

  drawingarea = gtk_drawing_area_new ();
  gtk_widget_ref (drawingarea);
  gtk_object_set_data_full (GTK_OBJECT (window), "drawingarea", drawingarea,
                            (GtkDestroyNotify) gtk_widget_unref);

  /* Connecting callbacks to the TOP-LEVEL Window and to the Drawingarea */
  gtk_signal_connect (GTK_OBJECT (drawingarea), "configure_event",
                      GTK_SIGNAL_FUNC (window_move_drawingarea),
                      NULL);

  
  gtk_signal_connect (GTK_OBJECT (window), "configure_event",
                      GTK_SIGNAL_FUNC (window_move_toplevel),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
                      GTK_SIGNAL_FUNC (gtk_main_quit),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (window), "destroy", 
		      GTK_SIGNAL_FUNC (gtk_main_quit), 
		      "WM destroy");
  
  set_window_icon(window,REALmagic_xpm);
  /* realize the TOP-LEVEL, it's needed for the pixmaps bg style in 
   * a included function */
  gtk_widget_realize (window);
  /* realize, map and show the drawing area */
  gtk_widget_show (drawingarea);

  /* V Box to pack the menubar, the drawing area and the toolbar */
  main_vbox = gtk_vbox_new (FALSE, 1);
  /* V Box's border width setting */
  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
  /* adding the V Box in the TOP-LEVEL window */
  gtk_container_add (GTK_CONTAINER (window), main_vbox);
  /* realize, map and show the V Box */
  gtk_widget_show (main_vbox);
  /* menubar by GtkItemFactory */
  get_main_menu (window, &menubar);
  /* pack into the V Box */
  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
  /* toolbar in a handlebox */
  handlebox = get_toolbar(window,main_vbox);
  /* pack into the V Box */
  gtk_box_pack_start (GTK_BOX (main_vbox), drawingarea, TRUE, TRUE, 0);
  /* showing the menubar and the hamdlebox with the toolbar */
  gtk_widget_show (menubar);
  gtk_widget_show (handlebox);
  
  return window;
}


void get_options_window()
{

  GtkWidget *frame;
  GtkWidget *box2;
  GtkWidget *radioTV, *radioVGA;
  GSList *group;

  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *vbox9;
  
  GtkWidget *hbox5;
  GtkWidget *frame1;
  GtkWidget *vbox10;
  GtkWidget *hbox6;
  GtkWidget *pixmap1;
  GtkWidget *volume;
  GtkWidget *pixmap2;
  GtkWidget *balance;
  GtkWidget *frame2;
  GtkWidget *table2;
  GtkWidget *pixmap5;
  GtkWidget *pixmap6;
  GtkWidget *pixmap7;
  GtkWidget *brightness;
  GtkWidget *saturation;
  GtkWidget *contrast;
  GtkWidget *vbox2;
  
  GtkWidget *logo_REALmagic;
  GtkWidget *hbuttonbox1;
  guint Ok_key;
  GtkWidget *Ok;
  guint Cancel_key;
  GtkWidget *Cancel;
  GtkAccelGroup *accel_group;

/* I test if there is already a visible or an iconified window of the file selector */
  if (Configuration!= NULL && Configuration->window != NULL)
    {
       gdk_window_show(Configuration->window);
       gdk_window_raise(Configuration->window);
    }
  else
    {
      accel_group = gtk_accel_group_new ();
      
      Configuration = gtk_window_new (GTK_WINDOW_TOPLEVEL);
      gtk_object_set_data (GTK_OBJECT (Configuration), "Configuration", Configuration);
      gtk_widget_set_usize (Configuration, 400, 315);
      gtk_window_set_title (GTK_WINDOW (Configuration), _("Options ..."));
      gtk_window_set_position (GTK_WINDOW (Configuration), GTK_WIN_POS_CENTER);
      gtk_window_set_policy (GTK_WINDOW (Configuration), FALSE, FALSE, FALSE);
      
      vbox1 = gtk_vbox_new (FALSE, 0);
      gtk_widget_ref (vbox1);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "vbox1", vbox1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (vbox1);
      gtk_container_add (GTK_CONTAINER (Configuration), vbox1);
      
      hbox1 = gtk_hbox_new (FALSE, 10);
      gtk_widget_ref (hbox1);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "hbox1", hbox1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (hbox1);
      gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
      gtk_container_set_border_width (GTK_CONTAINER (hbox1), 10);
      
      vbox9 = gtk_vbox_new (FALSE, 11);
      gtk_widget_ref (vbox9);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "vbox9", vbox9,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (vbox9);
      gtk_box_pack_start (GTK_BOX (hbox1), vbox9, TRUE, TRUE, 0);
      gtk_widget_set_usize (vbox9, 190, -2);
      
      hbox5 = gtk_hbox_new (TRUE, 10);
      gtk_widget_ref (hbox5);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "hbox5", hbox5,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (hbox5);
      gtk_box_pack_start (GTK_BOX (vbox9), hbox5, TRUE, TRUE, 0);
      
      frame1 = gtk_frame_new ("Volume");
      gtk_widget_ref (frame1);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "frame1", frame1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (frame1);
      gtk_box_pack_start (GTK_BOX (hbox5), frame1, FALSE, TRUE, 0);
      gtk_frame_set_label_align( GTK_FRAME(frame1), 0.5, 0.0);
      gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_ETCHED_OUT);
      
      vbox10 = gtk_vbox_new (FALSE, 10);
      gtk_widget_ref (vbox10);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "vbox10", vbox10,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (vbox10);
      gtk_container_add (GTK_CONTAINER (frame1), vbox10);
      gtk_container_set_border_width (GTK_CONTAINER (vbox10), 10);
      
      hbox6 = gtk_hbox_new (TRUE, 10);
      gtk_widget_ref (hbox6);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "hbox6", hbox6,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (hbox6);
      gtk_box_pack_start (GTK_BOX (vbox10), hbox6, TRUE, TRUE, 0);
      
      pixmap1 = create_pixmap (Configuration, audiol_xpm);
      gtk_widget_ref (pixmap1);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "pixmap1", pixmap1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (pixmap1);
      gtk_box_pack_start (GTK_BOX (hbox6), pixmap1, FALSE, FALSE, 0);

      volume = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (38.9, 0, 100, 1, 0, 0)));
      gtk_widget_ref (volume);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "volume", volume,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (volume);
      gtk_box_pack_start (GTK_BOX (hbox6), volume, FALSE, TRUE, 0);
      gtk_scale_set_draw_value (GTK_SCALE (volume), FALSE);
      
      pixmap2 = create_pixmap (Configuration, audior_xpm);
      gtk_widget_ref (pixmap2);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "pixmap2", pixmap2,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (pixmap2);
      gtk_box_pack_start (GTK_BOX (hbox6), pixmap2, FALSE, FALSE, 0);
      
      balance = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, -100, 100, 1, 0, 0)));
      gtk_widget_ref (balance);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "balance", balance,
                            (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (balance);
      gtk_box_pack_start (GTK_BOX (vbox10), balance, FALSE, FALSE, 0);
      gtk_widget_set_usize (balance, -2, 10);
      gtk_scale_set_draw_value (GTK_SCALE (balance), FALSE);
      gtk_scale_set_value_pos (GTK_SCALE (balance), GTK_POS_LEFT);
      gtk_scale_set_digits (GTK_SCALE (balance), 0);
      
      frame2 = gtk_frame_new ("Image B/S/C");
      gtk_widget_ref (frame2);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (frame2);
      gtk_box_pack_start (GTK_BOX (hbox5), frame2, TRUE, TRUE, 0);
      gtk_frame_set_label_align( GTK_FRAME(frame2), 0.5, 0.0);
      gtk_frame_set_shadow_type (GTK_FRAME (frame2), GTK_SHADOW_ETCHED_OUT);
      
      table2 = gtk_table_new (2, 3, FALSE);
      gtk_widget_ref (table2);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "table2", table2,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (table2);
      gtk_container_add (GTK_CONTAINER (frame2), table2);
      gtk_container_set_border_width (GTK_CONTAINER (table2), 10);

      pixmap5 = create_pixmap (Configuration, brightns_xpm);
      gtk_widget_ref (pixmap5);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "pixmap5", pixmap5,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (pixmap5);
      gtk_table_attach (GTK_TABLE (table2), pixmap5, 0, 1, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
      
      pixmap6 = create_pixmap (Configuration, saturatn_xpm);
      gtk_widget_ref (pixmap6);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "pixmap6", pixmap6,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (pixmap6);
      gtk_table_attach (GTK_TABLE (table2), pixmap6, 1, 2, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_FILL), 0, 0);
      
      pixmap7 = create_pixmap (Configuration, contrast_xpm);
      gtk_widget_ref (pixmap7);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "pixmap7", pixmap7,
                            (GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (pixmap7);
      gtk_table_attach (GTK_TABLE (table2), pixmap7, 2, 3, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_FILL), 0, 0);
      
      brightness = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 1000, 1, 0, 0)));
      gtk_widget_ref (brightness);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "brightness", brightness,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (brightness);
      gtk_table_attach (GTK_TABLE (table2), brightness, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
      gtk_scale_set_draw_value (GTK_SCALE (brightness), FALSE);
      
      saturation = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 1000, 1, 0, 0)));
      gtk_widget_ref (saturation);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "saturation", saturation,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (saturation);
      gtk_table_attach (GTK_TABLE (table2), saturation, 1, 2, 1, 2,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_FILL), 0, 0);
      gtk_scale_set_draw_value (GTK_SCALE (saturation), FALSE);
      
      contrast = gtk_vscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 1000, 1, 0, 0)));
      gtk_widget_ref (contrast);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "contrast", contrast,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (contrast);
      gtk_table_attach (GTK_TABLE (table2), contrast, 2, 3, 1, 2,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_FILL), 0, 0);
      gtk_scale_set_draw_value (GTK_SCALE (contrast), FALSE);
      
      vbox2 = gtk_vbox_new (FALSE, 0);
      gtk_widget_ref (vbox2);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "vbox2", vbox2,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (vbox2);
      gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
      
      /* ajoute ta frame debut */
      
      /* Create a Frame */
      frame = gtk_frame_new(NULL);
      gtk_box_pack_start (GTK_BOX (vbox2), frame, TRUE, TRUE, 0);
      /* Set the frame's label */
      gtk_frame_set_label( GTK_FRAME(frame), "VGA/TV out" );
      /* Align the label at the right of the frame */
      gtk_frame_set_label_align( GTK_FRAME(frame), 0.5, 0.0);
      /* Set the style of the frame */
      gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
      gtk_widget_show(frame);
      
      
      box2 = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
      gtk_container_add(GTK_CONTAINER(frame), box2);
      gtk_widget_show (box2);
  
      radioTV = gtk_radio_button_new_with_label (NULL, "TV");
      gtk_box_pack_start (GTK_BOX (box2), radioTV, TRUE, TRUE, 0);
      gtk_widget_show (radioTV);
      
      group = gtk_radio_button_group (GTK_RADIO_BUTTON (radioTV));
      radioVGA = gtk_radio_button_new_with_label(group, "VGA");
      gtk_box_pack_start (GTK_BOX (box2), radioVGA, TRUE, TRUE, 0);
      gtk_widget_show (radioVGA);
      
      if (b_vga == TRUE)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radioVGA), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radioTV), TRUE);
      
      /* ajoute ta frame fin */
      
      gtk_signal_connect(GTK_OBJECT (radioTV), "toggled",
			 GTK_SIGNAL_FUNC(switch_TVout_cb),
			 Configuration);

      logo_REALmagic = create_pixmap (Configuration, logo_xpm);
      gtk_widget_ref (logo_REALmagic);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "logo_REALmagic", logo_REALmagic,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (logo_REALmagic);
      gtk_box_pack_start (GTK_BOX (vbox2), logo_REALmagic, TRUE, TRUE, 0);
      
      hbuttonbox1 = gtk_hbutton_box_new ();
      gtk_widget_ref (hbuttonbox1);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "hbuttonbox1", hbuttonbox1,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (hbuttonbox1);
      gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, TRUE, TRUE, 0);
      gtk_widget_set_usize (hbuttonbox1, -2, 15);
      gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_SPREAD);
      
      Ok = gtk_button_new_with_label ("");
      Ok_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (Ok)->child),
				      _("_OK"));
      gtk_widget_add_accelerator (Ok, "clicked", accel_group,
				  Ok_key, GDK_MOD1_MASK, 0);
      gtk_widget_ref (Ok);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "Ok", Ok,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (Ok);
      gtk_container_add (GTK_CONTAINER (hbuttonbox1), Ok);
      gtk_widget_add_accelerator (Ok, "clicked", accel_group,
				  GDK_O, GDK_MOD1_MASK,
				  GTK_ACCEL_VISIBLE);
      
      Cancel = gtk_button_new_with_label ("");
      Cancel_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (Cancel)->child),
					  _("_Cancel"));
      gtk_widget_add_accelerator (Cancel, "clicked", accel_group,
				  Cancel_key, GDK_MOD1_MASK, 0);
      gtk_widget_ref (Cancel);
      gtk_object_set_data_full (GTK_OBJECT (Configuration), "Cancel", Cancel,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (Cancel);
      gtk_container_add (GTK_CONTAINER (hbuttonbox1), Cancel);
      gtk_widget_add_accelerator (Cancel, "clicked", accel_group,
				  GDK_C, GDK_MOD1_MASK,
				  GTK_ACCEL_VISIBLE);
      
      gtk_signal_connect_object (GTK_OBJECT (Configuration), "delete_event",
				 GTK_SIGNAL_FUNC (my_gtk_widget_destroy),
				 (GtkObject*)&Configuration);
      
      gtk_signal_connect_object (GTK_OBJECT (Configuration), "destroy",
				 GTK_SIGNAL_FUNC (my_gtk_widget_destroy),
				 (GtkObject*)&Configuration);
      
      gtk_signal_connect (GTK_OBJECT (Ok), "clicked",
			  GTK_SIGNAL_FUNC (C_Ok_clicked),
			  NULL);
      
      gtk_signal_connect_object (GTK_OBJECT (Cancel), "clicked",
				 GTK_SIGNAL_FUNC (my_gtk_widget_destroy),
				 (GtkObject*)&Configuration);
      
      gtk_window_add_accel_group (GTK_WINDOW (Configuration), accel_group);
      
      /* addon*/
      
      /* connect scales with her functions */
      gtk_signal_connect(GTK_OBJECT(GTK_RANGE(volume)->adjustment),    "value_changed", GTK_SIGNAL_FUNC(volume_changed),     GTK_OBJECT(GTK_RANGE(balance)->adjustment));
      gtk_signal_connect(GTK_OBJECT(GTK_RANGE(balance)->adjustment),   "value_changed", GTK_SIGNAL_FUNC(balance_changed),    GTK_OBJECT(GTK_RANGE(volume)->adjustment));
      gtk_signal_connect(GTK_OBJECT(GTK_RANGE(brightness)->adjustment),"value_changed", GTK_SIGNAL_FUNC(brightness_changed), NULL);
      gtk_signal_connect(GTK_OBJECT(GTK_RANGE(contrast)->adjustment),  "value_changed", GTK_SIGNAL_FUNC(contrast_changed),   NULL);
      gtk_signal_connect(GTK_OBJECT(GTK_RANGE(saturation)->adjustment),"value_changed", GTK_SIGNAL_FUNC(saturation_changed), NULL);
      
      /* set window icon*/
      set_window_icon(Configuration,REALmagic_xpm);
      
      loadconfig2(Configuration);
      
      /* end of addon*/
      gtk_widget_show(Configuration);
    }
  if(!b_msg_connect_ready)
    {
      gtk_widget_set_sensitive(frame,FALSE);
      gtk_widget_set_sensitive(frame1,FALSE);
      gtk_widget_set_sensitive(frame2,FALSE);      
    }

}

void loadconfig2( GtkWidget *widget)
{
	GtkAdjustment	*obj;
   
	// load contrast
	obj=GTK_RANGE(gtk_object_get_data (GTK_OBJECT (widget), "contrast"))->adjustment;
	gtk_adjustment_set_value(obj,(int) (1000-FMPGet(FMPI_CONTRAST)));

	// load saturation
	obj=GTK_RANGE(gtk_object_get_data (GTK_OBJECT (widget), "saturation"))->adjustment;
	gtk_adjustment_set_value(obj,(int) (1000-FMPGet(FMPI_SATURATION)));

	// load brightness
	obj=GTK_RANGE(gtk_object_get_data (GTK_OBJECT (widget), "brightness"))->adjustment;
	gtk_adjustment_set_value(obj,(int) (1000-FMPGet(FMPI_BRIGHTNESS)));

	// load balance
	obj=GTK_RANGE(gtk_object_get_data (GTK_OBJECT (widget), "balance"))->adjustment;
	// gtk_adjustment_set_value(obj,(int)(FMPGetProfileWord(RMREG_BALANCE)-100));
    
	// load volume
	obj=GTK_RANGE(gtk_object_get_data (GTK_OBJECT (widget), "volume"))->adjustment;
	// gtk_adjustment_set_value(obj,100-FMPGetProfileWord(RMREG_VOLUME));
 	
}
