/*****************************************************************************/

/*
 *      novaclient.c -- MPEG client on top of the NetStream 2000 driver
 *
 *      Copyright (C) 2000 Sigma Designs
 *                    written by Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *                    modified by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                                Pascal Cannenterre <pascal_cannenterre@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/
// non posix features of threads are used.
#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/user.h>
#include <sys/time.h>

#define _DEFINE_FMP_TYPES_
#include "fmp.h"

#include "novaclientserver.h"
#include "novaclient.h"

char defaultREALmagicdevice[] = DEFAULTREALMAGICDEVICE;
char *REALmagicdevice;

#define DEATHMESSAGE 0xdeaddead

int clientsidequeue=-1;

/* a good trick to debug semop (blocked is blocked!) */
#define MAXEINTRFAILURES 10000

#define EINTRPROOF( type , name , argswithparsandtypes, argswithoutparsandtypes... ) static type my_ ## name argswithparsandtypes \
{ \
	int rc,failurecount=0; \
	while (1) { \
		rc= name ( argswithoutparsandtypes ) ; \
		if (rc==-1) { \
			if (errno==EINTR) { \
				failurecount++; \
                                if (failurecount==MAXEINTRFAILURES) exit(1); \
	                } \
			else { perror( # name " failed" ); exit(1); } \
		} \
		else break; \
	} \
        return rc; \
} 
// DIRTY see above: we should return something useful as error code.
// warning: the function must return -1 according to perror style. This
// is not the case for pthreadcreate...!
// beware, these functions may also be called from RMPROXY/Class/Stream/os.h!
EINTRPROOF(int,msgsnd,(int msqid,struct msgbuf *msgp,int msgsz,int msgflg ),msqid,msgp,msgsz,msgflg)
EINTRPROOF(int,msgrcv,(int msqid,struct msgbuf *msgp,int msgsz,long msgtyp,int msgflg),msqid,msgp,msgsz,msgtyp,msgflg)

static void MsgSend(int msqid,char *msg,int length,long type)
{
	msgbigbuf B;

	if (length>MAXMSGLENGTH) _exit(1);

	B.mtype=type;
	memcpy(&B.mtext,msg,length);
	my_msgsnd(msqid,(struct msgbuf*)&B,length,REASONABLE_UMASK);   
}

static void MsgBlockingReceive(int msqid,char *msg,int length,long type)
{	
	msgbigbuf B;

	if (length>MAXMSGLENGTH) _exit(1);

	my_msgrcv(clientsidequeue,(struct msgbuf*)&B,length,type,REASONABLE_UMASK);
	memcpy(msg,&B.mtext,length);
}

DWORD FMPPlayFile(char * filename)
{
  ClientPlayRequest PQ;
  ServerAnswer A;
  memset (&PQ,0,sizeof(ClientPlayRequest));
  
  PQ.dwPlayCommand = FMP_PLAYFILE;
  strcpy(PQ.filename,filename);

  MsgSend(clientsidequeue,(char *)&PQ,sizeof(ClientPlayRequest),MSGFROMCLIENTTOPLAYFILESERVER);
  MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMPLAYFILESERVERTOCLIENT); 
  return A.dwReturn; 
}

DWORD FMPStopFile(void)
{
  ClientQuestion Q; 
  ServerAnswer A; 
  memset (&Q,0,sizeof(ClientQuestion)); 
  
  Q.dwCommand = FMP_STOPFILE;

  MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER);
  MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT);
  return A.dwReturn;
}

DWORD FMPCloseFile(void)
{
  ClientQuestion Q; 
  ServerAnswer A; 
  memset (&Q,0,sizeof(ClientQuestion)); 
  
  Q.dwCommand = FMP_CLOSEFILE;

  MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER);
  MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT);
  return A.dwReturn;
}

DWORD MPEGDriverUnload(void)
{
  ClientQuestion Q; 
  memset (&Q,0,sizeof(ClientQuestion)); 
  Q.dwCommand = FMP_DRIVERUNLOAD;
  MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER);
  return FMPE_OK;
}

#define FMPOPEN_TEMPLATE5(x,y) DWORD x (DWORD u1,DWORD u2,DWORD u3, PFMPCALLBACK u4,DWORD u5) \
{ \
        ClientQuestion Q; \
	ServerAnswer A; \
        memset (&Q,0,sizeof(ClientQuestion)); \
	\
        Q.dwCommand=y; \
	Q.dwArg1=u1; \
	Q.dwArg2=u2; \
	Q.dwArg3=u3; \
	Q.dwArg4= (DWORD) u4; \
	Q.dwArg5=u5; \
	MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
        \
	MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
	return A.dwReturn; \
}

FMPOPEN_TEMPLATE5(FMPOpen,FMP_OPEN)

#define TEMPLATEPLAY(x,y) DWORD x ( ) \
{ \
 ClientQuestion Q; \
 ServerAnswer A; \
 memset (&Q,0,sizeof(ClientQuestion)); \
  \
    Q.dwCommand=y; \
  \
  MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
  \
 MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
  return A.dwReturn; \
}

TEMPLATEPLAY(FMPPlay,FMP_PLAY)
TEMPLATEPLAY(FMPPause,FMP_PAUSE)
TEMPLATEPLAY(FMPStop,FMP_STOP)
TEMPLATEPLAY(FMPClose,FMP_CLOSE)
  
#define TEMPLATE1(x,y) DWORD x (DWORD u1) \
{ \
        ClientQuestion Q; \
	ServerAnswer A; \
        memset (&Q,0,sizeof(ClientQuestion)); \
	\
        Q.dwCommand=y; \
	Q.dwArg1=u1; \
	MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
        \
	MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
	return A.dwReturn; \
}

#define TEMPLATE2(x,y) DWORD x (DWORD u1,DWORD u2) \
{ \
        ClientQuestion Q; \
	ServerAnswer A; \
        memset (&Q,0,sizeof(ClientQuestion)); \
	\
        Q.dwCommand=y; \
	Q.dwArg1=u1; \
	Q.dwArg2=u2; \
	MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
        \
	MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
	return A.dwReturn; \
}

#define TEMPLATE4(x,y) DWORD x (DWORD u1,DWORD u2,DWORD u3,DWORD u4) \
{ \
        ClientQuestion Q; \
	ServerAnswer A; \
        memset (&Q,0,sizeof(ClientQuestion)); \
	\
        Q.dwCommand=y; \
	Q.dwArg1=u1; \
	Q.dwArg2=u2; \
	Q.dwArg3=u3; \
	Q.dwArg4=u4; \
	MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
        \
	MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
	return A.dwReturn; \
}

#define TEMPLATE6(x,y) DWORD x (DWORD u1,DWORD u2,DWORD u3,DWORD u4,DWORD u5,DWORD u6) \
{ \
        ClientQuestion Q; \
	ServerAnswer A; \
        memset (&Q,0,sizeof(ClientQuestion)); \
	\
        Q.dwCommand=y; \
	Q.dwArg1=u1; \
	Q.dwArg2=u2; \
	Q.dwArg3=u3; \
	Q.dwArg4=u4; \
	Q.dwArg5=u5; \
	Q.dwArg6=u6; \
	MsgSend(clientsidequeue,(char *)&Q,sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER); \
        \
	MsgBlockingReceive(clientsidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT); \
	return A.dwReturn; \
}

TEMPLATE1(FMPEnableFullScreen,FMP_ENABLE_FULLSCREEN)
TEMPLATE1(FMPGet,FMP_GET)
TEMPLATE1(FMPGetProfileWord,FMP_GET_PROFILE_WORD)
TEMPLATE2(FMPSet,FMP_SET)
TEMPLATE2(FMPWriteProfileWord,FMP_WRITE_PROFILE_WORD)
TEMPLATE4(FMPSetDestination,FMP_SET_DESTINATION)
TEMPLATE6(FMPAnalogOverlay,FMP_ANALOG_OVERLAY)

void FMPConnectToMessageQueue()
{
	int msgkey;
	
	REALmagicdevice=getenv(REALMAGICDEVICEFROMENV);
	if (REALmagicdevice==NULL)
		REALmagicdevice=defaultREALmagicdevice;
	
	msgkey=ftok(REALmagicdevice,IPC_PROJECT_KEY);
	if (msgkey==-1) { perror("ftok failed"); _exit(1); }
	
	clientsidequeue=msgget(msgkey,REASONABLE_UMASK);
	while (clientsidequeue==-1) {
		sleep(1);
		printf("FMPConnectToMessageQueue: trying again...\n");
		clientsidequeue=msgget(msgkey,REASONABLE_UMASK);
	}
	
	while (FMPAnalogOverlay(FMP_ANALOG_OVERLAY_ACCESS,0,0,0,0,0)!=FMPE_OK) {
		sleep(1);
		printf("FMPConnectToMessageQueue: not open...\n");
	}

	printf("FMPConnectToMessageQueue successful\n");
}

void FMPConnectToMSGQUEUE()
{
  int msgkey;
  
  REALmagicdevice=getenv(REALMAGICDEVICEFROMENV);
  if (REALmagicdevice==NULL)
    REALmagicdevice=defaultREALmagicdevice;
  
  msgkey=ftok(REALmagicdevice,IPC_PROJECT_KEY);
  if (msgkey==-1) { perror("ftok failed"); _exit(1); }
  
  clientsidequeue=msgget(msgkey,REASONABLE_UMASK);
  while (clientsidequeue==-1) {
    sleep(1);
    printf("FMPConnectToMessageQueue: trying again...\n");
    clientsidequeue=msgget(msgkey,REASONABLE_UMASK);
  }

}

void waitForAnalogOverlayAccess()
{
  
  while (FMPAnalogOverlay(FMP_ANALOG_OVERLAY_ACCESS,0,0,0,0,0)!=FMPE_OK) {
    sleep(1);
    printf("FMPConnectToMessageQueue: not open...\n");
  }
  
  printf("FMPConnectToMessageQueue successful\n");
}

void FMPDestroyMSGQUEUE()
{
// send messagequeuelistener a special message so that it dies.
  ClientQuestion Q;
  memset (&Q,0,sizeof(ClientQuestion));
  Q.dwCommand = DEATHMESSAGE;
  MsgSend(clientsidequeue,(char *)&Q,sizeof(Q),MSGFROMCLIENTTOSERVER);
}


#define ISO_END_CODE       0x1B9
#define SYSTEM_START_CODE  0x1BB
#define SYNC_BYTE		   0x47
#define EXTENSION_START_CODE 0xB5010000
#define PACK_START_CODE	0xBA010000
#define TRANSPORT_PACKET_LENGTH 0x00BC
#define SEQUENCE_HEADER	0xB3010000
#define AC3_HEADER		0x770B
#define AUDIO_MASK		0xF0FF
#define AUDIO_HEADER	0xF0FF
#define AUDIO_STREAM_MASK  0xC0010000
#define VIDEO_STREAM_MASK  0xE0010000
#define MIN_PACKET_HEADER_SEARCH 3000
#define DVD_PROGRAM_MUX_RATE 0xF8C38901
#define DVD_PACKET_SIZE 2048
#define UNDETERMINED_STREAM_TYPE 0xfeeddeaf

#define Malloc(type,n) (type *)malloc(n*sizeof(type))
#define BUFFER_SIZE 65536

DWORD max (DWORD a, DWORD b)
{
  if(a > b)
    return a;
  else
    return b;
}

DWORD findVOBheaderInBuffer(BYTE *pSearchBuffer, DWORD dwSearchBufferSize)
{
  DWORD dwBufferPos;
  BYTE	*pBuffer   = pSearchBuffer; 
  DWORD count = 0 ;

  for (dwBufferPos = 0; dwBufferPos < dwSearchBufferSize; dwBufferPos = dwBufferPos + DVD_PACKET_SIZE)
    {
      if ((*(DWORD *) (pBuffer + dwBufferPos)) == PACK_START_CODE) 
	{ 
	  if((*(DWORD *) (pBuffer + dwBufferPos + 10)) == DVD_PROGRAM_MUX_RATE)
	    {
	      count++;
	      if( count > 10)
		return FMPF_DVD;
	    }
	}
      else
	return UNDETERMINED_STREAM_TYPE;
    }
  return UNDETERMINED_STREAM_TYPE;
}

DWORD findHeaderInBuffer (BYTE *pSearchBuffer, DWORD dwSearchBufferSize)
{
  BYTE	*pBuffer   = pSearchBuffer;
  DWORD	dwCanBeSystem	 = 0;
  DWORD	dwCanBeSystem2	 = 0;
  DWORD	dwCanBeMPEGAudio = 0;
  DWORD	dwCanBeAC3Audio	 = 0;
  DWORD	dwCanBeVideo	 = 0;
  DWORD	dwCanBeVideo2	 = 0;

  DWORD dwBufferPos, dwCanbeMAX, dwAd;
  
  WORD wAudioStart = 0;
  
  BOOL bFound;
  DWORD count = 0;

  for (dwBufferPos = 0; dwBufferPos < dwSearchBufferSize; dwBufferPos++)
    {/* for#1 */
      /* debugging purpose ... */
      count++;
      if ((*(DWORD *) pBuffer) == PACK_START_CODE)
	{/* if PACK_... */
	  if ((* (pBuffer+4))>>6 == 0x01)
	    {
	      dwCanbeMAX = max (dwCanBeVideo, max( dwCanBeAC3Audio, dwCanBeMPEGAudio));
	      if (dwCanbeMAX > 8)
		{
		  /* May be not a system stream if we have found 8 headers before */
		  if (dwBufferPos == 0)
		    dwCanBeSystem2 += 6;	
		  else
		    dwCanBeSystem2 += 2; 
		}
	      else
		{
		  /* OK */
		  return FMPF_SYSTEM; /* mpeg2 system */
		}
	    }
	  else if ((* (pBuffer+4))>>4 == 0x02)
	    {
	      dwCanbeMAX = max (dwCanBeVideo, max( dwCanBeAC3Audio, dwCanBeMPEGAudio));
	      if (dwCanbeMAX > 8)
		{ 
		  /* May be not a system stream if we have found 8 headers before */
		  if (dwBufferPos == 0)
		    dwCanBeSystem += 6;	
		  else
		    dwCanBeSystem += 2;	
		}
	      else
		{
		  /* OK */
		  return FMPF_SYSTEM; /* mpeg system */
		}
	    }
	}
      else if ((*pBuffer) == SYNC_BYTE) 
	{/* SYNC_BYTE */
	  bFound = TRUE;
	  for (dwAd=0; dwBufferPos+dwAd < dwSearchBufferSize; dwAd+= TRANSPORT_PACKET_LENGTH)
	    {
	      bFound &= (*(pBuffer+dwAd) == SYNC_BYTE);
	      if (!bFound)
		break;
	    }
	  if ((bFound) && (dwAd > 10))
	    {
	      return FMPF_TRANSPORT; /* mpeg2 transport */
	    }
	}
      else if ((*(DWORD *) pBuffer) == SEQUENCE_HEADER)
	{/* SEQUENCE_HEADER  */
	  if (dwBufferPos == 0)
	    dwCanBeVideo += 6;	/* We need to wait at least a packet size... */
	  else
	    dwCanBeVideo += 2;	/* We need to wait at least a packet size... */
	}
      else if (dwCanBeVideo && ((*(DWORD *) pBuffer) == EXTENSION_START_CODE))
	{
	  dwCanBeVideo2 += 2;
	}
      else if (((*(WORD *) pBuffer)) == AC3_HEADER)
	{
	  if (dwBufferPos == 0)
	    dwCanBeAC3Audio += 6;	/* We need to wait at least a packet size... */
	  else
	    dwCanBeAC3Audio += 2;	/* We need to wait at least a packet size... */
	}
      else if (!dwCanBeVideo && (((*(WORD *) pBuffer) & AUDIO_MASK) == AUDIO_HEADER))
	{
	  if (dwCanBeMPEGAudio == 0)
	    {
	      wAudioStart = *(WORD *) pBuffer;
	    }
	  if (wAudioStart == *(WORD *) pBuffer)
	    {
	      if (dwBufferPos == 0)
		dwCanBeMPEGAudio += 6;	/* We need to wait at least a packet size...*/
	      else
		dwCanBeMPEGAudio += 1;	/* We need to wait at least a packet size...*/
	    }
	}
      
      if (dwBufferPos > MIN_PACKET_HEADER_SEARCH)
	{/* We have waited at least a packet size...*/
	  dwCanbeMAX = max (dwCanBeVideo, 
			    max (dwCanBeAC3Audio, 
				 max (dwCanBeMPEGAudio, 
				      max (dwCanBeSystem , dwCanBeSystem2))));
	  
	  if (dwCanBeSystem == dwCanbeMAX)
	    {
	      return FMPF_SYSTEM; /* mpeg system */
	    }
	  else if (dwCanBeSystem2 == dwCanbeMAX)
	    {
	      return FMPF_PROGRAM; /* mpeg system program */
	    }
	  else if (dwCanBeVideo == dwCanbeMAX)
	    {
	      return FMPF_VIDEO; /* mpeg ou mpeg2 video */
	    }
	  else if (dwCanBeAC3Audio == dwCanbeMAX)
	    {
	      return FMPF_AC3; /* ac3 audio */
	    }
	  else if (dwCanBeMPEGAudio == dwCanbeMAX)
	    {
	      return FMPF_MPEG_AUDIO; /* mpeg audio */
	    }
	}
      pBuffer++;	/* BYTE pointer inc => @+1  */
    }
  return UNDETERMINED_STREAM_TYPE;
}


DWORD getStreamType(char * filename)
{
  int fd;
  int bytesread_header = 1;
  FMP_BUFFER FMPBuf_header;
  BYTE * pBuf_header;
  DWORD streamtype;

  fd=open(filename,O_RDONLY);
      if (fd==-1)
	{
	  perror("Open failed !!"); 
	  _exit(1); 
	}
	  /* Starting by reading the file header */
	  // manually allocating my own buffer
	  pBuf_header = Malloc(BYTE,BUFFER_SIZE);
	  if(pBuf_header == NULL)
	    {
	      perror("\nThe buffer memory allocation has failed !! Damned !!\n");
	      _exit(1);
	    }
	  
	  FMPBuf_header.pBuffer = pBuf_header;
	  FMPBuf_header.dwBufferSize = BUFFER_SIZE*sizeof(BYTE);
	  // signals can cause harmless EINTR
	  while (((bytesread_header=read(fd,FMPBuf_header.pBuffer,FMPBuf_header.dwBufferSize))==-1)
		 &&(errno==EINTR));
	  if (bytesread_header==-1)
	    bytesread_header=0;
	  FMPBuf_header.dwDataSize=bytesread_header;
	  close(fd);
	  /* the buffer is ready to be analysed
	   * by the function findHeaderInBuffer */
	  streamtype = findVOBheaderInBuffer (FMPBuf_header.pBuffer, FMPBuf_header.dwBufferSize); 
	  if(streamtype == UNDETERMINED_STREAM_TYPE) 
	    streamtype = findHeaderInBuffer (FMPBuf_header.pBuffer, FMPBuf_header.dwBufferSize);
	  free(pBuf_header);
	  return streamtype;
}
