/*****************************************************************************/

/*
 *      mpegserver.c -- MPEG server on top of the NetStream 2000 driver
 *
 *      Copyright (C) 2000 Sigma Designs
 *                    written by Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *                    modified by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                                Pascal Cannenterre <pascal_cannenterre@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/user.h>
#include <sys/time.h>

#include <pthread.h>
#include <semaphore.h>

#define _DEFINE_FMP_TYPES_
#include "fmp.h"

#include "novaclientserver.h"
#include "novaserver.h"

char defaultREALmagicdevice[]=DEFAULTREALMAGICDEVICE;
char *REALmagicdevice;

MESSAGE_QUEUE_OBJECT serversidequeue;
/*
 * File descriptor of the file that we read
 */
int fd;
int b_go_back = 0;
THREADHANDLE fileplayer = NULL;
THREADHANDLE messagequeuelistener;
THREADHANDLE playfileserver;

void treatSIGUSR1()
{
  signal(SIGUSR1,treatSIGUSR1); /* reset signal */
  b_go_back = 1;
  INFOFORSTDOUT("\nGoing back at the beginning of the file ...\n");
}

DWORD CloseFile()
{
  DWORD FMPreturn = FMPE_OK;
  if(fileplayer != NULL)
  {
    // FMPStop(); 
    OSCancelThread(fileplayer);
    /*
     * No need to check the return value of close
     * Simply do the close to avoid any problem
     */
    close(fd);
    FMPreturn = FMPClose();
    fileplayer = NULL;
  }
  return FMPreturn;
}

void StopFile()
{
  if(fileplayer != NULL)
    pthread_kill(fileplayer->t,SIGUSR1);
}


void PlayFile(char * filename)
{
  int bytesread = 1, rc = FMPE_OK;
  FMP_BUFFER FMPBuf;

  b_go_back = 0;
  
  signal(SIGUSR1,treatSIGUSR1);

  /* Open the physic file related with filename */
  fd=open(filename,O_RDONLY);
  if (fd==-1)
    {
      perror("Open failed !!");
      fileplayer = NULL;
      pthread_exit(NULL); 
    }
 
  while (rc == FMPE_OK)
    {
      FMPPlay();
      if(b_go_back)
	{
	  FMPPause();
	  b_go_back = 0;
	}
      while (bytesread && (!b_go_back))
	{
	  // get a pointer to a free buffer
	  rc = FMPGetBuffer (&FMPBuf, TRUE);
	  if (rc != FMPE_OK)
	    {
	      INFOFORSTDOUT("FMPError : %s\n", FMPGetLastError(rc));
	      break;
	    }
	  // signals can cause harmless EINTR
	  while (((bytesread=read(fd,FMPBuf.pBuffer,FMPBuf.dwBufferSize))==-1)
		 &&(errno==EINTR));
	  if (bytesread==-1) bytesread=0;
	  FMPBuf.dwDataSize=bytesread;
	  // the buffer is ready to be queued
	  FMPPush (&FMPBuf);
	}
      FMPStop();
      lseek (fd,0,SEEK_SET);
      bytesread=1;
    }
}

void FMPonmessagequeueServer()
{
  while (1) {
    ClientQuestion Q;
    ServerAnswer A;
    memset (&A,0,sizeof(ServerAnswer));
    
    OSBlockingReceiveFrommessagequeue(serversidequeue,(char *)&Q,
				      sizeof(ClientQuestion),MSGFROMCLIENTTOSERVER);
#define CASETEMPLATEPLAY(x, y) case x: \
                                      A.dwReturn = y ( ); \
                                      break;
    
#define CASETEMPLATE(x, y, rest...) case x: \
                                      A.dwReturn = y ( rest ); \
                                      break;   
   
    switch (Q.dwCommand) {
      
      /* special message causes clean exit of FMPonmessagequeueServer thread.
	 (better than OSCancelThread) */
    case DEATHMESSAGE:
      // if someone is connected, there will be some side effect
      OSDestroymessagequeue(serversidequeue);
      return;
      break;
      CASETEMPLATEPLAY(FMP_PLAY,     FMPPlay); 
      CASETEMPLATEPLAY(FMP_STOP,     FMPStop);
      CASETEMPLATEPLAY(FMP_PAUSE,    FMPPause);
      CASETEMPLATEPLAY(FMP_CLOSE,    FMPClose);
   
      CASETEMPLATE(FMP_GET,                  FMPGet,                 Q.dwArg1);
      CASETEMPLATE(FMP_GET_PROFILE_WORD,     FMPGetProfileWord,      Q.dwArg1);
      CASETEMPLATE(FMP_SET,                  FMPSet,                 Q.dwArg1,Q.dwArg2);
      CASETEMPLATE(FMP_WRITE_PROFILE_WORD,   FMPWriteProfileWord,    Q.dwArg1,Q.dwArg2);
      CASETEMPLATE(FMP_SET_DESTINATION,      FMPSetDestination,      Q.dwArg1,Q.dwArg2,Q.dwArg3,Q.dwArg4);
      CASETEMPLATE(FMP_ANALOG_OVERLAY,       FMPAnalogOverlay,       Q.dwArg1,Q.dwArg2,Q.dwArg3,Q.dwArg4,Q.dwArg5,Q.dwArg6);
      CASETEMPLATE(FMP_OPEN,       FMPOpen,       Q.dwArg1,Q.dwArg2,Q.dwArg3,NULL,Q.dwArg5);
    case FMP_STOPFILE:
      StopFile();
      A.dwReturn = FMP_STOPFILE;
      break;
    case FMP_CLOSEFILE:
      A.dwReturn  = CloseFile();
      break;
    case FMP_DRIVERUNLOAD:    
      MPEGDriverUnload();
      return;
      break; 
    default:
      SUICIDE(ERRORUNKNOWNRPCONMSGQ);
      break;
    }
    
    OSPostInmessagequeue(serversidequeue,(char *)&A,sizeof(ServerAnswer),MSGFROMSERVERTOCLIENT);   
  }
}


void FMPonmessagequeuePlayFileServer()
{
  while(1)
    {
      ClientPlayRequest PQ;
      ServerAnswer B;
      memset (&B,0,sizeof(ServerAnswer));
      
      OSBlockingReceiveFrommessagequeue(serversidequeue,(char *)&PQ,
					sizeof(ClientPlayRequest),MSGFROMCLIENTTOPLAYFILESERVER);
      if(PQ.dwPlayCommand == FMP_PLAYFILE)
	{	    
	  fileplayer = OSCreateThread((TCHAR *)UNUSED_ARG,UNUSED_ARG,(FUNCPTR)PlayFile,(int) PQ.filename);
	  B.dwReturn = FMP_PLAYFILE;
	  OSPostInmessagequeue(serversidequeue,(char *)&B,sizeof(ServerAnswer),MSGFROMPLAYFILESERVERTOCLIENT);
	}  
    }
}




void FMPcreatemessagequeue()
{
	  int msgkey;

	  REALmagicdevice=getenv(REALMAGICDEVICEFROMENV);
	  if (REALmagicdevice==NULL)
		  REALmagicdevice=defaultREALmagicdevice;

	  msgkey=ftok(REALmagicdevice,IPC_PROJECT_KEY);
	  if (msgkey==-1) PERRORIZE("ftok failed");
	  serversidequeue=OSCreatemessagequeue(msgkey,IPC_CREAT|REASONABLE_UMASK);

	// creation of the message queue and thread listening for FMP commands on it

	messagequeuelistener =
	  OSCreateThread((TCHAR *)UNUSED_ARG,UNUSED_ARG,(FUNCPTR)FMPonmessagequeueServer,(int)UNUSED_ARG);
	playfileserver =
	  OSCreateThread((TCHAR *)UNUSED_ARG,UNUSED_ARG,(FUNCPTR)FMPonmessagequeuePlayFileServer,(int)UNUSED_ARG);
}
