/*****************************************************************************/

/*
 *      realmagicdev.c -- NetStream2000 EM8400 (quasar) ``major level'' driver
 *
 *      Copyright (C) 1999-2000  Sigma Designs
 *                    written by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                           and Howard G. Page
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include <linux/version.h>
#include <linux/module.h>

#include <linux/pci.h>

#include <asm/io.h>

#include "common.h"
#include "quasar.h"
#include "realmagicdev.h"

char kernel_version[] = UTS_RELEASE;

// this table stores up to REALMAGIC_MAX_DEV devices
realmagic_dev_t realmagic_dev_tab[REALMAGIC_MAX_DEV];
int realmagic_dev_idx;

// look in linux/fs.h for types
// int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
// int (*mmap) (struct file *, struct vm_area_struct *);
// int (*open) (struct inode *, struct file *);
// int (*release) (struct inode *, struct file *);

int realmagic_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
int realmagic_mmap(struct file *, struct vm_area_struct *);
int realmagic_open(struct inode *, struct file *);
int realmagic_release(struct inode *, struct file *);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
// so fun to change the order in function arrays.
struct file_operations realmagic_fops = 
{
	THIS_MODULE, // struct module *owner;
	NULL, // loff_t (*llseek) (struct file *, loff_t, int);
	NULL, // ssize_t (*read) (struct file *, char *, size_t, loff_t *);
	NULL, // ssize_t (*write) (struct file *, const char *, size_t, loff_t *);
	NULL, // int (*readdir) (struct file *, void *, filldir_t);
	NULL, // unsigned int (*poll) (struct file *, struct poll_table_struct *);
	realmagic_ioctl, // int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
	realmagic_mmap, // int (*mmap) (struct file *, struct vm_area_struct *);
	realmagic_open, // int (*open) (struct inode *, struct file *);
	NULL, // int (*flush) (struct file *);
	realmagic_release, // int (*release) (struct inode *, struct file *);
	NULL, // int (*fsync) (struct file *, struct dentry *, int datasync);
	NULL, // int (*fasync) (int, struct file *, int);
	NULL, // int (*lock) (struct file *, int, struct file_lock *);
	NULL, // ssize_t (*readv) (struct file *, const struct iovec *, unsigned long, loff_t *);
	NULL // ssize_t (*writev) (struct file *, const struct iovec *, unsigned long, loff_t *);
};
#else
// struct file_operations has 15 members for linux 2.2.12, 14, 16, 17.
static struct file_operations realmagic_fops =
{
    NULL,              /* llseek function.     */
    NULL,              /* read function.       */
    NULL,              /* write function.      */
    NULL,              /* read directory.      */
    NULL,              /* poll function.       */
    realmagic_ioctl,   /* ioctl function.      */
    realmagic_mmap,    /* mmap function.       */
    realmagic_open,    /* open function.       */
    NULL,              /* flush function.      */
    realmagic_release, /* release function.    */
    NULL,              /* sync file func.      */
    NULL,              /* async file func.     */
    NULL,              /* media change check.  */
    NULL,              /* revalidate function. */
    NULL               /* lock function.       */
};
#endif

// extract minor number when accessed by struct inode
int realmagic_get_dev(struct inode *i_node,struct file_operations **fops)
{
	int idx = MINOR(i_node->i_rdev)&REALMAGIC_MINOR_DEV_MASK;
	realmagic_dev_t *dev;

	ASSERT(DEBUG,(idx<realmagic_dev_idx),
	       -EINVAL,RAS);

	dev=&realmagic_dev_tab[idx];

	ASSERT(DEBUG,dev->fops,-EINVAL,RAS);
	
	*fops=dev->fops;

	return 0;
}

// extract minor number when accessed by struct file
int realmagic_get_dev_file(struct file *file, struct file_operations **fops)
{
	return realmagic_get_dev(file->f_dentry->d_inode,fops);
}

int realmagic_ioctl(struct inode* i_node, struct file* filp,
		    unsigned int arg1, unsigned long arg2)
{
	struct file_operations *dev_fops=0;

	VERBOSECALL(DEBUG,realmagic_get_dev(i_node,&dev_fops),
		    -EINVAL,RAS);
	
	ASSERT(DEBUG,dev_fops,-EINVAL,RAS);
	ASSERT(DEBUG,dev_fops->ioctl,-EINVAL,RAS);
	
	VERBOSECALL(DEBUG,(*dev_fops->ioctl)(i_node, filp, arg1, arg2),
		    -EINVAL,RAS);

	return 0;
}

int realmagic_mmap(struct file* filp,
		   struct vm_area_struct* vmptr)
{
	struct file_operations *dev_fops=0;

	VERBOSECALL(DEBUG,realmagic_get_dev_file(filp,&dev_fops),
		    -EINVAL,RAS);
	
	ASSERT(DEBUG,dev_fops,-EINVAL,RAS);
	ASSERT(DEBUG,dev_fops->mmap,-EINVAL,RAS);
	
	VERBOSECALL(DEBUG,(*dev_fops->mmap)(filp, vmptr),
		    -EINVAL,RAS);

	return 0;
}

int realmagic_open(struct inode *i_node, struct file *filp)
{
	struct file_operations *dev_fops=0;

	VERBOSECALL(DEBUG,realmagic_get_dev(i_node,&dev_fops),
		    -EINVAL,RAS);

	ASSERT(DEBUG,dev_fops,-EINVAL,RAS);
	ASSERT(DEBUG,dev_fops->open,-EINVAL,RAS);

	VERBOSECALL(DEBUG,(*dev_fops->open)(i_node, filp),
		    -EINVAL,RAS);

	MOD_INC_USE_COUNT;

	return 0;
}

int realmagic_release(struct inode *i_node, struct file *filp)
{
	struct file_operations *dev_fops=0;

	VERBOSECALL(DEBUG,realmagic_get_dev(i_node,&dev_fops),
		    -EINVAL,RAS);
	
	ASSERT(DEBUG,dev_fops,-EINVAL,RAS);
	ASSERT(DEBUG,dev_fops->release,-EINVAL,RAS);
	
	VERBOSECALL(DEBUG,(*dev_fops->release)(i_node, filp),
		    -EINVAL,RAS);

	MOD_DEC_USE_COUNT;
	
	return 0;
}

int init_module(void)
{
	realmagic_dev_idx=0;
	
	// look for hardware
	VERBOSECALL(DEBUG,quasar_init(),
		    -EINVAL,RAS);
	
	// register char devices
	VERBOSECALL(DEBUG,register_chrdev(REALMAGIC_MAJOR,
					  REALMAGIC_DEVICE_NAME,
					  &realmagic_fops),
		    -EINVAL,"ok for major %d",REALMAGIC_MAJOR);
	
	LOGINFO(1,"init_module success");

	return 0;
}

int cleanup_module_with_return_code()
{
	VERBOSECALL(DEBUG,quasar_cleanup(),
		    -EINVAL,RAS);
	
	VERBOSECALL(DEBUG,unregister_chrdev(REALMAGIC_MAJOR,
					    REALMAGIC_DEVICE_NAME),
		    -EINVAL,RAS);
	
	return 0;
}

void cleanup_module()
{
	int rc;
	
	rc=cleanup_module_with_return_code();

	LOGINFO(1,"cleanup_module over with %d",rc);
}
