/*
 * $Workfile: geodeioctl.c $
 * $Revision: 38 $
 *
 * File Contents: This file contains the  the Geode frame buffer
 *                device driver specific ioctl functions definitions.
 *
 * Notes:         All the geode frame buffer driver ioctl functions 
 *                use the durango library to access the hardware.
 *
 * Project:       Geode Frame buffer device driver
 *
 */

/* 
 * NSC_LIC_ALTERNATIVE_PREAMBLE
 *
 * Revision 1.0
 *
 * National Semiconductor Alternative GPL-BSD License
 *
 * National Semiconductor Corporation licenses this software 
 * ("Software"):
 *
 * Geode frame buffer driver
 *
 * under one of the two following licenses, depending on how the 
 * Software is received by the Licensee.
 * 
 * If this Software is received as part of the Linux Framebuffer or
 * other GPL licensed software, then the GPL license designated 
 * NSC_LIC_GPL applies to this Software; in all other circumstances 
 * then the BSD-style license designated NSC_LIC_BSD shall apply.
 *
 * END_NSC_LIC_ALTERNATIVE_PREAMBLE */

/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * Geode frame buffer driver
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * END_NSC_LIC_BSD */

/* NSC_LIC_GPL
 *
 * National Semiconductor Corporation Gnu General Public License for 
 *
 * Geode frame buffer driver
 *
 * (GPL License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted under the terms of the GNU General 
 * Public License as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version  
 *
 * In addition to the terms of the GNU General Public License, neither 
 * the name of the National Semiconductor Corporation nor the names of 
 * its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE, 
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. See the GNU General Public License for more details. 
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this file; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * END_NSC_LIC_GPL */


#define GEODEFBDEBUG

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/malloc.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/console.h>
#include <linux/selection.h>
#include <linux/ioport.h>
#include <linux/init.h>

#include <asm/io.h>
#include <asm/mtrr.h>
#include <asm/uaccess.h>

#include "gfx_rtns.h"
#include "gfx_defs.h"
#include "gfx_regs.h"

#include "panel.h"

#include "galproto.h"
#include "geodeioctl.h"

#ifdef GEODEFBDEBUG
#define DPRINTK(fmt, args...) printk(KERN_INFO ## fmt, args)
#else
#define DPRINTK(fmt, args...)
#endif

/*
 * Look for our signature and if subfunction is
 * within range
 */

#define IO_PACKET_ISVALID(x)\
	((x.dwSubfunction < GALFN_LASTFUNCTION_SUPPORTED) &&\
	(x.dwSignature == FBGAL_SIGNATURE))

typedef int (*PIOCTLFN)(void *);

static char bDispatchTableInitialized = 0;
static PIOCTLFN pIoctlFns[GALFN_LASTFUNCTION_SUPPORTED];

/*-----------------------------------------------------------------
 * Nscgfxfn_Unsupported 
 *
 * Description:	This function will be called when ioctl function 
 *				is not supported by geode frame buffer driver. 
 * parameters :
 *		pvarg :	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return:	It returns negative error value.
 *------------------------------------------------------------------*/
int Nscgfxfn_Unsupported(void *pvarg)
{
	return -EBADRQC;
}

/*--------------------------------------------------------------------
 * Nscgfxfn_getadapterinfo 
 * 
 * Description:	This function gets the galdevice adapter information
 *				to the user. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *		passed	to the ioctl by the application.
 *      return: 0 was return on getting the gal adapter information
 *				other wise -ve value will be returned.
 *--------------------------------------------------------------------*/
int Nscgfxfn_getadapterinfo(void *pvarg) 
{
	int retval;
	GAL_ADAPTERINFO sAdapterInfo;

    if (copy_from_user(&sAdapterInfo, 
                       (void *) pvarg, sizeof(sAdapterInfo)))
	{
		retval = -EFAULT;
	}
	else {
		sAdapterInfo.dwReturnValue = 0;
		sAdapterInfo.dwCPUVersion = gfx_detect_cpu();
		sAdapterInfo.dwVideoVersion = gfx_detect_video();
		sAdapterInfo.dwFrameBufferBase = gfx_get_frame_buffer_base();
		sAdapterInfo.dwFrameBufferSize = gfx_get_frame_buffer_size();
		sAdapterInfo.dwGfxRegisterBase = gfx_get_cpu_register_base();
		sAdapterInfo.dwVidRegisterSize = gfx_get_vid_register_base();
		sAdapterInfo.dwVipRegisterBase = gfx_get_vip_register_base();
		sAdapterInfo.dwMaxSupportedPixelClock =
					gfx_get_max_supported_pixel_clock();
		copy_to_user((void *) pvarg,&sAdapterInfo, 
					sizeof(sAdapterInfo));
		retval = 0;
	}    
	return retval;
}

/*----------------------------------------------------------------------
 * Nscgfxfn_setsoftvgastate
 * 
 * Description:	This function sets the softvga state
 *	parameters:
 *	     pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return:	0 was retured on success otherwise error value will be 
 *				returned.
 *--------------------------------------------------------------------*/
int Nscgfxfn_setsoftvgastate(void *pvarg)
{
	int retval;
	GAL_SOFTVGASTATE sSoftVgaState;
    
    if (copy_from_user(&sSoftVgaState, 
                       (void *) pvarg, sizeof(sSoftVgaState)))
	{
		retval = -EFAULT;
	}
	else {
		if (sSoftVgaState.bSoftVgaEnable)
			gfx_enable_softvga();
		else
			gfx_disable_softvga();
		retval = 0;
	}
	return retval;
}


/*----------------------------------------------------------------------
 * Nscgfxfn_getsoftvgastate
 * 
 * Description:	This function gets the softvga state
 *	parameters:
 *	     pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return:	0 was retured on success otherwise error value will be 
 *				returned.
 *--------------------------------------------------------------------*/
int Nscgfxfn_getsoftvgastate(void *pvarg)
{
	int retval;
	GAL_SOFTVGASTATE sSoftVgaState;
    
    if (copy_from_user(&sSoftVgaState, 
                       (void *) pvarg, sizeof(sSoftVgaState)))
	{
		retval = -EFAULT;
	}
	else {
		sSoftVgaState.bSoftVgaEnable = gfx_get_softvga_active();
		
		copy_to_user((void *) pvarg,&sSoftVgaState, 
					sizeof(sSoftVgaState));
		retval = 0;
	}
	return retval;
}


/*-------------------------------------------------------------------
 * Nscgfxfn_setcrtstate
 * 
 * Description:	This function enable the gal crt state based on 
 *				user information.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	0 was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcrtstate(void *pvarg)
{
	int retval;
	GAL_CRTSTATE sCrtState;
    
    if (copy_from_user(&sCrtState, 
                       (void *) pvarg, sizeof(sCrtState)))
	{
		retval = -EFAULT;
	}
	else {
		gfx_set_crt_enable(sCrtState.wCrtState);
		retval = 0;
	}
	return retval;
}


/*-----------------------------------------------------------------
 * Nscgfxfn_waituntilidle
 * 
 * Description:	This routine waits until the graphics engine is idle.
 *				It must be used before a display driver allows direct
 *				access to frame buffer data.	
 *  parameters:
 *		 pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	0 was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_waituntilidle(void *pvarg)
{
	int retval;
	GAL_WAITUNTILIDLE sWaitIdle;

    if (copy_from_user(&sWaitIdle, 
                       (void *) pvarg, sizeof(sWaitIdle)))
    {
		retval = -EFAULT;
	}
	else {
		gfx_wait_until_idle();
		sWaitIdle.dwReturnValue = 0;
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_waitverticalblank
 *
 * Description:	This function wait until start of vertical blank.
 *
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_waitverticalblank(void *pvarg)
{
	int retval;
	GAL_WAITVERTICALBLANK sWaitVBlank;

    if (copy_from_user(&sWaitVBlank, 
                       (void *) pvarg, sizeof(sWaitVBlank)))
	{
		retval = -EFAULT;
	}
	else {
		gfx_wait_vertical_blank();
		sWaitVBlank.dwReturnValue = 0;
		retval = 0;
	}    
	return retval;
}

/*--------------------------------------------------------------------
 * Nscgfxfn_writereg
 *
 * Description:	This function writes the user information to the
 *				gal hardware.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: 0 was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_writereg(void *pvarg)
{
	int retval;
	GAL_HWACCESS hwAccess;

    if (copy_from_user(&hwAccess, 
                       (void *) pvarg, sizeof(hwAccess)))
	{
		retval = -EFAULT;
	}
	else {
		if (hwAccess.dwType == GAL_REG) 
		{
			if (hwAccess.dwByteCount == 4) 
				WRITE_REG32(hwAccess.dwOffset, 
						hwAccess.dwValue);
			else 
			if (hwAccess.dwByteCount == 2) 
				WRITE_REG16(hwAccess.dwOffset, 
						hwAccess.dwValue);
			else 
			if (hwAccess.dwByteCount == 1) 
			{
				WRITE_REG8(hwAccess.dwOffset, hwAccess.dwValue);
			}
		} 
		else 
		if (hwAccess.dwType == GAL_VID) 
		{
			if(hwAccess.dwByteCount == 4) 
			{
				WRITE_VID32(hwAccess.dwOffset, 
						hwAccess.dwValue);
			}
		} 
		else
		if (hwAccess.dwType == GAL_VIP)
		{
			if (hwAccess.dwByteCount == 4) 
			{
				WRITE_VIP32(hwAccess.dwOffset, 
						hwAccess.dwValue);
			}
		}
		hwAccess.dwReturnValue = 0;
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_readreg
 *
 * Description:	This function reads the user information from the
 *				gal hardware.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_readreg(void *pvarg)
{
	int retval;
	GAL_HWACCESS hwAccess;

    if (copy_from_user(&hwAccess, 
                       (void *) pvarg, sizeof(hwAccess)))
	{
		retval = -EFAULT;
	} 
	else {
		hwAccess.dwReturnValue = 0;
		if(hwAccess.dwType == GAL_REG)
		{
			if(hwAccess.dwByteCount == 4) 
			{
				hwAccess.dwValue = 
						READ_REG32(hwAccess.dwOffset);
			}
		 	else 
			if (hwAccess.dwByteCount == 2) 
			{
				hwAccess.dwValue = 
						READ_REG16(hwAccess.dwOffset);
			}
		} 
		else 
		if(hwAccess.dwType == GAL_VID) 
		{
			if(hwAccess.dwByteCount == 4) 
			{
				hwAccess.dwValue = 
						READ_VID32(hwAccess.dwOffset);
			}
		} 
		else 
		if (hwAccess.dwType == GAL_VIP) 
		{
			if(hwAccess.dwByteCount == 4) 
			{
	                	hwAccess.dwValue = 
						READ_VIP32(hwAccess.dwOffset);
		     }
		}
		copy_to_user((void *) pvarg,&hwAccess, sizeof(hwAccess));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_ismodesupported
 *
 * Description:	This function verifies that a given display mode is
 *				supported or not.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_ismodesupported(void *pvarg)
{
	int retval;
	GAL_DISPLAYMODE sDisplayMode;
    
    if (copy_from_user(&sDisplayMode, 
                       (void *) pvarg, sizeof(sDisplayMode)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDisplayMode.dwSupported =
			gfx_is_display_mode_supported(sDisplayMode.wXres,
                                          sDisplayMode.wYres, 
                                          sDisplayMode.wBpp,
				          sDisplayMode.wRefresh);
		sDisplayMode.dwReturnValue = 1;
                copy_to_user((void *) pvarg, 
                       &sDisplayMode, sizeof(sDisplayMode));
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaymode
 *
 * Description	This function programs the display controller 
 *				for the specified display mode.	
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaymode(void *pvarg)
{
	int retval;
	GAL_DISPLAYMODE sDisplayMode;
	if (copy_from_user(&sDisplayMode, (void *) pvarg, sizeof(sDisplayMode)))
	{
		retval = -EFAULT;
	} 
	else {
		sDisplayMode.dwReturnValue =
		gfx_set_display_mode(sDisplayMode.wXres, sDisplayMode.wYres,
				sDisplayMode.wBpp, sDisplayMode.wRefresh);
		copy_to_user((void *) pvarg,&sDisplayMode, 
					sizeof(sDisplayMode));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaymode
 *
 * Description:	This function reads the current display parameters.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaymode(void *pvarg)
{
	int retval;
	GAL_DISPLAYMODE sDisplayMode;
	int xres, yres, bpp, hz;
	if (copy_from_user(&sDisplayMode, (void *) pvarg, sizeof(sDisplayMode)))
	{
		retval = -EFAULT;

	}
	else 
	{
		sDisplayMode.dwReturnValue = 
	 			gfx_get_display_mode(&xres, &yres, &bpp, &hz);
		sDisplayMode.wXres = xres;
		sDisplayMode.wYres = yres;
		sDisplayMode.wBpp = bpp;
		sDisplayMode.wRefresh = hz;
		copy_to_user((void *) pvarg,&sDisplayMode, 
						sizeof(sDisplayMode));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaybpp
 *
 * Description:	This function specifies the bpp of the current 
 *				mode so that the graphics engine can be programmed 
 *				accordingly.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which 
 *				was passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaybpp(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg,
					sizeof(sDisplayParams))) 
	{
		retval = -EFAULT;
	} 
	else {
		gfx_set_display_bpp(sDisplayParams.wBpp);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaybpp
 *
 * Description:	This function reads the bpp of the current mode 
 *				that the graphics engine can be programmed.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaybpp(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg,
					sizeof(sDisplayParams))) 
	{
		retval = -EFAULT;
	} 
	else {
		sDisplayParams.wBpp = gfx_get_display_bpp();
		copy_to_user((void *) pvarg, &sDisplayParams, 
					sizeof(sDisplayParams));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaypitch
 *
 * Description:	This function sets the pitch of the current
 *				display mode, which is the number of bytes between
 *				scanlines.
 *  parameters:
 *		 pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaypitch(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg,
					sizeof(sDisplayParams))) 
	{ 
		retval = -EFAULT;
	} 
	else 
	{
		gfx_set_display_pitch(sDisplayParams.wPitch);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaypitch
 *
 * Description:	This function returns the pitch of the current
 *				display mode, which is the number of bytes between
 *				scanlines.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaypitch(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg,
					sizeof(sDisplayParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDisplayParams.wPitch = gfx_get_display_pitch();
		copy_to_user((void *) pvarg, &sDisplayParams,
						sizeof(sDisplayParams));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplayoffset
 *
 * Description:	This function sets the frame buffer start address.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplayoffset(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg, 
					sizeof(sDisplayParams))) {
		retval = -EFAULT;
	} 
	else {
		gfx_set_display_offset(sDisplayParams.dwOffset);
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplayoffset
 *
 * Description:	This function returns the offset to the start of
 *				frame buffer in graphics memory.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplayoffset(void *pvarg)
{
	int retval;
	GAL_DISPLAYPARAMS sDisplayParams;
	if (copy_from_user(&sDisplayParams, (void *) pvarg,
					 sizeof(sDisplayParams))) {
		retval = -EFAULT;
	} 
	else {
		sDisplayParams.dwOffset = gfx_get_display_offset();
		copy_to_user((void *) pvarg, &sDisplayParams, 
						sizeof(sDisplayParams));
		retval = 0;
	}
    	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getrefreshfromdotclock
 *
 * Description:	This function takes display mode parameters,
 *				and returns the refresh rate of the closest display
 *				mode.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getrefreshfromdotclock(void *pvarg)
{
	int retval;
	GAL_DOTCLKTOREFRESH sDclk2refresh;
	int hz;
	if (copy_from_user(&sDclk2refresh, (void *) pvarg,
					 sizeof(sDclk2refresh))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDclk2refresh.dwReturnValue = gfx_get_refreshrate_from_frequency(sDclk2refresh.wXres,	
			sDclk2refresh.wYres, 
			sDclk2refresh.wBpp,&hz, sDclk2refresh.dwDotClock);
			sDclk2refresh.wRefreshRate = hz;
		copy_to_user((void *) pvarg, &sDclk2refresh,
						sizeof(sDclk2refresh));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaytiming
 *
 * Description:	This function gets display timings.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaytiming(void *pvarg)
{
	int retval;
	GAL_DISPLAYTIMING sDisplayTiming;
	if (copy_from_user(&sDisplayTiming, (void *) pvarg,
					 sizeof(sDisplayTiming))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDisplayTiming.dwReturnValue = 0;

		sDisplayTiming.dwDotClock = gfx_get_clock_frequency();
		sDisplayTiming.wPitch = gfx_get_display_pitch();
		sDisplayTiming.wBpp = gfx_get_display_bpp();
		sDisplayTiming.wHTotal = gfx_get_htotal();
		sDisplayTiming.wHActive = gfx_get_hactive();
		sDisplayTiming.wHSyncStart = gfx_get_hsync_start();
		sDisplayTiming.wHSyncEnd = gfx_get_hsync_end();
		sDisplayTiming.wHBlankStart = gfx_get_hblank_start();
		sDisplayTiming.wHBlankEnd = gfx_get_hblank_end();
		sDisplayTiming.wVTotal = gfx_get_vtotal();
		sDisplayTiming.wVActive = gfx_get_vactive();
		sDisplayTiming.wVSyncStart = gfx_get_vsync_start();
		sDisplayTiming.wVSyncEnd = gfx_get_vsync_end();
		sDisplayTiming.wVBlankStart = gfx_get_vblank_start();
		sDisplayTiming.wVBlankEnd = gfx_get_vblank_end();
		sDisplayTiming.wPolarity = gfx_get_sync_polarities();

                copy_to_user((void *) pvarg, 
                       &sDisplayTiming, sizeof(sDisplayTiming));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaytiming
 *
 * Description:	This function sets  display timings.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaytiming(void *pvarg)
{
	int retval;
	GAL_DISPLAYTIMING sDisplayTiming;
	if (copy_from_user(&sDisplayTiming, (void *) pvarg,
					 sizeof(sDisplayTiming))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDisplayTiming.dwReturnValue = 0;

		gfx_set_display_timings(sDisplayTiming.wBpp,
			sDisplayTiming.wPolarity, 
			sDisplayTiming.wHActive,
			sDisplayTiming.wHBlankStart, 
			sDisplayTiming.wHSyncStart,
			sDisplayTiming.wHSyncEnd, 
			sDisplayTiming.wHBlankEnd,
			sDisplayTiming.wHTotal, 
			sDisplayTiming.wVActive,
			sDisplayTiming.wVBlankStart, 
			sDisplayTiming.wVSyncStart,
			sDisplayTiming.wVSyncEnd, 
			sDisplayTiming.wVBlankEnd,
			sDisplayTiming.wVTotal, 
			sDisplayTiming.dwDotClock);
		gfx_set_display_pitch(sDisplayTiming.wPitch);
		retval = 0;
	}
	return retval;
}


/*-----------------------------------------------------------------
 * Nscgfxfn_setfixedtimings
 *
 * Description:	This function enables a flat panel and fixed timings.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setfixedtimings(void *pvarg)
{
	int retval;
	GAL_DISPLAYTIMING sDisplayTiming;
	if (copy_from_user(&sDisplayTiming, (void *) pvarg, 
					sizeof(sDisplayTiming))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sDisplayTiming.dwReturnValue = 0;

		gfx_set_fixed_timings(sDisplayTiming.wHActive, /* panel Xres */
				sDisplayTiming.wVActive, /* panel Yres */
				sDisplayTiming.wHTotal, /* Total Xres */
				sDisplayTiming.wVTotal, /* Total Yres */
			    	sDisplayTiming.wBpp);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaypaletteentry
 *
 * Description:	This function sets the entire palette used for 8 bpp
 *				graphics data.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaypaletteentry(void *pvarg)
{
	int retval;
	GAL_PALETTE_ENTRY sPalette;
	if (copy_from_user(&sPalette, (void *) pvarg, sizeof(sPalette)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPalette.dwReturnValue = 0;
		gfx_set_display_palette_entry(sPalette.dwIndex, 
							sPalette.dwPalette);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaypaletteentry
 *
 * Description:	This function returns a single 32-bit 8:8:8
 *				RGB entry of the 8 bpp graphics palette.
 *  parameters:
 *		 pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaypaletteentry(void *pvarg)
{
	int retval;
	GAL_PALETTE_ENTRY sPalette;
	if (copy_from_user(&sPalette, (void *) pvarg, sizeof(sPalette))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		unsigned long palette;

		sPalette.dwReturnValue = 0;

		gfx_get_display_palette_entry(sPalette.dwIndex, &palette);
		sPalette.dwPalette = palette;
		copy_to_user((void *) pvarg, &sPalette, sizeof(sPalette));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setdisplaypalette
 *
 * Description:	This function sets the entire palette used
 *				for 8bpp graphics data.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setdisplaypalette(void *pvarg)
{
	int retval;
	GAL_PALETTE sPalette;
	if (copy_from_user(&sPalette,(void *) pvarg, sizeof(sPalette)))
	{
		retval = -EFAULT;
	} 
	else {
		unsigned long *colors;
		sPalette.dwReturnValue = 0;
	        colors = (unsigned long *)sPalette.dwColors;
	        gfx_set_display_palette(colors);
	        retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getdisplaypalette
 *
 * Description:	This function returns the current palette used
 *				for 8bpp graphics data.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getdisplaypalette(void *pvarg)
{
	int retval;
	GAL_PALETTE sPalette;
	if (copy_from_user(&sPalette,(void *) pvarg, sizeof(sPalette))) {
		retval = -EFAULT;
	}
	else {

		unsigned long *colors;
		sPalette.dwReturnValue = 0;
		colors = (unsigned long *)sPalette.dwColors;
		gfx_get_display_palette(colors);
		copy_to_user((void *) pvarg,&sPalette, sizeof(sPalette));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setcompressionstate
 *
 * Description:	This function enables or disables display compression.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcompressionstate(void *pvarg)
{
	int retval;
	GAL_COMPRESSIONSTATE sCompState;
	if (copy_from_user(&sCompState,(void *) pvarg, sizeof(sCompState)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sCompState.dwReturnValue = 0;
		gfx_set_compression_enable(sCompState.bCompressionState);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getcompressionstate
 *
 * Description:	This function gets the display compression.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getcompressionstate(void *pvarg)
{
	int retval;
	GAL_COMPRESSIONSTATE sCompState;
	if (copy_from_user(&sCompState,(void *) pvarg, sizeof(sCompState)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sCompState.dwReturnValue = 0;
		sCompState.bCompressionState = gfx_get_compression_enable();

        copy_to_user((void *) pvarg, &sCompState, sizeof(sCompState));
		retval = 0;
	}
	return retval;
}


/*-----------------------------------------------------------------
 * Nscgfxfn_setcompressionparams
 *
 * Description:	This function sets the compression parameters of the
 *				frame buffer device.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcompressionparams(void *pvarg)
{
	int retval;
	GAL_COMPRESSIONPARAMS sCompParams;
	if (copy_from_user(&sCompParams,(void *) pvarg, sizeof(sCompParams)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sCompParams.dwReturnValue = 0;
		if (sCompParams.dwFlags & GAL_COMPRESSION_OFFSET)
			gfx_set_compression_offset(sCompParams.dwCompOffset);
		if (sCompParams.dwFlags & GAL_COMPRESSION_PITCH)
			gfx_set_compression_pitch(sCompParams.dwCompPitch);
		if (sCompParams.dwFlags & GAL_COMPRESSION_SIZE)
			gfx_set_compression_size(sCompParams.dwCompSize);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getcompressionparams
 *
 * Description:	This function gets the compression parameters of the
 *				frame buffer device.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getcompressionparams(void *pvarg)
{
    int retval;
    GAL_COMPRESSIONPARAMS sCompParams;
    
    if (copy_from_user(&sCompParams, 
                       (void *) pvarg, sizeof(sCompParams)))
    {
        retval = -EFAULT;
    }
    else
    {
        sCompParams.dwReturnValue = 0;
        
        if (sCompParams.dwFlags & GAL_COMPRESSION_OFFSET)
			sCompParams.dwCompOffset = gfx_get_compression_offset();
        
        if (sCompParams.dwFlags & GAL_COMPRESSION_PITCH)
			sCompParams.dwCompPitch = gfx_get_compression_pitch();
        
        if (sCompParams.dwFlags & GAL_COMPRESSION_SIZE)
			sCompParams.dwCompSize = gfx_get_compression_size();
        
        copy_to_user((void *) pvarg, &sCompParams, sizeof(sCompParams));
        retval = 0;
    }
    
    return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setcursorenable
 *
 * Description:	This function enable or disable the hardware cursor.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcursorenable(void *pvarg)
{
	int retval;
	GAL_CURSORENABLE sCursorEnable;
	if (copy_from_user(&sCursorEnable, (void *) pvarg, 
					sizeof(sCursorEnable)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		gfx_set_cursor_enable(sCursorEnable.bCursorEnable);
		sCursorEnable.dwReturnValue = 0;
        
        retval = 0;
    }
    
    return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getcursorenable
 *
 * Description:	This function gets the curent cursor state.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getcursorenable(void *pvarg)
{
    int retval;
    GAL_CURSORENABLE sCursorEnable;
    
    if (copy_from_user(&sCursorEnable, 
                       (void *) pvarg, sizeof(sCursorEnable)))
    {
        retval = -EFAULT;
    }
    else
    {
        sCursorEnable.dwReturnValue = 0;
		sCursorEnable.bCursorEnable = gfx_get_cursor_enable();
        
        copy_to_user((void *) pvarg, &sCursorEnable, sizeof(sCursorEnable));
		retval = 0;
     }
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setcursorcolors
 *
 * Description:	This function sets the colors of the hardware cursor.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcursorcolors(void *pvarg)
{
	int retval;
	GAL_CURSORCOLORS sCursorCol;
	if (copy_from_user(&sCursorCol,(void *) pvarg, sizeof(sCursorCol))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		gfx_set_cursor_colors(sCursorCol.dwBgColor, 
			sCursorCol.dwFgColor);
		sCursorCol.dwReturnValue = 0;

		retval = 0;
	}
    
        return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getcursorcolors
 *
 * Description:	This function sets the colors of the hardware cursor.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getcursorcolors(void *pvarg)
{
    int retval;
    GAL_CURSORCOLORS sCursorCol;
    
    if (copy_from_user(&sCursorCol, 
                       (void *) pvarg, sizeof(sCursorCol)))
    {
        retval = -EFAULT;
    }
    else
    {
        sCursorCol.dwReturnValue = 0;
		sCursorCol.dwBgColor = gfx_get_cursor_color(0);
		sCursorCol.dwFgColor = gfx_get_cursor_color(1);
        
        copy_to_user((void *) pvarg, &sCursorCol, sizeof(sCursorCol));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setcursorposition
 *
 * Description:	This function sets the position of the hardware cursor.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcursorposition(void *pvarg)
{
	int retval;
	GAL_CURSORPOSITION sCursorPos;
	if (copy_from_user(&sCursorPos,(void *) pvarg, sizeof(sCursorPos)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		gfx_set_cursor_position(sCursorPos.dwMemOffset, 
			sCursorPos.wXPos, 
			sCursorPos.wYPos, 
			sCursorPos.wXHot,
			sCursorPos.wYHot);
		sCursorPos.dwReturnValue = 0;
        
        retval = 0;
    }
    
    return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getcursorposition
 *
 * Description:	This function gets the position of the hardware cursor.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getcursorposition(void *pvarg)
{
    int retval;
    GAL_CURSORPOSITION sCursorPos;
    
    if (copy_from_user(&sCursorPos, 
                       (void *) pvarg, sizeof(sCursorPos)))
    {
        retval = -EFAULT;
    }
    else
    {
        sCursorPos.dwReturnValue = 0;
		sCursorPos.dwMemOffset = gfx_get_cursor_offset();
		sCursorPos.wXPos = 0;
		sCursorPos.wYPos = 0;
		sCursorPos.wXHot = 0;
		sCursorPos.wYHot = 0;
        
        copy_to_user((void *) pvarg, &sCursorPos, sizeof(sCursorPos));
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setcursorshape32
 *
 * Description:	This function loads 32x32 cursor pattern. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setcursorshape32(void *pvarg)
{
	int retval;
	GAL_SETCURSORSHAPE sCursorShape;
	if (copy_from_user(&sCursorShape,(void *) pvarg, 
					sizeof(sCursorShape))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		gfx_set_cursor_shape32(sCursorShape.dwMemOffset,
			(unsigned long *)sCursorShape.dwAndMask,
				(unsigned long *)sCursorShape.dwXorMask);
		sCursorShape.dwReturnValue = 0;
		retval = 0;
	}
	return retval;    
}


/*-----------------------------------------------------------------
 * Nscgfxfn_setsolidpattern
 *
 * Description:	This function used to specify a solid pattern color
 *				for future rendering. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setsolidpattern(void *pvarg)
{
	int retval;
	GAL_SETSOLIDPATTERN sSetSolidPat;
	if (copy_from_user(&sSetSolidPat,(void *) pvarg, sizeof(sSetSolidPat))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetSolidPat.dwReturnValue = 0;
		gfx_set_solid_pattern(sSetSolidPat.dwColor);
		retval = 0;
	}   
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setsolidsource
 *
 * Description:	This function specifies a constant source data value
 *				for raster operations that use both pattern and 
 *				source data. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setsolidsource(void *pvarg)
{
	int retval;
	GAL_SETSOLIDSOURCE sSetSolidSrc;
	if (copy_from_user(&sSetSolidSrc,(void *) pvarg, 
					sizeof(sSetSolidSrc))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetSolidSrc.dwReturnValue = 0;
		gfx_set_solid_source(sSetSolidSrc.dwColor);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setmonopattern
 *
 * Description:	This function specifies an 8x8 monochrome pattern 
 *				used in future rendering operations.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	'0' was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setmonopattern(void *pvarg)
{
	int retval;
	GAL_SETMONOPATTERN sSetMonoPat;
	if (copy_from_user(&sSetMonoPat,(void *) pvarg, sizeof(sSetMonoPat))) {
		retval = -EFAULT;
	} 
	else 
	{
        
		sSetMonoPat.dwReturnValue = 0;
		gfx_set_mono_pattern(sSetMonoPat.dwBgColor,
			sSetMonoPat.dwFgColor, 
			sSetMonoPat.dwData0, 
			sSetMonoPat.dwData1,
			sSetMonoPat.cTransparency);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setrasteroperation
 *
 * Description:	This function specifies the raster operation for 
 *				future rendering.  
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setrasteroperation(void *pvarg)
{
	int retval;
	GAL_SETRASTEROPERATION sSetRop;
	if (copy_from_user(&sSetRop,(void *) pvarg, sizeof(sSetRop))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetRop.dwReturnValue = 0;
		gfx_set_raster_operation(sSetRop.cRop);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_patternfill
 *
 * Description:	This function renders pattern data to a rectangular 
 *				region. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero  was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_patternfill(void *pvarg)
{
	int retval;
	GAL_PATTERNFILL sPatternFill;
	if (copy_from_user(&sPatternFill,(void *) pvarg, 
					sizeof(sPatternFill))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPatternFill.dwReturnValue = 0;
		gfx_pattern_fill(sPatternFill.wXPos, 
			sPatternFill.wYPos,
			sPatternFill.wWidth, 
			sPatternFill.wHeight);
		retval = 0;
	} 
	return retval;
}


/*-----------------------------------------------------------------
 * Nscgfxfn_screentoscreenblt
 *
 * Description:	This function is used to perform a screen to screen 
 *				BLT operation. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_screentoscreenblt(void *pvarg)
{
	int retval;
	GAL_SCREENTOSCREENBLT sScreen2ScreenBlt;
	if (copy_from_user(&sScreen2ScreenBlt, (void *) pvarg,
					sizeof(sScreen2ScreenBlt))) {
		retval = -EFAULT;
	} 
	else {
		sScreen2ScreenBlt.dwReturnValue = 0;
		gfx_screen_to_screen_blt(sScreen2ScreenBlt.wXStart,
			sScreen2ScreenBlt.wYStart, 
			sScreen2ScreenBlt.wXEnd,
			sScreen2ScreenBlt.wYEnd, 
			sScreen2ScreenBlt.wWidth,
				sScreen2ScreenBlt.wHeight);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_screentoscreenxblt
 *
 * Description:	This function is used to perform a screen to screen 
 *				BLT operation using a transparency color. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_screentoscreenxblt(void *pvarg)
{
	int retval;
	GAL_SCREENTOSCREENXBLT sScreen2ScreenXBlt;
	if (copy_from_user(&sScreen2ScreenXBlt, (void *) pvarg,
					sizeof(sScreen2ScreenXBlt))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sScreen2ScreenXBlt.dwReturnValue = 0;
		gfx_screen_to_screen_xblt(sScreen2ScreenXBlt.wXStart,
			sScreen2ScreenXBlt.wYStart, 
			sScreen2ScreenXBlt.wXEnd,
			sScreen2ScreenXBlt.wYEnd, 
			sScreen2ScreenXBlt.wWidth,
			sScreen2ScreenXBlt.wHeight, 
			sScreen2ScreenXBlt.wColor);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_bresenhamline
 *
 * Description:	This function is used to draw a single pixel line
 *				using the specified Bresenham parameters. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_bresenhamline(void *pvarg)
{
	int retval;
	GAL_BRESENHAMLINE sBresenhamLine;
	if (copy_from_user(&sBresenhamLine, (void *)pvarg, 
					sizeof(sBresenhamLine))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sBresenhamLine.dwReturnValue = 0;
		gfx_bresenham_line(sBresenhamLine.wX1, 
			sBresenhamLine.wY1,
			sBresenhamLine.wLength, 
			sBresenhamLine.wErr,
			sBresenhamLine.wE1, 
			sBresenhamLine.wE2,
			sBresenhamLine.wFlags);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgamodeswitch
 *
 * Description:	This function signals the beginning or end of a 
 *				mode switch.  
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgamodeswitch(void *pvarg)
{
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData,(void *) pvarg, sizeof(sVgaData)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sVgaData.dwReturnValue = 0;
		gfx_vga_mode_switch(sVgaData.dwFlags);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgaclearextended
 *
 * Description:	This will clear the Svga data.
 *
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgaclearextended(void *pvarg)
{ 
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData, (void *) pvarg, sizeof(sVgaData))) {
		retval = -EFAULT;
	} 
	else {
		sVgaData.dwReturnValue = 0;
		gfx_vga_clear_extended();
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgapitch
 *
 * Description:	This function sets VGA register values in VGA 
 *				structure for specified pitch. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgapitch(void *pvarg)
{
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData, (void *) pvarg, sizeof(sVgaData))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sVgaData.dwReturnValue = 0;
		gfx_vga_pitch((gfx_vga_struct *)&sVgaData.sVgaRegs, 
			sVgaData.dwFlags);
		retval = 0;
		copy_to_user((void *) pvarg,&sVgaData, sizeof(sVgaData));
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgarestore
 *
 * Description:	This function sets the VGA hardware to values in the
 *				VGA structure.  
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgarestore(void *pvarg)
{
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData,(void *) pvarg, sizeof(sVgaData)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sVgaData.dwReturnValue = 0;
		gfx_vga_restore((gfx_vga_struct *)&sVgaData.sVgaRegs, 
				sVgaData.dwFlags);
		retval = 0;
		copy_to_user((void *) pvarg,&sVgaData,sizeof(sVgaData));
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgasave
 *
 * Description:	This function saves the current VGA state in the
 *				VGA structure.  
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgasave(void *pvarg)
{
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData,(void *) pvarg, sizeof(sVgaData))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sVgaData.dwReturnValue = 0;
		gfx_vga_save((gfx_vga_struct *)&sVgaData.sVgaRegs,
			sVgaData.dwFlags);
		retval = 0;
		copy_to_user((void *) pvarg,&sVgaData, sizeof(sVgaData));
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_vgamode
 *
 * Description:This function sets VGA register values in VGA 
 *				structure for specified mode. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_vgamode(void *pvarg)
{
	int retval;
	GAL_VGAMODEDATA sVgaData;
	if (copy_from_user(&sVgaData,(void *) pvarg, sizeof(sVgaData))) {
		retval = -EFAULT;
	} 
	else {
		sVgaData.dwReturnValue = 0;
		gfx_vga_mode((gfx_vga_struct *)&sVgaData.sVgaRegs,
		 	 sVgaData.wXres, sVgaData.wYres, sVgaData.wBpp,
	                     			sVgaData.wRefresh);
		retval = 0;
		copy_to_user((void *) pvarg, &sVgaData, sizeof(sVgaData));
	}
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscpnlfn_enabledinbios
 *
 * Description:	This function gets the panel state from Bios. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_enabledinbios(void *pvarg)
{
	int retval;
	GAL_PNLBIOS sPnlBios;
	if (copy_from_user(&sPnlBios,(void *) pvarg, sizeof(sPnlBios)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPnlBios.dwReturnValue = 0;
		sPnlBios.state = Pnl_IsPanelEnabledInBIOS();
		copy_to_user((void *) pvarg, &sPnlBios, sizeof(sPnlBios));
		retval = 0;
	}
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscpnlfn_infofrombios
 *
 * Description:	This function gets the panel state from Bios. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_infofrombios(void *pvarg)
{
	int retval;
	GAL_PNLBIOS sPnlBios;
	if (copy_from_user(&sPnlBios,(void *) pvarg, sizeof(sPnlBios)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPnlBios.dwReturnValue = 0;
		Pnl_GetPanelInfoFromBIOS(&sPnlBios.XRes, &sPnlBios.YRes, 
			&sPnlBios.Bpp, &sPnlBios.Freq);
		copy_to_user((void *) pvarg, &sPnlBios, sizeof(sPnlBios));
		retval = 0;
	}
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscpnlfn_setpanelparams
 *
 * Description:	This function sets the panel parameters. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_setpanelparams(void *pvarg)
{
	int retval;
	GAL_PNLPARAMS sPnlParams;
	if (copy_from_user(&sPnlParams,(void *) pvarg, sizeof(sPnlParams)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPnlParams.dwReturnValue = 0;
		Pnl_SetPanelParam(&(sPnlParams.PanelParams));
		retval = 0;
	}
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscpnlfn_getpanelparams
 *
 * Description:	This function gets the panel parameters. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_getpanelparams(void *pvarg)
{
	int retval;
	GAL_PNLPARAMS sPnlParams;
	if (copy_from_user(&sPnlParams,(void *) pvarg, sizeof(sPnlParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sPnlParams.dwReturnValue = 0;
		Pnl_GetPanelParam(&(sPnlParams.PanelParams));
		retval = 0;
		copy_to_user((void *) pvarg, &sPnlParams, sizeof(sPnlParams));
	}    
	return retval;    
}

/*-----------------------------------------------------------------
 *  Nscpnlfn_initpanel
 *
 * Description:	This function intializes the panel. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_initpanel(void *pvarg)
{
	int retval;
	GAL_PNLPARAMS sPnlParams;
	if (copy_from_user(&sPnlParams,(void *) pvarg, sizeof(sPnlParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		Pnl_InitPanel((&(sPnlParams.PanelParams)));
		retval = 0;
	}
	return retval;    
}
  
/*-----------------------------------------------------------------
 *  Nscpnlfn_savepanel
 *
 * Description:	This function saves the current panel state. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_savepanel(void *pvarg)
{
    int retval;
    GAL_BASE sPnlParams;

    if (copy_from_user(&sPnlParams, 
                       (void *) pvarg, sizeof(sPnlParams)))
    {
        retval = -EFAULT;
    }
    else
    {
        sPnlParams.dwReturnValue = 0;
		Pnl_SavePanelState();

        retval = 0;
    }
    
    return retval;    
}

/*-----------------------------------------------------------------
 *  Nscpnlfn_restorepanel
 *
 * Description:	This function restores the current panel state. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_restorepanel(void *pvarg)
{
    int retval;
    GAL_BASE sPnlParams;

    if (copy_from_user(&sPnlParams, 
                       (void *) pvarg, sizeof(sPnlParams)))
    {
        retval = -EFAULT;
    }
    else
    {
        sPnlParams.dwReturnValue = 0;
		Pnl_RestorePanelState();

        retval = 0;
    }
    return retval;    
}

/*-----------------------------------------------------------------
 *  Nscpnlfn_powerup
 *
 * Description:	This function Powers up the 9211 for panel. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_powerup(void *pvarg)
{
    int retval;
    GAL_BASE sPnlParams;

    if (copy_from_user(&sPnlParams, 
                       (void *) pvarg, sizeof(sPnlParams)))
    {
        retval = -EFAULT;
    }
    else
    {
        sPnlParams.dwReturnValue = 0;
		Pnl_PowerUp();

        retval = 0;
    }
    return retval;    
}

/*-----------------------------------------------------------------
 *  Nscpnlfn_powerdown
 *
 * Description:	This function Powers down the 9211 for panel. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *		return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscpnlfn_powerdown(void *pvarg)
{
    int retval;
    GAL_BASE sPnlParams;

    if (copy_from_user(&sPnlParams, 
                       (void *) pvarg, sizeof(sPnlParams)))
    {
        retval = -EFAULT;
    }
    else
    {
        sPnlParams.dwReturnValue = 0;
		Pnl_PowerDown();

        retval = 0;
    }
    return retval;    
}

/*Panel**********************************************************/

/*TV**********************************************************/

/*-----------------------------------------------------------------
 * Nscgfxfn_settvparams
 *
 * Description:	This function sets the tv parameters of
 *				tvparameters structure. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_settvparams(void *pvarg)
{
	int retval;
	GAL_TVPARAMS sTvParams;
	if (copy_from_user(&sTvParams,(void *) pvarg, sizeof(sTvParams))) {
		retval = -EFAULT;
	} 
	else {
		sTvParams.dwReturnValue = 0;

		/* sequence might be important */
		if (sTvParams.dwFlags & GAL_TVRESOLUTION) {
			gfx_set_tv_display(sTvParams.wWidth, sTvParams.wHeight);
		}
	        if (sTvParams.dwFlags & GAL_TVFORMAT) {
			gfx_set_tv_format(sTvParams.wStandard, sTvParams.wType);
	        }
	        if (sTvParams.dwFlags & GAL_TVOUTPUT) {
			gfx_set_tv_output(sTvParams.wOutput);
	        }
	        if (sTvParams.dwFlags & GAL_TVSTATE) {
			gfx_set_tv_enable(sTvParams.bState);
	        }
		retval = 0;
	}    
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_gettvparams
 *
 * Description:	This function gets the tv parameters of the
 *				tvparameters structure to the user. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_gettvparams(void *pvarg)
{
	int retval;
	GAL_TVPARAMS sTvParams;
	if (copy_from_user(&sTvParams,(void *) pvarg, sizeof(sTvParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		unsigned int state=0;
		sTvParams.dwReturnValue = 0;
		if (sTvParams.dwFlags & GAL_TVSTATE) {
			sTvParams.bState = gfx_get_tv_enable(&state);
		}
		retval = 0;
        copy_to_user((void *) pvarg,
                     &sTvParams, 
                     sizeof(sTvParams));
    }
    
    return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_settvtiming
 *
 * Description:	This function sets the tv timing registers
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_settvtiming(void *pvarg)
{
    int retval;
    GAL_TVTIMING sTvTiming;

    if (copy_from_user(&sTvTiming, 
                       (void *) pvarg, sizeof(sTvTiming)))
    {
        retval = -EFAULT;
    }
    else
    {
		unsigned long VideoType = gfx_detect_video();
        sTvTiming.dwReturnValue = 0;
		if(VideoType == GFX_CPU_SC1200) {
			/* TV Format */
			WRITE_VID32(SC1200_TVOUT_HORZ_TIM, sTvTiming.HorzTim);
			WRITE_VID32(SC1200_TVOUT_HORZ_SYNC, sTvTiming.HorzSync);
			WRITE_VID32(SC1200_TVOUT_VERT_SYNC, sTvTiming.VertSync);
			WRITE_VID32(SC1200_TVOUT_LINE_END, sTvTiming.LineEnd);
			WRITE_VID32(SC1200_TVOUT_VERT_DOWNSCALE, sTvTiming.VertDownscale);
			WRITE_VID32(SC1200_TVOUT_HORZ_SCALING, sTvTiming.HorzScaling);
			WRITE_VID32(SC1200_TVENC_TIM_CTRL_1, sTvTiming.TimCtrl1);
			WRITE_VID32(SC1200_TVENC_TIM_CTRL_2, sTvTiming.TimCtrl2);
			WRITE_VID32(SC1200_TVENC_SUB_FREQ, sTvTiming.Subfreq);
			WRITE_VID32(SC1200_TVENC_DISP_POS, sTvTiming.DispPos);
			WRITE_VID32(SC1200_TVENC_DISP_SIZE, sTvTiming.DispSize);
			/* TV Output */
			WRITE_VID32(SC1200_TVENC_TIM_CTRL_2, sTvTiming.TimCtrl2 );
			WRITE_VID32(SC1200_TVOUT_DEBUG, sTvTiming.Debug);
			/* TV Enable */
			WRITE_VID32(SC1200_TVENC_DAC_CONTROL, sTvTiming.DacCtrl);
		}

        retval = 0;
	}
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_gettvtiming
 *
 * Description:	This function gets the tv timing registers
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_gettvtiming(void *pvarg)
{
    int retval;
    GAL_TVTIMING sTvTiming;

    if (copy_from_user(&sTvTiming, 
                       (void *) pvarg, sizeof(sTvTiming)))
    {
        retval = -EFAULT;
    }
    else
    {
		unsigned long VideoType = gfx_detect_video();
        sTvTiming.dwReturnValue = 0;
		if(VideoType == GFX_CPU_SC1200) {
			/* TV Format */
			sTvTiming.HorzTim = READ_VID32(SC1200_TVOUT_HORZ_TIM);
			sTvTiming.HorzSync = READ_VID32(SC1200_TVOUT_HORZ_SYNC);
			sTvTiming.VertSync = READ_VID32(SC1200_TVOUT_VERT_SYNC);
			sTvTiming.LineEnd = READ_VID32(SC1200_TVOUT_LINE_END);
			sTvTiming.VertDownscale = READ_VID32(SC1200_TVOUT_VERT_DOWNSCALE);
			sTvTiming.HorzScaling = READ_VID32(SC1200_TVOUT_HORZ_SCALING);
			sTvTiming.TimCtrl1 = READ_VID32(SC1200_TVENC_TIM_CTRL_1);
			sTvTiming.TimCtrl2 = READ_VID32(SC1200_TVENC_TIM_CTRL_2);
			sTvTiming.Subfreq = READ_VID32(SC1200_TVENC_SUB_FREQ);
			sTvTiming.DispPos = READ_VID32(SC1200_TVENC_DISP_POS);
			sTvTiming.DispSize = READ_VID32(SC1200_TVENC_DISP_SIZE);
			/* TV Output */
			sTvTiming.TimCtrl2 = READ_VID32(SC1200_TVENC_TIM_CTRL_2);
			sTvTiming.Debug = READ_VID32(SC1200_TVOUT_DEBUG);
			/* TV Enable */
			sTvTiming.DacCtrl = READ_VID32(SC1200_TVENC_DAC_CONTROL);
		}

        copy_to_user((void *) pvarg,
                     &sTvTiming, 
                     sizeof(sTvTiming));

        retval = 0;
    }
    
    return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_settvenable
 *
 * Description:	This function sets the tv enable
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_settvenable(void *pvarg)
{
	int retval;
	GAL_TVPARAMS sTvParams;
	if (copy_from_user(&sTvParams,(void *) pvarg, sizeof(sTvParams))) {
		retval = -EFAULT;
	} 
	else {
		sTvParams.dwReturnValue = 0;
		gfx_set_tv_enable(sTvParams.bState);
		retval = 0;
	}    
	return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_gettvenable
 *
 * Description:	This function gets the tv enable to the user. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_gettvenable(void *pvarg)
{
	int retval;
	GAL_TVPARAMS sTvParams;
	if (copy_from_user(&sTvParams,(void *) pvarg, sizeof(sTvParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		unsigned int state=0;
		sTvParams.dwReturnValue = 0;
		sTvParams.bState = 0;
		if (sTvParams.dwFlags & GAL_TVSTATE) {
			gfx_get_tv_enable(&state);
			sTvParams.bState = state;
		}
		retval = 0;
		copy_to_user((void *) pvarg,
                     &sTvParams, 
                     sizeof(sTvParams));
    }
    
    return retval;    
}

/*-----------------------------------------------------------------
 * Nscgfxfn_istvmodesupported
 *
 * Description:	This function verifies that a given tv mode is
 *				supported or not.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_istvmodesupported(void *pvarg)
{
	int retval;
	GAL_TVPARAMS sTvParams;
	if (copy_from_user(&sTvParams,(void *) pvarg, sizeof(sTvParams))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sTvParams.dwReturnValue = 0;
		sTvParams.bState = gfx_is_tv_display_mode_supported(sTvParams.wWidth, 
			sTvParams.wHeight, sTvParams.wStandard);
		retval = 0;
		copy_to_user((void *) pvarg,
                     &sTvParams, 
                     sizeof(sTvParams));
    }
    
    return retval;    
}

/*TV**********************************************************/

/*Video**********************************************************/
/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoenable
 *
 * Description:	This function enable or disables video overlay.
 *
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/

int Nscgfxfn_setvideoenable(void *pvarg)
{
	int retval;
	GAL_SETVIDEOENABLE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_enable(sSetVideo.enable);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoformat
 *
 * Description:	This function sets the format of the video 
 *				overlay data.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: 
 *				zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoformat(void *pvarg)
{
	int retval;
	GAL_SETVIDEOFORMAT sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_format(sSetVideo.format);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideosize
 *
 * Description:	This function sets the size of the source buffer. 
 *				It is used to determine how much data is 
 *				transferred between the display controller and the 
 *				video hardware. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideosize(void *pvarg)
{
	int retval;
	GAL_SETVIDEOSIZE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_size(sSetVideo.width, sSetVideo.height);
		retval = 0;
	}   
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideooffset
 *
 * Description:	This function sets the base offset of the video 
 *				buffer in graphics memory. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideooffset(void *pvarg)
{
	int retval;
	GAL_SETVIDEOOFFSET sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_offset(sSetVideo.offset);
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideowindow
 *
 * Description:	This function sets the location of the video 
 *				overlay window.  
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideowindow(void *pvarg)
{
	int retval;
	GAL_SETVIDEOWINDOW sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	}
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_window(sSetVideo.x, sSetVideo.y, sSetVideo.w,
					sSetVideo.h);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoscale
 *
 * Description:	This function sets the scale factors of the video 
 *				overlay window. 
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return:	zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoscale(void *pvarg)
{
	int retval;
	GAL_SETVIDEOSCALE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_scale(sSetVideo.srcw, sSetVideo.srch,
				 sSetVideo.dstw, sSetVideo.dsth);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideofilter
 *
 * Description:	This function sets the hardware filtering for the 
 *				video overlay. 
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideofilter(void *pvarg)
{
	int retval;
	GAL_SETVIDEOFILTER sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_filter(sSetVideo.xfilter, sSetVideo.yfilter);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideocolorkey
 *
 * Description:	This function sets the color key value used by the 
 *				video overlay hardware.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero  was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideocolorkey(void *pvarg)
{
	int retval;
	GAL_SETVIDEOCOLORKEY sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_color_key(sSetVideo.key, sSetVideo.mask,
					sSetVideo.bluescreen);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideodownscale_enable
 *
 * Description:	This function enables or disables horizontal
 *				downscaling.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideodownscale_enable(void *pvarg)
{
	int retval;
	GAL_SETVIDEODOWNSCALEENABLE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_downscale_enable(sSetVideo.enable);				
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideodownscale_config
 *
 * Description:	This function programs the horizontal video 
 *				downscale factor.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideodownscale_config(void *pvarg)
{
	int retval;
	GAL_SETVIDEODOWNSCALECONFIG sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_downscale_config(sSetVideo.type, sSetVideo.m); 
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideodownscale_coefficients
 *
 * Description:	This function programs the filter coefficients for 
 *				horizontal video downscaling.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideodownscale_coefficients(void *pvarg)
{
	int retval;
	GAL_SETVIDEODOWNSCALECOEFF sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_downscale_coefficients(sSetVideo.coef1,
				sSetVideo.coef2, sSetVideo.coef3,
						sSetVideo.coef4);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideosource
 *
 * Description:	This function selects the souce for incoming video
 *				data to the video processor.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideosource(void *pvarg)
{
	int retval;
	GAL_SETVIDEOSOURCE sSetVideo;
	VideoSourceType source = VIDEO_SOURCE_MEMORY;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		switch(sSetVideo.source) 
		{
			
			case GAL_VIDEO_SOURCE_MEMORY:
				
				source = VIDEO_SOURCE_MEMORY; 
				break;

			case GAL_VIDEO_SOURCE_DVIP:
                
				source = VIDEO_SOURCE_DVIP; 
				break;
		}
		gfx_set_video_source(source);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideointerlaced
 *
 * Description:	This function programs the video overlay mode for
 *				interlaced or progressive scanning.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideointerlaced(void *pvarg)
{
	int retval;
	GAL_SETVIDEOINTERLACED sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_interlaced(sSetVideo.enable);
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideocolorspace
 *
 * Description:	This function programs the color space for 
 *				graphics/video blending.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideocolorspace(void *pvarg)
{
	int retval;
	GAL_SETVIDEOCOLORSPACE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_color_space_YUV(sSetVideo.enable);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideocursor
 *
 * Description:	This function programs the video cursor color key
 *				and color mask.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideocursor(void *pvarg)
{
	int retval;
	GAL_SETVIDEOCURSOR sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		gfx_set_video_cursor(sSetVideo.key, sSetVideo.mask,
			sSetVideo.select_color2,  sSetVideo.color1,
							sSetVideo.color2);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideorequest
 *
 * Description:	This function sets the horizontal(pixel)
 *				and vertical(line) video request values.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideorequest(void *pvarg)
{
	int retval;
	GAL_SETVIDEOREQUEST sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
	retval = -EFAULT;
	} 
	else 
	{
        	sSetVideo.dwReturnValue = 0;
		gfx_set_video_request(sSetVideo.x, sSetVideo.y);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalphaenable
 *
 * Description:	This function enables or disables the currently 
 *				selected alpha region.
 *  parameters:
 *		pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalphaenable(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHAENABLE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		gfx_set_alpha_enable(sSetVideo.enable);
		retval = 0;
	}    
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalphawindow
 *
 * Description:This function sets the position and size of the 
 *			   currently selected alpha region.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalphawindow(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHAWINDOW sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_alpha_window(sSetVideo.x, sSetVideo.y, sSetVideo.width, 
					sSetVideo.height);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalphavalue
 *
 * Description:	This function the alpha of the currently selected 
 *				alpha region.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalphavalue(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHAVALUE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		gfx_set_alpha_value(sSetVideo.alpha, sSetVideo.delta);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalphapriority
 *
 * Description:	This function sets the priority of the currently 
 *				selected alpha region.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalphapriority(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHAPRIORITY sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
        
		sSetVideo.dwReturnValue = 0;
		gfx_set_alpha_priority(sSetVideo.priority);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalphacolor
 *
 * Description:	This function sets the color of the currently 
 *				selected alpha region.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalphacolor(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHACOLOR sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_alpha_color(sSetVideo.color);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideoalpharegion
 *
 * Description:	This function selects the alpha region for successive
 *				calls to set the parameters.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideoalpharegion(void *pvarg)
{
	int retval;
	GAL_SETVIDEOALPHAREGION sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo)))
	{
		retval = -EFAULT;
	} 
	else
	{
		sSetVideo.dwReturnValue = 0;
		gfx_select_alpha_region(sSetVideo.region);
		retval = 0;
	}
	return retval;
}


/*-----------------------------------------------------------------
 * Nscgfxfn_setvideooutsidealpha
 *
 * Description:	This function programs what is displayed inside the 
 *				video overlay area.
 *  parameters:
 *       pvarg:	This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideooutsidealpha(void *pvarg)
{
	int retval;
	GAL_SETVIDEOOUTSIDEALPHA sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) 
	{
		retval = -EFAULT;
	} 
	else 
	{
		sSetVideo.dwReturnValue = 0;
		gfx_set_no_ck_outside_alpha(sSetVideo.enable);
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_setvideopalette
 *
 * Description:	This function sets the palette in the video overlay 
 *				hardware.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: 
 *				zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_setvideopalette(void *pvarg)
{
	int retval;
	GAL_SETVIDEOPALETTE sSetVideo;
	if (copy_from_user(&sSetVideo,(void *) pvarg, sizeof(sSetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sSetVideo.dwReturnValue = 0;
		if (sSetVideo.identity) {
			gfx_set_video_palette(NULL);
		} 
		else {
			gfx_set_video_palette(sSetVideo.palette);
		}
		retval = 0;
	}
	return retval;
}

/*-----------------------------------------------------------------
 * Nscgfxfn_getvideoinfo
 *
 * Description:	This function reads the video information 
 *				of the hardware to the user.
 *  parameters:
 *       pvarg: This is void pointer  points to the data which was 
 *				passed to the ioctl by the application.
 *      return: zero was returned on success other wise error value
 *				will be returned.
 *------------------------------------------------------------------*/
int Nscgfxfn_getvideoinfo(void *pvarg)
{
	int retval;
	GAL_VIDEOINFO sGetVideo;
	if (copy_from_user(&sGetVideo,(void *) pvarg, sizeof(sGetVideo))) {
		retval = -EFAULT;
	} 
	else {
		sGetVideo.dwReturnValue = 0;
		sGetVideo.enable = gfx_get_video_enable();
		sGetVideo.format = gfx_get_video_format();
		sGetVideo.filter = gfx_get_video_filter();
		sGetVideo.src_size = gfx_get_video_src_size();
		sGetVideo.dst_size = gfx_get_video_dst_size();
		sGetVideo.line_size = gfx_get_video_line_size();
		sGetVideo.xclip = gfx_get_video_xclip();
		sGetVideo.offset = gfx_get_video_offset();
		sGetVideo.scale = gfx_get_video_scale();
		sGetVideo.position = gfx_get_video_position();
		sGetVideo.color_key_src = gfx_get_video_color_key_src();
		sGetVideo.color_key = gfx_get_video_color_key();
		sGetVideo.color_key_mask = gfx_get_video_color_key_mask();
		gfx_get_video_downscale_enable(&sGetVideo.downscale_enable);
		gfx_get_video_downscale_config(&sGetVideo.downscale_type,
						&sGetVideo.downscale_mask);
		gfx_get_video_downscale_coefficients(&sGetVideo.downscale_coef1,
			&sGetVideo.downscale_coef2,  &sGetVideo.downscale_coef3,
			&sGetVideo.downscale_coef4);
		copy_to_user((void *) pvarg, &sGetVideo, sizeof(sGetVideo));
		retval = 0;
		}
		return retval;
}

/*-----------------------------------------------------------------
 * Init_Dispatch_Table
 *
 * Description:	This function intializes the all the graphic ioctl 
 *				to the respective ioctl function pointer.
 *  parameters:	none
 *      return:	none
 *------------------------------------------------------------------*/
void Init_Dispatch_Table()
{
	int iFnIndex = 0;
	while (iFnIndex < GALFN_LASTFUNCTION_SUPPORTED)
		pIoctlFns[iFnIndex++] = Nscgfxfn_Unsupported;

	pIoctlFns[GALFN_GETADAPTERINFO] = Nscgfxfn_getadapterinfo;
	pIoctlFns[GALFN_SETSOFTVGASTATE] = Nscgfxfn_setsoftvgastate;
	pIoctlFns[GALFN_GETSOFTVGASTATE] = Nscgfxfn_getsoftvgastate;
	pIoctlFns[GALFN_WAITUNTILIDLE] = Nscgfxfn_waituntilidle;
	pIoctlFns[GALFN_WAITVERTICALBLANK] = Nscgfxfn_waitverticalblank;
	pIoctlFns[GALFN_SETCRTSTATE] = Nscgfxfn_setcrtstate;
	pIoctlFns[GALFN_WRITEREG] = Nscgfxfn_writereg;
	pIoctlFns[GALFN_READREG] = Nscgfxfn_readreg;

	/* display hardware related function */
	pIoctlFns[GALFN_ISDISPLAYMODESUPPORTED] = Nscgfxfn_ismodesupported;   
	pIoctlFns[GALFN_SETDISPLAYMODE] = Nscgfxfn_setdisplaymode;    
	pIoctlFns[GALFN_GETDISPLAYMODE] = Nscgfxfn_getdisplaymode;
	pIoctlFns[GALFN_SETDISPLAYBPP] = Nscgfxfn_setdisplaybpp;    
	pIoctlFns[GALFN_GETDISPLAYBPP] = Nscgfxfn_getdisplaybpp;    
	pIoctlFns[GALFN_SETDISPLAYPITCH] = Nscgfxfn_setdisplaypitch;    
	pIoctlFns[GALFN_GETDISPLAYPITCH] = Nscgfxfn_getdisplaypitch;    
	pIoctlFns[GALFN_SETDISPLAYOFFSET] = Nscgfxfn_setdisplayoffset;    
	pIoctlFns[GALFN_GETDISPLAYOFFSET] = Nscgfxfn_getdisplayoffset;    
	pIoctlFns[GALFN_DOTCLKTOREFRESH] = Nscgfxfn_getrefreshfromdotclock;    
	pIoctlFns[GALFN_GETDISPLAYTIMINGS] = Nscgfxfn_getdisplaytiming;    
	pIoctlFns[GALFN_SETDISPLAYTIMINGS] = Nscgfxfn_setdisplaytiming;
	pIoctlFns[GALFN_SETPALETTE] = Nscgfxfn_setdisplaypalette;
	pIoctlFns[GALFN_GETPALETTE] = Nscgfxfn_getdisplaypalette;
	pIoctlFns[GALFN_SETPALETTE_ENTRY] = Nscgfxfn_setdisplaypaletteentry;
	pIoctlFns[GALFN_GETPALETTE_ENTRY] = Nscgfxfn_getdisplaypaletteentry;
	pIoctlFns[GALFN_SETFIXEDTIMINGS] = Nscgfxfn_setfixedtimings;

	/* Hardware cursor funtions */
	pIoctlFns[GALFN_SETCURSORENABLE] = Nscgfxfn_setcursorenable;
	pIoctlFns[GALFN_GETCURSORENABLE] = Nscgfxfn_getcursorenable;
	pIoctlFns[GALFN_SETCURSORCOLORS] = Nscgfxfn_setcursorcolors;
	pIoctlFns[GALFN_GETCURSORCOLORS] = Nscgfxfn_getcursorcolors;
	pIoctlFns[GALFN_SETCURSORPOSITION] = Nscgfxfn_setcursorposition;
	pIoctlFns[GALFN_GETCURSORPOSITION] = Nscgfxfn_getcursorposition;
	pIoctlFns[GALFN_SETCURSORSHAPE] = Nscgfxfn_setcursorshape32;

	/* grafix rendering funtions */
	pIoctlFns[GALFN_SETSOLIDPATTERN] = Nscgfxfn_setsolidpattern;   
	pIoctlFns[GALFN_SETRASTEROPERATION] = Nscgfxfn_setrasteroperation;
	pIoctlFns[GALFN_SETSOLIDSOURCE] = Nscgfxfn_setsolidsource;    
	pIoctlFns[GALFN_PATTERNFILL] = Nscgfxfn_patternfill;       
	pIoctlFns[GALFN_SETMONOPATTERN] = Nscgfxfn_setmonopattern;    
	pIoctlFns[GALFN_SCREENTOSCREENBLT] = Nscgfxfn_screentoscreenblt; 
	pIoctlFns[GALFN_SCREENTOSCREENXBLT] = Nscgfxfn_screentoscreenxblt;
	pIoctlFns[GALFN_BRESENHAMLINE] = Nscgfxfn_bresenhamline;

	/* VGA Support functions */
	pIoctlFns[GALFN_VGAMODESWITCH] = Nscgfxfn_vgamodeswitch;
	pIoctlFns[GALFN_VGACLEARCRTEXT] = Nscgfxfn_vgaclearextended;
	pIoctlFns[GALFN_VGASETPITCH] = Nscgfxfn_vgapitch;
	pIoctlFns[GALFN_VGARESTORE] = Nscgfxfn_vgarestore;
	pIoctlFns[GALFN_VGASAVE] = Nscgfxfn_vgasave;
	pIoctlFns[GALFN_VGASETMODE] = Nscgfxfn_vgamode;

	/* Compression functions */
	pIoctlFns[GALFN_SETCOMPRESSIONSTATE] = Nscgfxfn_setcompressionstate;
	pIoctlFns[GALFN_GETCOMPRESSIONSTATE] = Nscgfxfn_getcompressionstate;
	pIoctlFns[GALFN_SETCOMPRESSIONPARAMS] = Nscgfxfn_setcompressionparams;
	pIoctlFns[GALFN_GETCOMPRESSIONPARAMS] = Nscgfxfn_getcompressionparams;

	/* Panel related stuff */
    pIoctlFns[GALFN_PNLSETPARAMS] = Nscpnlfn_setpanelparams;
    pIoctlFns[GALFN_PNLGETPARAMS] = Nscpnlfn_getpanelparams;
    pIoctlFns[GALFN_PNLINITPANEL] = Nscpnlfn_initpanel;
    pIoctlFns[GALFN_PNLSAVESTATE] = Nscpnlfn_savepanel;
    pIoctlFns[GALFN_PNLRESTORESTATE] = Nscpnlfn_restorepanel;
    pIoctlFns[GALFN_PNLPOWERUP] = Nscpnlfn_powerup;
    pIoctlFns[GALFN_PNLPOWERDOWN] = Nscpnlfn_powerdown;

    pIoctlFns[GALFN_PNLBIOSENABLE] = Nscpnlfn_enabledinbios;
    pIoctlFns[GALFN_PNLBIOSINFO] = Nscpnlfn_infofrombios;

	/* TV related stuff */
    pIoctlFns[GALFN_SETTVPARAMS] = Nscgfxfn_settvparams;
    pIoctlFns[GALFN_GETTVPARAMS] = Nscgfxfn_gettvparams;
    pIoctlFns[GALFN_SETTVTIMING] = Nscgfxfn_settvtiming;
    pIoctlFns[GALFN_GETTVTIMING] = Nscgfxfn_gettvtiming;
    pIoctlFns[GALFN_SETENABLE] = Nscgfxfn_settvenable;
    pIoctlFns[GALFN_GETENABLE] = Nscgfxfn_gettvenable;
    pIoctlFns[GALFN_ISTVMODESUPPORTED] = Nscgfxfn_istvmodesupported;

	/* Video related stuff */
	pIoctlFns[GALFN_SETVIDEOENABLE] = Nscgfxfn_setvideoenable;
	pIoctlFns[GALFN_SETVIDEOFORMAT] = Nscgfxfn_setvideoformat;
	pIoctlFns[GALFN_SETVIDEOSIZE] = Nscgfxfn_setvideosize;
	pIoctlFns[GALFN_SETVIDEOOFFSET] = Nscgfxfn_setvideooffset;
	pIoctlFns[GALFN_SETVIDEOWINDOW] = Nscgfxfn_setvideowindow;
	pIoctlFns[GALFN_SETVIDEOSCALE] = Nscgfxfn_setvideoscale;
	pIoctlFns[GALFN_SETVIDEOFILTER] = Nscgfxfn_setvideofilter;
	pIoctlFns[GALFN_SETVIDEOCOLORKEY] = Nscgfxfn_setvideocolorkey;
	pIoctlFns[GALFN_SETVIDEODOWNSCALEENABLE] = 
					Nscgfxfn_setvideodownscale_enable;
	pIoctlFns[GALFN_SETVIDEODOWNSCALECONFIG] =
					Nscgfxfn_setvideodownscale_config;
	pIoctlFns[GALFN_SETVIDEODOWNSCALECOEFF] =
					Nscgfxfn_setvideodownscale_coefficients;
	pIoctlFns[GALFN_SETVIDEOSOURCE] = Nscgfxfn_setvideosource;
	pIoctlFns[GALFN_SETVIDEOINTERLACED] = Nscgfxfn_setvideointerlaced;
	pIoctlFns[GALFN_SETVIDEOCOLORSPACE] = Nscgfxfn_setvideocolorspace;
	pIoctlFns[GALFN_SETVIDEOCURSOR] = Nscgfxfn_setvideocursor;
	pIoctlFns[GALFN_SETVIDEOREQUEST] = Nscgfxfn_setvideorequest;
	pIoctlFns[GALFN_SETVIDEOALPHAENABLE] = Nscgfxfn_setvideoalphaenable;
	pIoctlFns[GALFN_SETVIDEOALPHAWINDOW] = Nscgfxfn_setvideoalphawindow;
	pIoctlFns[GALFN_SETVIDEOALPHAVALUE] =  Nscgfxfn_setvideoalphavalue;
	pIoctlFns[GALFN_SETVIDEOALPHAPRIORITY] = Nscgfxfn_setvideoalphapriority;
	pIoctlFns[GALFN_SETVIDEOALPHACOLOR] = Nscgfxfn_setvideoalphacolor;
	pIoctlFns[GALFN_SETVIDEOALPHAREGION] = Nscgfxfn_setvideoalpharegion;
	pIoctlFns[GALFN_SETVIDEOOUTSIDEALPHA] = Nscgfxfn_setvideooutsidealpha;
	pIoctlFns[GALFN_SETVIDEOPALETTE] = Nscgfxfn_setvideopalette;
	pIoctlFns[GALFN_GETVIDEOINFO] = Nscgfxfn_getvideoinfo;

	bDispatchTableInitialized = 1;
}

/*-----------------------------------------------------------------
 * geodefb_ioctl_ext
 *
 * Description:	This function provide an interface to the Geode frame
 *				buffer specific input output control commands.
 *  parameters:
 *       inode:	Pointer to "inode" structure which contains the device
 *				reference.
 *        file:	Pointer to a "file" structure. 
 *     	   cmd:	The IOCTL code which was issued by the application.
 *         arg:	Pointer to the data which was passed to the IOCTL by
 *				the application.
 *         con:	The console from which the ioctl was issued.
 *        info:	This is a pointer to the frame buffer information 
 *				structure.  
 *      return:	Returns 0 if the ioctl was handled successfully.
 *----------------------------------------------------------------*/
int geodefb_ioctl_ext(struct inode *inode, struct file *file, u_int cmd,
			u_long arg, int con, struct fb_info *info)
{
	int retval = -EINVAL;
	GAL_BASE sNscgfxbase;

	if (!bDispatchTableInitialized) 
	{
		Init_Dispatch_Table();
	}
	if (copy_from_user(&sNscgfxbase,(void *) arg, sizeof(sNscgfxbase))) 
	{
		retval = -EFAULT;
	} 
	else {
		if (IO_PACKET_ISVALID(sNscgfxbase)) 
		{
			retval =
			 (pIoctlFns[sNscgfxbase.dwSubfunction])((void *)arg);  
		}
	}
	return retval;
}

