/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor OSS Audio Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */


//---------------------------
//   5530 - Global variables
//---------------------------

unsigned char   Underflow;
unsigned char   Overflow;
unsigned char   Running;
unsigned char   g_Irq;
unsigned char   g_DmaChan;
unsigned char   fInUse=FALSE;

unsigned long   SampleRate;
unsigned long  *prd_array;
unsigned long   prd_array_phys_adr;
unsigned long   dwtick;
unsigned long   v_nVolume=0;
unsigned long   gIntrAudio;

unsigned short  g_IoBase;
unsigned short  g_divisor;

const static unsigned long v_dma_num_samples = DMA_BUFFER_SIZE/2;

static PCM_TYPE      g_pcmtype[2];

static unsigned char *  dma_page[4]; 
static unsigned long  GLOBAL_DMAPageSize;

//
// These variables may be accessed by different processes or the hardware and
// are likely to change in a time critical fashion. Optimization may cause
// errors if not declared volatile.
//
volatile        unsigned char   v_fMoreData[2];
volatile static unsigned long   v_nNextPage[2];

//
// The volume curve translates the volume unsigned long into a multiplier and bit
// shift value to do software volume control.
//
const struct 
{
   unsigned short multiplier;
   unsigned short divisor;     // bits to right shift
} VOL[] = { {0, 0}, 
   {3, 5},
   {7, 6},
   {1, 3},
   {5, 5},
   {3, 4},
   {7, 5},
   {1, 2},
   {5, 4},
   {3, 3},
   {7, 4},
   {1, 1},
   {5, 3},
   {3, 2},
   {7, 3},
   {1, 0}
};  

unsigned char *  m_pIoSpace = 0;                                                     
unsigned char *  m_pAudioBase;

static unsigned short  tot_input_samples     = 0;
static unsigned short  tot_output_samples    = 0;

//
// PCI audio functions base register.(r/w via i/o).
//
unsigned long pci_audio_func_reg;  

//
// Pointer to memory mapped 5530 Audio regs.                        
//
unsigned long audio_regs_adr;      

//
// This is the PCI config address value used to probe
// PCI space for PCI devices.
//
unsigned long pci_config_addr;     

