/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    Video library for Geode SCx2x0
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2000-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */
 
/*--------------------------------------------------------------------------
 * Copyright (c) 2000-2001 by National Semiconductor Corporation
 * All rights reserved.
 *--------------------------------------------------------------------------
 * File Contents:
 *           Header file for OS dependent functions.
 * Project : Video library for Geode SCx2x0
 * Created : November, 2000
 * Author  : Galit Heller & Eyal Cohen
 *--------------------------------------------------------------------------
 * $Date: 8/10/01 6:58a $
 * $RCSfile: vl_os.h $
 * $ProjectRevision: 3.1 $
 * $Revision: 6 $
 *------------------------------------------------------------------------*/

#ifndef _vl_os_h
#define _vl_os_h

/*------------------------------------------------------------------------*/
/*----------------   Constants and macros definitions  -------------------*/
/*------------------------------------------------------------------------*/


/*------------------------------------------------------------------------*/
/*---------------------   Data Types definitions   -----------------------*/
/*------------------------------------------------------------------------*/
typedef void * VL_RING_ID;


/*------------------------------------------------------------------------*/
/*-----------------------   Function headers   ---------------------------*/
/*------------------------------------------------------------------------*/


/**************************************************************************/
/************************   Interrupt services   **************************/
/**************************************************************************/
VL_STATUS OS_InterruptConnect(int IRQL, void (* function)(int), const char *int_name, int param);
VL_STATUS OS_InterruptDisconnect(int IRQL);


/**************************************************************************/
/***********************   RingBuffer services   **************************/
/**************************************************************************/
VL_RING_ID OS_RingCreate (int nbytes);
void       OS_RingDelete (VL_RING_ID ringId);
int        OS_RingBufGet (VL_RING_ID ringId, char *buffer, int maxbytes);
int        OS_RingBufPut (VL_RING_ID ringId, char *buffer, int nbytes);


/**************************************************************************/
/**************************   PCI services   ******************************/
/**************************************************************************/
VL_STATUS OS_PciReadByte(int bus, int dev, int func, int address, unsigned char * data);
VL_STATUS OS_PciWriteByte(int bus, int dev, int func, int address, unsigned char data);
VL_STATUS OS_PciReadWord(int bus, int dev, int func, int address, unsigned short * data);
VL_STATUS OS_PciWriteWord(int bus, int dev, int func, int address, unsigned short data);
VL_STATUS OS_PciReadDWord(int bus, int dev, int func, int address, unsigned long * data);
VL_STATUS OS_PciWriteDWord(int bus, int dev, int func, int address, unsigned long data);


/**************************************************************************/
/*************************   I/O port access   ****************************/
/**************************************************************************/
unsigned char  OS_IoReadByte(unsigned short port);
void           OS_IoWriteByte(unsigned short port, unsigned char data);
unsigned short OS_IoReadWord(unsigned short port);
void           OS_IoWriteWord(unsigned short port, unsigned short data);
unsigned long  OS_IoReadDWord(unsigned short port);
void           OS_IoWriteDWord(unsigned short port, unsigned long data);

#endif /* _vl_os_h */
