/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    Video library for Geode SCx2x0
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2000-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */
 
/*--------------------------------------------------------------------------
 * Copyright (c) 2000-2001 by National Semiconductor Corporation
 * All rights reserved.
 *--------------------------------------------------------------------------
 * File Contents:
 *           External interface of the SCx2x0 Video Library.
 * Project : Video library for Geode SCx2x0
 * Created : November, 2000
 * Author  : Galit Heller & Eyal Cohen
 *--------------------------------------------------------------------------
 * $Date: 8/10/01 6:58a $
 * $RCSfile: vl_rtns.h $
 * $ProjectRevision: 3.1 $
 * $Revision: 6 $
 *------------------------------------------------------------------------*/

#ifndef _vl_rtns_h
#define _vl_rtns_h

/* COMPILER OPTION FOR C++ PROGRAMS */

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------*/
/*----------------   Constants and macros definitions  -------------------*/
/*------------------------------------------------------------------------*/
#define FALSE 0
#define TRUE  1

#define EVEN_FIELD 0
#define ODD_FIELD  1


/*------------------------------------------------------------------------*/
/*---------------------   Data Types definitions   -----------------------*/
/*------------------------------------------------------------------------*/
typedef unsigned char boolean;

typedef struct factor {
	unsigned short num;
	unsigned short denom;
} factor;

typedef enum VL_STATUS {
	VL_STATUS_UNSUPPORTED = -3,
	VL_STATUS_BAD_PARAMETER,
	VL_STATUS_ERROR,
	VL_STATUS_OK
} VL_STATUS;

typedef enum VL_TVStandardType {
	VL_TV_STANDARD_NTSC = 1,
    VL_TV_STANDARD_PAL,
    VL_TV_STANDARD_NONE
} VL_TVStandardType;

typedef enum VL_TVOutputType {
	VL_TV_OUTPUT_COMPOSITE = 1,
    VL_TV_OUTPUT_S_VIDEO,
    VL_TV_OUTPUT_YUV,
    VL_TV_OUTPUT_SCART
} VL_TVOutputType;

typedef enum VL_VideoDeinterlacingMethod {
	VL_VIDEO_DEINTERLACING_NONE = 1,   /* Send interlaced video to the video processor
									      and treat the data as interlaced */
	VL_VIDEO_DEINTERLACING_INTERLEAVE, /* Send interleaved video to the video processor,
								   	      and treat the data as progressive */
	VL_VIDEO_DEINTERLACING_BOB         /* Send interlaced video to the video processor, upscale x2 vertically,
									      and Bob the resulting frames according to the field */
} VL_VideoDeinterlacingMethod;

typedef enum VL_TVFlickerFilterType {
	VL_TV_FLICKER_FILTER_NONE = 1,        /* Flicker filter is disabled */
	
	VL_TV_FLICKER_FILTER_INTERLEAVED,     /* Flicker filter is configured for progressive video, to be
											 used when either Bob or Interleave de-interlacing are used */
	VL_TV_FLICKER_FILTER_INTERLACED       /* Flicker filter is configured for interlaced video, to be
											 used when no de-interlacing is done */
} VL_TVFlickerFilterType;

typedef enum VL_VOPSignalSelectType {
	VL_SELECT_VOP_SIGNALS = 1,
	VL_SELECT_TFT_SIGNALS
} VL_VOPSignalSelectType;

typedef enum VL_VideoWindowSize {
	VL_VIDEO_WINDOW_FULL = 1,
	VL_VIDEO_WINDOW_SCALED
} VL_VideoWindowSize;

typedef enum VL_VideoSourceType { // The source from which the video processor shows full screen video
	VL_VIDEO_SOURCE_MEMORY = 1,
	VL_VIDEO_SOURCE_DVIP,
	VL_VIDEO_SOURCE_DVIP_ONLY
} VL_VideoSourceType;

typedef enum VL_TVAspectRatioType { // The aspect ratio on TV display for graphics and/or video
	VL_TV_ASPECT_RATIO_GRAPHICS_ONLY = 1,
	VL_TV_ASPECT_RATIO_GRAPHICS_AND_VIDEO,
	VL_TV_ASPECT_RATIO_VIDEO_ONLY
} VL_TVAspectRatioType;

typedef enum VL_VideoOverlayModeType { // The aspect ratio on TV display for graphics and/or video
	VL_VIDEO_OVERLAY_COLOR_KEY = 1,
	VL_VIDEO_OVERLAY_CHROMA_KEY,
	VL_VIDEO_OVERLAY_COLOR_KEY_INSIDE_ALPHA_AND_VIDEO_OUTSIDE,
	VL_VIDEO_OVERLAY_CHROMA_KEY_INSIDE_ALPHA_AND_GRAPHICS_OUTSIDE,
	VL_VIDEO_OVERLAY_ALWAYS_ON
} VL_VideoOverlayModeType;

typedef struct VL_MemBuf {
    unsigned long address;
    unsigned short width;
    unsigned short height;
} VL_MemBuf;

typedef struct VL_GfxStruct {
    factor GfxAspectRatio;     // Graphics horizontal scaling for TV
	unsigned short GfxWidth;   // Gfx horizontal resolution
	unsigned short GfxHeight;  // Gfx vertical resolution
	unsigned short GfxPitch;   // Gfx pitch
} VL_GfxStruct;

typedef struct VL_VideoStruct {
    VL_TVStandardType TVStandard;
    VL_TVOutputType TVOutput;
    boolean CRTEnable;
    VL_VideoSourceType FullScreenSource;
    unsigned long GenlockDelay;
    boolean FieldPolarityInvert;
    unsigned char TimerIRQ;
    boolean SignalMonitorEnable;
    unsigned char SignalMonitorGPIONum;
    unsigned char SignalMonitorIRQ;
    // Graphics related settings
    VL_TVAspectRatioType TVAspectRatio; // Determine graphics/video aspect ratio on TV
    // Return values
	unsigned short Width;      // Video horizontal resolution
	unsigned short Height;     // Video vertical resolution
	VL_GfxStruct GfxStruct;
} VL_VideoStruct;

typedef struct VL_VbiStruct {
    unsigned char VipIRQ;
    unsigned char VipINT;                       // INTA = 1, INTB = 2, INTC = 3, INTD = 4
    void (* VL_VbiGetEmptyBuf)(unsigned long *phys_adr, unsigned long *virt_adr, char field_type);
    void (* VL_VbiNotifyFullBuf)(unsigned long buf, char field_type);
    unsigned long VbiLineSize;
} VL_VbiStruct;


/*------------------------------------------------------------------------*/
/*-----------------------   Function headers   ---------------------------*/
/*------------------------------------------------------------------------*/


/**************************************************************************/
/*******************   Initialization / Termination   *********************/
/**************************************************************************/

VL_STATUS VL_VideoInit(VL_VideoStruct * video_struct);

VL_STATUS VL_VideoEnd();

char * VL_Version();


/**************************************************************************/
/**************************   Alpha Blending   ****************************/
/**************************************************************************/

VL_STATUS VL_VideoSetAlphaWindow(unsigned char number, short x, short y,
   unsigned short width, unsigned short height);

VL_STATUS VL_VideoSetAlphaOperation(unsigned char number, unsigned char alpha,
   unsigned long color, boolean color_enable, int priority, char fade, boolean enable);


/**************************************************************************/
/*****************************   Color Key   ******************************/
/**************************************************************************/

VL_STATUS VL_VideoSetColorKey(unsigned long key, unsigned long mask);

VL_STATUS VL_VideoSetOverlay(VL_VideoOverlayModeType video_overlay_mode);


/**************************************************************************/
/**************************   Video controls   ****************************/
/**************************************************************************/

VL_STATUS VL_VideoEnable(boolean OnOff);
boolean VL_VideoIsEnabled();

VL_STATUS VL_VideoPause();
boolean VL_VideoIsPaused();

VL_STATUS VL_VideoResume();

unsigned short * VL_VideoGetRow(int j);

VL_STATUS VL_VideoSetDeinterlacingMethod(
   VL_VideoWindowSize size,
   VL_VideoDeinterlacingMethod method);
   
VL_STATUS VL_TVSetFlickerFilter(VL_TVFlickerFilterType FlickerFilter);   

VL_STATUS VL_VideoSetFullScreenSource(VL_VideoSourceType type, unsigned long genlock_delay);

VL_STATUS VL_VideoSourceActive(boolean OnOff);

VL_STATUS VL_FieldPolarityInvert(boolean OnOff);

VL_STATUS VL_TVAspectRatio(VL_TVAspectRatioType tv_aspect_ratio,
						   VL_GfxStruct * gfx_struct);

VL_STATUS VL_VOPSignalSelect(VL_VOPSignalSelectType type);


/**************************************************************************/
/****************************   VBI module   ******************************/
/**************************************************************************/

VL_STATUS VL_VbiInit(VL_VbiStruct * vbi_struct);

VL_STATUS VL_VbiEnd();

VL_STATUS VL_VbiCapture(boolean enable);

boolean VL_VbiIsCaptured();

VL_STATUS VL_VbiCCPassThruEnable(boolean OnOff, int cc_line_num);

boolean VL_VbiCCPassThruIsEnabled();

boolean VL_VbiGetCCData(unsigned char * ch1, unsigned char * ch2);


/**************************************************************************/
/***************************   Video Window   *****************************/
/**************************************************************************/

VL_STATUS VL_VideoSetWindow(VL_MemBuf * mem_buf,
   short x, short y, unsigned short width, unsigned short height);

VL_STATUS VL_VideoCfgCaptureBuf(VL_MemBuf * mem_buf);

boolean VL_VideoCaptureBufIsUpdated();

VL_STATUS VL_VideoCfgMemoryWindow(VL_MemBuf * mem_buf,
   unsigned short x, unsigned short y, unsigned short width, unsigned short height);

VL_STATUS VL_VideoCfgDisplayWindow(unsigned short x, unsigned short y,
   unsigned short width, unsigned short height);

/* CLOSE BRACKET FOR C++ COMPILATION */

#ifdef __cplusplus
}
#endif

#endif /* _vl_rtns_h */
