/*
 * $Workfile: crlf.c $
 * $Revision: 1 $
 * $Date: 10/17/01 4:14a $
 *
 * Copyright (c) 1999-2000 National Semiconductor Corporation.
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of National 
 * Semiconductor Corporation. ("Confidential Information").
 * You shall not disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered into
 * with National Semiconductor Corporation.
 * This code is supplied as is.
 *
 */

/*
 *$History: crlf.c $
 * 
 * *****************  Version 1  *****************
 * User: Sojan        Date: 10/17/01   Time: 4:14a
 * Created in $/Engineering_Releases/drivers/linux/geode_v4l2
 * 
 * *****************  Version 3  *****************
 * User: Sarma        Date: 3/29/01    Time: 4:08p
 * Updated in $/Engineering_Releases/drivers/linux/framebuffer/Linux-2.4.0/geode
 * Added recursive option to operate all the files in sub-directories.
 * 
 * *****************  Version 5  *****************
 * User: Sarma        Date: 3/29/01    Time: 4:06p
 * Updated in $/Engineering_Releases/drivers/linux/Xfree86/xfree86_v336/Kernel_2.2/xc/programs/Xserver/hw/xfree86/vga256/drivers/geode
 * Added recursive option to operate all the files in sub-directories.
 * 
 * *****************  Version 2  *****************
 * User: Sarma        Date: 12/25/00   Time: 8:23p
 * Updated in $/Engineering_Releases/drivers/linux/framebuffer/Linux-2.4.0-test1/geode
 * Added National Semiconductor Corp. copyright info.
*/


#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>

#include <unistd.h>
#include <errno.h>

int convert_crlf( const char* filename );

int recursive_crlf()
{
	DIR*            dir;
	struct dirent *direntp;

	printf( "converting all files...\n" );
	dir = opendir( "." );
	if( dir == NULL )
	{
		printf( "failed to open dir\n" );
		exit(1);
	}

	while( ( direntp = readdir(dir) ) != NULL ) 
	{
		/*printf( "%s\n", direntp->d_name );*/
		if( strcmp( direntp->d_name, "." ) == 0 )
		{
			continue;
		}
		else if( strcmp( direntp->d_name, ".." ) == 0 )
		{
			continue;
		}
		else
		{
			if(!convert_crlf( direntp->d_name ))
			{
				printf("Attempting to treat %s as a directory.\n", direntp->d_name);
				if(chdir( direntp->d_name )!=-1)
				{
					printf("Changed into directory %s.\n", direntp->d_name);
					recursive_crlf();
					chdir("..");
				}
				else 
				{
					printf("Cannot change into directory %s.\n", direntp->d_name);
					printf("%d\n", errno);
				}  
			}
		}
	}

	closedir( dir );
	return 0;
}


void usage()
{
	printf( "crlf [-a][-r][<filename>] converts CR/LF to CR\n" );
	printf( "\t-a converts all files in the current directory.\n" );
	printf( "\t-r converts all files recursively.\n" );
	printf( "\t[<filename>] converts the file provided.\n" );
}


int main( int argc, char* argv[] )
{
	DIR*  dir;
	struct dirent *direntp;

	if( argc < 2 )
	{
		usage();
		exit(1);
	}

	if( strcmp( argv[ 1 ], "-r" ) == 0 ){ 
		recursive_crlf();
	} else if( strcmp( argv[ 1 ], "-a" ) == 0 )
	{
		printf( "converting all files...\n" );
		dir = opendir( "." );
		if( dir == NULL )
		{
			printf( "failed to open dir\n" );
			exit(1);
		}

		while( ( direntp = readdir(dir) ) != NULL ) 
		{
			/*printf( "%s\n", direntp->d_name );*/
			if( strcmp( direntp->d_name, "." ) == 0 )
			{
				continue;
			}
			else if( strcmp( direntp->d_name, ".." ) == 0 )
			{
				continue;
			}
			else
			{
				convert_crlf( direntp->d_name );
			}
		}

		closedir( dir );
		exit(0);
	} else {
		convert_crlf( argv[ 1 ] );
	}

	return 1;
}


int convert_crlf( const char* filename )
{
	FILE* fr;
	char r_buffer[ 0xFFFFF ];
	char w_buffer[ 0xFFFFF ];
	int  wcntr;
	int  rcntr;
	int  i;

	if(!( fr = fopen( filename, "r" )))
	{
		printf( "failed to open for read %s\n", filename );
		return 0;
	}

	printf( "converting %s\n", filename );

	wcntr = 0;
	rcntr = fread( r_buffer, 1, sizeof(r_buffer), fr );

	for( i = 0; i < rcntr; i++ )
	{
		if( r_buffer[ i ] != 0x0D )
		{
			w_buffer[ wcntr++ ] = r_buffer[ i ];
		}
	}

	fclose(fr);

	if(!( fr = fopen( filename, "w" )))
	{
		printf( "failed to open for write %s\n", filename );
		return 0;
	}

	fwrite( w_buffer, 1, wcntr, fr );

	fclose(fr);

	return 1;
}