/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Geode Video 4 Linux 2 Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */
/* Test application for the V4l2 geode driver. 
   Author : Sojan James
   Date: Feb 23, 2001
 */
   
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>

#include <string.h>
#include<stdio.h>

#include "ioctl_geode_v.h"
#define OVERLAY_KEY 0x00f000f0
#define OVERLAY_MASK 0x00f0f0f0

/* Primitive test application. To see video on the screen, have something
   put a cyan background on the screen. 
   */

int fd_video = -1;
int fd_vout = -1;


void dvip(int enable)
{
	printf("DVIP:%d\n",ioctl(fd_vout,GEODE_IOCTL_S_DVIP,&enable));

}

		


void open_devices()
{
	fd_video = open("/dev/video", O_RDWR);
	fd_vout = open("/dev/vout", O_RDWR);

	printf("         FD: %d,%d\n",fd_video,fd_vout);

}

void close_devices()
{
	close(fd_video);
	close(fd_vout);
}

void get_vout_format(int * width, int* height)
{
	struct v4l2_format fmt;
	printf("G_FMT:%d\n",ioctl(fd_vout,VIDIOC_G_FMT,&fmt));
	*width = fmt.fmt.pix.width;
	*height = fmt.fmt.pix.height;
}

void set_overlay_info( unsigned long key, unsigned long mask, int graphics, 
			int enable)
{
	overlay_info_t ovr_info;
	ovr_info.ColorKey = key;
	ovr_info.ColorMask = mask;
	printf("%d:\n",ioctl(fd_vout, GEODE_IOCTL_SET_OVERLAY_INFO, &ovr_info));
}

void set_alpha_blend_info(int window, int x, int y, int w, int h, 
			  int priority, int alpha, int delta, int enable,
			  unsigned long color)
{
	alpha_blend_info_t alp_info;
	alp_info.WindowNumber = window;
	alp_info.StartX = x;
	alp_info.StartY = y;
	alp_info.Width = w;
	alp_info.Height = h;
	alp_info.Alpha = alpha;
	alp_info.Delta = delta;
	alp_info.Enable = enable;
	alp_info.Color = color;


	printf("%d:\n",ioctl(fd_vout, GEODE_IOCTL_SET_ALPHA_BLEND_INFO, &alp_info));
}

void set_video_fmt(int width, int height)
{
	struct v4l2_format fmt;
	fmt.fmt.pix.width = width;
	fmt.fmt.pix.height = height;

	printf("%d:\n",ioctl(fd_video, VIDIOC_S_FMT, &fmt));
}

void set_vout_window(int x, int y, int width, int height)
{
	struct v4l2_window win;
	win.x = x;
	win.y = y;
	win.width = width;
	win.height = height;

	win.clipcount = 0;

	printf("%d:\n",ioctl(fd_vout, VIDIOC_S_WIN, &win));
}

void set_input(int inp)
{
	int input = inp;
	printf("%d:\n",ioctl(fd_video, VIDIOC_S_INPUT,&input));
}

void capture_enable( int enable)
{
	int dummy;
	if(enable)
	{
		printf("%d:\n",ioctl(fd_video, VIDIOC_STREAMON, &dummy));
	}
	else
		printf("%d:\n",ioctl(fd_video, VIDIOC_STREAMOFF, &dummy));
}

void preview(int enable)
{
		printf("%d:\n",ioctl(fd_vout, VIDIOC_PREVIEW,&enable));

}

void set_chan(int ch)
{
	unsigned long c = ch;
	printf("%d:\n",ioctl(fd_video, VIDIOC_S_FREQ,&c));
}

void enum_fx(int index)
{
	struct v4l2_fxdesc fxdesc;
	int ret;
	fxdesc.index = index;
	ret = ioctl(fd_vout,VIDIOC_ENUMFX,&fxdesc);
	if(ret < 0) printf("Error on ENUMFX\n");
	printf("Effect : %s", fxdesc.name);
}

void set_vout_control(int val)
{
	int ret;
	int i = 0;
	struct v4l2_control ctrl;
	ret = ioctl(fd_vout,VIDIOC_S_EFFECT,&i);
	if(ret < 0) printf("Error on S_EFFECT\n");
	ctrl.id = GEODE_CID_OVERLAY_MODE;
	ctrl.value = val;
	ret = ioctl(fd_vout, VIDIOC_S_CTRL, &ctrl);
	if(ret < 0) printf("Error on S_CTRL\n");
}

int main()
{
	int width,height;
	int enable;
	int i;
	geode_ioctl_debug_i2c_t debug;
	printf("You shouldn't be running this test, there is a better one in the tests directory\n");
	open_devices();
	get_vout_format(&width,&height);
	printf("Width :%d, Height :%d\n",width,height);
	set_input(1);
	set_chan(4);
	set_video_fmt(400,200);
	capture_enable(1);
	set_overlay_info( OVERLAY_KEY, OVERLAY_MASK, 1, 1);
	enum_fx(0);

	
	set_vout_window(20,20,400,200);
	set_vout_control(5); /* no color key */
	preview(1); // show video

	getchar();
	printf("-----DECODER REGISTER DUMP------\n");
	for(i = 0; i <= 0xd0 ; i++)
	{
		debug.addr = (unsigned char)i;
		ioctl(fd_video,GEODE_IOCTL_GET_DECODER_REG,&debug);
		printf("%x : %x\n",i,debug.data);
	}

	getchar();


	set_input(3);
	printf("Set input to 3\n");

	preview(1);
	getchar();

	set_video_fmt(width,height);
	set_vout_window(0,0,width,height);

	printf("--------************************************DVIPing\n");
	dvip(1);
	//set_vout_window(0,0,width,height);
	getchar();

	printf("            UN DVIPing\n");
	dvip(0);
	//set_vout_window(0,0,width,height);
	getchar();
	set_input(2);

	printf("--------************************************DVIPing\n");
	dvip(1);
	//set_vout_window(0,0,width,height);
	getchar();

	printf("            UN DVIPing\n");
	dvip(0);
	//set_vout_window(0,0,width,height);
	getchar();


	set_video_fmt(400,200);
	capture_enable(1);
	set_overlay_info( OVERLAY_KEY, OVERLAY_MASK, 1, 1);
	set_vout_window(20,20,400,200);

	set_input(2);
	printf("Set input to 1\n");
	preview(1);
	capture_enable(0);
	getchar();
	capture_enable(1);
	getchar();
	capture_enable(0);
	getchar();
	capture_enable(1);
	getchar();

	//set_input(3);
	//printf("Set input to 3\n");
	//getchar();

	//set_input(2);
	//printf("Set input to 2\n");
	//getchar();

	//printf("Disabling Alpha\n");
	//set_alpha_blend_info(0,50, 50, 320,240, 1,125,0,0,125);
	//getchar();
	//printf("dis ovr\n");
	//set_overlay_info( OVERLAY_KEY, OVERLAY_MASK, 1, 0);
	//getchar();

	//set_input(3);
	//printf("Set input to 3\n");
	//getchar();


	set_input(2);
	printf("Set input to 1\n");
	getchar();
	
/*
	set_input(0);
	printf("Set input to 0\n");
	getchar();
*/
	
	//set_overlay_info( OVERLAY_KEY, OVERLAY_MASK, 1, 0);
	preview(0);
	set_vout_control(1); /* no color key */
	


	
	close_devices();
	printf("Cleaning up\n");
}
