/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to abstract the operating system from the 
 * drivers.  This includes handling sleep/delay, memory access, printing
 * debug output, system timers, etc.
 *
 * OS-abstraction functions:
 * 
 *    tv_os_dbg_msg
 *    tv_os_err_msg              
 *    tv_os_get_tick_count
 *    tv_os_sleep                
 *    tv_os_kmalloc              
 *    tv_os_kfree                
 *    tv_os_kmalloc_phys         
 *    tv_os_kfree_phys           
 *    tv_os_pci_dump             
 *    tv_os_pci_find_device      
 *    tv_os_pci_read_ulong       
 *    tv_os_pci_write_ulong      
 *    tv_os_pci_read_byte        
 *    tv_os_pci_write_byte       
 *    tv_os_pciconfig_read_byte  
 *    tv_os_pciconfig_write_byte 
 *    tv_os_pciconfig_read_word  
 *    tv_os_pciconfig_write_word 
 *    tv_os_pciconfig_read_ulong 
 *    tv_os_pciconfig_write_ulong
 *    tv_os_io_read_byte         
 *    tv_os_io_write_byte        
 *    tv_os_io_read_word         
 *    tv_os_io_write_word        
 *    tv_os_io_read_ulong        
 *    tv_os_io_write_ulong       
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#ifndef TV_OS_GEODE_H
#define TV_OS_GEODE_H

#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef LINUX
#include <asm/io.h> 
#include <linux/malloc.h>
#include <linux/pci.h>
#include <linux/interrupt.h>
#elif defined(WINCE)
#include <windows.h>
#include <stdlib.h>
#include <hal.h>
#include <ceddk.h>
#elif defined(WIN98)
#include <wdm.h>
#include <stdarg.h>
#include <stdio.h>
#elif defined(VXWORKS)
#include <vxWorks.h>
#include <intLib.h>
#include <iv.h>
#include <drv/pci/pciConfigLib.h>
#include <sysLib.h>
#include <rngLib.h>
#endif

#ifdef LINUX
//	Linux swaps port/data parameters
#define WRITE_PORT_UCHAR( x, y )  outb( (int)y, (int)x )
#define READ_PORT_UCHAR( x )      inb( (int)x )
#elif defined(WINCE)
#elif defined(WIN98)
#elif defined(VXWORKS)
#define WRITE_PORT_UCHAR( x, y )  sysOutByte( x, y )
#define READ_PORT_UCHAR( x )      sysInByte( x )
#endif

#ifndef BOOL
#define BOOL	unsigned char
#endif
#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif
#ifndef UCHAR
#define UCHAR	unsigned char
#endif
#ifndef BYTE
#define BYTE	unsigned char
#endif
#ifndef WORD
#define WORD	unsigned short
#endif
#ifndef ULONG
#ifdef LINUX
#define ULONG	unsigned long
#else
#define ULONG	unsigned long
#endif
#ifndef DWORD
#define DWORD unsigned long
#endif
#endif

#ifdef WINCE
#define inp(x)			_inp(x)
#define outp(x,y)		_outp(x,y)
#endif

#ifdef LINUX
#ifdef DEBUG
	#define tv_os_dbg_msg(fmt,arg...) printk(KERN_ERR "tv: " fmt,##arg)
#else
	#define tv_os_dbg_msg(fmt,arg...) 
#endif	
	//#define tv_os_dbg_msg(fmt,arg...) printk(KERN_DEBUG "tv: " fmt,##arg)
	#define tv_os_err_msg(fmt,arg...) printk(KERN_ERR "tv: " fmt,##arg)
	#define _T(x)	x
#elif defined(WINCE)
	void	tv_os_dbg_msg( LPCSTR szFormat, ... );
	void	tv_os_err_msg( LPCSTR szFormat, ... );
#elif defined(WIN98)
	void	tv_os_dbg_msg( LPCSTR szFormat, ... );
	void	tv_os_err_msg( LPCSTR szFormat, ... );
#elif defined(VXWORKS)
	#define _T(x)	x
#endif

#ifndef max
#define max(x,y)		( (x) > (y) ? (x) : (y) )
#endif
#ifndef min
#define min(x,y)		( (x) < (y) ? (x) : (y) )
#endif
#ifndef LOBYTE
#define LOBYTE(x)		( (x) & 0x00FF )
#endif
#ifndef HIBYTE
#define HIBYTE(x)		( ( (x) & 0xFF00 ) >> 8 )
#endif

ULONG tv_os_get_tick_count();
void tv_os_sleep( ULONG milliseconds );
void* tv_os_kmalloc( ULONG size );
void tv_os_kfree( void* data );
void* tv_os_kmalloc_phys(  ULONG address, ULONG size );
#ifdef WIN98
void tv_os_kfree_phys( void* data, ULONG size );
#else
void tv_os_kfree_phys( void* data );
#endif

#if defined(WINCE) || defined(WIN98)
BOOL tv_os_pci_dump( WORD bus, WORD vendor, WORD device, WORD fn );
BOOL tv_os_pci_find_device( WORD bus, WORD vendor, WORD device, WORD fn, ULONG* config_addr );
BOOL tv_os_pci_read_ulong( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, ULONG* data );
BOOL tv_os_pci_write_ulong( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, ULONG data );
BOOL tv_os_pci_read_byte( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, BYTE* data );
BOOL tv_os_pci_write_byte( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, BYTE data );
#endif

BOOL tv_os_pciconfig_read_byte( int bus, int dev, int func, int address, BYTE* data );
BOOL tv_os_pciconfig_write_byte( int bus, int dev, int func, int address, BYTE data );
BOOL tv_os_pciconfig_read_word( int bus, int dev, int func, int address, WORD* data );
BOOL tv_os_pciconfig_write_word( int bus, int dev, int func, int address, WORD data);
BOOL tv_os_pciconfig_read_ulong( int bus, int dev, int func, int address, ULONG* data );
BOOL tv_os_pciconfig_write_ulong( int bus, int dev, int func, int address, ULONG data );
BYTE tv_os_io_read_byte( WORD port );
void tv_os_io_write_byte( WORD port, BYTE data );
unsigned short tv_os_io_read_word( WORD port );
void tv_os_io_write_word( WORD port, WORD data );
ULONG tv_os_io_read_ulong( WORD port );
void tv_os_io_write_ulong( WORD port, ULONG data );

#ifdef __cplusplus
}
#endif

#endif	// TV_OS_GEODE_H
