/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to program the SAA7114 video decoder.
 *
 * Decoder functions:
 * 
 *    tv_saa7114_detect_hw
 *    tv_saa7114_software_reset
 *    tv_saa7114_set_input_win
 *    tv_saa7114_set_output_win
 *    tv_saa7114_set_decoder_vbi_upscale
 *    tv_saa7114_decoder_detect_macrovision
 *    tv_saa7114_decoder_detect_video
 *    tv_saa7114_set_decoder_analog_input
 *    tv_saa7114_set_decoder_brightness
 *    tv_saa7114_set_decoder_contrast
 *    tv_saa7114_set_decoder_hue
 *    tv_saa7114_set_decoder_saturation
 *    tv_saa7114_set_decoder_input_offset
 *    tv_saa7114_set_decoder_input_size
 *    tv_saa7114_set_decoder_output_size
 *    tv_saa7114_set_decoder_scale
 *    tv_saa7114_set_decoder_vbi_format
 *    tv_saa7114_set_decoder_vbi_enable
 *    tv_saa7114_get_decoder_brightness
 *    tv_saa7114_get_decoder_contrast
 *    tv_saa7114_get_decoder_hue
 *    tv_saa7114_get_decoder_saturation
 *    tv_saa7114_get_decoder_input_offset
 *    tv_saa7114_get_decoder_input_size
 *    tv_saa7114_get_decoder_output_size
 *    tv_saa7114_get_decoder_vbi_format
 *    tv_saa7114_set_tv_standard
 *    tv_saa7114_set_luminance
 *    tv_saa7114_initialize_ntsc
 *    tv_saa7114_xport_enable
 *    tv_saa7114_set_input
 *    tv_saa7114_set_scaling_parameters
 *    tv_saa7114_scale_video
 *    tv_saa7114_set_vbi_input_offset
 *    tv_saa7114_set_vbi_input_size
 *    tv_saa7114_set_vbi_output_size
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#include "tv_saa7114.h"

typedef struct tagDECODER_REG
{
	BYTE		ucRegister;
	BYTE		ucValue;
} DECODER_REG, *PDECODER_REG;

#if 1
static DECODER_REG tv_saa7114_init_vals_ntsc[] =
{
	{ 0x01, 0x08 }, { 0x02, 0xC0 }, { 0x03, 0x00 }, { 0x04, 0x90 },
        { 0x05, 0x90 }, { 0x06, 0xEB }, { 0x07, 0xE0 }, { 0x08, 0x88 },
        { 0x09, 0x45 }, { 0x0A, 0x80 }, { 0x0B, 0x44 }, { 0x0C, 0x40 },
        { 0x0D, 0x00 }, { 0x0E, 0x89 }, { 0x0F, 0x2E }, { 0x10, 0x0E },
        { 0x11, 0x00 }, { 0x12, 0x05 }, { 0x13, 0x00 }, { 0x14, 0x08 },
        { 0x15, 0x11 }, { 0x16, 0xFE }, { 0x17, 0x00 }, { 0x18, 0x40 },
        { 0x19, 0x80 }, { 0x30, 0xBC }, { 0x31, 0xDF }, { 0x32, 0x02 },
        { 0x34, 0xCD }, { 0x35, 0xCC }, { 0x36, 0x3A }, { 0x38, 0x03 },
        { 0x39, 0x10 }, { 0x3A, 0x00 }, { 0x40, 0x00 }, { 0x41, 0xFF },
        { 0x42, 0xFF }, { 0x43, 0xFF }, { 0x44, 0xFF }, { 0x45, 0xFF },
        { 0x46, 0xFF }, { 0x47, 0xFF }, { 0x48, 0xFF }, { 0x49, 0xFF },
        { 0x4A, 0xFF }, { 0x4B, 0xFF }, { 0x4C, 0xFF }, { 0x4D, 0xFF },
        { 0x4E, 0xFF }, { 0x4F, 0xFF }, { 0x50, 0xFF }, { 0x51, 0xFF },
        { 0x52, 0xFF }, { 0x53, 0xFF }, { 0x54, 0xFF }, { 0x55, 0xFF },
        { 0x56, 0xFF }, { 0x57, 0xFF }, { 0x58, 0x00 }, { 0x59, 0x47 },
        { 0x5A, 0x06 }, { 0x5B, 0x43 }, { 0x5D, 0x3E }, { 0x5E, 0x00 },
        { 0x80, 0x20 }, { 0x83, 0x00 }, { 0x84, 0x60 }, { 0x85, 0x00 },
        { 0x86, 0xe5 }, { 0x87, 0x01 }, { 0x88, 0xF8 },
	/* VBI task */

        { 0x90, 0x01 }, { 0x91, 0xC8 }, { 0x92, 0x08 }, { 0x93, 0x84 },
        { 0x94, 0x10 }, { 0x95, 0x00 }, { 0x96, 0xD0 }, { 0x97, 0x02 },
        { 0x98, 0x0A }, { 0x99, 0x00 }, { 0x9A, 0x0C }, { 0x9B, 0x00 },
        { 0x9C, 0xd0 }, { 0x9D, 0x02 }, { 0x9E, 0x0c }, { 0x9F, 0x00 },
        { 0xA0, 0x01 }, { 0xA1, 0x00 }, { 0xA2, 0x00 }, { 0xA4, 0x80 },
        { 0xA5, 0x40 }, { 0xA6, 0x40 }, { 0xA8, 0x00 }, { 0xA9, 0x04 },
        { 0xAA, 0x00 }, { 0xAC, 0x00 }, { 0xAD, 0x01 }, { 0xAE, 0x00 },
        { 0xB0, 0x00 }, { 0xB1, 0x04 }, { 0xB2, 0x00 }, { 0xB3, 0x04 },
        { 0xB4, 0x00 }, { 0xB8, 0x00 }, { 0xB9, 0x00 }, { 0xBA, 0x00 },
        { 0xBB, 0x00 }, { 0xBC, 0x00 }, { 0xBD, 0x00 }, { 0xBE, 0x00 },
        { 0xBF, 0x00 },

        /* Video task */

        { 0xC0, 0x80 }, { 0xC1, 0x08 }, { 0xC2, 0x00 }, { 0xC3, 0x80 },
        { 0xC4, 0x10 }, { 0xC5, 0x00 }, { 0xC6, 0xD0 }, { 0xC7, 0x02 },
        { 0xC8, 0x16 }, { 0xC9, 0x00 }, { 0xCA, 0xF1 }, { 0xCB, 0x00 },
        { 0xCC, 0xD0 }, { 0xCD, 0x02 }, { 0xCE, 0xF1 }, { 0xCF, 0x00 },
        { 0xD0, 0x01 }, { 0xD1, 0x00 }, { 0xD2, 0x00 }, { 0xD4, 0x80 },
        { 0xD5, 0x40 }, { 0xD6, 0x40 }, { 0xD8, 0x00 }, { 0xD9, 0x04 },
        { 0xDA, 0x00 }, { 0xDC, 0x00 }, { 0xDD, 0x02 }, { 0xDE, 0x00 },
        { 0xE0, 0x00 }, { 0xE1, 0x04 }, { 0xE2, 0x00 }, { 0xE3, 0x04 },
        { 0xE4, 0x00 }, { 0xE8, 0x00 }, { 0xE9, 0x00 }, { 0xEA, 0x00 },
        { 0xEB, 0x00 }, { 0xEC, 0x00 }, { 0xED, 0x00 }, { 0xEE, 0x00 },
        { 0xEF, 0x00 },
}; 

#else
static DECODER_REG tv_saa7114_init_vals_ntsc[] =
{                     
	{ 0x01, 0x08 },  
	{ 0x02, 0xC0 },  
	{ 0x03, 0x00 },  
	{ 0x04, 0x90 },  
	{ 0x05, 0x90 },  
	{ 0x06, 0xEB },  
	{ 0x07, 0xE0 },  
	{ 0x08, 0x88 },  
	{ 0x09, 0x45 },  
	{ 0x0A, 0x80 },  
	{ 0x0B, 0x44 },  
	{ 0x0C, 0x40 },  
	{ 0x0D, 0x00 },  
	{ 0x0E, 0x89 },  
	{ 0x0F, 0x2E },  
	{ 0x10, 0x0E },  
	{ 0x11, 0x00 },  
	{ 0x12, 0x05 },  
	{ 0x13, 0x00 },  
	{ 0x14, 0x08 },  
	{ 0x15, 0x11 },  
	{ 0x16, 0xFE },  
	{ 0x17, 0x00 },  
	{ 0x18, 0x40 },  
	{ 0x19, 0x80 },  
	{ 0x30, 0xBC },  
	{ 0x31, 0xDF },  
	{ 0x32, 0x02 },  
	{ 0x34, 0xCD },  
	{ 0x35, 0xCC },  
	{ 0x36, 0x3A },  
	{ 0x38, 0x03 },  
	{ 0x39, 0x10 },  
	{ 0x3A, 0x00 },  
	{ 0x40, 0x00 },  
	{ 0x41, 0xFF },  
	{ 0x42, 0xFF },  
	{ 0x43, 0xFF },  
	{ 0x44, 0xFF },  
	{ 0x45, 0xFF },  
	{ 0x46, 0xFF },  
	{ 0x47, 0xFF },  
	{ 0x48, 0xFF },  
	{ 0x49, 0xFF },  
	{ 0x4A, 0xFF },  
	{ 0x4B, 0xFF },  
	{ 0x4C, 0xFF },  
	{ 0x4D, 0xFF },  
	{ 0x4E, 0xFF },  
	{ 0x4F, 0xFF },  
	{ 0x50, 0xFF },  
	{ 0x51, 0xFF },  
	{ 0x52, 0xFF },  
	{ 0x53, 0xFF },  
	{ 0x54, 0xFF },  
	{ 0x55, 0xFF },  
	{ 0x56, 0xFF },  
	{ 0x57, 0xFF },  
	{ 0x58, 0x00 },  
	{ 0x59, 0x47 },  
	{ 0x5A, 0x06 },  
	{ 0x5B, 0x43 },  
	{ 0x5D, 0x3E },  
	{ 0x5E, 0x00 },  
	{ 0x80, 0x20 },  
	{ 0x83, 0x00 },  
	{ 0x84, 0x60 },  
	{ 0x85, 0x00 },  
	{ 0x86, 0xE5 },  
	{ 0x87, 0x01 },  
	{ 0x88, 0xF8 },  
	{ 0x90, 0x01 },  
	{ 0x91, 0xC8 },  
	{ 0x92, 0x08 },  
	{ 0x93, 0x84 },  
	{ 0x94, 0x10 },  
	{ 0x95, 0x00 },  
	{ 0x96, 0xD0 },  
	{ 0x97, 0x02 },  
	{ 0x98, 0x05 },  
	{ 0x99, 0x00 },  
	{ 0x9A, 0x0B },  
	{ 0x9B, 0x00 },  
	{ 0x9C, 0xA0 },  
	{ 0x9D, 0x05 },  
	{ 0x9E, 0x0B },  
	{ 0x9F, 0x00 },  
	{ 0xA0, 0x01 },  
	{ 0xA1, 0x00 },  
	{ 0xA2, 0x00 },  
	{ 0xA4, 0x80 },  
	{ 0xA5, 0x40 },  
	{ 0xA6, 0x40 },  
	{ 0xA8, 0x00 },  
	{ 0xA9, 0x02 },  
	{ 0xAA, 0x00 },  
	{ 0xAC, 0x00 },  
	{ 0xAD, 0x01 },  
	{ 0xAE, 0x00 },  
	{ 0xB0, 0x00 },  
	{ 0xB1, 0x04 },  
	{ 0xB2, 0x00 },  
	{ 0xB3, 0x04 },  
	{ 0xB4, 0x00 },  
	{ 0xB8, 0x00 },  
	{ 0xB9, 0x00 },  
	{ 0xBA, 0x00 },  
	{ 0xBB, 0x00 },  
	{ 0xBC, 0x00 },  
	{ 0xBD, 0x00 },  
	{ 0xBE, 0x00 },  
	{ 0xBF, 0x00 },  
	{ 0xC0, 0x80 },  
	{ 0xC1, 0x08 },  
	{ 0xC2, 0x00 },  
	{ 0xC3, 0x80 },  
	{ 0xC4, 0x10 },  
	{ 0xC5, 0x00 },  
	{ 0xC6, 0xD0 },  
	{ 0xC7, 0x02 },  
	{ 0xC8, 0x11 },  
	{ 0xC9, 0x00 },  
	{ 0xCA, 0xF1 },  
	{ 0xCB, 0x00 },  
	{ 0xCC, 0xD0 },  
	{ 0xCD, 0x02 },  
	{ 0xCE, 0xF1 },  
	{ 0xCF, 0x00 },  
	{ 0xD0, 0x01 },  
	{ 0xD1, 0x00 },  
	{ 0xD2, 0x00 },  
	{ 0xD4, 0x80 },  
	{ 0xD5, 0x40 },  
	{ 0xD6, 0x40 },  
	{ 0xD8, 0x00 },  
	{ 0xD9, 0x04 },  
	{ 0xDA, 0x00 },  
	{ 0xDC, 0x00 },  
	{ 0xDD, 0x02 },  
	{ 0xDE, 0x00 },  
	{ 0xE0, 0x00 },  
	{ 0xE1, 0x04 },  
	{ 0xE2, 0x00 },  
	{ 0xE3, 0x04 },  
	{ 0xE4, 0x00 },  
	{ 0xE8, 0x00 },  
	{ 0xE9, 0x00 },  
	{ 0xEA, 0x00 },  
	{ 0xEB, 0x00 },  
	{ 0xEC, 0x00 },  
	{ 0xED, 0x00 },  
	{ 0xEE, 0x00 },  
	{ 0x00, 0x00 },  
};			  
#endif
#define GFX_NUM_SAA7114_INIT_VALUES sizeof(tv_saa7114_init_vals_ntsc)/sizeof(DECODER_REG)
			  
int tv_saa7114_optimize_for_aliasing = 0;

TV_SAA7114_FIR_PREFILTER tv_saa7114_fir_values[] = {
	{ 0x01, 0x00, 0x00, 0x00, 0x00 }, { 0x02, 0x02, 0x5A, 0x01, 0x51 },
	{ 0x03, 0x04, 0xAB, 0x03, 0xA2 }, { 0x04, 0x07, 0xA3, 0x04, 0xAB },
	{ 0x05, 0x08, 0xAC, 0x07, 0xA3 }, { 0x06, 0x08, 0xFC, 0x07, 0xF3 }, 
	{ 0x07, 0x08, 0xFC, 0x07, 0xF3 }, { 0x08, 0x0F, 0xF4, 0x08, 0xFC }, 
	{ 0x09, 0x0F, 0xF4, 0x08, 0xFC }, { 0x0A, 0x10, 0xFD, 0x08, 0xFC },
	{ 0x0B, 0x10, 0xFD, 0x08, 0xFC }, { 0x0C, 0x10, 0xFD, 0x08, 0xFC },
	{ 0x0D, 0x10, 0xFD, 0x10, 0xFD }, { 0x0E, 0x10, 0xFD, 0x10, 0xFD },
	{ 0x0F, 0x1F, 0xF5, 0x10, 0xFD }, { 0x10, 0x20, 0xFE, 0x10, 0xFD },
	{ 0x11, 0x20, 0xFE, 0x10, 0xFD }, { 0x12, 0x20, 0xFE, 0x10, 0xFD },
	{ 0x13, 0x20, 0xFE, 0x20, 0xFE }, { 0x14, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x15, 0x20, 0xFE, 0x20, 0xFE }, { 0x16, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x17, 0x20, 0xFE, 0x20, 0xFE }, { 0x18, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x19, 0x20, 0xFE, 0x20, 0xFE }, { 0x1A, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x1B, 0x20, 0xFE, 0x20, 0xFE }, { 0x1C, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x1D, 0x20, 0xFE, 0x20, 0xFE }, { 0x1E, 0x20, 0xFE, 0x20, 0xFE },
	{ 0x1F, 0x20, 0xFE, 0x20, 0xFE }, { 0x20, 0x3F, 0xFF, 0x20, 0xFE },
	{ 0x21, 0x3F, 0xFF, 0x20, 0xFE }, { 0x22, 0x3F, 0xFF, 0x20, 0xFE },
	{ 0x23, 0x3F, 0xFF, 0x20, 0xFF }
};


void tv_saa7114_set_vbi_input_offset(WORD x, WORD y)
{
	tv_i2c_saa7114_write_byte_data(0x94, (BYTE)(x&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x95, (BYTE)(x >> 8) );
	tv_i2c_saa7114_write_byte_data(0x98, (BYTE)(y&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x99, (BYTE)(y >> 8) );
}

void tv_saa7114_set_vbi_input_size(WORD width, WORD height)
{
	tv_i2c_saa7114_write_byte_data(0x96, (BYTE)(width&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x97, (BYTE)(width >> 8) );
	tv_i2c_saa7114_write_byte_data(0x9a, (BYTE)(height&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x9b, (BYTE)(height >> 8) );
}
void tv_saa7114_set_vbi_output_size(WORD width, WORD height)
{
	tv_i2c_saa7114_write_byte_data(0x9c, (BYTE)(width&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x9d, (BYTE)(width >> 8) );
	tv_i2c_saa7114_write_byte_data(0x9e, (BYTE)(height&0x00FF) );
	tv_i2c_saa7114_write_byte_data(0x9f, (BYTE)(height >> 8) );
}

int tv_saa7114_detect_hw()
{
	BYTE		id = 0;
	id = tv_i2c_saa7114_read_byte_data( 0x00 );
	if( id != 0xFF )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

int tv_saa7114_software_reset()
{
	tv_i2c_saa7114_write_byte_data( 0x88, 0xF0 );
	tv_i2c_saa7114_write_byte_data( 0x88, 0x00 );
	tv_i2c_saa7114_write_byte_data( 0x88, 0xF0 );
	return 1;
}

void tv_saa7114_set_input_win( int std, WORD width, WORD height )
{
	if( std == TV_SAA7114_STANDARD_NTSC )
	{
		tv_saa7114_set_decoder_input_offset( 10, 18 );
	}
	else
	{
		tv_saa7114_set_decoder_input_offset( 10, 22 );
	}
	tv_saa7114_set_decoder_input_size( width, height );
}

void tv_saa7114_set_output_win( WORD srcWidth, WORD srcHeight, 
                         WORD dstWidth, WORD dstHeight )
{
	tv_saa7114_set_decoder_output_size( dstWidth, dstHeight );
	//tv_saa7114_set_decoder_scale( srcWidth, srcHeight, dstWidth, dstHeight,0 );
}

int tv_saa7114_set_decoder_vbi_upscale()
{
	/* Set horizontal output length to 1528 (720 * 2 * 14.318 / 13.5) */
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HORZ_OUTPUT_LO,   0xF8);
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HORZ_OUTPUT_HI,   0x05);

	/* Set horizontal luminance scaling increment to 484 (1024 * 13.5 / 28.636) */
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HSCALE_LUMA_LO,   0xE4);
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HSCALE_LUMA_HI,   0x01);

	/* Set horizontal chrominance scaling increment to 242 */
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HSCALE_CHROMA_LO, 0xF2);
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_TASK_A_HSCALE_CHROMA_HI, 0x00);

	return 1;
}

int tv_saa7114_decoder_detect_macrovision()
{
	BYTE macrovision = 0xff;
	macrovision = tv_i2c_saa7114_read_byte_data( TV_SAA7114_STATUS );
	return ((macrovision & 0x02) >> 1);
}

int tv_saa7114_decoder_detect_video()
{
	BYTE video = 0xff;
	video = tv_i2c_saa7114_read_byte_data( TV_SAA7114_STATUS );
	return !((video & 0x40) >> 6);
}

int tv_saa7114_set_decoder_analog_input( BYTE input )
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_ANALOG_INPUT_CTRL1, input );
	//tv_i2c_saa7114_write_byte_data( TV_SAA7114_ANALOG_INPUT_CTRL1, 0xC2);
	return 0;
}

int tv_saa7114_set_decoder_input_offset( WORD x, WORD y )
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_OFFSET_LO, (BYTE)(x & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_OFFSET_HI, (BYTE)(x >> 8) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_OFFSET_LO, (BYTE)(y & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_OFFSET_HI, (BYTE)(y >> 8) );
	tv_saa7114_software_reset();
	return 0;
}

int tv_saa7114_set_decoder_input_size( WORD width, WORD height )
{
	/* DIVIDE HEIGHT BY TWO FOR INTERLACING */
	height = (height + 1) >> 1;

	/* SET THE INPUT WINDOW SIZE */
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_INPUT_LO, (BYTE)(width & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_INPUT_HI, (BYTE)(width >> 8) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_INPUT_LO, (BYTE)(height & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_INPUT_HI, (BYTE)(height >> 8) );

	tv_saa7114_software_reset();
	return(0);
}

int tv_saa7114_set_decoder_output_size( WORD width, WORD height )
{
	/* DIVIDE HEIGHT BY TWO FOR INTERLACING */
	height = (height + 1) >> 1;

	/* SET THE OUTPUT WINDOW SIZE */
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_OUTPUT_LO, (BYTE)(width & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_OUTPUT_HI, (BYTE)(width >> 8) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_OUTPUT_LO, (BYTE)(height & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VERT_OUTPUT_HI, (BYTE)(height >> 8) );

	tv_saa7114_software_reset();
	return(0);
}
#if 0
int tv_saa7114_set_decoder_scale( WORD srcw, WORD srch, WORD dstw, WORD dsth )
{
	BYTE prescale = 0;
	int scale = 0;
    	int wUP_UP;
    	int wUP_LO; 
    	int wLO_UP ;
    	int wLO_LO;
	/* SET THE HORIZONTAL PRESCALE */
	/* Downscale from 1 to 1/63 source size. */
	if( dstw ) 
	{
		prescale = (BYTE) (srcw/dstw);
	}
	if( !prescale ) 
	{
		prescale = 1;
	}
	if( prescale > 63 ) 
	{
		return 1;
	}
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_PRESCALER, prescale );

	/* USE FIR PREFILTER FUNCTIONALITY (OPTIMISATION) */
	if( prescale < 36 ) 
	{
		if( tv_saa7114_optimize_for_aliasing ) 
		{
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_ACL,
						  tv_saa7114_fir_values[prescale-1].acl_low );
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_FIR_PREFILTER,
						  tv_saa7114_fir_values[prescale-1].prefilter_low );
		} 
		else 
		{
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_ACL,
						  tv_saa7114_fir_values[prescale-1].acl_high );
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_FIR_PREFILTER,
						  tv_saa7114_fir_values[prescale-1].prefilter_high );
		}
	} 
	else 
	{
		/* SAME SETTINGS FOR RATIO 1/35 DOWNTO 1/63 */
		if( tv_saa7114_optimize_for_aliasing ) 
		{
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_ACL,
						  tv_saa7114_fir_values[34].acl_low );
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_FIR_PREFILTER,
						  tv_saa7114_fir_values[34].prefilter_low );
		} 
		else 
		{
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_ACL,
						  tv_saa7114_fir_values[34].acl_high );
			tv_i2c_saa7114_write_byte_data( TV_SAA7114_HORZ_FIR_PREFILTER,
						  tv_saa7114_fir_values[34].prefilter_high );
		}
	}

	/* SET THE HORIZONTAL SCALING */
	if( !dstw ) 
	{
		return 1;
	}

	//<peter>BUGBUG: remove floating point code
	scale = (int)( ( ( 1024 * srcw ) / dstw ) / prescale );
	//scale = (int)(1024.*((1.*srcw)/(1.*dstw))*(1./prescale));
	//</peter>
	if( (scale > 8191) || (scale < 300) ) 
	{
		return 1;
	}
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HSCALE_LUMA_LO, (BYTE)(scale & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HSCALE_LUMA_HI, (BYTE)(scale >> 8) );
	scale >>= 1;
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HSCALE_CHROMA_LO, (BYTE)(scale & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HSCALE_CHROMA_HI, (BYTE)(scale >> 8) );

	/* SET THE VERTICAL SCALING (INTERPOLATION MODE) */
	if( !dsth ) 
	{
		return 1;
	}
	scale = (int)((1024 * (unsigned long)srch) / (unsigned long)dsth);

	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMA_LO, (BYTE)(scale & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMA_HI, (BYTE)(scale >> 8) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_LO, (BYTE)(scale & 0x00FF) );
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_HI, (BYTE)(scale >> 8) );
	scale = (int)((1000000 * (unsigned long)srch) / (unsigned long)dsth);
    	wUP_UP = min(0 + 16, 0xff);
    	wUP_LO = min(scale / 64 + 16,0xff);
    	wLO_UP = min (0, 0xff);
    	wLO_LO = min(scale / 64, 0xff);

	if (dsth >= (srch >> 1))
	{
		/* USE INTERPOLATION MODE FOR SCALE FACTOR ABOVE 0.5 */
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CONTROL, 0x00 );

		/* SET VERTICAL PHASE REGISTER FOR CORRECT SCALED INTERLACED OUTPUT (OPTIMISATION) */
		/* THE OPTIMISATION IS BASED ON OFIDC = 0 (REG 90h[6] = 0 ) */
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS0, TV_SAA7114_VSCALE_PHO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS1, TV_SAA7114_VSCALE_PHO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS2, (BYTE)(TV_SAA7114_VSCALE_PHO+scale/64-16) );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS3, (BYTE)(TV_SAA7114_VSCALE_PHO+scale/64-16) );

		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS0, TV_SAA7114_VSCALE_PHO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS1, TV_SAA7114_VSCALE_PHO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS2, (BYTE)(TV_SAA7114_VSCALE_PHO+scale/64-16) );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS3, (BYTE)(TV_SAA7114_VSCALE_PHO+scale/64-16) );

		/* RESTORE CONTRAST AND SATURATION FOR INTERPOLATION MODE */
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_FILTER_CONTRAST, (BYTE)0x40 );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_FILTER_SATURATION, (BYTE)0x40 );
	}
	else
	{
		/* USE ACCUMULATION MODE FOR DOWNSCALING BY MORE THAN 2x */
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CONTROL, 0x01 );

		/* SET VERTICAL PHASE OFFSETS OFF (OPTIMISATION) */
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS0, wUP_UP );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS1, wUP_LO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS2, wLO_UP );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_CHROMA_OFFS3, wLO_LO );

		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS0, wUP_UP );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS1, wUP_LO );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS2, wUP_UP );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_VSCALE_LUMINA_OFFS3, wLO_LO );

		/* ADJUST CONTRAST AND SATURATION FOR ACCUMULATION MODE */
		if( srch ) 
		{
			scale = (64 * dsth) / srch;
		}
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_FILTER_CONTRAST, (BYTE)scale );
		tv_i2c_saa7114_write_byte_data( TV_SAA7114_FILTER_SATURATION, (BYTE)scale );
	}

	tv_saa7114_software_reset();
	return 0;
}
#endif


int tv_saa7114_set_decoder_vbi_format( int start, int end, int format )
{
	int		i;
	BYTE		data;
	for( i = start; i <= end; i++ ) 
	{
		switch( format )
		{
			case VBI_FORMAT_VIDEO:	data = 0xFF; break;
			case VBI_FORMAT_RAW:	data = 0x77; break;
			case VBI_FORMAT_CC:		data = 0x55; break;
			case VBI_FORMAT_NABTS:	data = 0xCC; break;
			default:
				return 0;
		}
		tv_i2c_saa7114_write_byte_data( 0x3F + i, data );
	}
	return 1;
}

int tv_saa7114_set_decoder_vbi_enable( int enable )
{
	BYTE		data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_IPORT_CONTROL );
	if( enable )
	{
		data |= 0x80;
	}
	else
	{
		data &= ~0x80;
	}
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_IPORT_CONTROL, data );
	return 1;
}

int tv_saa7114_set_tv_standard( int standard )
{
	switch( standard ) 
	{
		case TV_SAA7114_STANDARD_NTSC:
			tv_i2c_saa7114_write_byte_data( 0x08, 0x88);
			tv_i2c_saa7114_write_byte_data( 0x09, 0x45);
			tv_i2c_saa7114_write_byte_data( 0x0E, 0x89);
			tv_i2c_saa7114_write_byte_data( 0x5A, 0x06);
			break;						
		case TV_SAA7114_STANDARD_PAL:			
			tv_i2c_saa7114_write_byte_data( 0x08, 0x98);
			tv_i2c_saa7114_write_byte_data( 0x09, 0x50);
			tv_i2c_saa7114_write_byte_data( 0x0E, 0x81);
			tv_i2c_saa7114_write_byte_data( 0x5A, 0x03);
			break;
		default:
			return 0;	
	}
	tv_saa7114_software_reset();
	return 1;
}

ULONG tv_saa7114_get_decoder_input_offset()
{
	ULONG value = 0;
	BYTE data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_OFFSET_LO );
	value = (ULONG) data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_OFFSET_HI );
	value |= ((ULONG) data) << 8;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_OFFSET_LO );
	value |= ((ULONG) data) << 16;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_OFFSET_HI );
	value |= ((ULONG) data) << 24;
	return value;
}

ULONG tv_saa7114_get_decoder_input_size()
{
	ULONG value = 0;
	BYTE data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_INPUT_LO );
	value = (ULONG) data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_INPUT_HI );
	value |= ((ULONG) data) << 8;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_INPUT_LO );
	value |= ((ULONG) data) << 17;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_INPUT_HI );
	value |= ((ULONG) data) << 25;
	return value;
}

ULONG tv_saa7114_get_decoder_output_size()
{
	ULONG value = 0;
	BYTE data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_OUTPUT_LO );
	value = (ULONG) data;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HORZ_OUTPUT_HI );
	value |= ((ULONG) data) << 8;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_OUTPUT_LO );
	value |= ((ULONG) data) << 17;
	data = tv_i2c_saa7114_read_byte_data( TV_SAA7114_VERT_OUTPUT_HI );
	value |= ((ULONG) data) << 25;
	return value;
}

int tv_saa7114_get_decoder_vbi_format( int line )
{
	BYTE format = 0;
	BYTE data = 0;
	data = tv_i2c_saa7114_read_byte_data( (BYTE)( 0x3F + (BYTE)line ) );
#if 0
	switch( data )
	{
		case 0xFF:	
			format = VBI_FORMAT_VIDEO;	
			break; 
		case 0x77:	
			format = VBI_FORMAT_RAW;	
			break; 
		case 0x55:	
			format = VBI_FORMAT_CC;		
			break;
		case 0xCC:	
			format = VBI_FORMAT_NABTS;	
			break;
	}
#endif
	return(format);
}

int tv_saa7114_set_decoder_brightness( BYTE brightness )
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_BRIGHTNESS, brightness ); 
	return 0;
}

BYTE tv_saa7114_get_decoder_brightness()
{
	BYTE brightness = 0;
	brightness = tv_i2c_saa7114_read_byte_data( TV_SAA7114_BRIGHTNESS );
	return brightness;
}

//void tv_saa7114_set_brightness( BYTE val )
//{
//	tv_i2c_saa7114_write_byte_data( TV_SAA7114_REG_BRIGHTNESS, val );		    
//}

int tv_saa7114_set_decoder_contrast(BYTE contrast)
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_CONTRAST, (BYTE)(contrast >> 1) ); 
	return 0;
}

BYTE tv_saa7114_get_decoder_contrast()
{
	BYTE contrast = 0;
	contrast = tv_i2c_saa7114_read_byte_data( TV_SAA7114_CONTRAST );
	contrast <<= 1;
	return contrast;
}

//void tv_saa7114_set_contrast( BYTE val )
//{
//	tv_i2c_saa7114_write_byte_data( TV_SAA7114_REG_CONTRAST, val );
//}

int tv_saa7114_set_decoder_hue( char hue )
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_HUE, (BYTE)hue ); 
	return 0;
}

char tv_saa7114_get_decoder_hue()
{
	BYTE hue = 0;
	hue = tv_i2c_saa7114_read_byte_data( TV_SAA7114_HUE );
	return((char)hue);
}


int tv_saa7114_set_decoder_saturation( BYTE saturation )
{
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_SATURATION, (BYTE)(saturation >> 1) ); 
	return 0;
}


BYTE tv_saa7114_get_decoder_saturation()
{
	BYTE saturation = 0;
	saturation = tv_i2c_saa7114_read_byte_data( TV_SAA7114_SATURATION );
	saturation <<= 1;
	return saturation;
}

/* ATTENTION */
/* This sets the Luminance filter characteristics of the decoder. 
   This should be called sharpness control. This will not have any
   affect when input is S-Video 
 */  

void tv_saa7114_set_luminance( BYTE val )
{
	unsigned char reg = 
		tv_i2c_saa7114_read_byte_data(TV_SAA7114_REG_LUMINANCE);
	/* change only bits [3:0] */
	val &= 0x0F; 
	reg &= 0xF0;
	val |= reg;
	tv_i2c_saa7114_write_byte_data( TV_SAA7114_REG_LUMINANCE, val );		    
}

void tv_saa7114_initialize_ntsc()
{
	int	i = 0;
	for (i = 0; i < GFX_NUM_SAA7114_INIT_VALUES; i++)
        {
		tv_i2c_saa7114_write_byte_data(
                        tv_saa7114_init_vals_ntsc[i].ucRegister,
                        tv_saa7114_init_vals_ntsc[i].ucValue);
        }
	
	tv_saa7114_set_tv_standard( TV_SAA7114_STANDARD_NTSC );
	//tv_saa7114_set_decoder_tv_standard( TV_SAA7114_STANDARD_NTSC );
	//gfx_set_decoder_analog_input(CVBS_IN);
	tv_saa7114_set_decoder_brightness(0x40 << 1);
	tv_saa7114_set_decoder_contrast(0x44 << 1);
	tv_saa7114_set_decoder_saturation(0x40 << 1);
	tv_saa7114_set_decoder_hue(0x0 << 1);
	tv_saa7114_software_reset();
}														   

void tv_saa7114_xport_enable( int enable )
{
	BYTE ucBValue;  
	BYTE ucAValue;
	if( enable )
	{
		tv_i2c_saa7114_write_byte_data( (BYTE)0x0e, (BYTE)0x09);    
		tv_i2c_saa7114_write_byte_data( (BYTE)0x0f, (BYTE)0x0f);    
		tv_i2c_saa7114_write_byte_data( (BYTE)0x1f, (BYTE)0xc8);    
		tv_i2c_saa7114_write_byte_data( (BYTE)0x61, (BYTE)0x23);    
		tv_i2c_saa7114_write_byte_data( (BYTE)0x62, (BYTE)0xbf);   
		tv_i2c_saa7114_write_byte_data( (BYTE)0x83, (BYTE)0x00);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x86, (BYTE)0x45);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x8f, (BYTE)0x08);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x91, (BYTE)0xd8);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x92, (BYTE)0x49);
		tv_i2c_saa7114_write_byte_data( (BYTE)0xc1, (BYTE)0x18);
		tv_i2c_saa7114_write_byte_data( (BYTE)0xc2, (BYTE)0x09);

//        tv_i2c_saa7114_write_byte_data( (BYTE)0x80, (BYTE)0x30);
      
		ucBValue = tv_i2c_saa7114_read_byte_data((BYTE)0x80);  
	    ucAValue = ucBValue; // save it
		ucBValue &= 0x01;
		if (ucBValue == 0) {// need to turn off
		ucAValue |= 0x01;
		tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)ucAValue);
		}
#if 0
		/* Bypasss Amplifier and anti-alias filter.Mode doesn't matter */ 
	//	tv_i2c_saa7114_write_byte_data((BYTE)0x02, (BYTE)0xc0);
		tv_i2c_saa7114_write_byte_data((BYTE)0x0f, (BYTE)0x24);
		tv_i2c_saa7114_write_byte_data((BYTE)0x83, (BYTE)0x00);
		tv_i2c_saa7114_write_byte_data((BYTE)0x86, (BYTE)0x45);
		tv_i2c_saa7114_write_byte_data((BYTE)0x91, (BYTE)0x18);
		tv_i2c_saa7114_write_byte_data((BYTE)0x92, (BYTE)0x09);
		tv_i2c_saa7114_write_byte_data((BYTE)0xc1, (BYTE)0x18);
		tv_i2c_saa7114_write_byte_data ((BYTE)0xc2, (BYTE)0x09);
		tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x11);
#endif
		
	}
	else
	{

#if 0
		tv_i2c_saa7114_write_byte_data((BYTE)0x0f, (BYTE)0x78);
		tv_i2c_saa7114_write_byte_data((BYTE)0x83, (BYTE)0x00);
		tv_i2c_saa7114_write_byte_data((BYTE)0x86, (BYTE)0xe5);
		tv_i2c_saa7114_write_byte_data((BYTE)0x91, (BYTE)0xc8);
		tv_i2c_saa7114_write_byte_data((BYTE)0x92, (BYTE)0x08);
		tv_i2c_saa7114_write_byte_data((BYTE)0xc1, (BYTE)0x08);
		tv_i2c_saa7114_write_byte_data((BYTE)0xc2, (BYTE)0x00);
		tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x10);
#endif
		tv_i2c_saa7114_write_byte_data( (BYTE)0x0e, (BYTE)0x09);    
		tv_i2c_saa7114_write_byte_data( (BYTE)0x0f, (BYTE)0x78);    
		tv_i2c_saa7114_write_byte_data((BYTE)0x1f, (BYTE)0xc8);	   
		tv_i2c_saa7114_write_byte_data( (BYTE)0x61, (BYTE)0x31);    
		tv_i2c_saa7114_write_byte_data((BYTE)0x62, (BYTE)0xbf);	    
		tv_i2c_saa7114_write_byte_data((BYTE)0x83, (BYTE)0x00);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x86, (BYTE)0xe5);
		tv_i2c_saa7114_write_byte_data( (BYTE)0x8f, (BYTE)0x0b);
		tv_i2c_saa7114_write_byte_data((BYTE)0x91, (BYTE)0xc8);
		tv_i2c_saa7114_write_byte_data((BYTE)0x92, (BYTE)0x08);
		tv_i2c_saa7114_write_byte_data((BYTE)0xc1, (BYTE)0x08);
		tv_i2c_saa7114_write_byte_data((BYTE)0xc2, (BYTE)0x00);
		ucBValue = tv_i2c_saa7114_read_byte_data((BYTE)0x80);  
	    ucAValue = ucBValue; // save it
		ucBValue &= 0x01;
		if (ucBValue == 1) {// need to turn off
		ucAValue &= ~0x01;
		tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)ucAValue);
		}

	}
}

#if 1
int tv_saa7114_set_input( int current_input, int new_input,
	int xoffset, int yoffset, int sourceW, int sourceH,
	 int width, int height, int standard )
{
     if( (new_input != 3)  && (current_input == 3) )  
     {
		// Going from digital X-port back to analog
		tv_i2c_saa7114_write_byte_data( 0x0E, 0x89 );
		tv_i2c_saa7114_write_byte_data( 0x0F, 0x21 ); //2b
 		tv_i2c_saa7114_write_byte_data( 0x86, 0xE5 ); //video data and text
		tv_i2c_saa7114_write_byte_data( 0x91, 0xC8 ); //select  analog
		tv_i2c_saa7114_write_byte_data( 0x92, 0x08 ); //was 40 for inverted field
		tv_i2c_saa7114_write_byte_data( 0xC1, 0xC8 );
		tv_i2c_saa7114_write_byte_data( 0xC2, 0x08 );
		tv_saa7114_initialize_ntsc();
     }
     switch( new_input )
     {
          case 0: //tuner
               tv_i2c_saa7114_write_byte_data( 0x88, 0x40 ); //scaler off, AD1 on
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#endif
               tv_i2c_saa7114_write_byte_data( 0x88, 0xF8 ); //scaler on, AD1 on
               tv_i2c_saa7114_write_byte_data( 0x09, 0x00 ); //use filter
  	    	tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x30);
               break;

          case 1: //video
               tv_i2c_saa7114_write_byte_data( 0x88, 0x80 ); //scaler off, AD2 on
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#endif
  	    	tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x30);
               tv_i2c_saa7114_write_byte_data( 0x88, 0xF8 ); //scaler on, AD2 on
               tv_i2c_saa7114_write_byte_data( 0x09, 0x00 ); //use filter
               break;

          case 2: //s-video
               tv_i2c_saa7114_write_byte_data( 0x88, 0xC0 ); //scaler off, AD1/2 on
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC9 );
               tv_i2c_saa7114_write_byte_data( 0x09, 0x80 ); //bypass filter and increase sharpness
  	    	tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x30);
               tv_i2c_saa7114_write_byte_data( 0x88, 0xF8 ); //scaler on, AD1/2 on
               break;

          case 3: //digital
               tv_i2c_saa7114_write_byte_data( 0x88, 0x00 ); //scaler off, AD1/2 off
	       
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#endif
	       
               tv_i2c_saa7114_write_byte_data( 0x0E, 0x09 );
               tv_i2c_saa7114_write_byte_data( 0x0F, 0x0F );
               tv_i2c_saa7114_write_byte_data( 0x83, 0x00 );
               tv_i2c_saa7114_write_byte_data( 0x91, 0x10 );
               tv_i2c_saa7114_write_byte_data( 0x92, 0x09 );
               tv_i2c_saa7114_write_byte_data( 0xC1, 0x10 );
               tv_i2c_saa7114_write_byte_data( 0xC2, 0x09 );
  	    	tv_i2c_saa7114_write_byte_data((BYTE)0x80, (BYTE)0x21);

               tv_i2c_saa7114_write_byte_data( 0x88, 0x30 ); //scaler on, AD1/2 off

               break;
     }
	if( ( width > 0 ) && ( height > 0 ) )
	{
		tv_saa7114_set_decoder_scale( xoffset, yoffset,
				sourceW,
				sourceH,
				width,
				height, standard );	
	} 
	tv_saa7114_software_reset(); 

     return 1;
}

#else

int tv_saa7114_set_input( int current_input, int new_input, int width, int height )
{
	//rigo

     if( new_input != 3 && current_input == 3)  
     {
		// Going from digital X-port back to analog
		tv_i2c_saa7114_write_byte_data( 0x0E, 0x89 );
		tv_i2c_saa7114_write_byte_data( 0x0F, 0x00 ); //2b
		tv_i2c_saa7114_write_byte_data( 0x86, 0xC5 ); //video data and text
		tv_i2c_saa7114_write_byte_data( 0x91, 0x08 ); //select  analog
		tv_i2c_saa7114_write_byte_data( 0x92, 0x00 ); //was 40 for inverted field
		tv_i2c_saa7114_write_byte_data( 0xC1, 0x08 );
		tv_i2c_saa7114_write_byte_data( 0xC2, 0x00 );
     }

     switch( new_input )
     {
          case 0: //tuner
               tv_i2c_saa7114_write_byte_data( 0x88, 0x40 ); //scaler off, AD1 on
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#endif
               tv_i2c_saa7114_write_byte_data( 0x09, 0x00 ); //use filter
               tv_i2c_saa7114_write_byte_data( 0x88, 0x70 ); //scaler on, AD1 on
			tv_saa7114_xport_enable( 0 );
			//tv_saa7114_xport_enable( 0, width, height );
               break;

          case 1: //video
               tv_i2c_saa7114_write_byte_data( 0x88, 0x80 ); //scaler off, AD2 on
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#endif
               tv_i2c_saa7114_write_byte_data( 0x09, 0x00 ); //use filter
               tv_i2c_saa7114_write_byte_data( 0x88, 0xB0 ); //scaler on, AD2 on
			tv_saa7114_xport_enable( 0 );
			//tv_saa7114_xport_enable( 0, width, height );
               break;

          case 2: //s-video
               tv_i2c_saa7114_write_byte_data( 0x88, 0xC0 ); //scaler off, AD1/2 on
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC9 );
               tv_i2c_saa7114_write_byte_data( 0x09, 0x84 ); //bypass filter and increase sharpness
               tv_i2c_saa7114_write_byte_data( 0x88, 0xF0 ); //scaler on, AD1/2 on
			tv_saa7114_xport_enable( 0 );
			//tv_saa7114_xport_enable( 0, width, height );
               break;

          case 3: //digital
               tv_i2c_saa7114_write_byte_data( 0x88, 0x00 ); //scaler off, AD1/2 off
#ifdef CYGNUS
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC2 );
#else
               tv_i2c_saa7114_write_byte_data( 0x02, 0xC0 );
#endif
               tv_i2c_saa7114_write_byte_data( 0x0E, 0x09 );
               tv_i2c_saa7114_write_byte_data( 0x0F, 0x0F );
               tv_i2c_saa7114_write_byte_data( 0x1F, 0xC8 );
               tv_i2c_saa7114_write_byte_data( 0x83, 0x00 );
               tv_i2c_saa7114_write_byte_data( 0x86, 0x45 );
               tv_i2c_saa7114_write_byte_data( 0x91, 0x18 );
               tv_i2c_saa7114_write_byte_data( 0x92, 0x09 );
               tv_i2c_saa7114_write_byte_data( 0xC1, 0x18 );
               tv_i2c_saa7114_write_byte_data( 0xC2, 0x09 );

               tv_i2c_saa7114_write_byte_data( 0x88, 0x30 ); //scaler on, AD1/2 off

			// This might be redundant.
			tv_saa7114_xport_enable( 1 );
			//tv_saa7114_xport_enable( 1, width, height );
               break;
     }
	tv_saa7114_software_reset(); 

     return 1;
}

#endif

static XSCALE XScale[MAX_XPSC] =	// .xACL, .xC2_1, .xDCGain
{
    { 0, 0, 0},     // Entry for XPSC 0
    { 0, 0, 0},     // Entry for XPSC 1
    { 1, 0, 1},     // Entry for XPSC 2
    { 3, 0, 2},     // Entry for XPSC 3
    { 4, 1, 3},     // Entry for XPSC 4
    { 7, 0, 3},     // Entry for XPSC 5
    { 8, 1, 4},     // Entry for XPSC 6
    { 8, 1, 4},     // Entry for XPSC 7
    {15, 0, 4},     // Entry for XPSC 8
    {15, 0, 4},     // Entry for XPSC 9
    {16, 1, 5},     // Entry for XPSC 10
    {16, 1, 5},     // Entry for XPSC 11
    {16, 1, 5},     // Entry for XPSC 12
    {16, 1, 5},     // Entry for XPSC 13
    {16, 1, 5},     // Entry for XPSC 14
    {31, 0, 5},     // Entry for XPSC 15
    {32, 1, 6},     // Entry for XPSC 16
    {32, 1, 6},     // Entry for XPSC 17
    {32, 1, 6},     // Entry for XPSC 18
    {32, 1, 6},     // Entry for XPSC 19
    {32, 1, 6},     // Entry for XPSC 20
    {32, 1, 6},     // Entry for XPSC 21
    {32, 1, 6},     // Entry for XPSC 22
    {32, 1, 6},     // Entry for XPSC 23
    {32, 1, 6},     // Entry for XPSC 24
    {32, 1, 6},     // Entry for XPSC 25
    {32, 1, 6},     // Entry for XPSC 26
    {32, 1, 6},     // Entry for XPSC 27
    {32, 1, 6},     // Entry for XPSC 28
    {32, 1, 6},     // Entry for XPSC 29
    {32, 1, 6},     // Entry for XPSC 30
    {32, 1, 6},     // Entry for XPSC 31
    {32, 1, 6},     // Entry for XPSC 32
    {32, 1, 6},     // Entry for XPSC 33
    {32, 1, 6},     // Entry for XPSC 34
    {32, 1, 6},     // Entry for XPSC 35
    {32, 1, 6},     // Entry for XPSC 36
    {32, 1, 6},     // Entry for XPSC 37
    {32, 1, 6},     // Entry for XPSC 38
    {32, 1, 6},     // Entry for XPSC 39
    {32, 1, 6},     // Entry for XPSC 40
    {32, 1, 6},     // Entry for XPSC 41
    {32, 1, 6},     // Entry for XPSC 42
    {32, 1, 6},     // Entry for XPSC 43
    {32, 1, 6},     // Entry for XPSC 44
    {32, 1, 6},     // Entry for XPSC 45
    {32, 1, 6},     // Entry for XPSC 46
    {32, 1, 6},     // Entry for XPSC 47
    {32, 1, 6},     // Entry for XPSC 48
    {32, 1, 6},     // Entry for XPSC 49
    {32, 1, 6},     // Entry for XPSC 50
    {32, 1, 6},     // Entry for XPSC 51
    {32, 1, 6},     // Entry for XPSC 52
    {32, 1, 6},     // Entry for XPSC 53
    {32, 1, 6},     // Entry for XPSC 54
    {32, 1, 6},     // Entry for XPSC 55
    {32, 1, 6},     // Entry for XPSC 56
    {32, 1, 6},     // Entry for XPSC 57
    {32, 1, 6},     // Entry for XPSC 58
    {32, 1, 6},     // Entry for XPSC 59
    {32, 1, 6},     // Entry for XPSC 60
    {32, 1, 6},     // Entry for XPSC 61
    {32, 1, 6},     // Entry for XPSC 62
    {32, 1, 6}      // Entry for XPSC 63
};


static ScaleBuf	Scale;


void tv_saa7114_set_scaling_parameters(int srcLeft, int srcTop,
                                       int srcWidth, int srcHeight,
				       int Xpixel, int Ypixel, int std )
{
	BYTE		ucValue;
	BYTE 	ucBValue;
	BYTE 	ucAValue;
	int		dwSAAHorzScalingRatio = 1;
	int		dwSAAVertScalingRatio = 1;
	InWindow  	rcInWindow;
	OutWindow 	prcOutWindow;
	ScaleParas 	pParameters;
	int	 	bInterlace = 1;
	int 		dwScalerModeOffset = 0;
	int 		nVideoStandard = std;
	int  	nYPO = 0;
	WORD 	wUP_UP = 0;
	WORD 	wUP_LO = 0;
	WORD 	wLO_UP = 0;
	WORD 	wLO_LO = 0;
	WORD 	wLuma00 = 0;
	WORD 	wLuma01 = 0;
	WORD 	wLuma10 = 0;
	WORD 	wLuma11 = 0;
	WORD	temp = 0;

	//tv_os_dbg_msg("tv_saa7114_set_scaling_parameters(): Xpixel = %d, Ypixel = %d\n", Xpixel, Ypixel);

	rcInWindow.nLeft = srcLeft;		//00
	rcInWindow.nWidth = srcWidth;
	rcInWindow.nTop = srcTop;
	rcInWindow.nHeight = srcHeight;

	// 	*********** Acquisition window position *************
	pParameters.wHorzInputStart = min(rcInWindow.nLeft, 0xfff);
	pParameters.wVertInputStart = min(rcInWindow.nTop, 0xfff);
	pParameters.wHorzInputLength = min(rcInWindow.nWidth, 0xfff);
	pParameters.wVertInputLength = min(rcInWindow.nHeight + 2, 0xfff);

	prcOutWindow.nWidth = Xpixel;
	prcOutWindow.nHeight = Ypixel;
	if( bInterlace )
	{
		prcOutWindow.nHeight = (prcOutWindow.nHeight+1)>>1;
	}

	pParameters.bTaskSelect = VIDEO_TASK;      // Task selected for video
	pParameters.bSkipFields = 0;
	pParameters.bRepeatTask = 0;

	pParameters.wHorzPhaseOffsetLuma = 0;
	pParameters.wHorzPhaseOffsetChroma = 0;

    	// scaler mode
//    	switch(m_nScalerMode)
//    	{
//            case VD_HIGH_BANDWIDTH:
//            	dwScalerModeOffset = -1000000;
//            	break;
//            case VD_LOW_BANDWIDTH:
//            	dwScalerModeOffset = 1000000;
//            	break;
//            case VD_DEFAULT:
//            default:
//            	dwScalerModeOffset = 0;
//            	break;
//    	}

	if (nVideoStandard == VD_NTSC)
	{
		pParameters.wVertInputLength =
			min( pParameters.wVertInputLength,
			262 - pParameters.wVertInputStart - 1 );
	}
	else if ( nVideoStandard == VD_PAL)
	{
		pParameters.wVertInputLength =
			min( pParameters.wVertInputLength,
			312 - pParameters.wVertInputStart - 1 );
	}
        if((DWORD) prcOutWindow.nWidth)
    	dwSAAHorzScalingRatio = (((DWORD) rcInWindow.nWidth) * 1000000) /
                        ((DWORD) prcOutWindow.nWidth);

        if((DWORD) prcOutWindow.nHeight)
    	dwSAAVertScalingRatio = (((DWORD) rcInWindow.nHeight) * 1000000) /
                             ((DWORD) prcOutWindow.nHeight);

    	pParameters.bHorzPrescale =
        max(1, min((BYTE) (((dwSAAHorzScalingRatio + dwScalerModeOffset) / 1000000)),
                   0x3f));

    	pParameters.bDCGain = XScale[pParameters.bHorzPrescale].xDCGain;
    	pParameters.bXC2_1 = XScale[pParameters.bHorzPrescale].xC2_1;

    	if (dwSAAHorzScalingRatio > (3900000 - dwScalerModeOffset))
    	{
        	pParameters.bFIRLuma = 3;
        	pParameters.bFIRChroma = 3;
    	}
    	else if (dwSAAHorzScalingRatio > (2000000 - dwScalerModeOffset))
    	{
        	pParameters.bFIRLuma = 2;
        	pParameters.bFIRChroma = 2;
    	}
    	else if (dwSAAHorzScalingRatio >= (1700000 - dwScalerModeOffset))
    	{
        	pParameters.bFIRLuma = 1;
        	pParameters.bFIRChroma = 1;
    	}
    	else
    	{
        	pParameters.bFIRLuma = 0;
        	pParameters.bFIRChroma = 0;
    	}

    	if (dwSAAVertScalingRatio < max(1000000,    // LPI mode
                                    (1250000 - dwScalerModeOffset)))
    	{
        	pParameters.bVertScalingMode = 0;
        	pParameters.bBrightness = 0x80;
        	pParameters.bContrast = 0x40;
        	pParameters.bSaturation = 0x40;
    	}
    	else                                        // ACM mode
    	{
        	pParameters.bVertScalingMode = 1;
        	pParameters.bBrightness = 0x80;
        	pParameters.bContrast = max(0,(64000000 / dwSAAVertScalingRatio));
        	pParameters.bSaturation = max(0,(64000000 / dwSAAVertScalingRatio));
        	//pParameters.bContrast = max(0, (BYTE)
                //                        (64000000 / dwSAAVertScalingRatio));
        	//pParameters.bSaturation = max(0, (BYTE)
                //                          (64000000 / dwSAAVertScalingRatio));
    	}

    	pParameters.wHorzScalingIncrementLuma =
        	min((WORD)(((dwSAAHorzScalingRatio / 1000) * 1024) /
                    (pParameters.bHorzPrescale * 1000)),
            	    0x1fff);
    	pParameters.wHorzScalingIncrementChroma =
        	min((pParameters.wHorzScalingIncrementLuma / 2), 0x1fff);
    	pParameters.wVertScalingIncrementLuma =
        	min((WORD)(((dwSAAVertScalingRatio / 1000) * 1024) / 1000), 0xffff);
    	pParameters.wVertScalingIncrementChroma =
        	min(pParameters.wVertScalingIncrementLuma, 0xffff);

    	nYPO = 0;

    	wUP_UP = min(nYPO + 16, 0xff);
    	wUP_LO = min(nYPO + pParameters.wVertScalingIncrementLuma / 64 + 16,
        	         0xff);
    	wLO_UP = min (nYPO, 0xff);
    	wLO_LO = min(nYPO + pParameters.wVertScalingIncrementLuma / 64, 0xff);

  	ucBValue = tv_i2c_saa7114_read_byte_data((BYTE)BTaskHandlingControl);
  	ucAValue = tv_i2c_saa7114_read_byte_data((BYTE)ATaskHandlingControl);

//    if (IsEnabledVBI() && (GetCurrentVBIMode() == VD_RAW_Y))
//    {                                       // Case raw VBI followed by video
//        if(! ((ucAValue & 0x40) && (ucBValue & 0x40))) //OFIDC = 0
//        {
//            if(pParameters.wVertScalingIncrementLuma >= 1024)
//            {
//                if(wLO_LO >= 16)
//                {
//                    wUP_UP = max((int)wUP_UP - 16, nYPO);
//                    wLO_LO = max((int)wLO_LO - 16, nYPO);
//                }
//            }
//            wUP_LO = wUP_UP;
//            wLO_UP = wLO_LO;
//        }
//    }
//    else
    {
        if(! (ucBValue & 0x40) ) //OFIDC = 0
        {
            if(pParameters.wVertScalingIncrementLuma >= 1024)
            {
                if(wLO_LO >= 16)
                {
                    wUP_UP = max((int)wUP_UP - 16, nYPO);
                    wLO_LO = max((int)wLO_LO - 16, nYPO);
                }
            }

            wUP_LO = wUP_UP;
            wLO_UP = wLO_LO;
        }
    }

    // scaler mode
//    switch(m_nScalerFID)
//    {
//       case VD_FID_INVERT:
//            wLuma00 = wLO_LO;
//            wLuma01 = wLO_UP;
//            wLuma10 = wUP_LO;
//            wLuma11 = wUP_UP;
//              break;
//
//        case VD_FID_DEFAULT:
//        default:
            wLuma00 = wUP_UP;
            wLuma01 = wUP_LO;
            wLuma10 = wLO_UP;
            wLuma11 = wLO_LO;
//            break;
//    }

    pParameters.wVertPhaseOffsetLuma00 = wLuma00;
    pParameters.wVertPhaseOffsetLuma01 = wLuma01;
    pParameters.wVertPhaseOffsetLuma10 = wLuma10;
    pParameters.wVertPhaseOffsetLuma11 = wLuma11;

    pParameters.wVertPhaseOffsetChroma00 = pParameters.wVertPhaseOffsetLuma00;
    pParameters.wVertPhaseOffsetChroma01 = pParameters.wVertPhaseOffsetLuma01;
    pParameters.wVertPhaseOffsetChroma10 = pParameters.wVertPhaseOffsetLuma10;
    pParameters.wVertPhaseOffsetChroma11 = pParameters.wVertPhaseOffsetLuma11;

//=============================================================================

	if( ( TaskSelect == VBI_TASK ) || ( TaskSelect == BOTH_TASKS ) )
	{
		temp = min(rcInWindow.nLeft, 0xfff);
		Scale.Buf_AHorzInputStart = LOBYTE(temp);
		Scale.Buf_AHorzInputStartMSB = HIBYTE(temp);
		//Scale.Buf_AHorzInputStart = LOBYTE(min(rcInWindow.nLeft, 0xfff));
		//Scale.Buf_AHorzInputStartMSB = HIBYTE(min(rcInWindow.nLeft, 0xfff));
		temp = min(rcInWindow.nWidth, 0xfff);
		Scale.Buf_AHorzInputLength = LOBYTE(temp);
		Scale.Buf_AHorzInputLengthMSB = HIBYTE(temp);
		//Scale.Buf_AHorzInputLength = LOBYTE(min(rcInWindow.nWidth, 0xfff));
		//Scale.Buf_AHorzInputLengthMSB = HIBYTE(min(rcInWindow.nWidth, 0xfff));

		temp = min(rcInWindow.nTop, 0xfff);
		Scale.Buf_AVertInputStart = LOBYTE(temp);
		Scale.Buf_AVertInputStartMSB = HIBYTE(temp);
		//Scale.Buf_AVertInputStart = LOBYTE(min(rcInWindow.nTop, 0xfff));
		//Scale.Buf_AVertInputStartMSB = HIBYTE(min(rcInWindow.nTop, 0xfff));
		temp = min(rcInWindow.nHeight, 0xfff);
		Scale.Buf_AVertInputLength = LOBYTE(temp);
		Scale.Buf_AVertInputLengthMSB = HIBYTE(temp);
		//Scale.Buf_AVertInputLength = LOBYTE(min(rcInWindow.nHeight, 0xfff));
		//Scale.Buf_AVertInputLengthMSB = HIBYTE(min(rcInWindow.nHeight, 0xfff));

		temp = min(prcOutWindow.nWidth, 0xfff);
		Scale.Buf_AHorzOutputLength = LOBYTE(temp);
		Scale.Buf_AHorzOutputLengthMSB = HIBYTE(temp);
		//Scale.Buf_AHorzOutputLength = LOBYTE(min(prcOutWindow.nWidth, 0xfff));
		//Scale.Buf_AHorzOutputLengthMSB = HIBYTE(min(prcOutWindow.nWidth, 0xfff));
		temp = min(prcOutWindow.nHeight, 0xfff);
		Scale.Buf_AVertOutputLength = LOBYTE(temp);
		Scale.Buf_AVertOutputLengthMSB = HIBYTE(temp);
		//Scale.Buf_AVertOutputLength = LOBYTE(min(prcOutWindow.nHeight, 0xfff));
		//Scale.Buf_AVertOutputLengthMSB = HIBYTE(min(prcOutWindow.nHeight, 0xfff));

		Scale.Buf_AHorzPrescale = pParameters.bHorzPrescale & 0x3f;
		Scale.Buf_AAcl = XScale[pParameters.bHorzPrescale].xACL & 0x3f;
		Scale.Buf_ADCGain = ((pParameters.bDCGain & 0x07)  |
							((pParameters.bXC2_1 & 0x01) << 3) |
							((pParameters.bFIRLuma & 0x03) << 4)|
							((pParameters.bFIRChroma  & 0x03) << 6));

		Scale.Buf_ALuminanceBrightness = pParameters.bBrightness;
		Scale.Buf_ALuminanceContrast = pParameters.bContrast;
		Scale.Buf_AChromaSaturation = pParameters.bSaturation;

		Scale.Buf_AHorzScalingLuma = LOBYTE(pParameters.wHorzScalingIncrementLuma);
		Scale.Buf_AHorzScalingLumaMSB = HIBYTE(pParameters.wHorzScalingIncrementLuma);
		Scale.Buf_AHorzPhaseLuma = pParameters.wHorzPhaseOffsetLuma;

		Scale.Buf_AHorzScalingChroma = LOBYTE(pParameters.wHorzScalingIncrementChroma);
		Scale.Buf_AHorzScalingChromaMSB = HIBYTE(pParameters.wHorzScalingIncrementChroma) & 0x1f;
		Scale.Buf_AHorzPhaseChroma = pParameters.wHorzPhaseOffsetChroma;

		Scale.Buf_AVertScalingLuma = LOBYTE(pParameters.wVertScalingIncrementLuma);
		Scale.Buf_AVertScalingLumaMSB = HIBYTE(pParameters.wVertScalingIncrementLuma);

		Scale.Buf_AVertScalingChroma = LOBYTE(pParameters.wVertScalingIncrementChroma);
		Scale.Buf_AVertScalingChromaMSB = HIBYTE(pParameters.wVertScalingIncrementChroma);

		Scale.Buf_AVertPhaseLuma00 = LOBYTE(pParameters.wVertPhaseOffsetLuma00);
		Scale.Buf_AVertPhaseLuma01 = LOBYTE(pParameters.wVertPhaseOffsetLuma01);
		Scale.Buf_AVertPhaseLuma10 = LOBYTE(pParameters.wVertPhaseOffsetLuma10);
		Scale.Buf_AVertPhaseLuma11 = LOBYTE(pParameters.wVertPhaseOffsetLuma11);

		Scale.Buf_AVertPhaseChroma00 = LOBYTE(pParameters.wVertPhaseOffsetChroma00);
		Scale.Buf_AVertPhaseChroma01 = LOBYTE(pParameters.wVertPhaseOffsetChroma01);
		Scale.Buf_AVertPhaseChroma10 = LOBYTE(pParameters.wVertPhaseOffsetChroma10);
		Scale.Buf_AVertPhaseChroma11 = LOBYTE(pParameters.wVertPhaseOffsetChroma11);

		ucValue = tv_i2c_saa7114_read_byte_data((BYTE)AVertScalingMode);  //Get A Vertical Scaling mode reg.
		Scale.Buf_AVertScalingMode = (ucValue & ~0x01) | (pParameters.bVertScalingMode & 0x01);

		ucValue = tv_i2c_saa7114_read_byte_data((BYTE)ATaskHandlingControl);  //Get A Task handling reg.
		Scale.Buf_ATaskHandlingControl = ((ucValue & ~0x3c) | ((pParameters.bRepeatTask  & 0x01) << 2) |
					((pParameters.bSkipFields & 0x07) << 3));
	}

	if( ( TaskSelect == VIDEO_TASK ) || ( TaskSelect == BOTH_TASKS ) )
	{
		temp = min(rcInWindow.nLeft, 0xfff);
		Scale.Buf_BHorzInputStart = LOBYTE(temp);
		Scale.Buf_BHorzInputStartMSB = HIBYTE(temp) & 0x0f;
		temp = min(rcInWindow.nWidth, 0xfff);
		Scale.Buf_BHorzInputLength = LOBYTE(temp);
		Scale.Buf_BHorzInputLengthMSB = HIBYTE(temp) & 0x0f;

		temp = min(rcInWindow.nTop, 0xfff);
		Scale.Buf_BVertInputStart = LOBYTE(temp);
		Scale.Buf_BVertInputStartMSB = HIBYTE(temp) & 0x0f;
		temp = min(rcInWindow.nHeight+2, 0xfff);
		Scale.Buf_BVertInputLength = LOBYTE(temp);
		Scale.Buf_BVertInputLengthMSB = HIBYTE(temp) & 0x0f;

		temp = min(prcOutWindow.nWidth, 0xfff);
		Scale.Buf_BHorzOutputLength = LOBYTE(temp);
		Scale.Buf_BHorzOutputLengthMSB = HIBYTE(temp) & 0x0f;
		temp = min(prcOutWindow.nHeight, 0xfff);
		Scale.Buf_BVertOutputLength = LOBYTE(temp);
		Scale.Buf_BVertOutputLengthMSB = HIBYTE(temp) & 0x0f;

		Scale.Buf_BHorzPrescale = pParameters.bHorzPrescale & 0x3f;
		Scale.Buf_BAcl = XScale[pParameters.bHorzPrescale].xACL & 0x3f;
		Scale.Buf_BDCGain = ((pParameters.bDCGain & 0x07)  |
				((pParameters.bXC2_1 & 0x01) << 3) |
				((pParameters.bFIRLuma & 0x03) << 4)|
				((pParameters.bFIRChroma  & 0x03) << 6));
		
		Scale.Buf_BLuminanceBrightness = pParameters.bBrightness;
		Scale.Buf_BLuminanceContrast = pParameters.bContrast;
		Scale.Buf_BChromaSaturation = pParameters.bSaturation;

		Scale.Buf_BHorzScalingLuma = 
			LOBYTE(pParameters.wHorzScalingIncrementLuma);
		Scale.Buf_BHorzScalingLumaMSB = 
			HIBYTE(pParameters.wHorzScalingIncrementLuma) & 0x1f;
		Scale.Buf_BHorzPhaseLuma = 
			pParameters.wHorzPhaseOffsetLuma;

		Scale.Buf_BHorzScalingChroma =
			 LOBYTE(pParameters.wHorzScalingIncrementChroma);
		Scale.Buf_BHorzScalingChromaMSB =
			 HIBYTE(pParameters.wHorzScalingIncrementChroma) & 0x1f;
		Scale.Buf_BHorzPhaseChroma = pParameters.wHorzPhaseOffsetChroma;

		Scale.Buf_BVertScalingLuma = LOBYTE(pParameters.wVertScalingIncrementLuma);
		Scale.Buf_BVertScalingLumaMSB = HIBYTE(pParameters.wVertScalingIncrementLuma);

		Scale.Buf_BVertScalingChroma = LOBYTE(pParameters.wVertScalingIncrementChroma);
		Scale.Buf_BVertScalingChromaMSB = HIBYTE(pParameters.wVertScalingIncrementChroma);

		Scale.Buf_BVertPhaseLuma00 = LOBYTE(pParameters.wVertPhaseOffsetLuma00);
		Scale.Buf_BVertPhaseLuma01 = LOBYTE(pParameters.wVertPhaseOffsetLuma01);
		Scale.Buf_BVertPhaseLuma10 = LOBYTE(pParameters.wVertPhaseOffsetLuma10);
		Scale.Buf_BVertPhaseLuma11 = LOBYTE(pParameters.wVertPhaseOffsetLuma11);

		Scale.Buf_BVertPhaseChroma00 = LOBYTE(pParameters.wVertPhaseOffsetChroma00);
		Scale.Buf_BVertPhaseChroma01 = LOBYTE(pParameters.wVertPhaseOffsetChroma01);
		Scale.Buf_BVertPhaseChroma10 = LOBYTE(pParameters.wVertPhaseOffsetChroma10);
		Scale.Buf_BVertPhaseChroma11 = LOBYTE(pParameters.wVertPhaseOffsetChroma11);

		ucValue = tv_i2c_saa7114_read_byte_data((BYTE)BVertScalingMode);  //Get B Vertical Scaling mode reg.
		Scale.Buf_BVertScalingMode = (ucValue & ~0x01) | (pParameters.bVertScalingMode & 0x01);

		ucValue = tv_i2c_saa7114_read_byte_data((BYTE)BTaskHandlingControl);  //Get B Task handling reg.
		Scale.Buf_BTaskHandlingControl = ((ucValue & ~0x3c) | ((pParameters.bRepeatTask  & 0x01) << 2) |
							((pParameters.bSkipFields & 0x07) << 3));
	}
}


void tv_saa7114_set_decoder_scale( int srcLeft, int srcTop,
 			     int srcWidth,int srcHeight,
			     int X, int Y, int standard )
{
	BYTE	ucTask;
	BYTE	ucSWRST;
	int	Xpixel = X;
	int	Ypixel = Y;

	//Calculate scaling parameters and fill in the register array
	tv_saa7114_set_scaling_parameters(srcLeft, srcTop, srcWidth, srcHeight,
	 				Xpixel, Ypixel, standard);

	ucSWRST = tv_i2c_saa7114_read_byte_data((BYTE)PowerSaveControl);
	tv_i2c_saa7114_write_byte_data((BYTE)PowerSaveControl, (BYTE)( (ucSWRST & ~0x20) | 0x00 ) );

	ucTask = tv_i2c_saa7114_read_byte_data((BYTE)GlobalControl1);
	tv_i2c_saa7114_write_byte_data(GlobalControl1, (BYTE)( ucTask & ~VIDEO_TASK ) );

	tv_i2c_saa7114_write_byte_data( BHorzInputStart, Scale.Buf_BHorzInputStart);
	tv_i2c_saa7114_write_byte_data( BHorzInputStartMSB, Scale.Buf_BHorzInputStartMSB);
	tv_i2c_saa7114_write_byte_data( BHorzInputLength, Scale.Buf_BHorzInputLength);
	tv_i2c_saa7114_write_byte_data( BHorzInputLengthMSB, Scale.Buf_BHorzInputLengthMSB);
	tv_i2c_saa7114_write_byte_data( BVertInputStart, Scale.Buf_BVertInputStart);
	tv_i2c_saa7114_write_byte_data( BVertInputStartMSB, Scale.Buf_BVertInputStartMSB);
	tv_i2c_saa7114_write_byte_data( BVertInputLength, Scale.Buf_BVertInputLength);
	tv_i2c_saa7114_write_byte_data( BVertInputLengthMSB, Scale.Buf_BVertInputLengthMSB);
	tv_i2c_saa7114_write_byte_data( BHorzOutputLength, Scale.Buf_BHorzOutputLength);
	tv_i2c_saa7114_write_byte_data( BHorzOutputLengthMSB, Scale.Buf_BHorzOutputLengthMSB);
	tv_i2c_saa7114_write_byte_data( BVertOutputLength, Scale.Buf_BVertOutputLength);
	tv_i2c_saa7114_write_byte_data( BVertOutputLengthMSB, Scale.Buf_BVertOutputLengthMSB);
	tv_i2c_saa7114_write_byte_data( BHorzPrescale, Scale.Buf_BHorzPrescale);
	tv_i2c_saa7114_write_byte_data( BAcl, Scale.Buf_BAcl);
	tv_i2c_saa7114_write_byte_data( BDCGain, Scale.Buf_BDCGain);
	tv_i2c_saa7114_write_byte_data( BLuminanceBrightness, Scale.Buf_BLuminanceBrightness);
	tv_i2c_saa7114_write_byte_data( BLuminanceContrast, Scale.Buf_BLuminanceContrast);
	tv_i2c_saa7114_write_byte_data( BChromaSaturation, Scale.Buf_BChromaSaturation);
	tv_i2c_saa7114_write_byte_data( BHorzScalingLuma, Scale.Buf_BHorzScalingLuma);
	tv_i2c_saa7114_write_byte_data( BHorzScalingLumaMSB, Scale.Buf_BHorzScalingLumaMSB);
	tv_i2c_saa7114_write_byte_data( BHorzPhaseLuma, Scale.Buf_BHorzPhaseLuma);
	tv_i2c_saa7114_write_byte_data( BHorzScalingChroma, Scale.Buf_BHorzScalingChroma);
	tv_i2c_saa7114_write_byte_data( BHorzScalingChromaMSB, Scale.Buf_BHorzScalingChromaMSB);
	tv_i2c_saa7114_write_byte_data( BHorzPhaseChroma, Scale.Buf_BHorzPhaseChroma);
	tv_i2c_saa7114_write_byte_data( BVertScalingLuma, Scale.Buf_BVertScalingLuma);
	tv_i2c_saa7114_write_byte_data( BVertScalingLumaMSB, Scale.Buf_BVertScalingLumaMSB);
	tv_i2c_saa7114_write_byte_data( BVertScalingChroma, Scale.Buf_BVertScalingChroma);
	tv_i2c_saa7114_write_byte_data( BVertScalingChromaMSB, Scale.Buf_BVertScalingChromaMSB);
	tv_i2c_saa7114_write_byte_data( BVertPhaseLuma00, Scale.Buf_BVertPhaseLuma00);
	tv_i2c_saa7114_write_byte_data( BVertPhaseLuma01, Scale.Buf_BVertPhaseLuma01);
	tv_i2c_saa7114_write_byte_data( BVertPhaseLuma10, Scale.Buf_BVertPhaseLuma10);
	tv_i2c_saa7114_write_byte_data( BVertPhaseLuma11, Scale.Buf_BVertPhaseLuma11);
	tv_i2c_saa7114_write_byte_data( BVertPhaseChroma00, Scale.Buf_BVertPhaseChroma00);
	tv_i2c_saa7114_write_byte_data( BVertPhaseChroma01, Scale.Buf_BVertPhaseChroma01);
	tv_i2c_saa7114_write_byte_data( BVertPhaseChroma10, Scale.Buf_BVertPhaseChroma10);
	tv_i2c_saa7114_write_byte_data( BVertPhaseChroma11, Scale.Buf_BVertPhaseChroma11);
	tv_i2c_saa7114_write_byte_data( BVertScalingMode, Scale.Buf_BVertScalingMode);
	tv_i2c_saa7114_write_byte_data( BTaskHandlingControl, Scale.Buf_BTaskHandlingControl);

	tv_i2c_saa7114_write_byte_data(GlobalControl1, (BYTE)( ( ucTask & ~BOTH_TASKS ) | VIDEO_TASK ) );
	tv_i2c_saa7114_write_byte_data((BYTE)PowerSaveControl, (BYTE)( (ucSWRST & ~0x20 ) | 0x20 ) );
}

															  

