/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to program TV tuners (various).
 *
 * Decoder functions:
 * 
 *    tv_tuner_channel_freq
 *    tv_tuner_get_signal_strength
 *    tv_tuner_get_freq
 *    tv_tuner_change_channel
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#include "tv_tuner.h"


static const ULONG cable_freq_list[] = 
{
  	0x0,    0x480,  0x360,  0x3c0,  0x420, 		
  	0x4e0,  0x540,  0xae0,  0xb40,  0xba0,		
  	0xc00,  0xc60,  0xcc0,  0xd20,  0x780, 		
  	0x7e0,  0x840,  0x8a0,  0x900,  0x960,		
  	0x9c0,  0xa20,  0xa80,  0xd80,  0xde0, 		
  	0xe40,  0xea0,  0xf00,  0xf60,  0xfc0,		
  	0x1020, 0x1080, 0x10e0, 0x1140, 0x11a0, 	
  	0x1200, 0x1260, 0x12c0, 0x1320, 0x1380,		
  	0x13e0, 0x1440, 0x14a0, 0x1500, 0x1560, 	
  	0x15c0, 0x1620, 0x1680, 0x16e0, 0x1740,		
  	0x17a0, 0x1800, 0x1860, 0x18c0, 0x1920,		
  	0x1980, 0x19e0, 0x1a40, 0x1aa0, 0x1b00,		
  	0x1b60, 0x1bc0, 0x1c20, 0x1c80, 0x1ce0, 	
  	0x1d40, 0x1da0, 0x1e00, 0x1e60, 0x1ec0,		
  	0x1f20, 0x1f80, 0x1fe0, 0x2040, 0x20a0, 	
  	0x2100, 0x2160, 0x21c0, 0x2220, 0x2280,		
  	0x22e0, 0x2340, 0x23a0, 0x2400, 0x2460, 	
  	0x24c0, 0x2520, 0x2580, 0x25e0, 0x2640,		
  	0x26a0, 0x2700, 0x2760, 0x27c0, 0x2820, 	
  	0x5a0,  0x600,  0x660,  0x6c0,  0x720,		
  	0x2880, 0x28e0, 0x2940, 0x29a0, 0x2a00,		
  	0x2a60, 0x2ac0, 0x2b20, 0x2b80, 0x2be0,		
  	0x2c40, 0x2ca0, 0x2d00, 0x2d60, 0x2dc0,		
  	0x2e20, 0x2e80, 0x2ee0, 0x2f40, 0x2fa0,		
  	0x3000, 0x3060, 0x30c0, 0x3120, 0x3180,		
	0x31e0, 0x3240, 0x32a0, 0x3300, 0x3360,		
	0x33c0, 0x3420, 0x3480, 0x34e0, 0x3540,		
	0x35a0, 0x3600, 0x3660, 0x36c0, 0x3720,		
	0x3780, 0x37e0, 0x3840, 0x38a0, 0x3900,		
	0x3960, 0x39c0, 0x3a20, 0x3a80, 0x3ae0,		
	0x3b40, 0x3ba0, 0x3c00, 0x3c60, 0x3cc0,		
	0x3d20, 0x3d80, 0x3de0, 0x3e40			
};


static const ULONG antenna_freq_list[] =
{
	0x0,    0x494,  0x374,  0x3D4,  0x434, 		
	0x4d4,  0x534,  0xaf4,  0xb54,  0xbb4,		
	0xc14,  0xc74,  0xcd4,  0xd34,  0x1d74,		
	0x1dd4, 0x1e34, 0x1e94, 0x1ef4, 0x1f54,		
	0x1fb4, 0x2014, 0x2074, 0x20d4, 0x2134,		
	0x2194, 0x21f4, 0x2254, 0x22b4, 0x2314,		
	0x2374, 0x23d4, 0x2434, 0x2494, 0x24f4, 	
	0x2554, 0x25b4, 0x2614, 0x2674, 0x26d4,		
	0x2734, 0x2794, 0x27f4, 0x2854, 0x28b4, 	
	0x2914, 0x2974, 0x29d4, 0x2a34, 0x2a94,		
	0x2af4, 0x2b54, 0x2bb4, 0x2c14, 0x2c74,		
	0x2cd4, 0x2d34, 0x2d94, 0x2df4, 0x2e54,		
	0x2eb4, 0x2f14, 0x2f74, 0x2fd4, 0x3034, 	
	0x3094, 0x30f4, 0x3154, 0x31b4, 0x3214,		
};


static tuner_table_entry_t	tuner_table[] = 
{
	{ 
	  IFFREQ_TEMIC_PAL,			    
	  VHF_LOW_TEMIC_PAL,
	  VHF_HIGH_TEMIC_PAL,
	  UHF_TEMIC_PAL,
	  TEMIC_PAL_LOWMID_THRESH,
	  TEMIC_PAL_MIDHIGH_THRESH
	},
	{ 
	  IFFREQ_PHILIPS_PAL_I,		    	
	  VHF_LOW_PHILIPS_PAL_I,
	  VHF_HIGH_PHILIPS_PAL_I,
	  UHF_PHILIPS_PAL_I,
	  PHILIPS_PAL_I_LOWMID_THRESH,
	  PHILIPS_PAL_I_MIDHIGH_THRESH
	},
	{ 
	  IFFREQ_PHILIPS_NTSC,		    
	  VHF_LOW_PHILIPS_NTSC,
	  VHF_HIGH_PHILIPS_NTSC,
	  UHF_PHILIPS_NTSC,
	  PHILIPS_NTSC_LOWMID_THRESH,
	  PHILIPS_NTSC_MIDHIGH_THRESH		    
	},
	{ 
	  IFFREQ_PHILIPS_SECAM,		    
	  VHF_LOW_PHILIPS_SECAM,
	  VHF_HIGH_PHILIPS_SECAM,
	  UHF_PHILIPS_SECAM,
	  PHILIPS_SECAM_LOWMID_THRESH,
	  PHILIPS_SECAM_MIDHIGH_THRESH
	},
	{ 
	  IFFREQ_PHILIPS_PAL,		    
	  VHF_LOW_PHILIPS_PAL,
	  VHF_HIGH_PHILIPS_PAL,
	  UHF_PHILIPS_PAL,
	  PHILIPS_PAL_LOWMID_THRESH,
	  PHILIPS_PAL_MIDHIGH_THRESH
	},
	{ 
	  IFFREQ_TEMIC_NTSC,		
	  VHF_LOW_TEMIC_NTSC,
	  VHF_HIGH_TEMIC_NTSC,
	  UHF_TEMIC_NTSC,
	  TEMIC_NTSC_LOWMID_THRESH,
	  TEMIC_NTSC_MIDHIGH_THRESH		
	},
	{ 
	  IFFREQ_TEMIC_PAL_I,		
	  VHF_LOW_TEMIC_PAL_I,
	  VHF_HIGH_TEMIC_PAL_I,
	  UHF_TEMIC_PAL_I,
	  TEMIC_PAL_I_LOWMID_THRESH,
	  TEMIC_PAL_I_MIDHIGH_THRESH		
	},
	{ 
	  IFFREQ_TEMIC_4036_FY5_NTSC,	
	  VHF_LOW_TEMIC_4036_FY5_NTSC,
	  VHF_HIGH_TEMIC_4036_FY5_NTSC,
	  UHF_TEMIC_4036_FY5_NTSC,
	  TEMIC_4036_FY5_NTSC_LOWMID_THRESH,
	  TEMIC_4036_FY5_NTSC_MIDHIGH_THRESH
	},
	{ 
	  IFFREQ_ALPS_TSBH1,		
	  VHF_LOW_ALPS_TSBH1,
	  VHF_HIGH_ALPS_TSBH1,
	  UHF_ALPS_TSBH1,
	  ALPS_TSBH1_LOWMID_THRESH,
	  ALPS_TSBH1_MIDHIGH_THRESH		
	},
	{ 
	  IFFREQ_ALPS_TSBE1,		
	  VHF_LOW_ALPS_TSBE1,
	  VHF_HIGH_ALPS_TSBE1,
	  UHF_ALPS_TSBE1,
	  ALPS_TSBE1_LOWMID_THRESH,
	  ALPS_TSBE1_MIDHIGH_THRESH		
	},
	{ 
	  IFFREQ_ALPS_TMDZ2,		
	  VHF_LOW_ALPS_TMDZ2,
	  VHF_HIGH_ALPS_TMDZ2,
	  UHF_ALPS_TMDZ2,
	  ALPS_TMDZ2_LOWMID_THRESH,
	  ALPS_TMDZ2_MIDHIGH_THRESH		
	}
};


static ULONG current_freq = 0;					 


ULONG tv_tuner_channel_freq( int cable, int channel )
{
	ULONG freq;
	ULONG mhz;

	if( cable == 1 )
	{
		/*	cable	*/
		freq = cable_freq_list[ channel ];

		if( ( channel != 5 ) && ( channel != 6 ) )
		{
			/*	BUGBUG: Why do we do this?	*/
			freq = freq + 0x14;
		}
	}
	else
	{
		/*	antenna	*/
		freq = antenna_freq_list[ channel ];
	}

	mhz =  ( freq * 62500 ) / 1000000;

	return mhz;
}


/*
	Sense the frequency.
	Return:
		0 = carrier not sensed
		1 = device busy
		2 = carrier sensed, must lower freq
		3 = carrier sensed, must lower freq
		4 = carrier sensed, must raise freq
		5 = locked on correct freq
*/
BYTE tv_tuner_get_signal_strength( BYTE tuner_i2c_read_addr )
{
	BYTE		status;
	status = tv_i2c_tuner_read_byte( tuner_i2c_read_addr );
	if( status & 0x40 ) 
	{
		switch( status & 0x07 ) 
		{
			case 0:
				/* carrier sensed, need to lower frequency */
				return 2;
			case 1:     
				/* carrier sensed, need to lower frequency */
				return 3;
			case 2:     
				/* carrier sensed, we are on correct frequency */
				return 5;
			case 3:     
				/* carrier sensed, need to raise frequency */
				return 4;
			case 4:     
			default:
				/* carrier not sensed */
				return 0;
		}
	}
	else
	{
		return 1;
	}
}


ULONG tv_tuner_get_freq()
{
	return current_freq;
}

int tv_tuner_change_channel( BYTE tuner_i2c_write_addr, int tuner_type, int cable, int channel )	
{
	WORD		control_code;
	WORD		divider;
	WORD		iffreq;	
	WORD		rffreq;	
	ULONG	freq = 0;								
	freq = tv_tuner_channel_freq( cable, channel );

//<peter>
	// This is so that the PAL VCR video will tune correctly.
	if( tuner_type == TUNER_TYPE_ALPS_TMDZ2 ) 
	{
		switch( channel )
		{
			case 36:
				freq = 606;
				break;
			case 38:
				freq = 614;
				break;
		}
	}
//</peter>
	
	current_freq = freq;

	iffreq = tuner_table[ tuner_type ].iffreq;
	
	if( freq < tuner_table[ tuner_type ].low_mid_thresh )
	{
		control_code = tuner_table[ tuner_type ].vhf_low_control;
	}
	else if( freq < tuner_table[ tuner_type ].mid_high_thresh )
	{
		control_code = tuner_table[ tuner_type ].vhf_high_control;
	}
	else
	{
		control_code = tuner_table[ tuner_type ].uhf_control;
	}

	rffreq = (WORD)freq;

	divider = (WORD)( ( rffreq * 16 ) + iffreq );

	tv_i2c_tuner_write( tuner_i2c_write_addr, divider, control_code );

	return 1;
}

int tv_tuner_change_frequency( BYTE tuner_i2c_write_addr, int tuner_type, int freq )	
{
	WORD		control_code;
	WORD		divider;
	WORD		iffreq;	
	WORD		rffreq;	
	
	current_freq = freq;

	iffreq = tuner_table[ tuner_type ].iffreq;
	if( freq < tuner_table[ tuner_type ].low_mid_thresh )
	{
		control_code = tuner_table[ tuner_type ].vhf_low_control;
	}
	else if( freq < tuner_table[ tuner_type ].mid_high_thresh )
	{
		control_code = tuner_table[ tuner_type ].vhf_high_control;
	}
	else
	{
		control_code = tuner_table[ tuner_type ].uhf_control;
	}
	rffreq = (WORD)freq;
	divider = (WORD)(  rffreq  + iffreq );
	tv_i2c_tuner_write( tuner_i2c_write_addr, divider, control_code );
	return 1;
}
