/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Geode Video 4 Linux 2 Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>
#include <signal.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>

#include <string.h>
#include "ioctl_geode_v.h"

volatile int	decoder_cc_enabled = 0;
volatile int	encoder_cc_enabled = 0;
unsigned char	encoder_cc_buf[ 64 ];
unsigned char	decoder_cc_screen[ 8 ][ 33 ];
unsigned char	decoder_cc_off_screen[ 8 ][ 33 ];

#define RAW_VBI_LEN		(720*10)
#include <signal.h>

void signal_handler(int sig)
{
	printf("Got signal\n");
}


int parse_cc( unsigned char cc1, unsigned char cc2 )
{  
	static unsigned char	offset = 0;
	static unsigned char	row = 0;
	static unsigned char	colidx = 0;
	int		update_screen = 0;
	int		i = 0;
	
	
	if( colidx > 32 )
	{
		colidx = 32;
	}
	
	cc1 &= 0x7F;
	cc2 &= 0x7F;
	
	//	check for row codes
	if( ( cc1 >= 0x11 ) && ( cc1 <= 0x14 ) && ( cc2 & 0x40 ) )
{
		offset = ( cc2 & 0x20 ) ? 1 : 0;
		colidx = 0;
		switch( cc1 )
		{
			case 0x11:
				row = offset;
				break;
			case 0x12:
				row = offset + 2;
				break;
			case 0x13:
				row = offset + 4;
				break;
			case 0x14:
				row = offset + 6;
				break;
		}
		return update_screen ;
	}
	
	if( ( cc1 != 0x11 ) && ( cc1 != 0x14 ) && ( cc1 != 0x17 ) )
	{
		//	regular data (text) byte
		if( cc1 )
		{
			decoder_cc_off_screen[ row ][ colidx++ ] = (char)cc1;
		}
		if( cc2 )
		{
			decoder_cc_off_screen[ row ][ colidx++ ] = (char)cc2;
		}
		return update_screen;
	}
	
	switch( cc1 )
	{
		case 0x11:
		case 0x19:
			//	attribute
			switch( cc2 )
			{
				case 0x20:
					//	white
					break;
				case 0x21:
					//	white underline
					break;
				case 0x22:
					//	green
					break;
				case 0x23:
					//	green underline
					break;
				case 0x24:
					//	blue
					break;
				case 0x25:
					//	blue underline
					break;
				case 0x26:
					//	cyan
					break;
				case 0x27:
					//	cyan underline
					break;
				case 0x28:
					//	red
					break;
				case 0x29:
					//	red underline
					break;
				case 0x2A:
					//	yellow
					break;
				case 0x2B:
					//	yellow underline
					break;
				case 0x2C:
					//	magenta
					break;
				case 0x2D:
					//	magenta underline
					break;
				case 0x2E:
					//	italics
					decoder_cc_off_screen[ row ][ colidx++ ] = ' ';
					break;
				case 0x2F:
					//	italics underline
					break;
			}
			break;
		case 0x14:
		case 0x1C:
			switch( cc2 )
			{
				case 0x20:
					for( i = 0; i < 32; i++ )
					{
						//	start of caption
						decoder_cc_off_screen[ 0 ][ i ] = 0x00;
						decoder_cc_off_screen[ 1 ][ i ] = 0x00;
						decoder_cc_off_screen[ 2 ][ i ] = 0x00;
						decoder_cc_off_screen[ 3 ][ i ] = 0x00;
						decoder_cc_off_screen[ 4 ][ i ] = 0x00;
						decoder_cc_off_screen[ 5 ][ i ] = 0x00;
						decoder_cc_off_screen[ 6 ][ i ] = 0x00;
						decoder_cc_off_screen[ 7 ][ i ] = 0x00;
					}
					break;
				case 0x21:
					//	backspace
					//printf("<backspace>\n");
					break;
				case 0x24:
					//	delete to end of row
					//printf("<delete to end of row>\n");
					break;
				case 0x25:
					//	roll-up captions 2 rows
					//printf("<roll-up captions 2 rows>\n");
					break;
				case 0x26:
					//	roll-up captions 3 rows
					//printf("<roll-up captions 3 rows>\n");
					break;
				case 0x27:
					//	roll-up captions 4 rows
					//printf("<roll-up captions 4 rows>\n");
					break;
				case 0x28:
					//	flash on
					break;
				case 0x29:
					//	resume direct captioning
					//printf("<resume direct captioning>\n");
					break;
				case 0x2A:
					//	text restart
					//printf("<text restart>\n");
					break;
				case 0x2B:
					//	resume text display
					//printf("<resume text display>\n");
					break;
				case 0x2C:
					update_screen = 1;
					for( i = 0; i < 32; i++ )
					{
						//	clear previous caption
						decoder_cc_screen[ 0 ][ i ] = 0x00;
						decoder_cc_screen[ 1 ][ i ] = 0x00;
						decoder_cc_screen[ 2 ][ i ] = 0x00;
						decoder_cc_screen[ 3 ][ i ] = 0x00;
						decoder_cc_screen[ 4 ][ i ] = 0x00;
						decoder_cc_screen[ 5 ][ i ] = 0x00;
						decoder_cc_screen[ 6 ][ i ] = 0x00;
						decoder_cc_screen[ 7 ][ i ] = 0x00;
					}
					break;
				case 0x2E:
					//	erase non-displayed memory
					break;
				case 0x2D:
					//	carriage return
					//	fall through
					//printf("<carriage return>\n");
				case 0x2F:
					update_screen = 1;
					for( i = 0; i < 32; i++ )
					{
						//	display caption
						decoder_cc_screen[ 0 ][ i ] = decoder_cc_off_screen[ 0 ][ i ];
						decoder_cc_screen[ 1 ][ i ] = decoder_cc_off_screen[ 1 ][ i ];
						decoder_cc_screen[ 2 ][ i ] = decoder_cc_off_screen[ 2 ][ i ];
						decoder_cc_screen[ 3 ][ i ] = decoder_cc_off_screen[ 3 ][ i ];
						decoder_cc_screen[ 4 ][ i ] = decoder_cc_off_screen[ 4 ][ i ];
						decoder_cc_screen[ 5 ][ i ] = decoder_cc_off_screen[ 5 ][ i ];
						decoder_cc_screen[ 6 ][ i ] = decoder_cc_off_screen[ 6 ][ i ];
						decoder_cc_screen[ 7 ][ i ] = decoder_cc_off_screen[ 7 ][ i ];
					}
					break;
				default:
					break;
			}
			break;
		case 0x17:
		case 0x1F:
			switch( cc2 )
			{
				case 0x21:
					//	tab offset 1 column
					break;
				case 0x22:
					//	tab offset 2 columns
					break;
				case 0x23:
					//	tab offset 3 columns
					break;
			}
			break;
	}
	return update_screen;
}

/*
	This is *very* simple.  It does not support color, italics,
	special characters, scrolling, etc.  It is based on a subset
	of the FCC telecommunications code Title 47, Volume 1, Part 15.
	HINT: It will render PBS CC well.
*/

void render_cc()
{
	char		row_buf[ 33 ];
	int		i;
	
	/* render the rows */
	for( i = 0; i < 8; i++ )
	{
		memset( row_buf, 0, sizeof(row_buf) );
		strncpy( row_buf, (const char*)decoder_cc_screen[ i ], 32 );
		row_buf[32]=0;
		if( strlen(row_buf) != 0 )
		{
			printf( "%s\n", row_buf );
		}
	}
}

int decode_cc( int vbi ) 
{
	__u8		vbi_data[ 48 ];
	/*  Each buffer has 4 bytes (2 odd, 2 even) */
	int		len;
	int i = 0;
	unsigned short header;
	memset( &vbi_data, 0, sizeof(vbi_data) );

        /* length of read is 4 because we requested two lines of 
           data per frame (Each read returns a frame). For CC, each
           line will contain 2 sliced bytes.
         */  
	len = read( vbi, vbi_data, 4);

	if( len < 0 )
	{
		printf("failed to read\n");
	}
	else
	{
			if( ( vbi_data[2] != 0x00 ) && ( vbi_data[3] != 0x00 ) )
			{
#if 0
				// This is used to debug order data received.
				printf("%d %d %c%c\n", vbi_data[0], vbi_data[1],vbi_data[2]&0x7f, vbi_data[3]&0x7f);
#else
				if( parse_cc( vbi_data[2], vbi_data[3] ) )
				{
					render_cc();
				}
#endif
			}
		//}
	}
	return 1;
}


void usage()
{
	printf("vbitest [-raw][-tune <channel>][-dump <size>]\n");
}

int
main(int argc, char *argv[])
{
	//struct v4l2_capability	cap;
	struct v4l2_format	fmt;
	__u32		i = 0;
	int			vid = 0;
	int vout ;
	//int			n = 0;
	int			args = 0;
	int			raw_mode = 0;
	int			mmap_mode = 0;
	int			dump_mode = 0;
	__u32		channel = 15;
	int			vbi = 0;
	__u32		dump_len = 0;
	int			streamon_val = 0;
	struct	signal_info_t	sig_info;
	geode_ioctl_debug_i2c_t debug;


	while( argc > ( args + 1 ) )
	{
		args++;
		if( strcmp(argv[args],"-input") == 0 )
		{
			if( argc > ( args + 1 ) )
			{
				args++;
				channel = atoi(argv[args]);
				printf("tuning to channel %d\n", channel );
			}
			else
			{
				usage();
				exit(1);
			}
		}
	}
	/* First, open the video device so we can select the input */
	vid = open("/dev/video",O_RDONLY); 
	if( vid == -1 )
	{
		printf( "failed to open video\n" );
		return 0;
	}
//	vout = open("/dev/vout",O_RDONLY|O_NOIO); 
//	if( vout == -1 )
//	{
//		printf( "failed to open vout\n" );
//		return 0;
//	}
	/* Set the input */
	if( ioctl(vid,VIDIOC_S_INPUT,&channel) < 0 )
	{
		printf("failed to set input\n");
	}
	
	if( ioctl(vid, VIDIOC_STREAMON, &channel) < 0)
	{
		printf("Failed streamon\n");
	}
	
	/* open the vbi device */
	/* Any further opens _must_ be with the flag O_NOIO */
	vbi = open( "/dev/vbi", O_RDONLY );
	if( vbi == -1 )
	{
		printf( "failed to open vbi\n" );
		return 0;
	}

		memset( &fmt, 0, sizeof(fmt) );
		fmt.fmt.vbi.samples_per_line = 2;
		fmt.fmt.vbi.sample_format =  GEODE_V4L2_VBI_SF_CC_US;
		fmt.fmt.vbi.start[0] = 20;
		fmt.fmt.vbi.start[1] = 283;
		fmt.fmt.vbi.count[0] = 1;
		fmt.fmt.vbi.count[1] = 1;
		fmt.fmt.vbi.flags = V4L2_VBI_APPEND_FMT;
		/* the default of the driver is line 21 and 284 for CC,
		   this call will add line 20 and 284 because of the 
		   V4L2_VBI_APPEND_FMT flag.
		*/   
                /* This is not really required, but has been put here just to
                   demonstrate how the call should be made. The defaults for
                   the driver works for CC (NTSC)
                 */
                   
		if( ioctl(vbi,VIDIOC_S_FMT,&fmt) < 0 )
		{
			printf("failed to set format\n");
		}

		sig_info.signal_number = SIGUSR1;
		sig_info.signal_on_length = 2;
		signal(SIGUSR1,signal_handler);
		/* uncomment if you want the driver to signal this app */
//		if( ioctl( vbi, GEODE_IOCTL_S_SIGNAL, &sig_info ) < 0 )
//		{
//			printf("failed to set signal\n");
//		}

		/* starting queuing the VBI buffers */
		if( ioctl( vbi, VIDIOC_STREAMON, &streamon_val ) < 0 )
		{
			printf("failed to stream on\n");
		}

		i = 100;
		do
		{
			decode_cc( vbi );
		} while(1);
		
		if( ioctl( vbi, VIDIOC_STREAMOFF, &streamon_val ) < 0 )
		{
			printf("failed to stream off\n");
		}
	close( vid );
	close( vbi );

	return 0;
}
