/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name	  : mouse.c						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains the exported functions which are *
 *		    used to do the interface with the bus mouse driver. *
 *									*
 ************************************************************************/

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>			// inb and outb
#include <linux/tqueue.h>	// for bottom-half
#include <linux/interrupt.h>
#include <linux/sched.h>	// for current
#include <linux/poll.h>		// poll method

#include "busmouse.h"

#define VERSION "1.0"
#define MOUSE_MINOR   12

static struct busmouse bus_ms=
{MOUSE_MINOR,"anscir",THIS_MODULE,NULL,NULL,0};

static int mouse_no;
static int open_count;



/************************************************************************
 * Syntax	: anscir_open                                           *
 *                                                                     	*
 * Description	: This function opens the basic IR mouse driver.        *
 *                                                                     	*
 * Parameters	: inode                                                	*
 *		    - The inode structure for the device special file.  *
 *                file                                                 	*
 *		    - The device-special file.                         	*
 *                                                                     	*
 * Return 	: 0 on sucess, error value on failure.                 	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

static int anscir_open (struct inode *inode, struct file *file)
{
	unsigned int minor = MINOR(file -> f_dentry -> d_inode -> i_rdev);

	MOD_INC_USE_COUNT;

	printk("\n ANSCIR :   OPEN      \n");

	if (open_count == 0)
	{
		printk("A National IR Driver: %s\n", VERSION);
	}
	
	return 0;
}


/************************************************************************
 * Syntax	: anscir_release                                        *
 *                                                                     	*
 * Description	: This function releases the basic IR mouse driver.     *
 *                                                                     	*
 * Parameters	: inode                                                	*
 *		    - The inode structure for the device special file.  *
 *                file                                                 	*
 *		    - The device-special file.                         	*
 *                                                                     	*
 * Return 	: 0 on sucess, error value on failure.                 	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

static int anscir_release (struct inode *inode, struct file *file)
{
	printk(" \n ANSCR release \n");
	MOD_DEC_USE_COUNT;
	return 0;
}



/************************************************************************
 * Syntax	: bus_init	                                        *
 *                                                                     	*
 * Description	: This function initializes basic mouse driver with the *
 *		  systems busmouse driver.				*
 *                                                                     	*
 * Parameters	: None                                                	*
 *                                                                     	*
 * Return 	: None.                 				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void  bus_init()
{
	bus_ms.open = anscir_open;
	bus_ms.release =  anscir_release;
}


ssize_t anscir_write (struct file *file, const char *buf, size_t count, loff_t *ppos)
{
	return 0;
}


ssize_t anscir_read (struct file *file, char *buf, size_t count, loff_t *ppos)
{
	unsigned int minor = MINOR(file -> f_dentry -> d_inode -> i_rdev);
		return -ENODEV;
}




/************************************************************************
 * Syntax	: anscir_ioctl	                                        *
 *                                                                     	*
 * Description	: Ioctls for the device.		.             	*
 *                                                                     	*
 * Parameters	: inode                                                	*
 *		    - The inode structure for the device special file. 	*
 *                file                                                 	*
 *		    - The device-special file.                         	*
 *		  cmd							*
 *		    - Command code.					*
 *		  arg							*
 *		    - argument to command.				*
 *                                                                     	*
 * Return 	: 0 on sucess, error value on failure.                 	*
 *                Error values are,					*
 *		    EINVAL: ioctl command invalid, or NULL passed	*
 *		    EFAULT: arg contains a bad address.			*
 *                                                                     	*
 * Comment	: None.	   						*
 *                                                                     	*
 ************************************************************************/

static int anscir_ioctl (struct inode *inode, struct file *file,
	unsigned int cmd, unsigned long arg)
{
	return 0;
}




/************************************************************************
 * Syntax	: anscir_poll                                           *
 *                                                                     	*
 * Description	: 					             	*
 *                                                                     	*
 * Parameters	: file                                                	*
 *		    - The device-special file.  			*
 *                wait                                                 	*
 *		    - The poll table.	                         	*
 *                                                                     	*
 * Return 	: Bitmask of permitted I/O operations.                 	*
 *		  ENODEV if file points to an invalid minor number.	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

static unsigned int anscir_poll (struct file *file, poll_table *wait)
{
	unsigned int minor = MINOR(file -> f_dentry -> d_inode -> i_rdev);
		return (0);
}



/************************************************************************
 * Syntax	: ansc_mv_mouse                                         *
 *                                                                     	*
 * Description	: This function calls exported function of system 	*
 *		  busmouse driver.					*
 *                                                                     	*
 * Parameters	: x                                                	*
 *		    - X axis position of the IR mouse.			*
 *                y                                                 	*
 *		    - Y axis position of IR mouse.                     	*
 *		  but							*
 *		    - Button status of the IR mouse.			*
 *                                                                     	*
 * Return 	: None.                 				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

unsigned char ansc_mv_mouse( int x, int y, int but)
{
    busmouse_add_movementbuttons(mouse_no,x,y,but);
}

EXPORT_SYMBOL_NOVERS(ansc_mv_mouse);


#ifdef MODULE

/************************************************************************
 * Syntax	: init_module                                           *
 *                                                                     	*
 * Description	: This function loads the driver as a module in the    	*
 *		  Linux kernel.						*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: 0 on success, errno on failure.	               	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int init_module (void)
{
	//	bus_init();
	mouse_no = register_busmouse(&bus_ms);
        printk("\n ARUN: registration done : %d \n",mouse_no);

	return 0;
}


/************************************************************************
 * Syntax	: cleanup_module                                        *
 *                                                                     	*
 * Description	: This function unloads the basic mouse driver module 	*
 *		  from the Linux kernel.				*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void cleanup_module (void)
{
	printk(" \n ANSCR cleanup \n");
	unregister_busmouse(mouse_no);

}

#endif

