/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name	  : rc5.h						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains the Rc5 protocol definitions.    *
 *									*
 ************************************************************************/

#ifndef __INCLUDED_RC5_H__
#define __INCLUDED_RC5_H__

/*--------------------------------------------------------------------------
 *            RC-5 Detection Information
 *------------------------------------------------------------------------*/

#define RC5_NUM_SAMPLES_IN_T_PERIOD        3
#define RC5_T_PERIOD_LENGTH                889    // usecs

// Maximal number of valid code bytes stored in the detection buffer.
// For RC-5 protocol significant bits in a code are
//        2 start bits
//        1 toggle bit
//        5 address bits
//        6 command bits
//        1 stop bit

#define RC5_NUM_START_BITS    2
#define RC5_NUM_TOGGLE_BITS   1
#define RC5_NUM_ADDRESS_BITS  5
#define RC5_NUM_COMMAND_BITS  6
#define RC5_NUM_STOP_BITS     1
#define RC5_NUM_CODE_BITS     \
            (RC5_NUM_START_BITS + RC5_NUM_TOGGLE_BITS + RC5_NUM_ADDRESS_BITS + \
             RC5_NUM_COMMAND_BITS)    // 14 bits


typedef union _RC5_CODE {
    DWORD Code;
    struct _Rc5Fields {
        DWORD Command : RC5_NUM_COMMAND_BITS;
        DWORD Address : RC5_NUM_ADDRESS_BITS;
        DWORD Toggle  : RC5_NUM_TOGGLE_BITS;
        DWORD Start   : RC5_NUM_START_BITS; 
    } Rc5Fields;
} RC5_CODE, *PRC5_CODE;

typedef DWORD REMOTE_CODE ; 

#endif /* __INCLUDED_RC5_H__ */
