/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name	  : sejin.h						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains the 4ppm and Sejin protocol      *
 *		    definitions. 					*
 *									*
 ************************************************************************/

#ifndef __INCLUDED_SEJIN_H__
  #define __INCLUDED_SEJIN_H__


/*--------------------------------------------------------------------------
 *            SEJIN Detection Information
 *------------------------------------------------------------------------*/
#define SEJIN_NUM_SAMPLES_IN_T_PERIOD     13
#define SEJIN_T_PERIOD_LENGTH_38K         312
#define SEJIN_T_PERIOD_LENGTH_56K         321

#define SEJIN_NUM_CODE_BITS               34
#define SEJIN_NUM_OF_CODE_DWORDS        \
            ((SEJIN_NUM_CODE_BITS + N_BITS_IN(DWORD) - 1) / N_BITS_IN(DWORD)) // 2

#define SEJIN_CHECKSUM_MASK               0x0f
#define SEJIN_START_BITS_VALUE            3

#define SEJIN_BREAK_CODE                  1
#define SEJIN_MAKE_CODE                   0


typedef union _SEJIN_CODE {
    DWORD Code[SEJIN_NUM_OF_CODE_DWORDS];
    BYTE  CodeBytes[sizeof(DWORD) * SEJIN_NUM_OF_CODE_DWORDS / sizeof(BYTE)];
    struct _SejinNibbleFields {
        DWORD              : 32;
        DWORD              : 2;
        DWORD Nibble6      : 4;
        DWORD Nibble5      : 4;
        DWORD Nibble4      : 4;
        DWORD Nibble3      : 4;
        DWORD Nibble2      : 4;
        DWORD Nibble1      : 4;
        DWORD Nibble0      : 4;
        DWORD              : 2;
    } SejinNibbleFields;
    struct _SejinCommonFields {
        DWORD             : 30;
        DWORD Checksum1   : 2;
        DWORD Checksum2   : 2;
        DWORD UserID      : 4;
        DWORD             : 4;
        DWORD             : 4;
        DWORD             : 4;
        DWORD             : 4;
        DWORD             : 3;
        DWORD DeviceID    : 5;
        DWORD StartBits   : 2;
    } SejinCommonFields;
    struct _SejinKbdFields {
        DWORD             : 30;
        DWORD Checksum1   : 2;
        DWORD Checksum2   : 2;
        DWORD UserID      : 4;
        DWORD KeyCode     : 8;
        DWORD RCtrl       : 1;
        DWORD LCtrl       : 1;
        DWORD RAlt        : 1;
        DWORD LAlt        : 1;
        DWORD RShift      : 1;
        DWORD LShift      : 1;
        DWORD FN          : 1;
        DWORD MakeBreak   : 1;
        DWORD RWindows    : 1;
        DWORD LWindows    : 1;
        DWORD Application : 1;
        DWORD DeviceID    : 5;
        DWORD StartBits   : 2;
    } SejinKbdFields;
    struct _SEJINMouseFields {
        DWORD             : 30;
        DWORD Checksum1   : 2;
        DWORD Checksum2   : 2;
        DWORD UserID      : 4;
        DWORD YMovement   : 8;
        DWORD XMovement   : 8;
        DWORD LButton     : 1;
        DWORD RButton     : 1;
        DWORD MButton     : 1;
        DWORD DeviceID    : 5;
        DWORD StartBits   : 2;
    } SejinMouseFields;
    struct _SEJINPointDiskFields {
        DWORD             : 30;
        DWORD Checksum1   : 2;
        DWORD Checksum2   : 2;
        DWORD UserID      : 4;
        DWORD MaxSpeed    : 4;
        DWORD Reserved    : 4;
        DWORD Direction   : 4;
        DWORD Speed       : 4;
        DWORD LButton     : 1;
        DWORD RButton     : 1;
        DWORD MButton     : 1;
        DWORD DeviceID    : 5;
        DWORD StartBits   : 2;
    } SejinPointDiskFields;
} SEJIN_CODE, *PSEJIN_CODE;

#endif /* __INCLUDED_SEJIN_H__ */
