/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 ********************************************************************************
 * File Name        : clientcom.cpp						*
 * 										*
 * Project Name	    : ASTB							*
 *                              						*
 * Subsystem Name   : Streamer	                                                *
 *                               						*
 * Module Name      : Clientcom							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 2/13/01							*
 *										*
 * Description      : This file defines the various functions to establish the	*
 *                    socket coneection with the server and then to send and 	*
 *		      receive data from it.					*
 *										*
 ********************************************************************************/


#include "clientcom.h"
#include "defines.h"


typedef struct sockaddr_in SOCKADDR_IN;
typedef struct hostent * PHOSTENT;
typedef struct sockaddr * PSOCKADDR;
int TERMIN=0;

/************************************************************************
 * Syntax	: ClientCom()						*
 *									*
 * Description	: This is a constructor which makes owner status as 	*
 *		  false when it is not connected to the server.		*
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

ClientCom::ClientCom()
{
	Owner = false;
}



/************************************************************************
 * Syntax	: ClientCom(CString ServerName)				*
 *									*
 * Description	: This is a function which calls to connect to the 	*
 *		  server when owner permission is there.		*
 *									*
 * Parameters	: ServerName                				*
 *			- Name of the server.                            *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

ClientCom::ClientCom(  CString ServerName ) 	
{	
	Owner = true ;
	if(!ConnectToServer(ServerName))	
	{
		Owner = false;	
	}
}





/************************************************************************
 * Syntax	: ClientCom(SOCKET prev_connection)			*
 *									*
 * Description	: This is a function which creates an object with    *
 *		  the previous connection.				*
 *									*
 * Parameters	: prev_connection					*
 *		      - The socket connection which is already existing.*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

ClientCom::ClientCom(  SOCKET prev_connection )
{
       Owner=false;
	ServerSock = prev_connection;
}		
		




/************************************************************************
 * Syntax	: ~ClientCom()						*
 *									*
 * Description	: This is a destructor which calls to disable send and  *
 *		  recive on socket and then to close the connection 	*
 *		  with the server.					*
 *									*
 * Parameters	: None.		                                        *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


ClientCom::~ClientCom()
{
	if(Owner)
	{
		CloseConnection();		// Disable sending & receiving on ServerSock.
		shutdown (ServerSock, 2); 	// Close the socket.
		closesocket(ServerSock);
	}
}


	


/************************************************************************
 * Syntax	: GetSocket()						*
 *									*
 * Description	: This is a function which returns the socket	*
 *		  descriptor.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: ServerSock						*
 *                    Socket descriptor.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

SOCKET ClientCom::GetSocket() 
{
	return ServerSock;
}




/************************************************************************
 * Syntax	: IsOwner()						*
 *									*
 * Description	: This is a function which reutrns the owner 	*
 *		  permission.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Owner							*
 *		     permission to access the 	network connection.     *
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::IsOwner()
{
	return Owner;
}





/************************************************************************
 * Syntax	: closesocket(int sockfd)				*
 *									*
 * Description	: This is a function calls function to close the     *
 *		  socket connection.					*
 *									*
 * Parameters	: sockfd						*
 *			The socket descriptor.				*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void ClientCom::closesocket (int sockfd)
{
	close(sockfd);	
}





/************************************************************************
 * Syntax	: shutdown(int, int )					*
 *									*
 * Description	: Can be used latter.					*
 *									*
 * Parameters	: sockfd						*
 *		      - Socket descriptor.				*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void  ClientCom::shutdown(int sockfd,int i)
{
// nothing to do

}



/************************************************************************
 * Syntax	: ConnectToServer(LPCTSTR serverName)			*
 *									*
 * Description	: This function is to estblish the socket connection *
 *		  with the server.					*
 *									*
 * Parameters	: serverName                				*
 *		       - Name of the server with which the coonection   *
 *			 is to be established.                          *
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


int ClientCom::ConnectToServer(LPCTSTR serverName)
{
	struct in_addr addr;
	char 		server_name[32];
	SOCKADDR_IN	destination_sin;// Server socket address
	struct   hostent    *phostent = NULL;   // Points to the HOSTENT structure of the server
	
	
	strncpy(server_name,serverName,31);

	// Create a TCP/IP socket that is bound to the server.
	if ( (ServerSock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{	
		printf( "Client: Error Opening socket: \n");
		return 0;
	}
	
	// Attempt to detect if we should call gethostbyname() or gethostbyaddr()
	if( (addr.s_addr = inet_addr((char *)server_name)) == -1)
	{
		if ((phostent = gethostbyname((char *)server_name)))
		{
			//If we find a host entry, set up the internet address
			addr.s_addr = *(long *)phostent->h_addr_list;
		}
		else{
			switch(h_errno)
			{
				case  HOST_NOT_FOUND :
						printf("host not found \n");
						break;
				 case  NO_DATA:
					 	printf(" no data \n"); 
					 	break;
				 case  TRY_AGAIN:
					 	printf(" try again \n"); 
					 	break;
				 case  NO_RECOVERY :
					 	printf(" no recov ..\n"); 
						break;
				 default:
					 	printf(" surprise ..\n");
						break;
			}
			
			closesocket(ServerSock);
			return 0;
			}
		    }
	else
	{
		# if 0
		printf("Connecting to %s %x\n",(char *)server_name, addr);
		phostent = gethostbyaddr((char *)&addr,sizeof(addr),AF_INET);	
		if(phostent == NULL)
		{
			printf("Route to %hs \n",inet_ntoa(addr));
			GetHostByAddrErrorMsg();
		 }
		#endif
	 }
		

	#if 0
		if (phostent == NULL )
		{
			printf("Client: Cannot resolve address [%s]%x: \n",server_name,addr);
			losesocket(ServerSock);
			eturn 0;
		}
	#endif


	// Copy the resolved information into the sockaddr_in structure
	memset(&destination_sin,0,sizeof(destination_sin));
	
	// Assign the socket IP address.
	memcpy(&(destination_sin.sin_addr), &addr, sizeof(addr));
	
	// Fill out the server socket's address information.
	destination_sin.sin_family = AF_INET;
	
	// Convert to network ordering.
	destination_sin.sin_port = htons(DEFAULT_PORT);

	// Establish a connection to the server socket.
	if (connect(ServerSock,(PSOCKADDR )&destination_sin, sizeof(destination_sin))
				== -1)
	{
	 	printf ( "connect() Failed: \n");
		closesocket(ServerSock);
		return 0;
	}
	
	//Send the block size
	commandWord = BLOCK_SZ;
	SendData((char *)&commandWord,sizeof(commandWord));
	
	return 1;
}


/************************************************************************
 * Syntax	: ReceiveData(char *pBuff, int len)			*
 *									*
 * Description	: This function is to receive the data from the 	*
 *		  server.						*
 *									*
 * Parameters	: *pBuff                				*
 *			- Buffer to receive the data.			*
 *		  len							*
 *		     - Length of the buffer.				*
 *									*
 * Returns	: 0 for failure, 1 for Sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


int ClientCom::ReceiveData(char *pBuff, int len)
{
	int retval, nSize=0;
	void ChangeTermin(int);
	int GetTermin();
    for (nSize = 0; nSize < len;)
	{
        	retval = recv (ServerSock, pBuff+nSize, len-nSize, 0);
        	if (retval == -1)
        	{
	        	RecvErrorMsg();
			// Disable sending & receiving on ServerSock.
			shutdown (ServerSock, 2);
			// Close the socket.
			closesocket(ServerSock);
			return 0;
	        }
		if( ((*(int*)pBuff) > ERR_BEGIN) && ((*(int*)pBuff) < ERR_END))
	        {
                        //print the error msg
       	         	ReadErrorMsg((*(int*)pBuff));
			ChangeTermin(1);
        	        return 0;
        	}
        	nSize += retval;
    	}
	return 1;
}

// the following two functions have been written by Chitra to
// allow the cliwnt to receive error messages from the server
// so that the change is reflected in the middleware API
/************************************************************************
 * Syntax	: ChangeTermin(int value)				*
 *									*
 * Description	: This function is to set the flag TERMIN to true. 	*
 *		  when the client receives an error message from	*
		  the server,the flag is set to true.			*
 *									*
 * Parameters	: int value						*
 *			- the value to which the flag is to be set	*
 *									*
 * Returns	: 0 for failure, 1 for Sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void ChangeTermin(int n)
{
	TERMIN=n;
}

/************************************************************************
 * Syntax	: GetTermin(int value)					*
 *									*
 * Description	: This function is to return the value of TERMIN	*
 *									*
 * Parameters	: None							*
 *									*
 * Returns	: Value of TERMIN which is either 0 or 1		*
 *			0 suggests no error				*
 *			1 suggests error msg from server		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int GetTermin()
{
	return TERMIN;
}
/************************************************************************
 * Syntax	: SendData(char *pBuff, int len)			*
 *									*
 * Description	: This function is to send the data from the server.	*
 *									*
 * Parameters	: *pBuff                				*
 *			- Buffer to receive the data.			*
 *		  len							*
 *		     - Length of the buffer.				*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


int ClientCom::SendData(char *pBuff, int len)
{
	int retval;

	retval = send(ServerSock, pBuff, len, 0);
	if (retval == -1  )
	{
		SendErrorMsg();
		// Disable sending & receiving on ServerSock.
		shutdown (ServerSock, 2);
		// Close the socket.
		closesocket(ServerSock);
		return 0;
	}
		
	return 1;
}




/************************************************************************
 * Syntax	: EndOfTransmission()					*
 *									*
 * Description	: This function is to send end of the transmission	*
 *		  to the server.					*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::EndOfTransmission()
{               		
	commandWord = END_OF_TRANSMISSION;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))	
		return false;
	return true;
}



/************************************************************************
 * Syntax	: AbortTransmission()					*
 *									*
 * Description	: This function is to send abort transmission to the	*
 *		  server.						*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
bool ClientCom::AbortTransmission()
{	
	commandWord = ABORT_TRANSMISSION;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))		
		return false;
	return true;
}




/************************************************************************
 * Syntax	: NextPacket()						*
 *									*
 * Description	: This function is to send next packet to the server *
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::NextPacket()
{
	commandWord = SEND_NEXT_PACKET;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))	
		return false;
	return true;
}
	



/************************************************************************
 * Syntax	: ResendFromStart()					*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  transmit from the begining.				*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::ResendFromStart()
{
	commandWord = RESEND_FROM_START;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))
			return false;
	return true;
}


/************************************************************************
 * Syntax	: SendFileData()					*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  transmit the file data.				*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::SendFileData()
{
	commandWord = SEND_FILE_DATA;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))
			return false;
	return true;
}



/************************************************************************
 * Syntax	: SendDirectoryListing()				*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  transmit the directory listing.			*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::SendDirectoryListing()
{
	commandWord = SEND_DIRECTORY_LISTING;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))
			return false;
	return true;
}
	




/************************************************************************
 * Syntax	: CloseConnection()					*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  close the socket connection.				*
 *									*
 * Parameters	: None.	                				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::CloseConnection()
{
	commandWord = CLOSE_CONNECTION;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))	
		return false;
	return true;
}





/************************************************************************
 * Syntax	: FastMode(int direction, unsigned long ByteLocation)	*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  put it in the fast mode either in the forward or	*
 *		  reverse direction.					*
 *									*
 * Parameters	: direction                                             *
 *			- Forward or reverse direction.          	*
 *		  ByteLocation                                          *
 *			- Postion of the byte.				*
 *									*
 * Returns	: True or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::FastMode(int direction, unsigned long ByteLocation)
{       	
	commandWord = FAST_MODE;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))	
		return false;
		//now send the direction
	if(!SendData((char *)&direction,sizeof(direction)))
		return false;
		//now send the processed byte location
	if(!SendData((char *)&ByteLocation,sizeof(ByteLocation)))
		return false;
	return true;
}




/************************************************************************
 * Syntax	: PlayMode(int direction, unsigned long PSOB,unsigned   *
 *                         long ByteLocation, unsigned long SkipTo)	*
 *									*
 * Description	: This function is to request to the server to 		*
 *		  put it in the normal play mode.			*
 *									*
 * Parameters	: direction                                     	*
 *			- Forward or reverse direction. 		*
 *		  PSOB							*
 *		     -							*
 *		  Bytelocation						*
 *			- Position of the byte.				*
 *		  SkipTo						*
 *		     - Postion where the control will jump.		*
 *									*
 * Returns	: True	or false.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

bool ClientCom::PlayMode(int direction, unsigned long PSOB,unsigned long ByteLocation, unsigned long SkipTo)
{

	commandWord = PLAY_MODE;
	if(!SendData((char *)&commandWord,sizeof(commandWord)))	
		return false;
        		//now send the direction

	if(!SendData((char *)&direction,sizeof(direction)))
		return false;


	if(direction == PLAY_NORMAL)    	//playmode
	{
		//now send the byte location to jump and play
		if(!SendData((char *)&SkipTo,sizeof(SkipTo)))
			return false;
        

	} else
	{	
		//now send the PSOB
		if(!SendData((char *)&PSOB,sizeof(PSOB)))
			return false;

		//now send the processed byte location
		if(!SendData((char *)&ByteLocation,sizeof(ByteLocation)))
			return false;
	}

	return true;
}





/************************************************************************
 * Syntax	: GetHostByAddrErrorMsg()				*
 *									*
 * Description	: Can be used latter.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void ClientCom::GetHostByAddrErrorMsg()
{
	/* Do Nothing */
}




/************************************************************************
 * Syntax	: SendErrorMsg()					*
 *									*
 * Description	: Can be used latter.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void ClientCom::SendErrorMsg()
{
	/* Do Nothing */
}



/************************************************************************
 * Syntax	: RecvErrorMsg()					*
 *									*
 * Description	: Can be used latter.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void ClientCom::RecvErrorMsg()
{
 	/* Do Nothing */
}

void ClientCom::ReadErrorMsg(int errcode)
{
	printf("\n######MSG_FROM_SERVER: ");
        switch(errcode)
        {
                case FILE_STAT_ERR:
                        printf("The Mentioned File is not present\n");
                        break;
                case FILE_OPEN_ERR:
                        printf("Selected File Not Present\n");
                        break;
		case DIR_OPEN_ERR:
			printf("Error in Opening Selected Directory\n");
			break;
        } //end of switch
	printf("\n");
}


