/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 ********************************************************************************
 * File Name        : clientcom.h						*
 * 										*
 * Project Name	    : ASTB							*
 *										*
 * Subsystem Name   : Streamer	                                                *
 *                               						*
 * Module Name      : clientcom							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This header file defines the function prototypes, whcih   *
 *		      are  used for estblishing a socket connection with the 	*
 *		      server.							*
 ********************************************************************************/


#ifndef _CLIENTCOM_h_
#define _CLIENTCOM_h_


#include   <sys/types.h>
#include   <sys/socket.h>
#include   <netinet/in.h>
#include   <arpa/inet.h>
#include  <stdio.h>
#include <netdb.h>
#include  <string.h>
#include  <unistd.h>
#include   "command.h"


#define MPEG_PACKET_SIZE 188
#define PACKETS_IN_BLOCK 349
#define BLOCK_SZ (MPEG_PACKET_SIZE * PACKETS_IN_BLOCK)

typedef int  SOCKET ;

typedef  char *	CString;
typedef  char * LPCTSTR;




/************************************************************************
 * Class Description : The class contains various functions declaration *
 *		       which are used to establish a connection with the*
 *		       server.						*
 ************************************************************************/


class ClientCom
{
public:
	ClientCom(); // standard constructor
	~ClientCom();
	ClientCom(CString ServerName);
	ClientCom(SOCKET prev_connection);

	SOCKET GetSocket() ;
	bool	IsOwner() ;
	int ReceiveData(char *pBuff, int len);
	int SendData(char *pBuff, int len);
	bool EndOfTransmission();
	bool AbortTransmission();
	bool NextPacket();
	bool ResendFromStart();
	bool SendFileData();
	bool SendDirectoryListing();
	bool CloseConnection();
	bool FastMode(int direction, unsigned long ByteLocation);
	bool PlayMode(int direction, unsigned long PSOB,
	unsigned long ByteLocation, unsigned long SkipTo=0);

private:
	SOCKET ServerSock;
	bool Owner;
	int commandWord;
	int ConnectToServer(LPCTSTR server_name);
	//Error message handlers
	void RecvErrorMsg();
	void ReadErrorMsg(int);
	void SendErrorMsg();
	void GetHostByAddrErrorMsg();
	void closesocket(int sockfd) ;
	void shutdown(int sockfd,int i);
};


#endif
