/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 ********************************************************************************
 * File Name        : lnx_clnt.cpp						*
 * 										*
 * Project Name     : ASTB							*
 *										*
 * Subsystem Name   : Streamer							*
 *										*
 * Module Name      : lnx_clnt							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This module is the main module from which the action      *
 *		      starts.							*
 *										*
 ********************************************************************************/


#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
                       
#include "clientcom.h"
#include "defines.h"
#include "nscparam.h"
#include "nscsigma.h"		       
#include "nscplayer.h"


Param *pParam;




/************************************************************************
 * Syntax	: GetCommand()						*
 *									*
 * Description	: This function returns the status of the key press 	*
 *		  buffer.						*
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: Return the status of the key press buffer.idicates the*
 *		  keypress(button press is accepted or not,  non	*
 *		  negative for sucess.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int GetCommand()
{ 	return pParam->GetStatus();
}


/************************************************************************
 * Syntax	: MainControl()						*
 *									*
 * Description	: This function is responsible for envoking functions  	*
 *		  depending on the button pressed.			*
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

 void  *MainControl(void *a)
 {	
	char serverNameDot[60];
	char fileName[100];
	int  PrevMode =0;
	int value;
	int retvalue =1;
	static int cntflg=0; 
	NscPlayer *pPlayer; 	
	ClientCom *connection;

	pPlayer		= new NscPlayer();
	pPlayer->Setparam(pParam);

	
	 while(1)
	 {       
		 if( GetCommand())
		 {
		 switch( pParam->GetKeyWithValue(& value))
		 {
		    case CGETDIR :
				
			if(0==pPlayer->ShowDirectory(pParam->GetRmtCWD()))
			{ pParam->postErMsg("SHOW LIST : Fail to Show List");
			}
				
		        PrevMode =CGETDIR;
			break;
						
 		    case CCNTSER:   printf("\n CCNTSER \n");
			if(cntflg ==0)
			{
			pParam->GetServNameDot(serverNameDot);
			 printf("\n servername = <%s>\n\n \n",serverNameDot);
		 	connection 	= new ClientCom(serverNameDot);     
					
			pPlayer->SetConnection(connection);

			if(connection->IsOwner())
				{
				cntflg = 1;
			 	pParam->SetOwner(1);
			 	pParam->postErMsg("CONNECT:connected ");
				 printf("\n connected to server \n");
				}
				else 
				{
				pParam->SetOwner(0);
				pParam->postErMsg("CONNECT:Fails to connect");
				 printf("\n fail to connect \n");
				}
			}
			else
			{
				if((connection->IsOwner())) 
				{
				cntflg=0;
			  	pParam->SetOwner(0);
				connection-> EndOfTransmission();
				usleep(1000);
				delete(connection);
				}
				else
				{
							
			  	pParam->SetOwner(0);
			        pParam->postErMsg("DISCONNECT:Fails to disconnect ");
				}
			}
			PrevMode =CCNTSER;
			break;
						
		   case COPEN:
			pParam->GetFlName(fileName);
			pPlayer->SetupFileTransfer(fileName); 
			pPlayer->ResetBuffers() ;
			PrevMode = COPEN;
			break;
						
		    case CPLAY:	
			if( (PrevMode == CFF) || (PrevMode ==CREV))
				pPlayer->SetInPlayMode(1);
			else    pPlayer->SetInPlayMode(0);
			if( 0 == pPlayer->NSCNormalPlay())
			{	pParam->postErMsg(" PLAY : Fail to Play \n");
				retvalue  =0;
			}
			PrevMode=CPLAY;
			break;
						
		   case  CSTP:	
			connection-> EndOfTransmission();
			usleep(1000);
			PrevMode = CSTP;
			break;		
						
		   case CQUT:
			delete pPlayer;
			PrevMode = CQUT;
			break;

		   case  CSCROLL:	
			pPlayer->SetSliderPoint(pParam->GetScrolpt());	
			pPlayer->SetInPlayMode(1);
			if( 0 == pPlayer->NSCNormalPlay())
			{	pParam->postErMsg("SKIP : Fail to Play ");
			 	retvalue =0;
			}
			PrevMode = CSCROLL;
			break;
						
		    case CFF:	
			pPlayer->SetFastFarward();
			if( 0 == pPlayer->NSCFastPlay())
			{	pParam->postErMsg(" FastFarward:Fail to Play");
			 	retvalue =0;
			}
			PrevMode = CFF;
			break;
						
		   case CREV:	
			pPlayer->SetReverseMode();
			if(0 == pPlayer->NSCFastPlay())
			{	
				pParam->postErMsg(" Reverse : Fail to Play ");
			 	retvalue =0;
			}
			PrevMode = CREV;
			break;
						
		   case CMUTE:    
			break;
		   case CADIOL:   
		   	pPlayer->SetAudioLeft(pParam->GetAudioL());
			break;
		   case CADIOR:	
		   	pPlayer->SetAudioRight(pParam->GetAudioR());
			break;				
						
		   default :	
		   	break; 
		 }
		 }
	 }
 }




/************************************************************************
 * Syntax	: GetPtrToParam()					*
 *									*
 * Description	: This function returns the pointer to the param object.*
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: Pointer to the param object.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

Param * GetPtrToParam()
{
	return  pParam;
}



/************************************************************************
 * Syntax	: main(int , char ** )					*
 *									*
 * Description	: - create a thread where GUI runs independently	*
 *                - main thread controls the actions depending on  	*
 *                   the button press                                   *
 *	          - Gui thread and main thread communicate via common   *
 *	            object PARAm.                                       *
 *             								*
 *									*
 * Parameters	: argc,argv                                             *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
/*
void main(int argc, char ** argv)
{

  	int    thdHandle;
	pthread_t       f_th;
 	char  msg[] = " ";


	GlbArgc	= argc;
	GlbArgv = argv;

	pParam = new Param();
	
 	thdHandle = pthread_create(&f_th,NULL,ThdFunc,(void *) msg);
	if(thdHandle != 0)
                {      printf("InitPlayer:Failed to create Feeder Thread !!!\n");
			exit(1);
                }
	if( ! MainControl() )
	{printf(" Terminating streamer software ");
	}

	
}
*/





/************************************************************************
 * Syntax	: NscOpenPlayer()					*
 *									*
 * Description	: - create a thread .					*
 *									*
 * Parameters	: None.		                                        *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscOpenPlayer()
{

  	int    thdHandle;
	pthread_t       f_th;
 	char  msg[] = " ";

	pParam = new Param();
 	thdHandle = pthread_create(&f_th,NULL,MainControl,(void *) msg);
	if(thdHandle != 0)
        {      
	  printf("InitPlayer:Failed to create Feeder Thread !!!\n");
	  exit(1);
        }
	
}


