/**********************************************************************************
 *
 * National Semiconductor Corporation Open Source License for
 * NSC Digital Video 
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is licensee's responsibility to comply with
 * any export regulations applicable in licensee's jurisdiction. Under
 * CURRENT 2000  U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 *
 **********************************************************************************/			  

#include<stdio.h>
#include "mpeg_api.h"

/*********************************************************************************
 * Syntax	:  U32 VMW_DVM_GetBufferPointer( DECODE_BUFFER_INFO* pDecodeBufferInfo)
 *
 * Description	:  This function fetches a buffer from the memory manager. The 
 *                 decoded mpeg data is placed in this buffer.
 *
 * Parameters	:  DECODE_BUFFER_INFO*   A pointer to the Buffer Information.
 *
 * Returns	:  VMW_SUCCESS on Success ; VMW_DVM_ENOMEMORY on failure.
 *  
 * Comment	:
 *********************************************************************************/
  
U32 VMW_DVM_GetBufferPointer( DECODE_BUFFER_INFO* pDecodeBufferInfo)
{ 
	U32 fmpErr;       
        fmpErr = FMPGetBuffer((FMP_BUFFER*)pDecodeBufferInfo,TRUE);
        if (fmpErr == 0 )
        {
           DBG("Buffer Allocation Successful \n");
	   return VMW_SUCCESS;
        }
        else
        {
           DBG("Buffer Allocation Failed  \n");
           return VMW_DVM_ENOMEMORY;
        }
} 
        
/*********************************************************************************
 * Syntax       :  U32  VMW_DVM_StartMPEGDecode(U32* pBuffer)
 *
 * Description	:  This function starts MPEG decoding. 
 *`
 * Parameters	:  A pointer to PFMP_BUFFER. 
 *
 * Returns	:  VMW_SUCCESS on Success , else VMW_DVM_EPUSH.
 *  
 * Comment	:
 *********************************************************************************/

U32  VMW_DVM_StartMPEGDecode(U32* pBuffer)
{
	U32 fmpErr;       
        fmpErr = FMPPush((PFMP_BUFFER)pBuffer);

        if (fmpErr == 0 )
        {
           DBG("Data Pushed to Decoder Successfully \n");
           return VMW_SUCCESS ;
        }
        else
        {
           DBG("Failed to Push Data to Decoder  \n");
           return VMW_DVM_EPUSH;
        }
}
       
/*********************************************************************************
 * Syntax       :  U32 VMW_DVM_Open(U32 flags,U32 size,U32 count,void* Callback)
 *
 * Description	:  This functions Opens the MPEG Decoder. 
 *
 * Parameters	:
 *                 Flags  : This specifies the Stream type to play.
 *
 *                 This could be any of the following
 *		   MPEG_TRANSPORT  - for transport Stream
 *                 MPEG_SYSTEM     - for system / program Stream
 *                 MPEG_VIDEO      - for video Stream
 *                 MPEG_AUDIO      - for Mpeg Audio Stream
 *                 MPEG_DVD        - for DVD Stream
 *                 MPEG_SVCD       - for SVCD/VCD Stream
 *
 *                 size		: Specifies the size in buffer.
 *                 count	: Specifies the number of buffers to be allocated.
 *                 Callback	: Function pointer to the call back function to receive
 *                                the strean notification.         	
 *
 * Returns	:  VMW_SUCCESS on success ; VMW_DVM_EOPEN on failure.
 *  
 * Comment	:
 *********************************************************************************/

U32 VMW_DVM_Open(U32 flags,U32 size,U32 count,void* Callback)
{
        if(MPEGDriverEntry(NO_DRIVE)== 0)
        {
          if( FMPOpen(flags,size,count,Callback,0))
          {
            DBG("Failed in Driver Open  \n");
            return VMW_DVM_EOPEN;
          }
          else
          {
            DBG("Driver Opened Successfully  \n");
            return VMW_SUCCESS;
          }
        }
        else
        {
          DBG("Failed in Driver Entry  \n");
          return VMW_DVM_EMPEGOPEN; 
        }
}

/*********************************************************************************
 * Syntax       :  U32 VMW_DVM_Play()
 *
 * Description	:  This function puts the driver in play mode.
 *
 * Parameters	:  None 
 *
 * Returns	:  VMW_SUCCESS on success ; VMW_DVM_EPLAY on failure.
 *  
 * Comment	:
 *********************************************************************************/

U32 VMW_DVM_Play()
{      
       U32 fmpErr;       
       fmpErr= FMPPlay();
       if (fmpErr ==0)
       {
          DBG(" Play Call Successful \n");
          return VMW_SUCCESS;
       }
       else
       {
          DBG(" Play Call Failed \n");
          return VMW_DVM_EPLAY;
       }

}

/*********************************************************************************
 * Syntax       :  U32 VMW_DVM_Pause(U32 pauseOnOff)
 *
 * Description	:  Depending on pauseOnOff , This function either pauses MPEG 
 *                 decoding Or puts the driver in play mode.
 *
 * Parameters	:  pauseOnOff : if this is 1 , MPEG decoding is paused; else
 *                 the driver is put in the play mode.
 *
 * Returns	:  VMW_SUCCESS on success; VMW_DVM_EPAUSE on failure.
 *  
 * Comment	:
 *********************************************************************************/

U32 VMW_DVM_Pause(U32 pauseOnOff)
{
        U32 fmpErr;       
	if(pauseOnOff == 1)
	{
	   fmpErr = FMPPause();
	}
	else
        {
	   fmpErr = FMPPlay();
        }
        if ( fmpErr == 0)
        {
            DBG(" Pause Call Successful \n");
            return VMW_SUCCESS;
        }
        else
        {
            DBG(" Pause Call Failed \n");
            return VMW_DVM_EPAUSE;
        }
}

/*********************************************************************************
 * Syntax       : U32 VMW_DVM_Stop()
 *
 * Description	: This function stops MPEG Decoder. 
 *
 * Parameters	: None.
 *
 * Returns	: VMW_SUCCESS on success ; VMW_DVM_ESTOP on failure.
 *  
 * Comment	:
 *********************************************************************************/
 
U32 VMW_DVM_Stop()
{      
       U32 fmpErr;       
       fmpErr = FMPStop();
       if ( fmpErr == 0 )
       {
          DBG(" Stop Call Successful \n");
          return VMW_SUCCESS;
       }
       else
       {
          DBG(" Stop Call Failed \n");
          return VMW_DVM_ESTOP;
       }
}
      
/*********************************************************************************
 * Syntax	:  U32 VMW_DVM_Close()
 *
 * Description	:  This function Closes the MPEG Decoder.
 *
 * Parameters	:  None
 *
 * Returns	:  VMW_SUCCESS on success; VMW_DVM_ECLOSE on failure.
 *  
 * Comment	:
 *********************************************************************************/

U32 VMW_DVM_Close()
{
      U32 fmpErr;       
      fmpErr = FMPStop();
      if (fmpErr == 0)
      {
        fmpErr = FMPClose();
        if(fmpErr == 0)
        {
         fmpErr = MPEGDriverUnload();
        }
      }
      if ( fmpErr == 0)
      {
          DBG(" DVM_Close Successful \n");
          return VMW_SUCCESS;
      }
      else
      {
          DBG(" DVM_Close Failed \n");
          return VMW_DVM_ECLOSE ;
      }

}
