 /**********************************************************************************
 *
 * National Semiconductor Corporation Open Source License for
 * NSC Digital Video  
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2001.
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is licensee's responsibility to comply with
 * any export regulations applicable in licensee's jurisdiction. Under
 * CURRENT 2000 U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 **********************************************************************************/
 			
#ifndef __MPEG_API_H
#define __MPEG_API_H

#ifdef __cplusplus
extern "C"{
#endif

#define _DEFINE_FMP_TYPES_
#include "fmp.h"


#ifdef DEBUG
#define DBG(args...) printf(__FILE__":%d:"args, __LINE__)
#else
#define DBG(args...)
#endif

#ifdef MESSAGE
#define MSG(args...) printf(__FILE__":%d:"args, __LINE__)
#else
#define MSG(args...)
#endif


typedef enum {
                VMW_SUCCESS,                    /* Error returned if the call was successful */
                VMW_DVM_ENOMEMORY,       	/* Error returned by getbufferpointer for not having enough memory */		
         	VMW_DVM_EPUSH,		   	/* Error returned by push function in startmpegdecode  */
		VMW_DVM_EMPEGOPEN,		/* Error returned by MpegDriverEntry */
		VMW_DVM_EOPEN, 		       	/* Error returned by mpeg open */
		VMW_DVM_EPLAY,			/* Error returned by play */
		VMW_DVM_EPAUSE,			/* Error returned by pause */
		VMW_DVM_ESTOP,			/* Error returned by stop */
		VMW_DVM_ECLOSE,			/* Error returned by close */
 		VMW_DVM_EMPEGUNLOAD		/* Error returned by mpegunload */

             } ErrCode;
               



typedef unsigned int U32;

typedef struct 
{
     void* pBuffer;
     U32   bufferSize; 
     U32   dataSize;
     U32   flags;
     U32   reserved[8];
}DECODE_BUFFER_INFO; 


/***************************IMPORTANT*******************************

ALL THE FUNCTIONS RETURNS ZERO ON SUCCESS AND NON-ZERO ON FAILURE

********************************************************************/



U32 VMW_DVM_GetBufferPointer(DECODE_BUFFER_INFO *);	/* Gets the pointer to the buffer where the decoded */
							/* data is stored */

U32 VMW_DVM_Pause(U32);					/* Parameter can have only 2 values either 1 or 0 */
							/* which tells whether to pause or resume */

U32 VMW_DVM_Play();					/* Puts the driver in the play mode */

U32 VMW_DVM_StartMPEGDecode(U32*);			/* Initiates decoding the buffer(where decoded data to be placed)*/    
							/* whose pointer is passed as a parameter */

U32 VMW_DVM_Open(U32 stream_type, U32 size, U32 count, void* pCallback);
							/**************************************************
							        stream_type could be:
                      						MPEG_TRANSPORT  - for transport Stream
                      						MPEG_SYSTEM     - for system / program Stream
                      						MPEG_VIDEO      - for video Stream
                      						MPEG_AUDIO      - for Mpeg Audio Stream
                      						MPEG_DVD	      - for DVD Stream
                      						MPEG_SVCD       - for SVCD/VCD Stream
                  
                  					 size  : specifying the size of the buffer
                  					 count : specifying the no. of buffers to be allocated 
                  					 pCallback : the function pointer to the callback function
                      					 to receive the stream notification
							 Purpose is to open the MPEG decoder driver

							***************************************************/

U32 VMW_DVM_Stop();					/* stops the MPEG decoder driver */

U32 VMW_DVM_Close();					/* closes the MPEG decoder driver */


#ifdef __cplusplus
}
#endif
#endif /* __MPEG_API_H_ */
