/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 


 ********************************************************************************
 * File Name        : nscsigma.cc						*
 * 										*
 * Project Name     : ASTB							*
 *										*
 * Subsystem Name   : Streamer	                                                *
 *                               						*
 * Module Name      : nscsigma							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This module is responsible for communicating with the 	*
 *		      sigma driver  via sigma APIs.				*
 *										*
 ********************************************************************************/


#include <stdlib.h>

#include "nscsigma.h"
#include "clientcom.h"
#include "defines.h"

  		
/************************************************************************
 * Syntax	: sigma()						*
 *									*
 * Description	: This constructor initialises an Instance odf sigma 	*
 *		  class.	 					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

sigma::	sigma()
{	
	GenMode = 0;
	NoOfBuf = NUMOFBUFFERS;
 	BlockSz = BLOCK_SZ;
	MuteStatus = TRUE;
	MPEGDriverEntry(NO_DRIVE);
		
}


/************************************************************************
 * Syntax	: InitSigma()						*
 *									*
 * Description	: This function initialises sigma driver in TV mode.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::InitSigma()
{
	char* tvstd;
	FMPOpen(FMPF_TRANSPORT, BlockSz,NoOfBuf ,NULL,0); 
	//FMPOpen(FMPF_SYSTEM, BlockSz,NoOfBuf ,NULL,0); 
	FMPPlay();
	// FMPSet(FMPI_VIDEOOUT,FMPV_VIDEOOUT_TV);
        tvstd = getenv("TVSTD");
        if(strcmp(tvstd,"pal") == 0)
	{
		printf("TVSTD set to %s\n",tvstd);
 		FMPSet(FMPI_VIDEOOUT,FMPV_VIDEOOUT_PAL);
	}
	else 
		FMPSet(FMPI_VIDEOOUT,FMPV_VIDEOOUT_NTSC);
}

/************************************************************************
 * Syntax	: InitSigmaSwitch( int flg )				*
 *									*
 * Description	: This function initialises sigma driver in TV/video	*
 *		  mode.							*
 *									*
 * Parameters	: flg							*
 *		   - Flag to select video out mode.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: For cygnus platform this function is not useful as the*
 *		  digital video is interfaced  through VIP.		*
 ************************************************************************/

void
sigma::InitSigmaSwitch( int flg )
{
	FMPPlay();
}
	


/************************************************************************
 * Syntax	: PlaySigma( char * buf, long expect, int mode )	*
 *									*
 * Description	: This function plays the buffer in the specified 	*
 *		  mode.		 					*
 *									*
 * Parameters	: *buf							*
 *		    - Buffer to play.					*
 *		  expect						*
 *		    - Size of data.					*
 *		  mode							*
 *		    - Mode Play/Skip/FF/Rev				*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: Ref: trick data/time discontinuty mode  of sigma.	*
 ************************************************************************/

void
sigma::PlaySigma( char * buf, long expect, int mode )
{
       	int BufStatus;
	
	BufStatus = FMPGetBuffer(&FMPBuf,TRUE);
	if ( BufStatus != 0 )
	{  
		printf(" Sigma not able to alloc mem \n"); 
	   	return ;
	}

      	switch ( mode )
	 {
		case SIG_FF_REV:       	//  FF / REV  
		{   
		   if( GenMode != 1 )	
		   {	
			FMPBuf.dwFlags = FMP_TRICKMODE_START;     // Trick mode 
			GenMode =1;
		   }
		}
		break;
			
	   	case SIG_SKIP:	// SKIP
		{
			if(GenMode == 1)
			{
			 	FMPBuf.dwFlags=FMP_TRICKMODE_END | FMP_DATADISCONTINUITY
				            | FMP_TIMEDISCONTINUITY;//   END TRICK MODE
				FMPBuf.dwFlagsEx=0x00;   
				GenMode =3;
			}else
			{
				FMPBuf.dwFlags=FMP_DATADISCONTINUITY | FMP_TIMEDISCONTINUITY;
				            	                   //  Data discontinuty 
				FMPBuf.dwFlagsEx=0x00;   
		        	GenMode =3;
			}
		}
		break;
	default	:
		{	if(GenMode ==1)
			{
				FMPBuf.dwFlags= FMP_TRICKMODE_END;     //   END TRICK MODE
				FMPBuf.dwFlagsEx= FMP_VIDEO_PES | FMP_AUDIO_PES;

				GenMode =2;
			}
			else
			if(GenMode == 2)
			{	
				FMPBuf.dwFlags=0x00;     //  Normal mode 
				GenMode =0;
			}

		}break;
	}	

			
  	FMPBuf.dwDataSize=expect; 		
     	memcpy(FMPBuf.pBuffer,buf,expect); 			
 	FMPPush(&FMPBuf); 			
}

/************************************************************************
 * Syntax	: StopSigma()						*
 *									*
 * Description	: This function stops playing and closes sigma.		*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::StopSigma()
{	
	printf("Entering into the stopSigma \n");
	FMPStop();
 	FMPClose();
	MPEGDriverUnload();
	printf("Coming out of stopsigma \n");
}



/************************************************************************
 * Syntax	: AudioOnSigma() 					*
 *									*
 * Description	: This function switch on  audio.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::AudioOnSigma() 
{ 	
	FMPSet(FMPI_MUTE,0); MuteStatus =TRUE;
}


/************************************************************************
 * Syntax	: AudioOffSigma()					*
 *									*
 * Description	: This function switch off  audio.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::AudioOffSigma() 
{ 	
	FMPSet(FMPI_MUTE,1); MuteStatus =FALSE;
}




/************************************************************************
 * Syntax	: AudioSwitch()						*
 *									*
 * Description	: This function switch on/off  audio.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::AudioSwitch() 
{ 
	if (MuteStatus)  
		AudioOffSigma();
	else	AudioOnSigma();
}




/************************************************************************
 * Syntax	: SetLVolSigma(int vol)					*
 *									*
 * Description	: This function sets left audio volume of sigma.	*
 *									*
 * Parameters	: vol							*
 *		    - Value of the volume to be set.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
void  
sigma::SetLVolSigma(int vol)  
{	
	FMPSet(FMPI_LEFT_VOLUME,vol); 
}



/************************************************************************
 * Syntax	: SetRVolSigma(int vol)					*
 *									*
 * Description	: This function sets right audio volume of sigma. 	*
 *		  class.	 					*
 *									*
 * Parameters	: vol							*
 *		    - Value of the volume to be set.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::SetRVolSigma(int vol)  
{	
	FMPSet(FMPI_RIGHT_VOLUME,vol); 
}



/************************************************************************
 * Syntax	: FlushSigma()						*
 *									*
 * Description	: This function Flushes sigma's internal buffer. 	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void
sigma::FlushSigma()
{	
	FMPFlush(TRUE);
}


/************************************************************************
 * Syntax	: ClearSigma()						*
 *									*
 * Description	: None.		 					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: Not used.						*
 ************************************************************************/

void
sigma::ClearSigma()
{;
}


/************************************************************************
 * Syntax	: ClearSigma_new()					*
 *									*
 * Description	: This destructor clears sigam driver. 		  	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


void 
sigma::ClearSigma_new()
{
	FMPStop();  
	
	sleep(1);
	FMPPlay();
}

	
/************************************************************************
 * Syntax	: ~sigma()						*
 *									*
 * Description	: This destructor stops sigam driver. 		  	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

sigma::~sigma()
{
	StopSigma();
}
	




