 /*
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    NSC Digital Video
 *
 *
 * Copyright (c) 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is licensee's responsibility to comply with
 * any export regulations applicable in licensee's jurisdiction. Under
 * CURRENT 2000 U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 */
#include <stdio.h>
#include<stdio.h>
#include<linux/fb.h>
#include<stdlib.h>
#include<fcntl.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<sys/mman.h>
#include<unistd.h>
#include<linux/ioctl.h>

#include "vmwdvdapi.h"


/******************************************************************************
 Syntax       : U32 VMW_DVD_Callback( DWORD dwContext. DWORD dwMsg , DWORD dwValue)

 Description  : This function captures the driver events and updates the dvd_info
                structure accordingly.
		

 Param1       : dwContext   The driver Context ( This function is called by the 
                            driver)

 Param2	      : dwMsg       The driver event.
 
 Param3	      : dwValue     This is the data that comes with the event.  	
                  

 Returns      : Returns Zero.


*******************************************************************************/

DWORD VMW_DVD_Callback( DWORD dwContext, DWORD dwMsg , DWORD dwValue)
{
	#ifdef DEBUG
		printf("Entering VMW_DVD_Callback");
	#endif

	switch(dwMsg)
        {
		case FMPM_DVD_TITLE_CHANGE :
                	dvd_info.title_no = dwValue;
	#ifdef DEBUG		
                        printf( "Changing Title %u \n",dvd_info.title_no);
	#endif		
                     	break;

		case FMPM_DVD_CURRENT_TIME:
                     	dvd_info.time=dwValue;
	#ifdef DEBUG		
                        printf( "Changing Time %x \n",dvd_info.time);
	#endif		
		    	break;

		case FMPM_DVD_PLAYBACK_STOPPED:
			dvd_info.dvd_state= STOP;
	#ifdef DEBUG		
                        printf( "Changing State %d \n",dvd_info.dvd_state);
	#endif		
			break;

		case FMPM_DVD_PROGRAM_START:
			dvd_info.chapter_no =dwValue;
	#ifdef DEBUG		
                        printf( "Changing Chapter %u \n",dvd_info.chapter_no);
	#endif		
			break;
                 
                case FMPM_DVD_ANGLES_BLOCK:
		case FMPM_DVD_AUDIO_STREAM_CHANGE:
		case FMPM_DVD_BUTTONS_CHANGE:
		case FMPM_DVD_BUTTON_CHANGE:
		case FMPM_DVD_DOMAIN_CHANGE:
		case FMPM_DVD_NO_FP_PGC:
		case FMPM_DVD_PARENTAL_LEVEL_CHANGE:
		case FMPM_DVD_STILL_OFF:
		case FMPM_DVD_STILL_ON:
    		case FMPM_DVD_SUBPICTURE_STREAM_CHANGE: 
		case FMPM_DVD_VALID_UOPS_CHANGE: 
		default : break;
	}
	
	#ifdef	DEBUG
		printf("Coming out of VMW_DVD_Callback()");
	#endif
	return 0;
} 

/*
******************************************************************************
 Syntax       : U32 VMW_DVD_TitlePlay(U32 param)

 Description  : The function  selects the specified title in the DVD disc and
		plays it.

 Param        : Title No; (1..99)

 Returns      : Returns Zero on success and  play error on Failure.

 comments     : Some of the DVD disc will not have multiple titles.

*******************************************************************************
*/

U32 VMW_DVD_TitlePlay(U32 title)
{
	#ifdef DEBUG	
		printf("Entering into VMW_DVD_TitlePlay");
	#endif
        U32 maxTitle;
        maxTitle = FMPGet(FMPI_TITLES_AVAILABLE);
        if(maxTitle== 0xffffffff)
              return VMW_DVDE_TITLENUMBER;
        if (title >= maxTitle) title = maxTitle;
        if (!TitlePlay(title))
        {
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_TitlePlay");
		#endif
	        return VMW_OK;
	}
        else
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_TitlePlay");
		#endif
                return VMW_DVDE_TITLENUMBER;
	}

}

/*
******************************************************************************
 Syntax       : U32 VMW_DVD_ChapterPlay(U32 param1,U32 Param2)

 Description  : The function selects the chapter from the selected  title in
	                                             the DVD disc and plays it..
 Param1        :   Title No; (1..99)

 Param2        :   Chapter No; (1..999)

 Returns       :   Returns Zero on success and  play error on Failure.

 comments      :  Some of the DVD disc will not have multiple titles but under
                  a tltle there may be a diffrent chapters.

*******************************************************************************
*/

U32 VMW_DVD_ChapterPlay(U32 title, U32 chapter)
{
        #ifdef DEBUG	
		printf("Entering into VMW_DVD_ChapterPlay");
	#endif

	U32 maxTitle;
        maxTitle = FMPGet(FMPI_TITLES_AVAILABLE);
        if(maxTitle== 0xffffffff)
              return VMW_DVDE_CHAPTER;
        if (title >= maxTitle) title=maxTitle;
        if (!ChapterPlay(title, chapter))
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_ChapterPlay");
		#endif
                return VMW_OK;
	}
        else
	{
        	#ifdef DEBUG	
			printf("Coming out of VMW_DVD_ChapterPlay");
		#endif
		return VMW_DVDE_CHAPTER;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_PauseOn(void)

 Description  	: The function pause the DVD playback
..
 Param       	: void

 Returns          : Returns Zero on success and  play error on Failure.

 comments     	: This will stop the mpeg2decoder and also stops reading
											 straems.
*******************************************************************************
*/

U32 VMW_DVD_PauseOn(void)
{
	#ifdef DEBUG	
		printf("Entering into VMW_DVD_PauseOn");
	#endif
        if (!FMPPause())
        {
        	#ifdef DEBUG	
			printf("Going out of VMW_DVD_PauseOn");
		#endif
	        dvd_info.dvd_state = PAUSE;
                return VMW_OK;
        } 
        else
	{
		#ifdef DEBUG	
			printf("Going out of VMW_DVD_PauseOn");
		#endif
                return VMW_DVDE_PAUSE;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_PauseOff(void)

 Description  	: The function starts the DVD playback after pause.
..
 Param       	: void

 Returns          : Returns Zero on success and  pause error on failure.

 comments     	: This will resume the mpeg2decoder and also startss reading
	                      streams fromwhere it paused.
*******************************************************************************
*/

U32 VMW_DVD_PauseOff(void)
{
 	#ifdef DEBUG	
		printf("Entering into VMW_DVD_PauseOff");
	#endif
        if(!FMPPlay())
        {
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_PauseOff");
		#endif
                dvd_info.dvd_state= PLAY;
                return VMW_OK;
        }
        else
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_PauseOff");
		#endif
                return VMW_DVDE_PAUSE;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Stop()

 Description  	:  Function stops the playback
                                                                        
 Param       	:  None.

 Returns        : Returns Zero on success and stop error on Failure.

 comments     	: None.
	
*******************************************************************************
*/

U32 VMW_DVD_Stop(void)
{
	#ifdef DEBUG	
		printf("Entering into VMW_DVD_Stop");
	#endif
	U32 i = FMPStop();    
        if (i == 0)
        {
                dvd_info.dvd_state = STOP;
		printf("In dvd stop %d\n",dvd_info.dvd_state);
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_Stop");
		#endif
                return VMW_OK;
        }
        else
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_Stop");
		#endif
                return VMW_DVDE_STOP;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ForwardScan(float param, int fast_slow)

 Description  	:  Function sets scan speed to navigate through the program
                                                                        chain.
 Param       	: float value to set the forward scan speed, and fast_slow is
		  to change the motion to fast or slow. If it is TRUE fast else			  slow.

 Returns        : Returns Zero on success and scan error on Failure.

 comments     	: This will sets by  user at diffrent speeds for navigation.
	
*******************************************************************************
*/

U32 VMW_DVD_ForwardScan(float speed, int fast_slow)
{
	#ifdef DEBUG	
		printf("Entering into VMW_DVD_ForwardScan");
	#endif

        if (!ForwardScan((int)speed, fast_slow))
        {

		if(fast_slow == TRUE ) dvd_info.dvd_state= FF; 
		else	dvd_info.dvd_state = SLOW;
			
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_ForwardScan");
		#endif
                return VMW_OK;
        }
        else	
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_ForwardScan");
		#endif
                return VMW_DVDE_SCAN;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_BackwardScan(float param, int fast_slow)

 Description  	:  Function sets scan speed to navigate through the program
                                                  chain in forward direction.

 Param       	: float value to set the backward scan speed, if fast_slow is
		  true plays fast else slow motion.

 Returns        : Returns Zero on success and scan error on Failure.

 comments     	: This will sets by  user at diffrent speeds for navigation.
	
*******************************************************************************
*/

U32 VMW_DVD_BackwardScan(float speed, int fast_slow)
{
	#ifdef DEBUG	
		printf("Entering into of VMW_DVD_BackwardScan");
	#endif
	
        if (!BackwardScan((int)speed, fast_slow))
        {
                dvd_info.dvd_state= REWIND; 
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_BackwardScan");
		#endif
                return VMW_OK;
        }
        else
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_BackwardScan");
		#endif
                return VMW_DVDE_SCAN;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_AudioStreamSelect(U32 param)

 Description  	:Fuction selects audio stream from maximum availlable streams


 Param       	: value specifies the stream id.

 Returns        : Returns Zero on success and stream error on failure

 comments     	: none
	
*******************************************************************************
*/
U32 VMW_DVD_AudioStreamSelect(U32 streamId)
{
        #ifdef DEBUG	
		printf("Entering into of VMW_DVD_AudioStreamSelect");
	#endif

	U32 max_audstrm;
        max_audstrm = FMPGet(FMPI_AUDIO_STREAMS_AVAILABLE);
        if(max_audstrm == 0xffffffff)
                return VMW_DVDE_STREAM;

        if ( streamId  >=  max_audstrm) streamId=max_audstrm;
        if (!AudioStreamChange(streamId))
	{
		#ifdef DEBUG	
			printf("Coming out of VMW_DVD_AudioStreamSelect");
		#endif
                return VMW_OK;
	}
        else
	{
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_AudioStreamSelect");
                #endif
                return VMW_DVDE_STREAM;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_AngleSelect(U32 param)

 Description  	:Fuction sets the difrent angle


 Param       	: value specifies the angle id.

 Returns        : Returns Zero on success and stream error on failure

 comments     	: maximum angle supported at present is 0-7.
	
*******************************************************************************
*/

U32 VMW_DVD_AngleSelect(U32 angle)
{
        U32 max_angle;
	#ifdef DEBUG
       		 printf("Coming out of VMW_DVD_AngleSelect");
        #endif
        max_angle = FMPGet(FMPI_ANGLES_AVAILABLE);
	if( max_angle== 0xffffffff)
                return VMW_DVDE_ANGLE;
	       
        if (angle  >= max_angle) angle=max_angle;
        if (!AngleChange(angle))
	{
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_AngleSelect");
                #endif
                return VMW_OK;
	}
        else
	{
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_AngleSelect");
                #endif
                return VMW_DVDE_ANGLE;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_SubPictureStreamSelect(U32 param1,U32 Param2)

 Description  	:Fuction selects the subpicturestream(subtitle) of availlable
                    and enable or disable it based on setdisplay.


 Param1      	: value specifies the subtiltle id [range 0-31]

 Param2       	: Enabels or disables the subtitle[true/false].


 Returns        : Returns Zero on success and DVDE_Stream error on failure.

 comments     	: none
	
*******************************************************************************
*/
U32 VMW_DVD_SubPictureStreamSelect(U32 subPicStream,U32 setDisplay)
{
        U32 max_subpic;
	#ifdef DEBUG
       		printf("Entering into VMW_DVD_SubPictureStreamSelect");
        #endif

        max_subpic = FMPGet(FMPI_SUBPICTURE_STREAM_AVAILABLE);
	if( max_subpic== 0xffffffff)
                return VMW_DVDE_SUBSTREAM;
  
        if (subPicStream >= max_subpic) subPicStream=max_subpic;
        if (!SubPictureStreamChange (subPicStream,setDisplay))
	{
       		#ifdef DEBUG
       			printf("Coming out of VMW_DVD_SubPictureStreamSelect");
	        #endif
	        return VMW_OK;
	}
        else
	{
		#ifdef DEBUG
       			printf("Coming out of VMW_DVD_SubPictureStreamSelect");
	        #endif
                return VMW_DVDE_SUBSTREAM;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Karaoke(U32 param)

 Description  	:Fuction sets the dvd playback into karaoke mode

 Param      	: value specifies the mixing mode of diffrent tracks of sound

 Returns        : Returns Zero on success and DVDE_Stream error on failure.

 comments     	: none
	
*******************************************************************************
*/

U32 VMW_DVD_Karaoke(U32 mixingMode)
{

	   #ifdef DEBUG
                  printf("Entering into VMW_DVD_Karoake");
           #endif
 
     if (!KaraokeAudioPresentationMode (mixingMode))
	{
		 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_Karaoke");
                #endif
                return VMW_OK;
	}
        else
	{
		 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_Karaoke");
                #endif
                return VMW_DVDE_KARAOKE;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_SetVideoPresentationMode(U32 param1,U32
                                                                  param2);

 Description  	:Fuction sets the dvd playback presentation screen size and
                   various patterns like pan-scan and letter_box etc.

 Param1      	: specifies the size if video presentation

 Param2      	: sets diffrent style of window like letter box etc.

 Returns        : Returns Zero on success and DVDE_Stream error on failure.

 comments     	: none
	
*******************************************************************************
*/

U32 VMW_DVD_SetVideoPresentationMode(U32 aspectRatio,U32 displayMode)
{
       	 #ifdef DEBUG
         	printf("Coming out of VMW_DVD_SetVideoPresentationMode");
	#endif
 
	if (!VideoPresentationModeChange(aspectRatio,displayMode))
	{
		 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_SetVideoPresentationMode");
                #endif
                return VMW_OK;
	}
        else
	{
        	#ifdef DEBUG
                        printf("Coming out of VMW_DVD_SetVideoPresentationMode");
                #endif
		return  VMW_DVDE_STREAM;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ActivateButton(U32 param)

 Description  	:Fuction activates the menu buttons sppecified


 Param       	: value specifies the button Id.

 Returns          : Returns Zero on success and DVDE_Search error on failure

 comments     	: This will be useful when the select button returns the dvd
                    button id.
	
*******************************************************************************
*/

U32 VMW_DVD_ActivateButton(U32 param)
{
	 #ifdef DEBUG
		printf("Coming out of VMW_DVD_ActivateButton");
         #endif

        if (!ButtonSelectAndActivate(param))
	{
        	 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_ActivateButton");
                #endif
	        return VMW_OK;
	}
        else
	{
	       	 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_ActivateButton");
                #endif
	       return VMW_DVDE_MENU;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ActivateMenu(U32 param)

 Description  	: Activtes the menu on the screen based on menuId.


 Param       	: value specifies the menuid.

 Returns        : Returns Zero on success and menu error on failure.

 comments     	: MenuID's are supported are root,subpicture,title,chapter
                    and audio etc.
	
*******************************************************************************
*/

U32 VMW_DVD_ActivateMenu(DVDMenu_t menuId)
{
        #ifdef DEBUG
                 printf("Entering into VMW_DVD_ActivateMenu");
        #endif
	 if (!MenuCall(menuId))
	 {
        	 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_ActivateMenu");
                #endif
	       return VMW_OK;
	}
         else
	{
		 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_ActivateMenu");
                #endif
                return VMW_DVDE_MENU;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Search(U32 searchMode, U32 *pSearchParam);

 Description  	:  

 Param          : Selects the button left,right.up,down relative to current
                    button

 Returns          : Returns Zero on success and DVDE_Menu error on failure.

 comments     	: none
******************************************************************************
*/

U32 VMW_DVD_Search(U32 searchMode,U32 *pSearchParam)
{
        #ifdef DEBUG
	        printf("Enter into VMW_DVD_Search");
        #endif
 
	switch(searchMode)
        {
                case DVD_CHAPTER_SRCH:
                        if (!ChapterSearch(*pSearchParam))
                                return VMW_OK;
                        else
                                break;
                case DVD_TIME_SRCH:
                        if (!TimeSearch((TCHAR*)pSearchParam))
                                return VMW_OK;
                        else
                                break;
                case DVD_GOUP_SRCH:
                        if (!GoUp())
                                return VMW_OK;
                        else
                                break;
                case DVD_TOP_SRCH:
                         if (!TopPGSearch())
                                return VMW_OK;
                         else
                                break;
                case DVD_NEXT_SRCH:
                         if (!NextPGSearch())
                                return VMW_OK;
                        else
                                break;
                case DVD_PREV_SRCH:
                        if (!PrevPGSearch())
                                return VMW_OK;
                        else
                                break;

                default:
                                break;

        }
	 #ifdef DEBUG
         	printf("Coming out of VMW_DVD_Search");
         #endif
        return  VMW_DVDE_INSEARCH;
}
/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Play(void);

 Description  	: This fuction puts the driver in playmode. 

 Returns        : None.

 comments     	: None.
	
******************************************************************************
*/
U32 VMW_DVD_Play()
{
	#ifdef DEBUG
		printf("Entering into of VMW_DVD_Play");
        #endif

	if(!FMPPlay())
	{
		dvd_info.dvd_state = PLAY;
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_Play");
                #endif
		return VMW_OK;
	}
	else
	{
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_Play");
                #endif
		return VMW_DVDE_PLAY;
	}
}
/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Open(void);

 Description  	: This fuction invokes the mpeg2decoder driver and sets dvd-rom
                    ready to play.

 Returns        : Returns Zero on success and DVDE_Open error on failure.

 comments     	: some of the paremeters in this functions are set to EM8400
                    Mpeg2decoder to paly
	
******************************************************************************
*/

U32 VMW_DVD_Open(void)
{
	#ifdef DEBUG
        	printf("Entering into VMW_DVD_Open");
        #endif
	int fbfd;

	struct fb_var_screeninfo vinfo;

        DWORD dwError;
        int driveNo = 5;
        int dataBuffSize = 32*1024;
        int noBuffer = 8;

	FMP_OPENSTRUCT DVDOpenStruct;

	/* Open the file for reading and writing */
        fbfd = open("/dev/fb0", O_RDWR);
        if (!fbfd)
        {
                printf("Error: cannot open framebuffer device.\n");
                exit(1);
        }
        printf("The framebuffer device  opened successfully. %d \n",vinfo.yres);

        /* Get variable screen information */
        if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo))
        {
                printf("Error reading variable information.\n");
                exit(3);
        }
        close(fbfd); /* Closing the frame buffer device */	
	

        MPEGDriverEntry(driveNo);

        DVDOpenStruct.dwStructSize  = sizeof(FMP_OPENSTRUCT);
        DVDOpenStruct.bDriverNumber = driveNo;
        DVDOpenStruct.dwSize        = dataBuffSize;
        DVDOpenStruct.dwCount       = noBuffer;
        DVDOpenStruct.pCallback     = VMW_DVD_Callback;  
        DVDOpenStruct.dwContext     = 0;
        DVDOpenStruct.sFileName     = 0;
        DVDOpenStruct.dwFlags       = FMPF_DVD;
        DVDOpenStruct.bDiscContent  = FMPC_DVD_VIDEO;
        DVDOpenStruct.bFileSystem   = 0x00;

	if(vinfo.yres == 576)
	{
		printf("It is in PAL Mode\n");
		FMPSet(FMPI_VIDEOOUT,FMPV_VIDEOOUT_PAL);
	}
	
        dwError = FMPOpenDiscPlayback (&DVDOpenStruct);

        if (dwError != FMPE_OK || DVDOpenStruct.bDiscContent != FMPC_DVD_VIDEO)
        { 
		#ifdef DEBUG
			printf("Not a valid disc\n");
	        #endif 
	        return VMW_DVDE_OPEN;
        }
        else
	{
                dvd_info.dvd_state= OPEN;
		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_Open");
                #endif
       		return  VMW_OK ;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_Close(void);

 Description  	: This fuction unloads the mpeg2decoder driver


 Returns        : Returns Zero on success and DVDE_Close error on failure.

 comments     	: none
******************************************************************************
*/

U32 VMW_DVD_Close(void)
{
         #ifdef DEBUG
                printf("Entering into VMW_DVD_Close");
         #endif

	if ( (!FMPStop()) && (!FMPClose()) && (!MPEGDriverUnload()))
        {
                dvd_info.dvd_state= CLOSE;
		 #ifdef DEBUG
                        printf("Coming out of VMW_DVD_Close");
                #endif
                return VMW_OK;
        }
        else
	{
       		#ifdef DEBUG
                        printf("Coming out of VMW_DVD_Close");
                #endif
	        return VMW_DVDE_CLOSE;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_LanguageSelect(char *param);

 Description  	: This function sets the specified lang for menu

 param            : specifies the laguage of the menu

 Returns          : Returns Zero on success and DVDE_Stream error on failure.

 comments     	: none
******************************************************************************
*/

U32 VMW_DVD_LanguageSelect(char *pLanguageCode)
{
         #ifdef DEBUG
                printf("Entering into VMW_DVD_LanguageSelect");
         #endif

	if(!MenuLanguageSelect(*pLanguageCode))
	{
		#ifdef DEBUG
                	printf("Coming out of VMW_DVD_LanguageSelect");
        	#endif
                return VMW_OK;
	}
        else
	{
		#ifdef DEBUG
                	printf("Coming out of VMW_DVD_LanguageSelect");
	        #endif
                 return VMW_DVDE_STREAM;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ButtonSelect(U32 param);

 Description  	: This function selects the button relative current button

 Param            : Selects the button left,right.up,down relative to current
                    button

 Returns          : Returns Zero on success and DVDE_Menu error on failure.

 comments     	: none
******************************************************************************
*/

U32 VMW_DVD_SetParentalControl(char *pCountry,U32 level)
{
       #ifdef DEBUG
      		printf("Entering into VMW_DVD_SetParentalControl");
       #endif

	if (!ParentalCountrySelect(*pCountry) &&((!ParentalLevelSelect(level))))
       {
		#ifdef DEBUG
                	printf("Coming out of VMW_DVD_SetParentalControl");
        	#endif
	         return VMW_OK;	
	}
        else
	{
		#ifdef DEBUG
                	printf("Coming out of VMW_DVD_SetParentalControl");
         	#endif
                return  VMW_DVDE_STREAM;
	}
}


/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ButtonSelect(U32 param);

 Description  	: This function selects the button relative current button

 Param            : Selects the button left,right.up,down relative to current
                    button

 Returns          : Returns Zero on success and DVDE_Menu error on failure.

 comments     	: none
******************************************************************************
*/


U32 VMW_DVD_ButtonSelect(U32 button)
{
	#ifdef DEBUG
                printf("Entering into VMW_DVD_ButtonSelect");
        #endif

        switch (button)
        {
                case DVD_UPPERBUTTON:
                        if (!UpperButtonSelect())
                                return VMW_OK;
                        else
                                break;
                case DVD_LOWERBUTTON:
                        if (!LowerButtonSelect())
                                return VMW_OK;
                        else
                                break;
                case DVD_LEFTBUTTON:
                       if (!LeftButtonSelect())
                                return VMW_OK;
                        else
                                break;
                case DVD_RIGHTBUTTON:
                        if (!RightButtonSelect())
                                return VMW_OK;
                         else
                                break;
               default:
                                break;
        }
	#ifdef DEBUG
                printf("Coming out of VMW_DVD_ButtonSelect");
        #endif

        return  VMW_DVDE_BUTTON;
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_ButtonActivate(void);

 Description  	: This functionwill be used with the button select

 Returns          : Returns Zero on success and DVDE_Menu error on failure.

 comments     	:Selcted button is the one highlighted button by button
                                                                    select.

******************************************************************************
*/

U32 VMW_DVD_ButtonActivate(void)
{
	#ifdef DEBUG
                printf("Entering into VMW_DVD_ButtonActivate");
        #endif
	
	if (!ButtonActivate())
	{
		#ifdef DEBUG
			printf("Coming out of VMW_DVD_ButtonActivate");
		#endif	
                return VMW_OK;
	}
        else
	{
		#ifdef DEBUG
                	printf("Coming out of VMW_DVD_ButtonActivate");
	        #endif
                return VMW_DVDE_MENU;
	}
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_GetCurrentChapter();

 Description  	: None.

 Returns        : Returns the current chapter number being played.

 comments     	: None.

******************************************************************************
*/
U32 VMW_DVD_GetCurrentChapter()
{
        #ifdef DEBUG
		printf("Entering into VMW_DVD_GetCurrentChapter");
	#endif 
	return dvd_info.chapter_no;
}

/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_GetCurrentTitle();

 Description  	: None.

 Returns        : Returns the current title number being played.

 comments     	: None.

******************************************************************************
*/
U32 VMW_DVD_GetCurrentTitle()
{
        #ifdef DEBUG
		printf("Entering into VMW_DVD_GetCurrentTitle");
	#endif 
	return dvd_info.title_no;
}
/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_GetCurrentPlayState();

 Description  	: None.

 Returns        : Returns the status of the play like play/pause/stop/pause/ff/rewind etc.

 comments     	: None.

******************************************************************************
*/
U32 VMW_DVD_GetCurrentPlayState()
{
        #ifdef DEBUG
		printf("Entering into VMW_DVD_GetCurrentPlayState");
	#endif 
	return dvd_info.dvd_state;
}
/*
******************************************************************************
 Syntax       	: U32 VMW_DVD_GetCurrentTime();

 Description  	: None.

 Returns        : Returns the current time in bcd format. 

 comments     	: None.

******************************************************************************
*/
U32 VMW_DVD_GetCurrentTime()
{
	#ifdef DEBUG
		printf("Entering into VMW_DVD_GetCurrentTime");
	#endif
	return dvd_info.time;
}

