/*
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    NSC Digital video
 *
 *
 * Copyright (c) 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is licensee's responsibility to comply with
 * any export regulations applicable in licensee's jurisdiction. Under
 * CURRENT 2000 U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 */
#ifndef _VMWDVDAPI_H_
#define _VMWDVDAPI_H_

#define _DEFINE_FMP_TYPES_

#include <fmp.h>

#ifndef U32
        typedef unsigned long U32;
#endif

#ifdef _cplusplus
        extern "C" {
#endif

typedef enum{ DVD_CHAPTER_SRCH = 1001, DVD_TIME_SRCH, DVD_GOUP_SRCH,
                         DVD_TOP_SRCH, DVD_NEXT_SRCH, DVD_PREV_SRCH} DVD_SRCH;

typedef enum {VMW_OK=0,VMW_DVDE_STREAM = 10, VMW_DVDE_PLAY,VMW_DVDE_STOP,VMW_DVDE_PAUSE,
              VMW_DVDE_SCAN,VMW_DVDE_MENU,VMW_DVDE_INSEARCH,VMW_DVDE_CLOSE, 			
              VMW_DVDE_OPEN,VMW_DVDE_TITLENUMBER,VMW_DVDE_CHAPTER, VMW_DVDE_ANGLE, VMW_DVDE_BUTTON, VMW_DVDE_SUBSTREAM, VMW_DVDE_KARAOKE} DVD_ERR_t;

typedef enum { DVD_LOWERBUTTON = 100, DVD_UPPERBUTTON,
                     DVD_LEFTBUTTON,DVD_RIGHTBUTTON } DVD_BUTTONS_t;

typedef enum { UNDEFINED = 200,OPEN, PLAY,FF,SLOW,REWIND,PAUSE,STOP,
               CLOSE } DVD_State_t; 

typedef enum { DVD_MENU_ID_TITLE=2, DVD_MENU_ID_ROOT, DVD_MENU_ID_SUBPICTURE,
               DVD_MENU_ID_AUDIO, DVD_MENU_ID_ANGLE, DVD_MENU_ID_RESUME  
             } DVDMenu_t;


/* Dvd status update structure info */
typedef struct 
{
	U32 chapter_no;
	U32 title_no;
	U32 time;
	DVD_State_t dvd_state;
} DVD_Info_t; 

/* The variable is static to make the updations continuous */
static DVD_Info_t dvd_info;


	U32 VMW_DVD_TitlePlay(U32 title);

	U32 VMW_DVD_Play();

 	U32 VMW_DVD_ChapterPlay(U32 title,U32 chapter);
  	
  	U32 VMW_DVD_PauseOn(void);

   	U32 VMW_DVD_PauseOff(void);

  	U32 VMW_DVD_Stop(void);

   	U32 VMW_DVD_ForwardScan(float speed,int fast_slow);

    	U32 VMW_DVD_BackwardScan(float speed,int fast_slow);

    	U32  VMW_DVD_ActivateMenu(DVDMenu_t menuId);

     	U32 VMW_DVD_AudioStreamSelect(U32 streamId);

        U32 VMW_DVD_AngleSelect(U32 angle);

        U32  VMW_DVD_ActivateButton(U32 param);

        U32 VMW_DVD_SubPictureStreamSelect(U32 subPicStream,U32 setDisplay);

        U32 VMW_DVD_Karaoke(U32 mixingMode);

        U32 VMW_DVD_SetVideoPresentationMode(U32 aspectRatio,U32 displayMode);

        U32 VMW_DVD_Search(U32 searchMode,U32 *pSearchParam);

        U32 VMW_DVD_Open(void);

        U32 VMW_DVD_Close(void);

        U32 VMW_DVD_LanguageSelect(char *pLanguageCode);

        U32 VMW_DVD_SetParentalControl(char *pCountry,U32 level);

        U32 VMW_DVD_ButtonSelect(U32 param);

        U32 VMW_DVD_ButtonActivate(void);
	
	U32 VMW_DVD_GetCurrentChapter();

	U32 VMW_DVD_GetCurrentTitle();

	U32 VMW_DVD_GetCurrentPlayState();

	U32 VMW_DVD_GetCurrentTime();

/* end of function prototypes */


#ifdef _cplusplus
}
#endif /* Extern endif */

#endif /* vmwdvdapi.h */
