/*
 * $Workfile: galproto.h $
 * $Revision: 30 $
 *
 * File Contents: This file contains the main functions of the Geode 
 *                frame buffer device drivers GAL function prototypes and  
 *                data structures.
 *
 * Project:       Geode Frame buffer device driver
 *
 */

/* 
 * NSC_LIC_ALTERNATIVE_PREAMBLE
 *
 * Revision 1.0
 *
 * National Semiconductor Alternative GPL-BSD License
 *
 * National Semiconductor Corporation licenses this software 
 * ("Software"):
 *
 * Geode frame buffer driver
 *
 * under one of the two following licenses, depending on how the 
 * Software is received by the Licensee.
 * 
 * If this Software is received as part of the Linux Framebuffer or
 * other GPL licensed software, then the GPL license designated 
 * NSC_LIC_GPL applies to this Software; in all other circumstances 
 * then the BSD-style license designated NSC_LIC_BSD shall apply.
 *
 * END_NSC_LIC_ALTERNATIVE_PREAMBLE */

/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * Geode frame buffer driver
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * END_NSC_LIC_BSD */

/* NSC_LIC_GPL
 *
 * National Semiconductor Corporation Gnu General Public License for 
 *
 * Geode frame buffer driver
 *
 * (GPL License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted under the terms of the GNU General 
 * Public License as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version  
 *
 * In addition to the terms of the GNU General Public License, neither 
 * the name of the National Semiconductor Corporation nor the names of 
 * its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE, 
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. See the GNU General Public License for more details. 
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this file; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * END_NSC_LIC_GPL */

 
#ifndef __GALPROTO_SEP_20_2000
#define __GALPROTO_SEP_20_2000

/* durango reg definitions and type's */
#include <gfx_type.h>
#include <gfx_regs.h>

/* Panel related definition */
#include <pnl_defs.h>

typedef unsigned int    DWORD;
typedef unsigned short  WORD;
typedef unsigned char   CHAR;
typedef unsigned char   BOOLEAN;
typedef unsigned int    *PDWORD;

/***************************************/
/* Applications/User mode drivers use this ioctl to
 * send a graphics device request to the frame buffer
 * driver
 */
#define FBIOGAL_API      0x4700

/*
 * Applications must sign the I/O packet with this value
 */

#define FBGAL_SIGNATURE 0xC0C0BABE

/*
 * Version is a 16:16 fixed value
 * Current version is 1.0000
 */

#define FBGAL_VERSION 0x10000

/*
 * Definitions for Graphics Subfunctions
 *
 */

/* General Adapter level functions */

#define GALFN_GETADAPTERINFO		0
#define GALFN_SETSOFTVGASTATE           1
#define GALFN_GETSOFTVGASTATE           2
#define GALFN_WAITUNTILIDLE             3
#define GALFN_WAITVERTICALBLANK         4
#define GALFN_SETCRTSTATE               5
#define GALFN_WRITEREG                  6
#define GALFN_READREG                   7

/* Change/Get Display hardware state */

#define GALFN_ISDISPLAYMODESUPPORTED    10
#define GALFN_SETDISPLAYMODE            11
#define GALFN_GETDISPLAYMODE            12
#define GALFN_SETDISPLAYBPP             13
#define GALFN_GETDISPLAYBPP             14
#define GALFN_SETDISPLAYPITCH           15
#define GALFN_GETDISPLAYPITCH           16
#define GALFN_SETDISPLAYOFFSET          17
#define GALFN_GETDISPLAYOFFSET          18
#define GALFN_DOTCLKTOREFRESH           19
#define GALFN_GETDISPLAYTIMINGS         20
#define GALFN_SETDISPLAYTIMINGS         21
#define GALFN_SETPALETTE                22
#define GALFN_GETPALETTE                23
#define GALFN_SETPALETTE_ENTRY          24
#define GALFN_GETPALETTE_ENTRY          25
#define GALFN_SETFIXEDTIMINGS           26

/* Hardware cursor funtions */

#define GALFN_SETCURSORENABLE           30
#define GALFN_GETCURSORENABLE           31
#define GALFN_SETCURSORPOSITION         32
#define GALFN_GETCURSORPOSITION         33
#define GALFN_SETCURSORSHAPE            34
#define GALFN_SETCURSORCOLORS           35
#define GALFN_GETCURSORCOLORS           36

/* grafix rendering funtions */
#define GALFN_SETSOLIDPATTERN           40
#define GALFN_SETRASTEROPERATION        41
#define GALFN_SETSOLIDSOURCE            42
#define GALFN_PATTERNFILL               43
#define GALFN_SETMONOPATTERN            44
#define GALFN_SCREENTOSCREENBLT         45
#define GALFN_SCREENTOSCREENXBLT        46
#define GALFN_BRESENHAMLINE             47

/* VGA Support functions */

#define GALFN_VGAMODESWITCH             50
#define GALFN_VGACLEARCRTEXT            51
#define GALFN_VGASETPITCH               52
#define GALFN_VGARESTORE                53
#define GALFN_VGASAVE                   54
#define GALFN_VGASETMODE                55

/* Compression functions */
#define GALFN_SETCOMPRESSIONSTATE       56
#define GALFN_GETCOMPRESSIONSTATE       57
#define GALFN_SETCOMPRESSIONPARAMS      58
#define GALFN_GETCOMPRESSIONPARAMS      59

/* Panel Support functions */

#define GALFN_PNLSETPARAMS              60
#define GALFN_PNLGETPARAMS              61
#define GALFN_PNLINITPANEL              62
#define GALFN_PNLSAVESTATE              63
#define GALFN_PNLRESTORESTATE           64
#define GALFN_PNLPOWERUP                65
#define GALFN_PNLPOWERDOWN              66
#define GALFN_PNLBIOSENABLE             67
#define GALFN_PNLBIOSINFO               68

/* TV Support functions */

#define GALFN_SETTVPARAMS               70
#define GALFN_GETTVPARAMS               71
#define GALFN_SETTVTIMING               72
#define GALFN_GETTVTIMING               73
#define GALFN_SETENABLE                 74
#define GALFN_GETENABLE                 75
#define GALFN_ISTVMODESUPPORTED         76

/* Video Support functions */

#define GALFN_SETVIDEOENABLE            80
#define GALFN_SETVIDEOFORMAT            81
#define GALFN_SETVIDEOSIZE              82
#define GALFN_SETVIDEOOFFSET            83
#define GALFN_SETVIDEOWINDOW            84
#define GALFN_SETVIDEOSCALE             85
#define GALFN_SETVIDEOFILTER            86
#define GALFN_SETVIDEOCOLORKEY          87
#define GALFN_SETVIDEODOWNSCALEENABLE   88
#define GALFN_SETVIDEODOWNSCALECONFIG   89
#define GALFN_SETVIDEODOWNSCALECOEFF    90
#define GALFN_SETVIDEOSOURCE            91
#define GALFN_SETVIDEOINTERLACED        92
#define GALFN_SETVIDEOCOLORSPACE        93
#define GALFN_SETVIDEOCURSOR            94
#define GALFN_SETVIDEOREQUEST           95
#define GALFN_SETVIDEOALPHAENABLE       96
#define GALFN_SETVIDEOALPHAWINDOW       97
#define GALFN_SETVIDEOALPHAVALUE        98
#define GALFN_SETVIDEOALPHAPRIORITY     99
#define GALFN_SETVIDEOALPHACOLOR        100
#define GALFN_SETVIDEOALPHAREGION       101
#define GALFN_SETVIDEOOUTSIDEALPHA      102
#define GALFN_SETVIDEOPALETTE           103
#define GALFN_GETVIDEOINFO              104  

#define GALFN_LASTFUNCTION_SUPPORTED    105 /* end of GAL function list */


#define GAL_HEADER\
	DWORD  dwSignature;    /* Sign all structs with FBGAL_SIGNATURE    */\
	DWORD  dwSize;         /* Size of struct for that subfunction      */\
	DWORD  dwVersion;      /* Current version of the API               */\
	DWORD  dwSubfunction;  /* GAL subfunction                          */\
	DWORD  dwReturnValue;  /* Return value from subfunction */

/*
 * #define GALFN_PNLPOWERUP
 * #define GALFN_PNLPOWERDOWN
 */
typedef struct __GAL_BASE
{
	GAL_HEADER

} GAL_BASE, *PGAL_BASE;

/*
 * #define GALFN_GETADAPTERINFO
 */
typedef struct __GAL_GETADAPTERINFO
{
	GAL_HEADER
	DWORD  dwCPUVersion;
	DWORD  dwFrameBufferBase;
	DWORD  dwFrameBufferSize;
	DWORD  dwGfxRegisterBase;
	DWORD  dwVidRegisterSize;
	DWORD  dwVipRegisterBase;
	DWORD  dwVideoVersion;
	DWORD  dwMaxSupportedPixelClock;

} GAL_ADAPTERINFO, *PGAL_ADAPTERINFO;

#define GAL_SOFTVGASTATE_ENABLE   1
#define GAL_SOFTVGASTATE_DISABLE  0
/*
 * #define GALFN_SOFTVGASTATE
 */
typedef struct __GAL_SOFTVGASTATE
{
	GAL_HEADER
	BOOLEAN   bSoftVgaEnable;

} GAL_SOFTVGASTATE, *PGAL_SOFTVGASTATE;

/*
 * #define GALFN_WAITUNTILIDLE
 */
typedef struct __GAL_WAITUNTILIDLE
{
	GAL_HEADER

} GAL_WAITUNTILIDLE, *PGAL_WAITUNTILIDLE;

/*
 * #define GALFN_WAITVERTICALBLANK
 */
typedef struct __GAL_WAITVERTICALBLANK
{
	GAL_HEADER

} GAL_WAITVERTICALBLANK, *PGAL_WAITVERTICALBLANK;

#define GAL_REG 0x1
#define GAL_VID 0x2
#define GAL_VIP 0x4
/*
 * #define GALFN_WRITEREG
 * #define GALFN_READREG
 */
typedef struct __GAL_HWACCESS
{
	GAL_HEADER
	DWORD  dwType;
	DWORD  dwOffset;
	DWORD  dwValue;
	DWORD  dwByteCount;

} GAL_HWACCESS, *PGAL_HWACCESS;

/*
 * #define GALFN_ISDISPLAYMODESUPPORTED
 * #define GALFN_SETDISPLAYMODE
 * #define GALFN_GETDISPLAYMODE
 */
typedef struct __GAL_DISPLAYMODE
{
	GAL_HEADER
	WORD   wXres;
	WORD   wYres;
	WORD   wBpp;
	WORD   wRefresh;
	DWORD  dwSupported;

} GAL_DISPLAYMODE, *PGAL_DISPLAYMODE;

/*
 * #define GALFN_SETBPP
 * #define GALFN_GETBPP
 * #define GALFN_SETPITCH                  
 * #define GALFN_GETPITCH                  
 * #define GALFN_SETOFFSET                 
 * #define GALFN_GETOFFSET                 
 */
typedef struct __GAL_DISPLAYPARAMS
{
	GAL_HEADER
	DWORD  dwOffset;
	WORD   wBpp;
	WORD   wPitch;

} GAL_DISPLAYPARAMS, *PGAL_DISPLAYPARAMS;

/*
 * #define GALFN_DOTCLKTOREFRESH
 */
typedef struct __GAL_DOTCLKTOREFRESH
{
	GAL_HEADER
	DWORD  dwDotClock;
	WORD   wXres;
	WORD   wYres;
	WORD   wBpp;
	WORD   wRefreshRate;

} GAL_DOTCLKTOREFRESH, *PGAL_DOTCLKTOREFRESH;
 
/*
 * #define GALFN_GETDISPLAYTIMINGS
 * #define GALFN_SETDISPLAYTIMINGS
 */
typedef struct __GAL_DISPLAYTIMING
{
	GAL_HEADER
	DWORD  dwDotClock;
	WORD   wPitch;
	WORD   wBpp;
	WORD   wHTotal;
	WORD   wHActive;
	WORD   wHSyncStart;
	WORD   wHSyncEnd;
	WORD   wHBlankStart;
	WORD   wHBlankEnd;
	WORD   wVTotal;
	WORD   wVActive;
	WORD   wVSyncStart;
	WORD   wVSyncEnd;
	WORD   wVBlankStart;
	WORD   wVBlankEnd;
	WORD   wPolarity;

} GAL_DISPLAYTIMING, *PGAL_DISPLAYTIMING;

/*
 * #define GALFN_SETPALETTE_ENTRY
 * #define GALFN_GETPALETTE_ENTRY
 */
typedef struct __GAL_PALETTE_ENTRY
{
	GAL_HEADER
	DWORD  dwIndex;
	DWORD  dwPalette;
} GAL_PALETTE_ENTRY, *PGAL_PALETTE_ENTRY;

/*
 * #define GALFN_SETPALETTE
 * #define GALFN_GETPALETTE
 */
typedef struct __GAL_PALETTE
{
	GAL_HEADER
	DWORD dwColors[256];
} GAL_PALETTE, *PGAL_PALETTE;

/*
 * #define GALFN_COMPRESSIONSTATE  
 */
typedef struct __GAL_COMPRESSIONSTATE
{
	GAL_HEADER
	BOOLEAN   bCompressionState;
} GAL_COMPRESSIONSTATE, *PGAL_COMPRESSIONSTATE;

#define GAL_COMPRESSION_ENABLE   1
#define GAL_COMPRESSION_DISABLE  0

#define GAL_COMPRESSION_OFFSET  1
#define GAL_COMPRESSION_PITCH   2
#define GAL_COMPRESSION_SIZE    4
#define GAL_COMPRESSION_ALL     7

/*
 * #define GALFN_COMPRESSIONPARAMS
 */
typedef struct __GAL_COMPRESSIONPARAMS
{
	GAL_HEADER
	DWORD dwFlags;
	DWORD dwCompOffset;
	WORD dwCompPitch;
	WORD dwCompSize;
} GAL_COMPRESSIONPARAMS, *PGAL_COMPRESSIONPARAMS;

#define GAL_SETCURSORENABLE_ENABLE    1
#define GAL_SETCURSORENABLE_DISABLE   0
/*
 * #define GALFN_CURSORENABLE
 */
typedef struct __GAL_CURSORENABLE
{
	GAL_HEADER
	BOOLEAN   bCursorEnable;
} GAL_CURSORENABLE, *PGAL_CURSORENABLE;

/*
 * #define GALFN_CURSORPOSITION
 */
typedef struct __GAL_CURSORPOSITION
{
	GAL_HEADER
	DWORD  dwMemOffset;
	WORD   wXPos;
	WORD   wYPos;
	WORD   wXHot;
	WORD   wYHot;
} GAL_CURSORPOSITION, *PGAL_CURSORPOSITION;

/*
 * #define GALFN_SETCURSORSHAPE
 */
typedef struct __GAL_SETCURSORSHAPE
{
	GAL_HEADER
	DWORD  dwMemOffset;
	DWORD  dwAndMask[32]; /* Most gfx hardware support only 32x32 */
	DWORD  dwXorMask[32];
} GAL_SETCURSORSHAPE, *PGAL_SETCURSORSHAPE;


/*
 * #define GALFN_SETCURSORCOLORS
 */
typedef struct __GAL_CURSORCOLORS
{
	GAL_HEADER
	DWORD  dwBgColor;
	DWORD  dwFgColor;
} GAL_CURSORCOLORS, *PGAL_CURSORCOLORS;

/*
 * #define GALFN_SETSOLIDPATTERN
 */
typedef struct __GAL_SETSOLIDPATTERN
{
	GAL_HEADER
	DWORD  dwColor;
} GAL_SETSOLIDPATTERN, *PGAL_SETSOLIDPATTERN;

/*
 * #define GALFN_SETRASTEROPERATION
 */
typedef struct __GAL_SETRASTEROPERATION
{
	GAL_HEADER
	CHAR   cRop;
} GAL_SETRASTEROPERATION, *PGAL_SETRASTEROPERATION;

/*
 * #define GALFN_SETSOLIDSOURCE
 */
typedef struct __GAL_SETSOLIDSOURCE
{
	GAL_HEADER
	DWORD  dwColor;
} GAL_SETSOLIDSOURCE, *PGAL_SETSOLIDSOURCE;

/*
 * #define GALFN_PATTERNFILL
 */
typedef struct __GAL_PATTERNFILL
{
	GAL_HEADER
	WORD   wXPos;
	WORD   wYPos;
	WORD   wWidth;
	WORD   wHeight;
} GAL_PATTERNFILL, *PGAL_PATTERNFILL;

/*
 * #define GALFN_SETMONOPATTERN
 */
typedef struct __GAL_SETMONOPATTERN
{
	GAL_HEADER
    	DWORD  dwBgColor;
	DWORD  dwFgColor;
	DWORD  dwData0;
	DWORD  dwData1;
	CHAR   cTransparency;
} GAL_SETMONOPATTERN, *PGAL_SETMONOPATTERN;

/*
 * #define GALFN_SCREENTOSCREENBLT
 */
typedef struct __GAL_SCREENTOSCREENBLT
{
	GAL_HEADER
	WORD   wXStart;
	WORD   wYStart;
	WORD   wXEnd;
	WORD   wYEnd;
	WORD   wWidth;
	WORD   wHeight;
} GAL_SCREENTOSCREENBLT, *PGAL_SCREENTOSCREENBLT;


/*
 * #define GALFN_SCREENTOSCREENXBLT
 */
typedef struct __GAL_SCREENTOSCREENXBLT
{
	GAL_HEADER
	WORD   wXStart;
	WORD   wYStart;
	WORD   wXEnd;
	WORD   wYEnd;
	WORD   wWidth;
	WORD   wHeight;
	WORD   wColor;
} GAL_SCREENTOSCREENXBLT, *PGAL_SCREENTOSCREENXBLT;

/*
 * #define GALFN_BRESENHAMLINE
 */
typedef struct __GAL_BRESENHAMLINE
{
	GAL_HEADER
	WORD   wX1;
	WORD   wY1;
	WORD   wLength;
	WORD   wErr;
	WORD   wE1;
	WORD   wE2;
	WORD   wFlags;
} GAL_BRESENHAMLINE, *PGAL_BRESENHAMLINE;

/*
 * #define GALFN_VGAMODESWITCH 
 * #define GALFN_VGACLEARCRTEXT
 * #define GALFN_VGASETPITCH   
 * #define GALFN_VGARESTORE
 * #define GALFN_VGASAVE   
 * #define GALFN_VGASETMODE
 */

typedef struct __GAL_VGAREGS
{
	int xsize;
	int ysize;
	int hz;
	int clock;
	unsigned char miscOutput;
	unsigned char stdCRTCregs[GFX_STD_CRTC_REGS];
	unsigned char extCRTCregs[GFX_EXT_CRTC_REGS];
} GAL_VGAREGS, *PGAL_VGAREGS;


typedef struct __GAL_VGAMODEDATA
{
	GAL_HEADER
	DWORD  dwFlags;       /* Flags for this subfunction */
	GAL_VGAREGS sVgaRegs; /* CRT+SEQ+SEQ register data block */
	WORD   wXres;
	WORD   wYres;
	WORD   wBpp;
	WORD   wRefresh;
} GAL_VGAMODEDATA, *PGAL_VGAMODEDATA;


/******** Panel Support functions *********************/
/*
* #define GALFN_PNLSETPARAMS
* #define GALFN_PNLGETPARAMS
* #define GALFN_PNLINITPANEL
* #define GALFN_PNLSAVESTATE
* #define GALFN_PNLRESTORESTATE
*/
typedef struct __GAL_PNLPARAMS
{
	GAL_HEADER
	Pnl_PanelParams PanelParams;
} GAL_PNLPARAMS, *PGAL_PNLPARAMS;

/*
* #define GALFN_PNLBIOSENABLE
* #define GALFN_PNLBIOSINFO
*/
typedef struct __GAL_PNLBIOS
{
	GAL_HEADER
	int state;
	int XRes;
	int YRes;
	int Bpp;
	int Freq;
} GAL_PNLBIOS, *PGAL_PNLBIOS;

/*
 * #define GALFN_SETCRTSTATE
 * #define GALFN_GETCRTSTATE
 */
typedef struct __GAL_CRTSTATE
{
	GAL_HEADER
	WORD wCrtState;
} GAL_CRTSTATE, *PGAL_CRTSTATE;

#define GAL_TVSTATE       0x01
#define GAL_TVOUTPUT      0x02
#define GAL_TVFORMAT      0x04
#define GAL_TVRESOLUTION  0x08
#define GAL_TVALL         0x0F
/*
 * #define GALFN_SETTVPARAMS
 * #define GALFN_GETTVPARAMS
 * #define GALFN_SETENABLE
 * #define GALFN_GETENABLE
 * #define GALFN_ISTVMODESUPPORTED
 */
typedef struct __GAL_TVPARAMS
{
	GAL_HEADER
	DWORD dwFlags;
	WORD wWidth;
	WORD wHeight;
	WORD wStandard;
	WORD wType;
	WORD wOutput;
	WORD wResolution;
	BOOLEAN bState;

}GAL_TVPARAMS, *PGAL_TVPARAMS;

/*
 * #define GALFN_SETTVTIMING
 * #define GALFN_GETTVTIMING
 */
typedef struct __GAL_TVTIMING
{
    GAL_HEADER
	DWORD dwFlags; /* not used currently */
	unsigned long HorzTim;
	unsigned long HorzSync;
	unsigned long VertSync;
	unsigned long LineEnd;
	unsigned long VertDownscale;
	unsigned long HorzScaling;
	unsigned long TimCtrl1;
	unsigned long TimCtrl2;
	unsigned long Subfreq;
	unsigned long DispPos;
	unsigned long DispSize;
	unsigned long Debug;
	unsigned long DacCtrl;
	unsigned long DotClock;
}GAL_TVTIMING, *PGAL_TVTIMING;

/******** Video Support functions *********************/

typedef struct __GAL_SETVIDEOENABLE
{
 	GAL_HEADER
	BOOLEAN enable;
} GAL_SETVIDEOENABLE, *PGAL_SETVIDEOENABLE;

typedef struct __GAL_SETVIDEOFORMAT
{
	GAL_HEADER
	int format;
} GAL_SETVIDEOFORMAT, *PGAL_SETVIDEOFORMAT;

typedef struct __GAL_SETVIDEOSIZE
{
	GAL_HEADER
	unsigned short width;
	unsigned short height;
} GAL_SETVIDEOSIZE, *PGAL_SETVIDEOSIZE;

typedef struct __GAL_SETVIDEOOFFSET
{
	GAL_HEADER
	unsigned long offset;
} GAL_SETVIDEOOFFSET, *PGAL_SETVIDEOOFFSET;

typedef struct __GAL_SETVIDEOWINDOW
{
	GAL_HEADER
	short x;
	short y;
    	short w;
	short h;
} GAL_SETVIDEOWINDOW, *PGAL_SETVIDEOWINDOW;

typedef struct __GAL_SETVIDEOSCALE
{
	GAL_HEADER
	unsigned short srcw;
	unsigned short srch;
	unsigned short dstw;
	unsigned short dsth;
} GAL_SETVIDEOSCALE, *PGAL_SETVIDEOSCALE;

typedef struct __GAL_SETVIDEOFILTER
{
	GAL_HEADER
	int xfilter;
	int yfilter;
} GAL_SETVIDEOFILTER, *PGAL_SETVIDEOFILTER;

typedef struct __GAL_SETVIDEOCOLORKEY
{
	GAL_HEADER
	unsigned long key;
	unsigned long mask;
	int bluescreen;
} GAL_SETVIDEOCOLORKEY, *PGAL_SETVIDEOCOLORKEY;

typedef struct __GAL__SETVIDEODOWNSCALEENABLE
{
	GAL_HEADER
	int enable;
} GAL_SETVIDEODOWNSCALEENABLE, *PGAL_SETVIDEODOWNSCALEENABLE;

typedef struct __GAL__SETVIDEODOWNSCALECONFIG
{
	GAL_HEADER
	unsigned short type;
	unsigned short m;
} GAL_SETVIDEODOWNSCALECONFIG, *PGAL_SETVIDEODOWNSCALECONFIG;

typedef struct __GAL__SETVIDEODOWNSCALECOEFF
{
	GAL_HEADER
	unsigned short coef1;
	unsigned short coef2;
	unsigned short coef3;
	unsigned short coef4;
} GAL_SETVIDEODOWNSCALECOEFF, *PGAL_SETVIDEODOWNSCALECOEFF;

#define GAL_VIDEO_SOURCE_MEMORY 0x0
#define GAL_VIDEO_SOURCE_DVIP   0x1
typedef struct __GAL__SETVIDEOSOURCE
{
	GAL_HEADER
	int source;
} GAL_SETVIDEOSOURCE, *PGAL_SETVIDEOSOURCE;

typedef struct __GAL__SETVIDEOINTERLACED
{
	GAL_HEADER
	int enable;
} GAL_SETVIDEOINTERLACED, *PGAL_SETVIDEOINTERLACED;

typedef struct __GAL__SETVIDEOCOLORSPACE
{
	GAL_HEADER
	int enable;
} GAL_SETVIDEOCOLORSPACE, *PGAL_SETVIDEOCOLORSPACE;

typedef struct __GAL__SETVIDEOCURSOR
{
	GAL_HEADER
 	unsigned long key;
	unsigned long mask;
	unsigned short select_color2;
	unsigned long color1;
	unsigned long color2;
} GAL_SETVIDEOCURSOR, *PGAL_SETVIDEOCURSOR;

typedef struct __GAL__SETVIDEOREQUEST
{
	GAL_HEADER
	short x;
	short y;
} GAL_SETVIDEOREQUEST, *PGAL_SETVIDEOREQUEST;

typedef struct __GAL__SETVIDEOALPHAENABLE
{
	GAL_HEADER
	int enable;
} GAL_SETVIDEOALPHAENABLE, *PGAL_SETVIDEOALPHAENABLE;

typedef struct __GAL__SETVIDEOALPHAWINDOW
{
	GAL_HEADER
	short x;
	short y;
	unsigned short width;
	unsigned short height;
} GAL_SETVIDEOALPHAWINDOW, *PGAL_SETVIDEOALPHAWINDOW;

typedef struct __GAL__SETVIDEOALPHAVALUE
{
	GAL_HEADER
	unsigned char alpha;
	char delta;
} GAL_SETVIDEOALPHAVALUE, *PGAL_SETVIDEOALPHAVALUE;

typedef struct __GAL__SETVIDEOALPHAPRIORITY
{
	GAL_HEADER
	int priority;
} GAL_SETVIDEOALPHAPRIORITY, *PGAL_SETVIDEOALPHAPRIORITY;

typedef struct __GAL__SETVIDEOALPHACOLOR
{
	GAL_HEADER
	unsigned long color;
} GAL_SETVIDEOALPHACOLOR, *PGAL_SETVIDEOALPHACOLOR;

typedef struct __GAL__SETVIDEOALPHAREGION
{
	GAL_HEADER
	int region;
} GAL_SETVIDEOALPHAREGION, *PGAL_SETVIDEOALPHAREGION;

typedef struct __GAL__SETVIDEOOUTSIDEALPHA
{
	GAL_HEADER
	int enable;
} GAL_SETVIDEOOUTSIDEALPHA, *PGAL_SETVIDEOOUTSIDEALPHA;

typedef struct __GAL__SETVIDEOPALETTE
{
	GAL_HEADER
	int identity;
	unsigned long palette[256];
} GAL_SETVIDEOPALETTE, *PGAL_SETVIDEOPALETTE;


typedef struct __GAL_VIDEOINFO
{
	GAL_HEADER
	int enable;
	int format;
	int filter;

	unsigned long src_size;
	unsigned long dst_size;
	unsigned long line_size;
	unsigned long xclip;
	unsigned long offset;
	unsigned long scale;
	unsigned long position;

	int color_key_src;
	unsigned long color_key;
	unsigned long color_key_mask;

	int downscale_enable;

	unsigned short downscale_type;

	unsigned short downscale_mask;
	unsigned short downscale_coef1;
	unsigned short downscale_coef2;
	unsigned short downscale_coef3;
	unsigned short downscale_coef4;
} GAL_VIDEOINFO, *PGAL_VIDEOINFO;

/* Gal device function's prototye declarations */

/** Init **********************************************************/
BOOLEAN Gal_initialize_interface(void);
BOOLEAN Gal_cleanup_interface(void);
BOOLEAN Gal_get_adapter_info(PGAL_ADAPTERINFO pAdapterInfo);
BOOLEAN Gal_set_softvga_state(BOOLEAN);
BOOLEAN Gal_get_softvga_state(int *bState);
BOOLEAN Gal_set_crt_state(int);
BOOLEAN Gal_wait_until_idle(void);
BOOLEAN Gal_wait_vertical_blank(void);
BOOLEAN Gal_write_register(int type, unsigned long offset, unsigned long value,
			int size);

BOOLEAN Gal_read_register(int type, unsigned long offset, unsigned long *value,
			int size);
/** Display Engine ******************************************************/
BOOLEAN Gal_is_display_mode_supported(int xres, int yres, int bpp, int hz,
					int *supported);
BOOLEAN Gal_set_display_mode(int xres, int yres, int bpp, int hz);
BOOLEAN Gal_get_display_mode(int *xres, int *yres, int *bpp, int *hz);
BOOLEAN Gal_set_display_bpp(unsigned short bpp);
BOOLEAN Gal_get_display_bpp(unsigned short *bpp);
BOOLEAN Gal_set_display_pitch(unsigned short pitch);
BOOLEAN Gal_get_display_pitch(unsigned short *pitch);
BOOLEAN Gal_set_display_offset(unsigned long offset);
BOOLEAN Gal_get_display_offset(unsigned long *offset);
BOOLEAN Gal_get_refreshrate_from_dotclock(int xres, int yres, int bpp,int *hz,
					unsigned long frequency);
BOOLEAN Gal_get_display_timing(PGAL_DISPLAYTIMING pDisplayTiming);
BOOLEAN Gal_set_display_timing(PGAL_DISPLAYTIMING pDisplayTiming);
BOOLEAN Gal_set_fixed_timings( int pnlXres, int pnlYres, int totXres,
				int totYres, int bpp);
BOOLEAN Gal_set_display_palette_entry(unsigned long index,
					 unsigned long palette);
BOOLEAN Gal_get_display_palette_entry(unsigned long index, 
					unsigned long *palette);
BOOLEAN Gal_set_display_palette(PGAL_PALETTE);
BOOLEAN Gal_get_display_palette(PGAL_PALETTE);
BOOLEAN Gal_set_cursor_enable(int enable);
BOOLEAN Gal_get_cursor_enable(int *enable);
BOOLEAN Gal_set_cursor_colors(unsigned long bkcolor, unsigned long fgcolor);
BOOLEAN Gal_get_cursor_colors(unsigned long *bkcolor, unsigned long *fgcolor);
BOOLEAN Gal_set_cursor_position(unsigned long memoffset, 
                             unsigned short xpos, unsigned short ypos, 
                             unsigned short xhotspot, unsigned short yhotspot);
BOOLEAN Gal_get_cursor_position(unsigned long *memoffset, 
                             unsigned short *xpos, 
                             unsigned short *ypos, 
                             unsigned short *xhotspot, 
                             unsigned short *yhotspot);
BOOLEAN Gal_set_cursor_shape32(unsigned long memoffset, 
                            unsigned long *andmask, unsigned long *xormask);

/** Render ********************************************************/
BOOLEAN Gal_set_solid_pattern(unsigned long color);

BOOLEAN Gal_set_mono_pattern(unsigned long bgcolor, unsigned long fgcolor, 
				unsigned long data0, unsigned long data1,
				 unsigned char transparency);

BOOLEAN Gal_set_raster_operation(unsigned char rop);

BOOLEAN Gal_pattern_fill(unsigned short x, unsigned short y,
	 unsigned short width, unsigned short height);

BOOLEAN Gal_set_solid_source(unsigned long color);

BOOLEAN Gal_screen_to_screen_blt(unsigned short srcx, unsigned short srcy,
		unsigned short dstx, unsigned short dsty, unsigned short width,
		unsigned short height);

BOOLEAN Gal_screen_to_screen_xblt(unsigned short srcx, unsigned short srcy,
		unsigned short dstx, unsigned short dsty, unsigned short width, 
		unsigned short height, unsigned short color);

BOOLEAN Gal_bresenham_line(unsigned short x, unsigned short y,
		unsigned short length, unsigned short initerr,
		unsigned short axialerr, unsigned short diagerr,
		unsigned short flags);

/** Compression*******************************************************/
BOOLEAN Gal_set_compression_state(BOOLEAN);
BOOLEAN Gal_get_compression_state(int *flag);
BOOLEAN Gal_set_compression_parameters(unsigned long flags,
                unsigned long offset,
                unsigned short pitch, 
		unsigned short size);
BOOLEAN Gal_get_compression_parameters(unsigned long flags, 
                unsigned long *offset,
                unsigned short *pitch, 
		unsigned short *size);

/** VGA **********************************************************/
BOOLEAN Gal_vga_mode_switch(int active);
BOOLEAN Gal_vga_clear_extended(void);
BOOLEAN Gal_vga_pitch(PGAL_VGAMODEDATA pvregs, unsigned short pitch);
BOOLEAN Gal_vga_restore(PGAL_VGAMODEDATA pvregs);
BOOLEAN Gal_vga_save(PGAL_VGAMODEDATA pvregs);
BOOLEAN Gal_vga_mode(PGAL_VGAMODEDATA pvregs);

/** Panel **********************************************************/
BOOLEAN Gal_pnl_set_params(unsigned long flags, PPnl_PanelParams pParam);
BOOLEAN Gal_pnl_get_params(unsigned long flags, PPnl_PanelParams pParam);
BOOLEAN Gal_pnl_init(PPnl_PanelParams pParam);
BOOLEAN Gal_pnl_save(void);
BOOLEAN Gal_pnl_restore(void);
BOOLEAN Gal_pnl_powerup(void);
BOOLEAN Gal_pnl_powerdown(void);

BOOLEAN Gal_pnl_enabled_in_bios(int *state);
BOOLEAN Gal_pnl_info_from_bios(int *xres, int *yres, int *bpp, int *hz);

/** TV **********************************************************/
BOOLEAN Gal_tv_set_params(unsigned long flags, PGAL_TVPARAMS pTV);
BOOLEAN Gal_tv_get_params(unsigned long flags, PGAL_TVPARAMS pTV);
BOOLEAN Gal_tv_set_timings(unsigned long flags, PGAL_TVTIMING pTV);
BOOLEAN Gal_tv_get_timings(unsigned long flags, PGAL_TVTIMING pTV);
BOOLEAN Gal_set_tv_enable(int bState);
BOOLEAN Gal_get_tv_enable(int *bState);
BOOLEAN Gal_is_tv_mode_supported(unsigned long flags, PGAL_TVPARAMS pTV,
		int *bState);

/** Video **********************************************************/
BOOLEAN Gal_set_video_enable(int enable);
BOOLEAN Gal_set_video_format(int format);
BOOLEAN Gal_set_video_size(unsigned short width, unsigned short height);
BOOLEAN Gal_set_video_offset(unsigned long offset);
BOOLEAN Gal_set_video_window(short x, short y, short w, short h);
BOOLEAN Gal_set_video_scale(unsigned short srcw, unsigned short srch,
                            unsigned short dstw, unsigned short dsth);
BOOLEAN Gal_set_video_filter(int xfilter, int yfilter);
BOOLEAN Gal_set_video_color_key(unsigned long key, 
                             unsigned long mask, int bluescreen);
BOOLEAN Gal_set_video_downscale_enable(int enable);
BOOLEAN Gal_set_video_downscale_config(unsigned short type, unsigned short m);
BOOLEAN Gal_set_video_downscale_coefficients(unsigned short coef1,
                             unsigned short coef2,
                             unsigned short coef3,
                             unsigned short coef4);
BOOLEAN Gal_set_video_source(int source);
BOOLEAN Gal_set_video_interlaced(int enable);
BOOLEAN Gal_set_video_color_space(int enable);
BOOLEAN Gal_set_video_cursor(unsigned long key,
                             unsigned long mask,
                             unsigned short select_color2,
                             unsigned long color1,
                             unsigned long color2);
BOOLEAN Gal_set_video_request(short x, short y);
BOOLEAN Gal_set_alpha_enable(int enable);
BOOLEAN Gal_set_alpha_window(short x, short y, 
                             unsigned short width,
                             unsigned short height);
BOOLEAN Gal_set_alpha_value(unsigned char alpha, char delta);
BOOLEAN Gal_set_alpha_priority(int priority);
BOOLEAN Gal_set_alpha_color(unsigned long color);
BOOLEAN Gal_select_alpha_region(int region);
BOOLEAN Gal_set_video_outside_alpha(int enable);
BOOLEAN Gal_set_video_palette(unsigned long* palette);

#endif
