/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name 	  : app_window.h			   *
 *							   *
 * Project Name   : ASTB				   *
 *							   *
 * Module Name    : AppWindow				   *
 * 							   *
 * Author Name	  : Lakshmi B G				   *
 *							   *
 * Language	  : C++					   *
 * 							   *
 * Date		  : 2/02/01				   *
 *							   *
 * Description	  : This file defines the class for the    *
 * 		    the gui creation of the application    *
 * 		    window. The user interacts with the    *
 * 		    gui created by this class.		   *
 **********************************************************/

#ifndef _APPWINDOW_H
#define _APPWINDOW_H

/* File from installed netstream library */
#include <nscparam.h>	
#include <nscstrapi.h>

/* User includes */
#include "tv_screen.h"
#include "includes.h"
#include "tv_tuner_gui.h"
#include "netstrm_gui.h"
#include "dvd_gui.h"


/* Defines for selected input */
typedef enum { DVD = 1, NETSTREAM, VIDEO, TV} Input_t;

/* Defines to swith between normal and fullscreen */
typedef enum {GUI_MODE_NORMAL, GUI_MODE_FULLSCREEN} guiMode_t;

/* Few global function callbacks */
/* The main application callback */
void app_callback(Fl_Widget*, void* dum);

/* Callback for updating the progress bar */
void updatePrgBar(void *, Fl_Value_Slider*, Fl_Output*);

/* Application Main Window class declaration */
class AppMainWindow : public Fl_Window 
{
	public:
		/* The class constructor */
		AppMainWindow(int x,int y, int w,int h,int end_x,int end_y,
				char* label=0);
		/* Creates gui at the specified x ann y position with the 
		 * the specified width and height and the label named for 
                 * the application window 
		 */
		int create_gui(int x,int y, int w,int h,char* label=0);
	
		/* Callback functions defined for the specified widget */
		void callback_fun(Fl_Widget*);
		
		/* I/O device handle */	
		int handle(int event);

		/* Destructor */
		~AppMainWindow() {};

		/* Main window variables */	
		Fl_Box* 	main_window;	
		Fl_Group* 	main_grp;
		
		/* Bottom menu widgets. */
		/* Includes TvTuner, DVD, Netstream, Video buttons */
		Fl_Box* 	menu_box;	
		Fl_Group* 	menu_group;
		Fl_Button* 	main_button;
		Fl_Button* 	tuner_button;	
		Fl_Button* 	vcp_button;
		Fl_Button* 	net_button;
		Fl_Button* 	dvd_button;
		Fl_Button* 	exit_button;
		Fl_Button* 	pixbut;

		/* Option box for video controls, message box*/
		/* The box with the oval shape remote */
		Fl_Box* 	option_box;
		Fl_Group* 	option_group;

		/* Message box */
		Fl_Output* 	message_out;
		Fl_Scrollbar* 	hor;
		Fl_Scrollbar* 	ver;
	
		/* Video controls having sliders */
		Fl_Group* 	slider_group;
		Fl_Slider* 	slider_bright;
  		Fl_Slider* 	slider_saturation;
  		Fl_Slider* 	slider_hue;
		Fl_Button* 	videoctl_done;

		/* Interface with the  middleware objects */
		TV_Screen* 	screen;	   /* video screen class */
		TV_Tuner* 	tvtuner;   /* tuner class */
		NetStrmGUI* 	netstream; /* Netstream gui class */
		DvdGUI* 	dvdgui;	   /* Dvd gui class */
		Param* 		pParam;	   /* Parameter pass class */

		/* Window co-ordinates */
		int 		sx_win;    /* Starting x & y co-ordinates */
		int 		sy_win;
		int 		wid_win;   /* width and height of the win */
		int 		hei_win;
		int 		xres,yres; /* Resolution of the frame buffer */
};

#endif
