/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name      : keypad_dvd.cc                          *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Subsystem Name : Integrated Test App                    *
 *                                                         *
 * Module Name    : Keypad 				   *
 *                                                         *
 * Author Name    : Lakshmi B G                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           : 2/02/01                                *
 *      
 * Description	  : This file contains the definitions of  *
 *		    keypad features			   *
 **********************************************************/
  
#include <vmwdvdapi.h>

#include "keypad_dvd.h"

/* Global variables to toggle between one digit and two digit 
   numbered channels */
int g_two_dvd=0, g_one_dvd=0;
int val_dvd;

/************************************************************************
 * Syntax	: Keypad_Dvd(int,int,int,int,char*)			*
 * Description	: This is the class constructor				*
 * Parameters	: x,y
 *		  - Window starting co-ordinates 			*
 *		  w,h							*
 *		  - width and height of the co-ordinates 		*
 ***********************************************************************/
Keypad_Dvd::Keypad_Dvd(int x, int y, int w, int h, char *label=0)
	:Fl_Box(x,y,w,h,label)
{
      button_grp = new Fl_Group(x,y,w,h);
      topx = x;
      topy = y;
      create_gui();
      hide();
}

/************************************************************************
 * Syntax       : create_gui()                                          *
 * Description  : this function creates all the keypad components       *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Dvd::create_gui()
{
	button_grp->begin();
		b1_b = new Fl_Button(topx+10, topy+5, 20, 20, "1");
		b2_b = new Fl_Button(topx+30, topy+5, 20, 20,"2");
		b3_b = new Fl_Button(topx+50, topy+5, 20, 20, "3");
		b4_b = new Fl_Button(topx+70, topy+5, 20, 20, "4");
		b5_b = new Fl_Button(topx+10, topy+25, 20, 20, "5");
		b6_b = new Fl_Button(topx+30, topy+25, 20, 20, "6");
		b7_b = new Fl_Button(topx+50, topy+25, 20, 20, "7");
		b8_b = new Fl_Button(topx+70, topy+25, 20, 20, "8");
		b9_b = new Fl_Button(topx+10, topy+45, 20, 20, "9");
		b0_b = new Fl_Button(topx+30, topy+45, 20, 20, "0");
		two_b = new Fl_Button(topx+50, topy+45, 20, 20, "--");
	button_grp->end();

	b1_b->callback(keypad_keypress_dvd, this);
	b2_b->callback(keypad_keypress_dvd, this);
	b3_b->callback(keypad_keypress_dvd, this);
	b4_b->callback(keypad_keypress_dvd, this);
	b5_b->callback(keypad_keypress_dvd, this);
	b6_b->callback(keypad_keypress_dvd, this);
	b7_b->callback(keypad_keypress_dvd, this);
	b8_b->callback(keypad_keypress_dvd, this);
	b9_b->callback(keypad_keypress_dvd, this);
	b0_b->callback(keypad_keypress_dvd, this);
	two_b->callback(keypad_keypress_dvd, this);
}

/************************************************************************
 * Syntax       : hide()                                                *
 * Description  : this function hide all the keypad components          *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Dvd::hide()
{
 	Fl_Box::hide();
 	button_grp->hide();
}

/************************************************************************
 * Syntax       : show()                                                *
 * Description  : this function shows all the keypad components         *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Dvd::show()
{
 	Fl_Box::show();
 	button_grp->show();
}

void BTHEN(int val1, void* ptr)
{
	printf("Button 1 is pressed \n");	
	Keypad_Dvd* parent =  ptr;
	
	if(g_two_dvd == 1)
	{
		val_dvd = 10*val1;	
		g_two_dvd = 0;
		g_one_dvd = 1;
	}
	else 
	if(g_one_dvd == 1)
	{
		val_dvd = val_dvd + val1;
		printf("Change channel to %d \n",val_dvd);
		g_one_dvd = 0;
		VMW_DVD_Search(DVD_CHAPTER_SRCH,(unsigned long)&val_dvd);
		parent->hide();
	}
	else
	{
		val_dvd = val1;
		VMW_DVD_Search(DVD_CHAPTER_SRCH,(unsigned long)&val_dvd);
		parent->hide();
		
	}
}

/************************************************************************
 * Syntax       : keypad_keypress_dvd(Fl_Widget*, void*)                *
 * Description  : this is the callback function for the keys on dvd     *
 *                      channel keypad					*
 *									*
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void keypad_keypress_dvd(Fl_Widget* wid, void* ptr)
{
	Keypad_Dvd* parent = ptr;
	printf("key callback called\n");
 	if(wid == parent->b1_b)
		BTHEN(1,ptr);
	else if(wid == parent->b2_b)
		BTHEN(2,ptr);
	else if(wid == parent->b3_b)
		BTHEN(3,ptr);
	else if(wid == parent->b4_b)
		BTHEN(4,ptr);
	else if(wid == parent->b5_b)
		BTHEN(5,ptr);
	else if(wid == parent->b6_b)
		BTHEN(6,ptr);
	else if(wid == parent->b7_b)
		BTHEN(7,ptr);
	else if(wid == parent->b8_b)
		BTHEN(8,ptr);
	else if(wid == parent->b9_b)
		BTHEN(9,ptr);
	else if(wid == parent->b0_b)
		BTHEN(0,ptr);
 	else if(wid == parent->two_b)
	{
		g_two_dvd = 1;
	}
	else
		printf ("some invalid button pressed\n");
}
