/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name      : keypad_tun.cc                          *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Subsystem Name : Integrated Test App                    *
 *                                                         *
 * Module Name    : KeypadTuner 			   *
 *                                                         *
 * Author Name    : Lakshmi B G                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           : 2/02/01                                *
 *                 					   *
 * Description 	  : This file contains definition for 	   *
		    the keypad tuner class	           *
 **********************************************************/

#include <anlg_video.h>

#include "keypad_tun.h"

/* Global variables for selecting double digit channel or single digit
channel number */
int g_two=0, g_one=0;

/* Holds the number of the channel */
unsigned long int val;

/************************************************************************
 * Syntax       : Keypad_Tun(int,int,int,int,char*)                     *
 * Description  : Constructor						*
 * Parameters   : x,y - starting window positions 			*
 *		  w,h - width and height of the application		*
 *		  label - label for the widget			        *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
Keypad_Tun::Keypad_Tun(int x, int y, int w, int h, char *label=0):
	Fl_Box(x,y,w,h,label)
{
	button_grp = new Fl_Group(x,y,w,h);
        topx = x;
        topy = y;
        create_gui();
        hide();
}

/************************************************************************
 * Syntax       : create_gui()                                          *
 * Description  : this function creates all the keypad components	*
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Tun::create_gui()
{
	button_grp->begin();
		b1_b = new Fl_Button(topx+10, topy+15, 20, 20, "1");
		b1_b->labelfont(FL_BOLD);
		b2_b = new Fl_Button(topx+30, topy+15, 20, 20,"2");
		b2_b->labelfont(FL_BOLD);
		b3_b = new Fl_Button(topx+50, topy+15, 20, 20, "3");
		b3_b->labelfont(FL_BOLD);
		b4_b = new Fl_Button(topx+70, topy+15, 20, 20, "4");
		b5_b = new Fl_Button(topx+10, topy+35, 20, 20, "5");
		b6_b = new Fl_Button(topx+30, topy+35, 20, 20, "6");
		b7_b = new Fl_Button(topx+50, topy+35, 20, 20, "7");
		b8_b = new Fl_Button(topx+70, topy+35, 20, 20, "8");
		b9_b = new Fl_Button(topx+10, topy+55, 20, 20, "9");
		b0_b = new Fl_Button(topx+30, topy+55, 20, 20, "0");
		two_b = new Fl_Button(topx+50, topy+55, 20, 20, "--");

		b4_b->labelfont(FL_BOLD);
		b5_b->labelfont(FL_BOLD);
		b6_b->labelfont(FL_BOLD);
		b7_b->labelfont(FL_BOLD);
		b8_b->labelfont(FL_BOLD);
		b9_b->labelfont(FL_BOLD);
		b0_b->labelfont(FL_BOLD);
		two_b->labelfont(FL_BOLD);

	button_grp->end();

	b1_b->callback(keypad_keypress_tun, this);
	b2_b->callback(keypad_keypress_tun, this);
	b3_b->callback(keypad_keypress_tun, this);
	b4_b->callback(keypad_keypress_tun, this);
	b5_b->callback(keypad_keypress_tun, this);
	b6_b->callback(keypad_keypress_tun, this);
	b7_b->callback(keypad_keypress_tun, this);
	b8_b->callback(keypad_keypress_tun, this);
	b9_b->callback(keypad_keypress_tun, this);
	b0_b->callback(keypad_keypress_tun, this);
	two_b->callback(keypad_keypress_tun, this);
}
/************************************************************************
 * Syntax       : hide()                                                *
 * Description  : this function hides all the keypad components         *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Tun::hide()
{
 	Fl_Box::hide();
 	button_grp->hide();
}

/************************************************************************
 * Syntax       : show()                                                *
 * Description  : this function shows all the keypad components         *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Keypad_Tun::show()
{
 	Fl_Box::show();
 	button_grp->show();
}

#define BTHEN(val1)  {\
		printf ("button 1 pressed\n");\
		if (g_two == 1)\
		{\
			val = 10*val1;\
			g_two = 0;\
			g_one = 1;\
		}\
		else if(g_one == 1)\
		{\
			val = val+val1;\
printf ("change channel to %d\n", val);\
			VMW_AVT_SetTunerFrequency(&val);\
			g_one = 0;\
			parent->hide();\
		}\
		else\
		{\
printf ("channel changed to 1\n");\
			val = val1;\
			VMW_AVT_SetTunerFrequency(&val);\
			parent->hide();\
		}\
	}

/************************************************************************
 * Syntax       : keypad_keypress_tun(Fl_Widget* wid, void* ptr)	*
 * Description  : this is the callback function for the keys pressed in *
 *                      tv tuner keypad					*
 * Parameters   : pointer to the component on which action happened     *
 *                      pointer to the parent on the component		*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void keypad_keypress_tun(Fl_Widget* wid, void* ptr)
{
	Keypad_Tun* parent = ptr;
	printf("key callback called\n");
	SetAudio(AV_CHAN_TUNER,0,1);
 	if(wid == parent->b1_b)
		BTHEN(1)
	else if(wid == parent->b2_b)
		BTHEN(2)
	else if(wid == parent->b3_b)
		BTHEN(3)
	else if(wid == parent->b4_b)
		BTHEN(4)
	else if(wid == parent->b5_b)
		BTHEN(5)
	else if(wid == parent->b6_b)
		BTHEN(6)
	else if(wid == parent->b7_b)
		BTHEN(7)
	else if(wid == parent->b8_b)
		BTHEN(8)
	else if(wid == parent->b9_b)
		BTHEN(9)
	else if(wid == parent->b0_b)
		BTHEN(0)
 	else if(wid == parent->two_b)
	{
		g_two = 1;
	}
	else
		printf ("some invalid button pressed\n");
}

