/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name      : tv_screen.h                            *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Module Name    : Integrated Test App                    *
 *                                                         *
 * Author Name    : Sojan James                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           :                                        *
 *							   *
 * Description	  : Screen class for video display  	   *
 **********************************************************/	

#ifndef _TVSCREEN_H
#define _TVSCREEN_H


#include<iostream.h>
#include<FL/Fl_Tile.H>
#include<FL/fl_draw.H>
#include<string>

#include "includes.h"

#define TV_MAX_LABELS	5

/* Class for the tv-screen display on the application */
class TV_Screen : public Fl_Widget
{
	public:
		
	TV_Screen(int x, int y, int w, int h, char * label = 0);
	
	void formTile(TV_Screen* t);	
	void setImage(const unsigned char *data,int w, int h,int x, int y);
	void displayImage();
	void displayLevelBar();
	void setLevel(int level);
	void setLabel(const char * str,int lab);
	
	/* set color, face and size of font */
	void setLabelAttrib(Fl_Color clr,int face, int size,int lab);
	void setLabelPos(int x, int y, int lab);
	void drawLabels();
	void setBackground(unsigned char r, unsigned char g, unsigned char b);

	protected:
		void draw(); /* draw the widget */

	private:
		Fl_Tile* screen_tile;
		Fl_Output* tv_msg_output;
	
		int level_bar_color;
		int text_color;
	
		unsigned char * image_data;
		int image_height;
		int image_width;
		int image_x;
		int image_y;

		int level;
		Fl_Color bgcol;
		
		/* The color for the labels */
		Fl_Color labelColor[TV_MAX_LABELS];

		/* The text for the labels */
		string label[TV_MAX_LABELS];
	
		/* The x and y co-ordinates of the labels */
		int labelx[TV_MAX_LABELS],labely[TV_MAX_LABELS];

		/* The face and size of the labels */
		int face[TV_MAX_LABELS];
		int size[TV_MAX_LABELS];
};

#endif
